package ubiware.core;

import ubiware.core.commands.AddBeliefsN3Command;
import ubiware.core.commands.BlockingCommand;
import ubiware.core.commands.HasBeliefsN3Command;
import ubiware.core.commands.RemoveBeliefsN3Command;
import ubiware.core.commands.UbiwareAgentCommand;
import ubiware.util.saplbuilder.SaplDocument;

/**
 * NEVER use this class from inside the UbiwareAgent thread (This means, not from in the thread which goes trough RAB's) This will cause a
 * deadlock! Use in a newly created thread. example:
 * 
 * <pre>
 * <code>
 * class RabRunnableExampleBehavior extends ReusableAtomicBehavior {
 * 
 * 	
 * 	protected void doAction() {
 * 		//........
 * 		new Thread(new RabRunnable(this.myAgent) {
 * 			
 * 			public void run() {
 * 				//access to the agents beliefs
 * 				this.AddBeliefsN3Blocking(":bill :hasSon :John");
 * 				//in the next line, we're sure that the beliefs have been added, since this was the blocking variety!
 * 
 * 			}
 * 		}).start();
 * 		//..........
 * 
 * 	}
 * 
 * 	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {...}
 * 
 * }
 * </pre>
 * 
 * </code>
 * 
 * 15.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class AgentWrapper {
	/**
	 * the agent this is interacting with.
	 */
	private UbiwareAgent agent;

	public AgentWrapper(UbiwareAgent myAgent) {
		this.agent = myAgent;
	}

	/**
	 * Adds a command to the agent which will be executed in the future.
	 * 
	 * @see UbiwareAgent#addCommand(UbiwareAgentCommand)
	 * @param command
	 *            The command for the Ubiware agent
	 */
	public void addCommand(UbiwareAgentCommand<?> command) {
		this.agent.addCommand(command);
	}

	/**
	 * Add beliefs to the agent, after this call returns, it is guaranteed that the beliefs will be added in the future. The user cannot
	 * expect the add to happen immediately.
	 * 
	 * @param n3string
	 *            {@link UbiwareAgent#addBeliefsN3(String)}
	 */
	public void addBeliefsN3(SaplDocument doc) {
		UbiwareAgentCommand<?> command = new AddBeliefsN3Command(doc);
		this.agent.addCommand(command);
	}

	/**
	 * Add beliefs to the agent, after this call returns, the beliefs are added to the agents beliefs.
	 * 
	 * @param n3string
	 *            {@link UbiwareAgent#addBeliefsN3(String)}
	 * @return {@link UbiwareAgent#addBeliefsN3(String)}
	 */
	public boolean addBeliefsN3Blocking(SaplDocument doc) {
		return this.executeCommandInBlockingMode(new AddBeliefsN3Command(doc));
	}

	/**
	 * Removes beliefs from the agent, after this call returns, it is guaranteed that the beliefs will be removed in the future. The user
	 * cannot expect the removal to happen immediately.
	 * 
	 * @param n3String
	 *            {@link UbiwareAgent#removeBeliefsN3(String)}
	 */
	public void removeBeliefsN3(SaplDocument n3String) {
		UbiwareAgentCommand<?> command = new RemoveBeliefsN3Command(n3String);
		this.agent.addCommand(command);
	}

	/**
	 * Remove beliefs from the agent, after this call returns, the beliefs are removed from the agents beliefs.
	 * 
	 * @param n3string
	 *            {@link UbiwareAgent#removeBeliefsN3(String)}
	 */
	public void removeBeliefsN3Blocking(SaplDocument n3string) {
		this.executeCommandInBlockingMode(new RemoveBeliefsN3Command(n3string));
	}

	//this method would not make any sense, it is not a method which causes results on the agent, it just queries.
	//	@SuppressWarnings("unused")
	//	private void hasBeliefsN3(String n3String) {
	//		//this method would not make any sense, it is not a method which causes results on the agent, it just queries.
	//	}

	/**
	 * Check whether the agent has a certain belief and get values bound to variables.
	 * 
	 * @param n3string
	 *            {@link UbiwareAgent#hasBeliefsN3(String)}
	 * @return n3string {@link UbiwareAgent#hasBeliefsN3(String)}
	 */
	public BindingsSet hasBeliefsN3Blocking(SaplDocument n3string) {
		return this.executeCommandInBlockingMode(new HasBeliefsN3Command(n3string));
	}

	/**
	 * Wraps the command in a {@link BlockingCommand} wrapper and passes it to the agent. Then waits for execution and returns the result of
	 * the execution.
	 * 
	 * @param <E>
	 *            The type of the {@link UbiwareAgentCommand}
	 * @param command
	 *            the {@link UbiwareAgentCommand}
	 * @return the return value of {@link UbiwareAgentCommand#getResult()} after execution of the command
	 */
	private <E> E executeCommandInBlockingMode(UbiwareAgentCommand<E> command) {
		BlockingCommand<E> blockingCommand = new BlockingCommand<E>(command);
		this.agent.addCommand(blockingCommand);
		blockingCommand.waitForExecution();
		return blockingCommand.getResult();
	}
}
