package ubiware.core;

import java.io.Serializable;
import java.util.Map;

import ubiware.util.IDGenerator;
import ubiware.util.saplbuilder.Resource;

import com.google.common.collect.Maps;

/**
 * Representation of the blackboard of an agent, the agent uses its blackboard to keep references to java objects. The blackboard works
 * thread safe.
 * 
 * 12.2.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class Blackboard implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * The actual blackboard, mapping Strings (IDs) to Objects
	 */
	final Map<Resource, Object> blackboard = Maps.newHashMap();

	/**
	 * this object is kept for generating new IDs for objects put to the {@link Blackboard}
	 */
	final static IDGenerator idgenerator = new IDGenerator(UbiwareAgent.BLACKBOARD_OBJECT_PREFIX);

	/**
	 * Add an object to the blackboard, the returned ID can be used later to retrieve the object from the Blackboard
	 * 
	 * @param object
	 *            The object which should be put on the Blackboard.
	 * @return The ID assigned to the object put on the Blackboard
	 */
	public synchronized Resource addObject(Object object) {
		Resource id = Blackboard.idgenerator.getNewResource();
		this.blackboard.put(id, object);
		return id;
	}

	/**
	 * remove an object from the Blackboard if it exists there. If the object existed, it will be returned.
	 * 
	 * @param ID
	 *            The ID of the desired object.
	 * @return The Object of which removal is requested.
	 */
	public synchronized Object removeObject(Resource ID) {
		return this.blackboard.remove(ID);
	}

	/**
	 * get an object from the Blackboard if it exists there. Otherwise null will be returned.
	 * 
	 * @param ID
	 *            The ID of the desired object.
	 * @return The Object which is requested.
	 */
	public synchronized Object getObject(Resource ID) {
		return this.blackboard.get(ID);
	}
}
