package ubiware.core;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Extracted form Ubiware agent, this class provides methods for replacing the whole name of a resource by their prefix example:
 * <http://www.ubiware.jyu.fi/sapl#MyResource> to sapl:MyResource
 * 
 * This class is mainly used for producing nicer output.
 * 
 * 8.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class ResourcePrefixer {
/**
	 * The namespaces known by this prefixer
	 * maps namespaces (actually '<' + namespace) to prefixes 
	 */
	private Map<String, String> knownNamespaces = new HashMap<String, String>();

	/**
	 * prefix a resource using known namespaces
	 * 
	 * @param resource
	 *            The resource whose url should be replaced by its prefix
	 * @return The prefixed string
	 */
	public String prefixResource(String resource) {
		if (resource.startsWith("<" + UbiwareAgent.BLANK_NODE_NS)) {
			return resource.substring(1, resource.length() - 1);
		}

		for (String namespace : this.knownNamespaces.keySet()) {
			if (resource.startsWith(namespace)) {
				String prefix = this.knownNamespaces.get(namespace);
				//remove the namespace from the resource:
				String resourceWithoutNamespace = resource.substring(namespace.length(), resource.length() - 1);
				return prefix + resourceWithoutNamespace;
			}
		}
		//no prefix for the resource found, return integrally.
		return resource;
	}

	/**
	 * prefix a resource using known namespaces
	 * 
	 * @param resource
	 *            The resource whose url should be replaced by its prefix
	 * @param usedNS
	 *            a map to which the namespace - prefix pair which was used will be added
	 * @return The prefixed string
	 */
	public String prefixResource(String resource, Map<String, String> usedNS) {
		if (resource.startsWith("<" + UbiwareAgent.BLANK_NODE_NS)) {
			return resource.substring(1, resource.length() - 1);
		}

		for (String namespace : this.knownNamespaces.keySet()) {
			if (resource.startsWith(namespace)) {
				String prefix = this.knownNamespaces.get(namespace);
				usedNS.put(namespace.substring(1), prefix);
				//remove the namespace from the resource:
				String resourceWithoutNamespace = resource.substring(namespace.length(), resource.length() - 1);
				return prefix + resourceWithoutNamespace;
			}
		}
		//no prefix for the resource found, return integrally.
		return resource;
	}

	/**
	 * Adds namespaces to this ResourcePrefixer. If a previous namespace was defined with the another prefix, the old value will be
	 * overwritten.
	 * 
	 * @param namespaces
	 *            The pairs of prefix -> namespace
	 */
	void addNameSpaces(HashMap<String, String> namespaces) {
		for (Entry<String, String> entry : namespaces.entrySet()) {
			String prefix = entry.getKey();
			String namespace = entry.getValue();
			this.knownNamespaces.put('<' + namespace, prefix); //We add '<' to speed up matching later
		}
	}

}
