package ubiware.core;

import jade.core.AID;
import jade.core.behaviours.Behaviour;
import jade.lang.acl.ACLMessage;

import java.util.HashMap;
import java.util.List;
import java.util.Set;

import ubiware.core.commands.UbiwareAgentCommand;
import ubiware.util.saplbuilder.ParseException;
import ubiware.util.saplbuilder.Parser;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

/**
 * {@link ReusableAtomicBehavior} provides a base class for all behaviours which this agent is running. These behaviours are normally
 * instantiated by the {@link UbiwareAgent} live cycle. The behaviour implements a function for the agent.</br> Note: when using
 * addBeliefsN3, removeBeliefsN3, and hasBeliefsN3 with String arguments, one must either use the full URIs or include needed @prefix
 * statements into the Notation3 string.
 * 
 * @author Michael Cochez for UBIWARE project
 */
public abstract class ReusableAtomicBehavior implements ParametrizedAsset {

	/**
	 * Is this RAB finished? Should it still run? Since every RAB is by default a one shot behavior, initially set to true. Deriving RAB's
	 * can change this value to false to indicate that the behavior has not finished yet.
	 */
	protected boolean finished = true; //
	/**
	 * Did the RAB execution end in a success? the default is that it ends in success, once the flag is set by {@link #setFailed()}, the RAB
	 * will be failed forever. The status of the flag can be requested by {@link #hasSucceededTillNow()}
	 */
	private boolean success = true; //by default, ends in success

	/**
	 * Reference to the agent on which this RAB is scheduled. Enables behaviors to call public methods of UbiwareAgent such as send(),
	 * receive(), doMove() and other
	 * 
	 * @deprecated This reference is deprecated since RAB's should not depend directly on the implementation of the agent for functionality.
	 */
	@Deprecated
	protected UbiwareAgent myAgent;

	/**
	 * The parameters for execution of the RAB
	 */
	private BehaviorStartParameters startParameters;

	/**
	 * The ID of this RAB.
	 */
	String ID = null;
	/**
	 * If I understand correctly, the linkedcontexts keeps IDs of contexts which are referenced from inside the rab in order to prevent
	 * removal by the Ubiware Agent
	 */
	Set<String> linkedContexts;
	/**
	 * Indicates if this RAB still waits its first (maybe the only) execution
	 */
	private boolean executed = false;
	/**
	 * The behavior which encapsulates this RAB to be added to the underlying jade agent.
	 */
	private Behaviour encapsulatingBehavior;

	/**
	 * Every RAB must have at least a default constructor. This because the agent runtime will instantiate the behavior using java
	 * reflection searching for a default constructor.
	 */
	public ReusableAtomicBehavior() {
		super();
	}

	/**
	 * This method is called by the agent when this RAB must perform its action. The action will only be called if this RAB was successful
	 * until now. This means that parameterinitialisation and any previous doAction call returned without trowing exceptions.
	 */
	final void action() {
		if (this.hasSucceededTillNow()) {
			try {
				try {
					this.doAction();
				} catch (Exception e1) {
					throw new Exception(String.format("Behavior %s from agent %s throws exception from doAction()", this.getClass()
							.getName(), this.myAgent.getName()), e1);
				}
			} catch (Exception e) {
				this.setFailed();
				e.printStackTrace();
			} catch (Throwable e) {
				this.setFailed();
				throw new Error(String.format("Behavior %s from agent % s throws exception from doAction", this.getClass().getName(),
						this.myAgent.getName()), e);
			}
		}
	}

	/**
	 * Derived RAB's should implement this method and do their action within it. The derived RAB can assume that doAction is never called
	 * without a successful call to {@link ReusableAtomicBehavior#initializeRAB(BehaviorStartParameters)}
	 */
	protected abstract void doAction() throws Exception;

	/**
	 * Set the agent which is executing this RAB. This is done by the ubiware agent itself once this RAB is added to the agent.
	 * 
	 * @param agent
	 */
	final void setAgent(UbiwareAgent agent) {
		this.myAgent = agent;
	}

	/**
	 * Set the behavior which encapsulates this RAB. This is done by the ubiware agent itself once this RAB is added to the agent.
	 * 
	 * @param behavior
	 */
	final void setEncapsulatingBehavior(Behaviour behavior) {
		this.encapsulatingBehavior = behavior;
	}

	/**
	 * Marks this RAB as being failed, this used to be done by setting the success flag to false (<CODE>this.success = false</CODE>) This
	 * new approach must ensure that once the RAB has been marked as failed, it cannot be marked as a success again. (The flag can only be
	 * set, not unset)
	 */
	public final void setFailed() {
		this.success = false;
	}

	/**
	 * returns true iff the RAB has succeeded in its operation up to this point. (The setFailed() method has not been called)
	 * 
	 * @return only true as long as setfailed has not been called.
	 */
	public final boolean hasSucceededTillNow() {
		return this.success;
	}

	/**
	 * Blocks this behaviour. If this method is called from within action() method, behaviour suspension occurs as soon as action() returns.
	 * And as long as no new message arrives or no explicit restart is called on the behavior.
	 */
	public final void block() {
		this.encapsulatingBehavior.block();
	}

	/**
	 * Blocks this behaviour the specified amount of time. If this method is called from within action() method, behaviour suspension occurs
	 * as soon as action() returns. The behaviour will be restarted when among the three following events happens. A time of millis
	 * milliseconds has passed since the call to block(). An ACL message is received by the agent this behaviour belongs to. Method
	 * restart() is called explicitly on this behaviour object.
	 * 
	 * @param millis
	 *            How many milliseconds should this behavior be blocked.
	 */
	public final void block(long millis) {
		this.encapsulatingBehavior.block(millis);
	}

	/**
	 * Restarts a blocked behaviour.
	 */
	public final void restart() {
		this.encapsulatingBehavior.restart();
	}

	/**
	 * Set the start parameters for this behavior.
	 */
	@Override
	public final void setStartParameters(BehaviorStartParameters startParams) {
		this.startParameters = startParams;
	}

	/**
	 * callled by the ubiware agent when this RAB starts, calls in turn this{@link #onRABStart()} and this
	 * {@link #initializeRAB(BehaviorStartParameters)}
	 */
	final void onStart() {
		//print(getClass().getName());		

		this.executed = true;
		List<String> addBeliefs = this.startParameters.addOnStart;
		List<String> removeBeliefs = this.startParameters.removeOnStart;

		for (String contextID : removeBeliefs) {
			this.removeBeliefs(contextID);
		}

		for (String contextID : addBeliefs) {
			this.addBeliefs(contextID, null);
		}

		if ((addBeliefs.size() != 0) || (removeBeliefs.size() != 0)) {
			this.wakeAgent();
		}
		//this.onRABStart();
		
		
		try {
			try {
				this.initializeRAB(this.startParameters);
			} catch (Exception e1) {
				throw new Exception(String.format("Behavior %s from agent %s throws exception from doAction()", this.getClass()
						.getName(), this.myAgent.getName()), e1);
			}
		} catch (Exception e) {
			this.setFailed();
			e.printStackTrace();
		} catch (Throwable e) {
			this.setFailed();
			throw new Error(String.format("Behavior %s from agent % s throws exception from doAction", this.getClass().getName(),
					this.myAgent.getName()), e);
		}
	}

	/**
	 * Derived RAB's should implement this method and initialise all the needed parameters to run the RAB. The supplied parameters object
	 * might become invalid after this call.
	 * 
	 * @param parameters
	 *            The parameters specified by the user of this RAB.
	 * 
	 * @throws IllegalParameterConfigurationException
	 *             If one of the parameters isn't correctly given.
	 */
	protected abstract void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException;

	//	/**
	//	 * Method called once when this RAB is started. Hook to be overridden if desired.
	//	 */
	//	public void onRABStart() {
	//		//hook
	//	}

	/**
	 * Callled by the ubiware agent when this RAB ends, calls in turn onRABEnd
	 */
	final int onEnd() {
		this.onRABEnd();
		List<String> addBeliefs;
		List<String> removeBeliefs;
		if (this.success) {
			addBeliefs = this.startParameters.addOnSuccess;
			removeBeliefs = this.startParameters.removeOnSuccess;
		} else {
			addBeliefs = this.startParameters.addOnFail;
			removeBeliefs = this.startParameters.removeOnFail;
		}

		addBeliefs.addAll(this.startParameters.addOnEnd);
		removeBeliefs.addAll(this.startParameters.removeOnEnd);

		for (String contextID : removeBeliefs) {
			this.removeBeliefs(contextID);
		}

		for (String contextID : addBeliefs) {
			this.addBeliefs(contextID, null);
		}

		if (this.linkedContexts != null) {
			for (String linkedContextID : this.linkedContexts) {
				ContextContainer context = this.myAgent.contexts.get(linkedContextID);
				if (context != null) {
					context.referenced--;
				}
			}
		}

		//if(addBeliefs.size()!=0 || removeBeliefs.size()!=0) wakeAgent();
		if (this.ID != null) {
			this.myAgent.removeRABInstance(this);
		}
		return 0;
	}

	/**
	 * Method called once when this RAB is ended. Hook to be overridden if desired for example if resources have to be cleaned up. This
	 * method will also be called if the initialisation of the RAB failed.
	 */
	public void onRABEnd() {
		//hook
	}

	/**
	 * @return Has this behavior finished its job and can it be removed from the agent scheduler.
	 */
	public final boolean done() {
		return this.finished;
	}

	// Interface to subclasses, i.e. RABs

	/**
	 * Prints to the screen (and to the log) specified text. Text is printed preceded by �[<current time>] <agent�s name>:"
	 * 
	 * @param text
	 *            The Object from which the text must be made (using {@link Object#toString()})
	 */
	public void print(Object text) {
		this.myAgent.print(text);
	}

	/**
	 * Informs the agent that it must wake up its Live behavior to check whether new rules became applicable.
	 */
	public final void wakeAgent() {
		this.myAgent.restartLive();
	}

	// parameters cannot be queried anymore. They are supplied once in the call to initialiseRAB.
	//	/**
	//	 * Get one value for the given parameter name.
	//	 * 
	//	 * @param name
	//	 *            the name of the parameter.
	//	 * @return its associated value.
	//	 */
	//	public String getParameterValue(String name) {
	//		return this.startParameters.getParameterValue(name);
	//	}
	//
	//	/**
	//	 * Get one value for the given parameter name taking the namespace into account.
	//	 * 
	//	 * @param name
	//	 *            the name of the parameter.
	//	 * @param ns
	//	 *            the namespace the parameter name resides in.
	//	 * @return its associated value.
	//	 */
	//	public String getParameterValue(String name, String ns) {
	//		return this.startParameters.getParameterValue(name, ns);
	//	}
	//
	//	/**
	//	 * Get all values for the given parameter name.
	//	 * 
	//	 * @param name
	//	 *            the name of the parameter.
	//	 * @return its associated value.
	//	 */
	//	public List<String> getParameterValues(String name) {
	//		return this.startParameters.getParameterValues(name);
	//	}
	//
	//	/**
	//	 * Get all values for the given parameter name taking the namespace into account.
	//	 * 
	//	 * @param name
	//	 *            the name of the parameter.
	//	 * @param ns
	//	 *            the namespace the parameter name resides in.
	//	 * @return its associated value.
	//	 */
	//	public List<String> getParameterValues(String name, String ns) {
	//		return this.startParameters.getParameterValues(name, ns);
	//	}

	/**
	 * adds beliefs (copies a sub-graph given with ID can be of a context or a statement) with binding variables (if given)
	 * 
	 * @param contextOrStatementID
	 *            the the statement or context which should be copied
	 * @param vars
	 *            The variable association which should be used in the copy action.
	 * @deprecated Use the version with S-APL document if possible!
	 */
	@Deprecated
	public final void addBeliefs(String contextOrStatementID, HashMap<String, String> vars) {
		this.myAgent.addBeliefs(contextOrStatementID, vars);
	}

	/**
	 * adds beliefs defined by a N3 string
	 * 
	 * @param n3string
	 *            the S-APL string which must be added to the agent.
	 * @param doChecks
	 *            should checks be performed for joining containers
	 * @return true is beliefs are added. False if either adding beliefs failed or no beliefs where added.
	 * @deprecated use {@link #addBeliefsN3(SaplDocument)} instead for type safety
	 */
	@Deprecated
	public final boolean addBeliefsN3(String n3string, boolean doChecks) {
		return this.myAgent.addBeliefsN3(n3string, doChecks);
	}

	/**
	 * Same as {@link #addBeliefsN3(String, boolean)} with doChecks set to true
	 * 
	 * @param n3string
	 *            The string of S-APL which should get added to the agent's beliefs structure.
	 * @return true if beliefs are added. False if either adding beliefs failed or no beliefs where added.
	 * @deprecated use {@link #addBeliefsN3(SaplDocument)} instead for type safety
	 */
	@Deprecated
	public final boolean addBeliefsN3(String n3string) {
		return this.myAgent.addBeliefsN3(n3string, true);
	}

	/**
	 * This is the advised way to add S-APL beliefs to the agent. Create an S-APL document and feed it to this function from within the
	 * agent's thread. For example from inside an RAB.
	 * 
	 * @param document
	 *            The S-APL beliefs' structure which should be added to the agent.
	 */
	public final void addBeliefsN3(SaplDocument document) {
		boolean addingSuccessFull = this.addBeliefsN3(document.generateSapl());
		if (!addingSuccessFull) {
			throw new Error(
					"adding s-apl originating from a s-apl document must succeed since sapldocument should onlyproduce valid s-apl.\nViolating code\n"
							+ document.generateSapl());
		}
	}

	/**
	 * remove beliefs by pattern given (ID can be of a context or a statement)
	 * 
	 * @param beliefID
	 *            the ID of the context or statement specifying the beliefs to be removed.
	 * @deprecated Use the version with S-APL document if possible!
	 */
	@Deprecated
	public final void removeBeliefs(String beliefID) {
		this.myAgent.removeBeliefs(beliefID);
	}

	/**
	 * Removes beliefs according to pattern specified in an S-APL (Notation3) string.
	 * 
	 * @param n3string
	 *            The String containing the S-APL code to be added to the agent.
	 * @deprecated Use {@link #removeBeliefsN3(SaplDocument)} instead
	 */
	@Deprecated
	public final void removeBeliefsN3(String n3string) {
		this.myAgent.removeBeliefsN3(n3string);
	}

	/**
	 * Remove beliefs according to pattern specified in a S-APL document
	 * 
	 * @param document
	 *            The document containing the S-APL pattern to be removed to the agent.
	 */
	public final void removeBeliefsN3(SaplDocument document) {
		this.myAgent.removeBeliefsN3(document.generateSapl());
	}

	/**
	 * Erases a statement or a context specified by its ID.: ID can be of a context or a statement
	 * 
	 * @param beliefID
	 */
	public final void eraseBeliefs(String beliefID) {
		this.myAgent.eraseBeliefs(beliefID);
	}

	/**
	 * returns possible bindings of variables to beliefs given (with ID can be of a context or a statement)
	 * 
	 * @param contextOrStatementID
	 * @param queryContextID
	 * @return The set with bindings for the query.
	 */

	public final BindingsSet hasBeliefs(String contextOrStatementID, String queryContextID) {
		return this.myAgent.hasBeliefs(contextOrStatementID, queryContextID);
	}

	/**
	 * Same as {@link #hasBeliefs(String, String)} with queryContextID set to {@link UbiwareAgent#GENERAL_CONTEXT}
	 * 
	 * @param statementID
	 * @return The set with bindings for the query.
	 */
	public final BindingsSet hasBeliefs(String statementID) {
		return this.myAgent.hasBeliefs(statementID);
	}

	/**
	 * Returns all possible bindings of variables (from the specified queryContext) to the query specified in the S-APL document
	 * 
	 * @param document
	 *            The document containing the query
	 * @param queryContextID
	 *            The context into which matches to the query must be searched.
	 * @return A {@link BindingsSet} containing all possible bindings to the variables specified.
	 */

	public final BindingsSet hasBeliefsN3(SaplDocument document, String queryContextID) {
		return this.myAgent.hasBeliefsN3(document.generateSapl(), queryContextID);
	}

	/**
	 * Returns all possible bindings of variables (from the agents global context G) to the query specified in the S-APL document Returns
	 * possible solutions (combinations of bindings of variables) to match beliefs specified in an S-APL (Notation3) string. Returns null if
	 * no match is found.
	 * 
	 * @param document
	 *            The document containing the query
	 * @return A {@link BindingsSet} containing all possible bindings to the variables specified.
	 */
	public final BindingsSet hasBeliefsN3(SaplDocument document) {
		return this.myAgent.hasBeliefsN3(document.generateSapl());
	}

	/**
	 * Returns all possible bindings of variables (from the specified queryContext) to the query specified in the S-APL string Returns
	 * possible solutions (combinations of bindings of variables) to match beliefs specified in an S-APL (Notation3) string. Returns null if
	 * no match is found.
	 * 
	 * @param n3string
	 *            The String containing the query
	 * @param queryContextID
	 *            The context into which matches to the query must be searched.
	 * @return A {@link BindingsSet} containing all possible bindings to the variables specified.
	 * @deprecated Use {@link #hasBeliefsN3(SaplDocument, String)} instead
	 */
	@Deprecated
	public final BindingsSet hasBeliefsN3(String n3string, String queryContextID) {
		return this.myAgent.hasBeliefsN3(n3string, queryContextID);
	}

	/**
	 * Returns all possible bindings of variables (from the agents global context G) to the query specified in the S-APL document Returns
	 * possible solutions (combinations of bindings of variables) to match beliefs specified in an S-APL (Notation3) string. Returns null if
	 * no match is found.
	 * 
	 * @param n3string
	 *            The String containing the query
	 * @return A {@link BindingsSet} containing all possible bindings to the variables specified.
	 * @deprecated Use {@link #hasBeliefsN3(SaplDocument, String)} instead
	 */
	@Deprecated
	public final BindingsSet hasBeliefsN3(String n3string) {
		return this.myAgent.hasBeliefsN3(n3string);
	}

	/**
	 * moves all links from a context to General Context and removes the former context
	 * 
	 * @param contextID
	 */
	public final void moveBeliefs(String contextID) {
		this.myAgent.moveBeliefs(contextID);
	}

	/**
	 * Same as {@link #produceN3(String, ProduceN3Options)} with the default options.
	 * 
	 * @param contextID
	 *            The context for which S-APL must be generated.
	 * @return The produced S-APL code
	 * @deprecated use the version which returns a document instead.
	 */
	@Deprecated
	public final String produceN3(String contextID) {
		return this.myAgent.produceN3(contextID);
	}

	/**
	 * Returns an S-APL (Notation3) string corresponding to a sub-graph starting with context container given. The class ProduceN3Options
	 * has following boolean fields: indents (default true), cleanContent (default false), and useNameSpaces (default true). If indents =
	 * true, the line breaks and indentation are used. If cleanContent = true, the content is cleaned in the sense that potential query
	 * constructs such as Optional, or, filtering are not included. If useNameSpaces = true, the known namespaces are substituted with their
	 * prefixes.
	 * 
	 * @param contextID
	 *            The context ID for which N3 must be produced.
	 * @param options
	 *            The options for production.
	 * @return The produced N3
	 * @deprecated use the version which returns a document instead.
	 */
	@Deprecated
	public final String produceN3(String contextID, ProduceN3Options options) {
		return this.myAgent.produceN3(contextID, options);
	}

	//FIXME instead of producing and then re-parsing, the generation should immediately create a proper document.
	public final SaplDocument produceN3Document(String contextID) {

		String sapl = this.myAgent.produceN3(contextID);
		if (sapl == null) {
			throw new Error("result from produceN3 was null");
		}
		try {
			return Parser.parse(sapl);
		} catch (ParseException e) {
			throw new Error("sapl parsed from the generation should never be wrong.");
		}
	}

	/**
	 * * Returns an S-APL (Notation3) string corresponding to a sub-graph starting with context container given. The class ProduceN3Options
	 * has following boolean fields: indents (default true), cleanContent (default false), and useNameSpaces (default true). If indents =
	 * true, the line breaks and indentation are used. If cleanContent = true, the content is cleaned in the sense that potential query
	 * constructs such as Optional, or, filtering are not included. If useNameSpaces = true, the known namespaces are substituted with their
	 * prefixes.
	 * 
	 * @param contextID
	 *            The context ID for which N3 must be produced.
	 * @param options
	 *            The options for production.
	 * @return The produced N3
	 */
	//FIXME instead of producing and then re-parsing, the generation should immediately create a proper document.
	public final SaplDocument produceN3Document(String contextID, ProduceN3Options options) {

		String sapl = this.myAgent.produceN3(contextID, options);
		if (sapl == null) {
			throw new Error("result from produceN3 was null");
		}
		try {
			return Parser.parse(sapl);
		} catch (ParseException e) {
			throw new Error("sapl parsed from the generation should never be wrong.");
		}
	}

	/**
	 * returns the vector of copies of statements in a context container (+ a fills a vector of their original IDs)
	 * 
	 * @param contextID
	 *            the context which should be copied.
	 * @param ids
	 *            This is an in-out parameter! To this list, the IDs of the original statements will be added.
	 * @return The copies.
	 */
	public final List<SemanticStatement> getStatements(String contextID, List<String> ids) {
		return this.myAgent.getStatements(contextID, ids);
	}

	/**
	 * Returns the vector of copies of statements in a context container. SemanticStaments objects have three String data members: subject,
	 * predicate and object.
	 * 
	 * @param contextID
	 *            The context ID which should eb copied.
	 * @return The list of semantic statements. Keep in mind that this list is in unspecified order.
	 */
	public final List<SemanticStatement> getStatements(String contextID) {
		return this.myAgent.getStatements(contextID);
	}

	/**
	 * Add an object to blackboard, return the ID of the created object
	 * 
	 * @param o
	 *            the Object which should be put on the {@link Blackboard} of the {@link UbiwareAgent}
	 * @return The ID of the created object which can be used for later retrieval.
	 */
	public final Resource addOnBlackboard(Object o) {
		return this.myAgent.addOnBlackboard(o);
	}

	/**
	 * Removes an object from blackboard.
	 * 
	 * @param id
	 *            The ID of the {@link Blackboard} object
	 */
	public final void removeFromBlackboard(Resource id) {
		this.myAgent.removeFromBlackboard(id);
	}

	/**
	 * returns an object from blackboard of the {@link UbiwareAgent}
	 * 
	 * @param id
	 *            the id of the {@link Blackboard} object
	 * @return The Object associated with this ID.
	 */
	public final Object getFromBlackboard(Resource id) {
		return this.myAgent.getFromBlackboard(id);
	}

	/**
	 * Has this RAB been executed at least once?
	 * 
	 * @return only true if this RAB has been executed before.
	 */
	public final boolean hasBeenExecuted() {
		return this.executed;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return super.hashCode();
	}

	/**
	 * Two rab's are only equal if they are the same RAB!
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		return super.equals(obj);
	}

	/**
	 * Forwards the message this.getClass().getName() + "error: " + message to {@link ReusableAtomicBehavior#print(Object)}
	 */
	protected final void printError(String message) {
		this.print(this.getClass().getName() + "error: " + message);
	}

	protected void sendMessage(int performative, Resource receiver, SaplDocument messageContent, String ontology, String conversationID) {
//		if (!CheckAgentExistance.check(this.myAgent, receiver)) {
//			throw new IllegalArgumentException("No such agent on the platform " + receiver);
//		}
//		ACLMessage message = new ACLMessage(performative);
//		message.addReceiver(new AID(receiver.toString(), AID.ISLOCALNAME));
//		message.setContent(messageContent.generateSapl());
//		message.setOntology(ontology);
//		message.setConversationId(conversationID);
		this.sendMessage(performative, receiver, messageContent.generateSapl(), ontology, conversationID);
	}

	/**
	 * Same as the other send message, but with string content
	 * 
	 * @param performative
	 * @param receiver
	 * @param messageContent
	 * @param ontology
	 * @param conversationID
	 * @deprecated Use the version with sapl document instead.
	 */
	@Deprecated
	protected void sendMessage(int performative, Resource receiver, String messageContent, String ontology, String conversationID) {
		if (!CheckAgentExistance.check(this.myAgent, receiver)) {
			throw new IllegalArgumentException("No such agent on the platform " + receiver);
		}
		ACLMessage message = new ACLMessage(performative);
		message.addReceiver(new AID(receiver.toString(), AID.ISLOCALNAME));
		message.setContent(messageContent);
		message.setOntology(ontology);
		message.setConversationId(conversationID);
		this.myAgent.send(message);
	}
	
	protected void addCommand(UbiwareAgentCommand<?> command){
		this.myAgent.addCommand(command);
	}
	
}
