package ubiware.core;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import ubiware.util.StringEscape;
import ubiware.util.WhiteSpaceString;

/**
 * This class produces a S-APL N3 String from the beliefs strucure of the {@link UbiwareAgent}.
 * 
 * This is a faster implementation of the version by Arthem Katasonov. Speed improvement is mainly achieved by not generating unused ID's
 * 
 * 23.6.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class SaplN3Producer {
	/**
	 * The size of an indentation
	 */
	private final static int indent_size = 4;
	//private final ProduceN3Options options;
	/**
	 * The agent this producer is producing for.
	 */
	private final UbiwareAgent agent;
	//private final Map<String, String> ids = new HashMap<String, String>();
	//private final Vector<String> ids2 = new Vector<String>();
	//private final Vector<String> ids3 = new Vector<String>();
	//private long stCount = 0;
	//private final Map<String, String> usedNS = new HashMap<String, String>();
	private final ResourcePrefixer actualPrefixer;
	/**
	 * @see ProduceN3Options
	 */
	private final boolean useNameSpaces;
	private final boolean indents;
	private final boolean cleanContent;
	private final boolean includeRules;

	public SaplN3Producer(UbiwareAgent agent, ResourcePrefixer prefixer, ProduceN3Options options) {
		this.agent = agent;
		//this.options = options;
		this.actualPrefixer = prefixer;
		this.useNameSpaces = options.useNameSpaces;
		this.indents = options.indents;
		this.cleanContent = options.cleanContent;
		this.includeRules = options.includeRules;
	}

	/**
	 * Produce S-APL N3 for the given {@link ContextContainer}. This method produces less nice looking S-APL since for performance reasons,
	 * the prefixes are generated after the S-APL code. This S-APL is valid but looks less nice. Use the version which does not take a
	 * StringBuilder as an argument to generate S-APL for visual presentation.
	 * 
	 * @param c
	 *            The {@link ContextContainer}
	 * @param builder
	 *            The StringBuilder into which the sapl code should be ge
	 * @return A string containing the S-APL N3 code.
	 */
	public StringBuilder produceN3(ContextContainer c, StringBuilder builder) {
		if (c == null) {
			throw new NullPointerException("The context container to produce N3 from mustn't be null.");
		}
		//check which beliefs are referenced more as once in the structure:
		Set<String> duplicateStatements = this.searchDuplicates(c);
		//the stringbuilder in which the whole code will be put.
		//		StringBuilder result = new StringBuilder();
		//use of a conditional prefixer, depending on the useNamespaces option and used later to get the used namespaeces.
		ConditionalResourcePrefixer prefixer = new ConditionalResourcePrefixer(this.actualPrefixer, this.useNameSpaces);
		this.produceN3Recursion(c, duplicateStatements, new HashMap<String, String>(), builder, prefixer, 0);
		//prepend namespaces.
		if (this.useNameSpaces) {
			//StringBuilder ns_str = new StringBuilder();
			for (Entry<String, String> entry : prefixer.getUsedNameSpaces().entrySet()) {
				String ns = entry.getKey();
				String prefix = entry.getValue();
				if (!ns.equals("#") || !prefix.equals(":")) {
					builder.append("@prefix ");
					builder.append(prefix);
					builder.append(" <");
					builder.append(ns);
					builder.append("> .");
					if (this.indents) {
						builder.append('\n');
					} else {
						builder.append(' ');
					}
				}
			}
			//ns_str.append(result);//is similar to a prepending the namespaces in the beginning, but this might be slightly faster because less data has to be copied
			//return ns_str.toString();//return the string to which we appended everything.
			//} else {
			//return result.toString();
		}
		return builder;

	}

	/**
	 * Produce S-APL N3 for the given {@link ContextContainer}.
	 * 
	 * @param c
	 *            The {@link ContextContainer}
	 * @return A string containing the S-APL N3 code.
	 */
	public String produceN3(ContextContainer c) {
		if (c == null) {
			throw new NullPointerException("The context container to produce N3 from mustn't be null.");
		}
		//check which beliefs are referenced more as once in the structure:
		Set<String> duplicateStatements = this.searchDuplicates(c);
		//the stringbuilder in which the whole code will be put.
		StringBuilder result = new StringBuilder();
		//use of a conditional prefixer, depending on the useNamespaces option and used later to get the used namespaeces.
		ConditionalResourcePrefixer prefixer = new ConditionalResourcePrefixer(this.actualPrefixer, this.useNameSpaces);
		this.produceN3Recursion(c, duplicateStatements, new HashMap<String, String>(), result, prefixer, 0);
		//prepend namespaces.
		if (this.useNameSpaces) {
			StringBuilder ns_str = new StringBuilder();
			for (Entry<String, String> entry : prefixer.getUsedNameSpaces().entrySet()) {
				String ns = entry.getKey();
				String prefix = entry.getValue();
				if (!ns.equals("#") || !prefix.equals(":")) {
					ns_str.append("@prefix ");
					ns_str.append(prefix);
					ns_str.append(" <");
					ns_str.append(ns);
					ns_str.append("> .");
					if (this.indents) {
						ns_str.append('\n');
					} else {
						ns_str.append(' ');
					}
				}
			}
			ns_str.append(result);//is similar to a prepending the namespaces in the beginning, but this might be slightly faster because less data has to be copied
			return ns_str.toString();//return the string to which we appended everything.
		} else {
			return result.toString();
		}
	}

	/**
	 * Recursive step of the generation. Generates the N3 for the given {@link ContextContainer}
	 * 
	 * @param context
	 *            the {@link ContextContainer} for which N3 should be generated.
	 * @param duplicateStatements
	 *            statements which appear more as once in the document.
	 * @param generatedReferableStatements
	 *            statements which are already generated previously and can be referred to.
	 * @param result
	 *            the StringBuilder to which the result must be appended.
	 * @param prefixer
	 *            the prefixer to use when prefixing a resource.
	 * @param recursiondepth
	 *            the current recursion depth.
	 */
	private void produceN3Recursion(final ContextContainer context, final Set<String> duplicateStatements,
			final Map<String, String> generatedReferableStatements,//map, mapping statementIDs on local IDs, only referable statements are in the map. 
			final StringBuilder result, ConditionalResourcePrefixer prefixer, final int recursiondepth) {
		boolean generateDotInBegining = false; //this is the first statement in this container
		for (String statementID : context.members) {
			//generate dot if this is not the first statement in the container
			if (generateDotInBegining) {
				result.append(" .");
				if (this.indents) {
					result.append('\n');
				} else {
					result.append(' ');
				}
			}
			if (this.indents) {
				this.putWhiteSpace(result, recursiondepth * SaplN3Producer.indent_size);
			}
			//if the statement has been referenced earlier, it does not have to be created again:
			String localID = generatedReferableStatements.get(statementID);
			if (localID != null) {
				this.produceN3localIDReference(localID, result, prefixer);
				generateDotInBegining = true;
				continue; //<<<<<<< if a localIDReference is created we are done for this statement
			}
			final SemanticStatement statement = this.agent.beliefs.get(statementID);
			final String subject = statement.subject;
			final String predicate = statement.predicate;
			final String object = statement.object;
			//if the flag cleancontent is set, certain content should not be displayed on the 0-th recursion level:
			if (this.cleanContent && (recursiondepth == 0)) {
				//if the statement matches '* embeddedPredicate *' with the first * not a variable
				if (UbiwareAgent.embedded_predicates.contains(predicate) && !subject.startsWith("?")) {
					generateDotInBegining = false; // this results in not generating a dot twice since nothing is generated.
					continue;
				}
				//matches '* sapl:hasMember *' and subject is not a variable
				if (predicate.equals(SaplN3Constants.hasMember) && !subject.startsWith("?")) {
					generateDotInBegining = false; // this results in not generating a dot twice since nothing is generated.
					continue;
				}
				//matches '* sapl:memberOf *' and object is not a variable
				if (predicate.equals(SaplN3Constants.memberOf) && !object.startsWith("?")) {
					generateDotInBegining = false; // this results in not generating a dot twice since nothing is generated.
					continue;
				}
				//matches '* sapl:is Optional' , '* sapl:Limit *', '* sapl:Offset *'
				if ((predicate.equals(SaplN3Constants.is) && object.equals(SaplN3Constants.Optional))
						|| predicate.equals(SaplN3Constants.Limit) || predicate.equals(SaplN3Constants.Offset)) {

					if (this.indents) {
						this.putWhiteSpace(result, recursiondepth * SaplN3Producer.indent_size);
					}
					//recursiondepth does not have to be increased because we're unwrapping a container.
					this.produceN3Recursion(this.agent.contexts.get(subject), duplicateStatements, generatedReferableStatements, result,
							prefixer, recursiondepth);
					//String str = SaplN3.produceN3Recursion(st.subject, recursionLevel, options, recursionVariables).toString();
					//if (result.length() > 0) {
					//	result.append((options.indents) ? ".\n" : ". ");
					//}

					//result.append(str);
					generateDotInBegining = true;
					continue;
				}
				//matches '* sapl>or *' , generates the two containers the sapl:or is connecting as if they where on the sapl:or level.
				if (predicate.equals(SaplN3Constants.or)) {
					if (this.indents) {
						this.putWhiteSpace(result, recursiondepth * SaplN3Producer.indent_size);
					}
					//generate the code for the subject
					this.produceN3Recursion(this.agent.contexts.get(subject), duplicateStatements, generatedReferableStatements, result,
							prefixer, recursiondepth);
					//because we are joining the two containers, an extra dot is needed.
					result.append(" .");
					if (this.indents) {
						result.append('\n');
					} else {
						result.append(' ');
					}
					//repeat for object container
					if (this.indents) {
						this.putWhiteSpace(result, recursiondepth * SaplN3Producer.indent_size);
					}
					//generate the code for the object
					this.produceN3Recursion(this.agent.contexts.get(object), duplicateStatements, generatedReferableStatements, result,
							prefixer, recursiondepth);

					generateDotInBegining = true;
					continue;
				}

			}

			//from here on we have to generate the whole statement.

			//if the statement will be referenced later, we've to generate an ID for it:
			boolean duplicateStatementsContainsStatementID = duplicateStatements.contains(statementID);
			if (duplicateStatementsContainsStatementID) {
				localID = '_' + UbiwareAgent.STATEMENT_PREFIX + generatedReferableStatements.size();
				generatedReferableStatements.put(statementID, localID);
				result.append('{'); //start bracket to enclose the statement in order to be able to add the id later.
			}
			//generate subject
			this.produceN3Subject(subject, duplicateStatements, generatedReferableStatements, result, prefixer, recursiondepth);
			result.append(' ');
			//generate predicate
			this.produceN3Predicate(predicate, result, prefixer);
			result.append(' ');
			//generate object
			this.produceN3Object(object, statement.objectDatatype, statement.objectLanguage, duplicateStatements,
					generatedReferableStatements, result, prefixer, recursiondepth);
			//if the statement will be referenced later, we've to generate an ID for it:
			if (duplicateStatementsContainsStatementID) {
				result.append('}');
				result.append(' ');
				result.append(prefixer.prefixResource(SaplN3Constants.ID));
				result.append(' ');
				//append the previously generated ID
				result.append(localID);
			}

			generateDotInBegining = true;//the next statement will not be first.
		}

	}

	/**
	 * Append the requested amount of whitespace to the StringBuilder
	 * 
	 * @param result
	 * @param length
	 */
	private void putWhiteSpace(StringBuilder result, int length) {
		result.append(WhiteSpaceString.getWhiteSpaceString(length));

	}

	/**
	 * Produce a reference to another statement (which is previousely generated in the document)
	 * 
	 * @param localID
	 *            The ID local to the document
	 * @param result
	 *            The StringBuilder to which the code must be appended.
	 * @param prefixer
	 *            The Prefixer to use to prefix possible resources.
	 */
	private void produceN3localIDReference(final String localID, final StringBuilder result, ConditionalResourcePrefixer prefixer) {
		result.append(localID);
		result.append(' ');
		result.append(prefixer.prefixResource(SaplN3Constants.is));
		result.append(' ');
		result.append(prefixer.prefixResource(SaplN3Constants.TRUE));

	}

	/**
	 * Produce the code of a subject
	 * 
	 * @param subject
	 *            The subject to produce
	 * @param duplicateStatements
	 *            statements which appear more as once in the document.
	 * @param generatedReferableStatements
	 *            statements which are already generated previously and can be referred to.
	 * @param result
	 *            the StringBuilder to which the result must be appended.
	 * @param prefixer
	 *            the prefixer to use when prefixing a resource.
	 * @param recursiondepth
	 *            the current recursion depth.
	 */
	private void produceN3Subject(String subject, final Set<String> duplicateStatements,
			final Map<String, String> generatedReferableStatements,//map, mapping statementIDs on local IDs, the local ID will be null if no ID is generated. 
			final StringBuilder result, ConditionalResourcePrefixer prefixer, final int recursiondepth) {
		if (subject.startsWith("<") && subject.endsWith(">")) {
			//it is normal uri
			result.append(prefixer.prefixResource(subject));
		} else if (subject.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
			//it is a container
			result.append('{');
			//here is a slight difference with the original implementation : when indentation is on and rules are skipped and there are no rules,
			//that empty container will show as '{\n neededwhitespace }\n' while the original implementation just showed {} 
			if (this.indents) {
				result.append('\n');
			}
			//end of the code which is different. original : ((options.indents && (str.length() > 0)) ? "\n" : "") where str was the produced content.
			if (!this.includeRules && (subject.equals(UbiwareAgent.RULES_CONTEXT) || subject.equals(UbiwareAgent.METARULES_CONTEXT))) {
				//skip the rules.
				if (this.agent.contexts.get(subject).members.size() > 0) {
					result.append("Contents are skipped");
				} else {
					//nothing to do
				}
			} else {
				//do recursion
				ContextContainer container = this.agent.contexts.get(subject);
				this.produceN3Recursion(container, duplicateStatements, generatedReferableStatements, result, prefixer, recursiondepth + 1);
			}
			//here is a slight difference with the original implementation : when indentation is on and rules are skipped and there are no rules,
			//that empty container will show as '{\n neededwhitespace }\n' while the original implementation just showed {}
			if (this.indents) {
				result.append('\n');
				this.putWhiteSpace(result, recursiondepth * SaplN3Producer.indent_size);
			}
			//end of the code which is different. original :((options.indents && (str.length() > 0)) ? "\n"	+ WhiteSpaceString.getWhiteSpaceString((recursionLevel) * SaplN3.indent_size) : "") where str was the produced content. 
			result.append("}");
		} else {
			//it is a literal
			result.append("\"");
			result.append(StringEscape.escape(subject));
			result.append("\"");
		}

	}

	/**
	 * Generate the code of a predicate
	 * 
	 * @param predicate
	 *            the predicate to produce code for
	 * @param result
	 *            The StringBuilder to which the code shoud be appended
	 * @param prefixer
	 *            the prefixer which must be used to prefix resources.
	 */
	private void produceN3Predicate(final String predicate, final StringBuilder result, ConditionalResourcePrefixer prefixer) {
		if (predicate.startsWith("<") && predicate.endsWith(">")) {
			//it is a proper uri
			result.append(prefixer.prefixResource(predicate));
		} else {
			result.append("\"");
			result.append(StringEscape.escape(predicate));
			result.append("\"");
		}

	}

	/**
	 * Produce the code of a object
	 * 
	 * @param object
	 *            The subject to produce
	 * @param duplicateStatements
	 *            statements which appear more as once in the document.
	 * @param generatedReferableStatements
	 *            statements which are already generated previously and can be referred to.
	 * @param result
	 *            the StringBuilder to which the result must be appended.
	 * @param prefixer
	 *            the prefixer to use when prefixing a resource.
	 * @param recursiondepth
	 *            the current recursion depth.
	 * 
	 * @param objectDatatype
	 * @see {@link SemanticStatement#objectDatatype}
	 * @param objectLanguage
	 * @see {@link SemanticStatement#objectLanguage}
	 */
	private void produceN3Object(final String object, final String objectDatatype, final String objectLanguage,
			final Set<String> duplicateStatements, final Map<String, String> generatedReferableStatements,//map, mapping statementIDs on local IDs, the local ID will be null if no ID is generated. 
			final StringBuilder result, ConditionalResourcePrefixer prefixer, final int recursiondepth) {

		if (object.startsWith(UbiwareAgent.CONTEXT_PREFIX) && (objectDatatype == null)) {
			//it is a container
			result.append('{');
			//original : + ((options.indents && (str.length() > 0)) ? "\n" : "")
			if (this.indents) {
				result.append('\n');
			}
			//recursive step
			ContextContainer container = this.agent.contexts.get(object);
			this.produceN3Recursion(container, duplicateStatements, generatedReferableStatements, result, prefixer, recursiondepth + 1);
			//original		+ ((options.indents && (str.length() > 0)) ? "\n"+ WhiteSpaceString.getWhiteSpaceString((recursionLevel) * SaplN3.indent_size) : "") + 
			if (this.indents) {
				result.append('\n');
				this.putWhiteSpace(result, recursiondepth * SaplN3Producer.indent_size);
			}
			result.append("}");
		} else if (!(object.startsWith("<") && object.endsWith(">")) || (objectDatatype != null) || (objectLanguage != null)) {
			//it is a literal, depending on the type, do a different thing:
			if (objectDatatype != null) {
				result.append("\"");
				result.append(StringEscape.escape(object));
				result.append("\"^^");
				result.append(objectDatatype);
			} else if (objectLanguage != null) {
				result.append("\"");
				result.append(StringEscape.escape(object));
				result.append("\"@");
				result.append(objectLanguage);
			} else {
				result.append("\"");
				result.append(StringEscape.escape(object));
				result.append("\"");
			}
		} else {
			//it is a proper uri
			result.append(prefixer.prefixResource(object));
		}

	}

	/**
	 * returns a set of all statementIDs which are used more as once in the ContextContainer
	 * 
	 * @param c
	 *            the {@link ContextContainer}
	 * @return the set of statement ID's
	 */
	private Set<String> searchDuplicates(ContextContainer c) {
		Set<String> duplicates = new HashSet<String>();
		this.searchDuplicates(c, new HashSet<String>(), duplicates);
		return duplicates;
	}

	/**
	 * performs a preorder traversal of the tree of contextcontainers to find duplicates.
	 * 
	 * @param c
	 *            the {@link ContextContainer}
	 * @param referencedEarlier
	 *            Set of earlier referenced {@link SemanticStatement}
	 * @param duplicates
	 *            Set of duplicate {@link SemanticStatement}
	 */
	private void searchDuplicates(ContextContainer c, Set<String> referencedEarlier, Set<String> duplicates) {
		for (String statementID : c.members) {
			SemanticStatement statement = this.agent.beliefs.get(statementID);
			//we only consider this statement as a possible candidate if it has more as one reference
			//In a later implementation phase when referenced is not an attribute of statement anymore, this check can be safely removed.
			if (statement.referenced > 1) {
				if (referencedEarlier.contains(statementID)) {
					duplicates.add(statementID);
				} else {
					referencedEarlier.add(statementID);
				}
			}
			//if subject or object of the statement are containers, continue recursively.
			String subject = statement.subject;
			String object = statement.object;
			if (subject.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
				ContextContainer subjectContainer = this.agent.contexts.get(subject);
				this.searchDuplicates(subjectContainer, referencedEarlier, duplicates);
			}
			if (object.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
				ContextContainer objectContainer = this.agent.contexts.get(object);
				this.searchDuplicates(objectContainer, referencedEarlier, duplicates);
			}
		}
	}
}
