package ubiware.core;

import jade.util.leap.Serializable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * Representation of a triple (subject, predicate, object).
 * 
 * @author Michael Cochez for Ubiware project
 * @since 3 Aug 2010
 */
public class SemanticStatement implements Serializable {

	private static final long serialVersionUID = 1L;
	public String subject = null;
	public String predicate = null;
	public String object = null;

	/**
	 * for explicitly typed literals, the type of the object will be set.
	 */
	public String objectDatatype = null;
	/**
	 * for literals with language defined, the language will be set
	 */
	public String objectLanguage = null;

	/**
	 * List of ID's of referencing Contexts if there are several.
	 */
	public List<String> memberOf;

	/**
	 * The number of referencing contexts.
	 */
	public int referenced = 0;

	/**
	 * Fixes this statement : set ? in all not set S P O
	 */
	private void fix() {
		if (this.subject == null) {
			this.subject = "?";
		}
		if (this.predicate == null) {
			this.predicate = "?";
		}
		if (this.object == null) {
			this.object = "?";
		}
	}

	/**
	 * 
	 * @param s
	 *            Subject of the statement.
	 * @param p
	 *            Predicate of the statement.
	 * @param o
	 *            Object of the statement.
	 */
	public SemanticStatement(String s, String p, String o) {
		this(s, p, o, null, null);
	}

	/**
	 * 
	 * @param s
	 *            Subject of the statement.
	 * @param p
	 *            Predicate of the statement.
	 * @param o
	 *            Object of the statement.
	 * @param oD
	 *            Datatype of the object.
	 * @param oL
	 *            Language of the object
	 */
	public SemanticStatement(String s, String p, String o, String oD, String oL) {
		this.subject = s;
		this.predicate = p;
		this.object = o;
		this.objectDatatype = oD;
		this.objectLanguage = oL;
		this.fix();

		this.memberOf = new ArrayList<String>(1);
	}

	/**
	 * creates an identical semantic statement (deep copy)
	 * 
	 * @param toCopy
	 *            the semantic statement to copy from.
	 */
	public SemanticStatement(SemanticStatement toCopy) {
		this.subject = toCopy.subject;
		this.predicate = toCopy.predicate;
		this.object = toCopy.object;
		this.objectDatatype = toCopy.objectDatatype;
		this.objectLanguage = toCopy.objectLanguage;
		this.memberOf = new ArrayList<String>(1);
	}

	/**
	 * Is the subject '*' or a variable?
	 * 
	 * @return true if the subject is '*' or a variable, false otherwise
	 */
	public boolean isSubjectAny() {
		return this.subject.equals("*") || this.isSubjectVar();
	}

	/**
	 * Is the predicate '*' or a variable?
	 * 
	 * @return true if the predicate is '*' or a variable, false otherwise
	 */
	public boolean isPredicateAny() {
		return this.predicate.equals("*") || this.isPredicateVar();
	}

	/**
	 * Is the object '*' or a variable?
	 * 
	 * @return true if the object is '*' or a variable, false otherwise
	 */
	public boolean isObjectAny() {
		return this.object.equals("*") || this.isObjectVar();
	}

	/**
	 * Is the subject a variable?
	 * 
	 * @return true if the subject is a variable, false otherwise
	 */
	public boolean isSubjectVar() {
		return (this.subject.length() > 1) && this.subject.startsWith("?");
	}

	/**
	 * Is the predicate a variable?
	 * 
	 * @return true if the predicate is a variable, false otherwise
	 */
	public boolean isPredicateVar() {
		return (this.predicate.length() > 1) && this.predicate.startsWith("?");
	}

	/**
	 * Is the object a variable?
	 * 
	 * @return true if the object is a variable, false otherwise
	 */
	public boolean isObjectVar() {
		return (this.object.length() > 1) && this.object.startsWith("?");
	}

	/**
	 * Does this statement have variables?
	 * 
	 * @return true if this statement has a variable, false otherwise
	 */
	public boolean hasVariables() {
		return this.isSubjectVar() || this.isPredicateVar() || this.isObjectVar();
	}

	/**
	 * Returns the name of the variable of the subject if it is a variable.
	 * 
	 * @return the name of the variable if the subject is a variable, null otherwise.
	 */
	public String getSubjectVariableName() {
		return this.isSubjectVar() ? this.subject.substring(1) : null;
	}

	/**
	 * Returns the name of the variable of the predicate if it is a variable.
	 * 
	 * @return the name of the variable if the predicate is a variable, null otherwise.
	 */
	public String getPredicateVariableName() {
		return this.isPredicateVar() ? this.predicate.substring(1) : null;
	}

	/**
	 * Returns the name of the variable of the object if it is a variable.
	 * 
	 * @return the name of the variable if the object is a variable, null otherwise.
	 */
	public String getObjectVariableName() {
		return this.isObjectVar() ? this.object.substring(1) : null;
	}

	/**
	 * returns true this statement is equal (taking universalities into account) to the 'obj' statement.
	 */
	//	@Override
	public boolean equalsUniversal(SemanticStatement other) {
		if (other == null) {
			return false;
		}
		if (!this.isSubjectAny() && !other.isSubjectAny()) {
			if (!this.subject.equals(other.subject)) {
				return false;
			}
		}
		if (!this.isPredicateAny() && !other.isPredicateAny()) {
			if (!this.predicate.equals(other.predicate)) {
				return false;
			}
		}
		if (!this.isObjectAny() && !other.isObjectAny()) {
			if (!this.object.equals(other.object)) {
				return false;
			}
		}

		return true;
	}

	//replaces variables with their values from vars
	public boolean[] bindVars(HashMap<String, String> vars) {
		boolean[] changed = new boolean[3];
		changed[0] = false;
		changed[1] = false;
		changed[2] = false;
		if (vars.size() > 0) {
			String var = this.getSubjectVariableName();
			if (var != null) {
				String value = vars.get(var);
				if ((value != null) && !value.equals("")) {
					this.subject = value;
					changed[0] = true;
				}
			}
			var = this.getPredicateVariableName();
			if (var != null) {
				String value = vars.get(var);
				if ((value != null) && !value.equals("")) {
					this.predicate = value;
					changed[1] = true;
				}
			}
			var = this.getObjectVariableName();
			if (var != null) {
				String value = vars.get(var);
				if ((value != null) && !value.equals("")) {
					this.object = value;
					changed[2] = true;
				}
			}
		}
		return changed;
	}

	@Override
	public int hashCode() {
		String str = this.subject + " " + this.predicate + " " + this.object;
		return str.hashCode();
	}

	@Override
	public String toString() {
		//return subject+" "+predicate+" "+object+" ("+String.valueOf(referenced)+")";
		return this.subject + " " + this.predicate + " " + this.object;
	}

	/**
	 * Only true if the subject of this semantic statement starts with the CONTEXT_PREFIX
	 * 
	 * @return is the subject of this statement a context container?
	 */
	public boolean isSubjectContext() {
		return this.subject.startsWith(UbiwareAgent.CONTEXT_PREFIX);
	}

	/**
	 * Only true if the object of this semantic statement starts with the CONTEXT_PREFIX
	 * 
	 * @return is the object of this statement a context container?
	 */
	public boolean isObjectContext() {
		return this.object.startsWith(UbiwareAgent.CONTEXT_PREFIX);
	}
}
