package ubiware.core;

import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.CyclicBehaviour;

import java.net.URI;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.StringTokenizer;

import ubiware.core.behaviors.BeliefsLoadBehavior;
import ubiware.core.behaviors.DebugBehavior;
import ubiware.core.commands.AddBeliefsN3Command;
import ubiware.core.commands.UbiwareAgentCommand;
import ubiware.core.policies.AllowAllPolicyChecker;
import ubiware.core.policies.PolicyChecker;
import ubiware.core.policies.Status;
import ubiware.ontology.param;
import ubiware.util.ContextIDGenerator;
import ubiware.util.SetOperation;
import ubiware.util.StatementIDGenerator;
import ubiware.util.StringEscape;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

import com.google.common.collect.Multimap;

public class UbiwareAgent extends Agent {

	private static final long serialVersionUID = 1L;

	//creating standard beliefs and contexts
	final Map<String, SemanticStatement> beliefs = new HashMap<String, SemanticStatement>();
	final Map<String, ContextContainer> contexts = new HashMap<String, ContextContainer>();

	private static final RABLoader loader = new RABLoader();

	private final Set<ReusableAtomicBehavior> runningRABs = new HashSet<ReusableAtomicBehavior>();
	private final Map<String, String> rabInstanceIDs = new HashMap<String, String>();

	/**
	 * The blackboard of the agent
	 */
	private final Blackboard blackboard = new Blackboard();

	/**
	 * The resource prefixer of the agent
	 */
	private final ResourcePrefixer prefixer = new ResourcePrefixer();

	Live live;

	//long contextCount = 0;
	public final ContextIDGenerator contextIDGenerator = new ContextIDGenerator();

	//long statementCount = 0;
	public final StatementIDGenerator statementIDGenerator = new StatementIDGenerator();

	private long lifeCycleCount = 0;

	public long getLifeCycleCount() {
		return this.lifeCycleCount;
	}

	//in the future, the agent is not creating the log itself anymore.
	//	static boolean logCreated = false;

	//the agent does itself not have any knowledge of external GUI's, this variable is not needed any longer.
	//Map<String, AgentGUI> myGUIs; //agent's graphical user interfaces, if needed
	//The agent does not need the following variables any longer because of the new Server model
	//AgentServer myServer; //agent's HTTP server (e.g. for getting events from external applications like Google Earth or IE), if needed

	//ServerEventsHandler serverHandler; //behavior handling events from HTTP
	//creating a queue for HTTP events
	//	private final Vector<ServerEvent> serverEventQueue = new Vector<ServerEvent>();
	//	private final Object serverEventQueueLock = new Object();

	public static final String CONTEXT_PREFIX = "_C";
	public static final String QUERY_CONTEXT_PREFIX = "?_C";
	public static final String STATEMENT_PREFIX = "_S";
	public static final String BLACKBOARD_OBJECT_PREFIX = "_BLACKBOARD";

	public static final String GENERAL_CONTEXT = "G";

	final static String RULES_CONTEXT = "_C_RULES";
	final static String METARULES_CONTEXT = "_C_METARULES";
	final static String GOALS_CONTEXT = "_C_GOALS";
	final static String ACHIEVE_CONTEXT = "_C_ACHIEVE";
	final static String DO_CONTEXT = "_C_DO";
	final static String FALSE_CONTEXT = "_C_FALSE";

	final static String RULES_BELIEF = "_S_RULES";
	final static String METARULES_BELIEF = "_S_METARULES";
	final static String GOALS_BELIEF = "_S_GOALS";
	final static String ACHIEVE_BELIEF = "_S_ACHIEVE";
	final static String DO_BELIEF = "_S_DO";
	final static String FALSE_BELIEF = "_S_FALSE";

	public static final String RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	public static final String RDFS_NS = "http://www.w3.org/2000/01/rdf-schema#";
	public static final String OWL_NS = "http://www.w3.org/2002/07/owl#";
	public static final String XMLS_NS = "http://www.w3.org/2001/XMLSchema#";

	public static final String BLANK_NODE_NS = "_:";

	public static final String UNBOUND_VARIABLE = "{@$unbound$@}";

	final static String NAME_BELIEF = "_S_NAME";
	final static String TIME_BELIEF = "_S_TIME";
	final static String DATE_BELIEF = "_S_DATE";
	final static String NOW_BELIEF = "_S_NOW";

	//to speed up some common matching procedures 
	/**
	 * Set containing all the protected entities of the agents, this set is unmodifiable and can thus safely be shared
	 */
	private final static Set<String> protected_entities = UbiwareAgent.create_protected_entities();

	private static Set<String> create_protected_entities() {
		//those cannot be removed
		Set<String> entities = new HashSet<String>();
		entities.add(UbiwareAgent.GENERAL_CONTEXT);
		entities.add(UbiwareAgent.RULES_CONTEXT);
		entities.add(UbiwareAgent.METARULES_CONTEXT);
		entities.add(UbiwareAgent.GOALS_CONTEXT);
		entities.add(UbiwareAgent.ACHIEVE_CONTEXT);
		entities.add(UbiwareAgent.DO_CONTEXT);
		entities.add(UbiwareAgent.FALSE_CONTEXT);
		entities.add(UbiwareAgent.RULES_BELIEF);
		entities.add(UbiwareAgent.METARULES_BELIEF);
		entities.add(UbiwareAgent.GOALS_BELIEF);
		entities.add(UbiwareAgent.DO_BELIEF);
		//		entities.add(UbiwareAgent.DO_CONTEXT);
		entities.add(UbiwareAgent.FALSE_BELIEF);
		//Setup the protected entities , use an unmodifiableSet to protect from modification.
		return Collections.unmodifiableSet(entities);
	}

	/**
	 * This is an unmodifiable set containing the embedded predicates of the agent, those predicates are processed by the engine rather than
	 * matched against beliefs
	 */
	public static final Set<String> embedded_predicates = UbiwareAgent.create_embedded_predicates();

	private static Set<String> create_embedded_predicates() {
		//those processed by the engine rather than matched against beliefs
		Set<String> predicates = new HashSet<String>();
		predicates.add(SaplN3Constants.gt);
		predicates.add(SaplN3Constants.lt);
		predicates.add(SaplN3Constants.gte);
		predicates.add(SaplN3Constants.lte);
		predicates.add(SaplN3Constants.eq);
		predicates.add(SaplN3Constants.neq);
		predicates.add(SaplN3Constants.regex);
		predicates.add(SaplN3Constants.expression);
		predicates.add(SaplN3Constants.count);
		predicates.add(SaplN3Constants.countGroupedBy);
		predicates.add(SaplN3Constants.max);
		predicates.add(SaplN3Constants.min);
		predicates.add(SaplN3Constants.sum);
		predicates.add(SaplN3Constants.startsWith);
		predicates.add(SaplN3Constants.endsWith);
		predicates.add(SaplN3Constants.contains);
		predicates.add(SaplN3Constants.doesNotStartWith);
		predicates.add(SaplN3Constants.doesNotEndWith);
		predicates.add(SaplN3Constants.doesNotContain);
		return Collections.unmodifiableSet(predicates);
	}

	/**
	 * This is an unmodifiable set containing the set_predicats of the agent, solution set modifiers that must have a variable as object
	 */
	private final static Set<String> set_predicates = UbiwareAgent.create_set_predicates();

	private static Set<String> create_set_predicates() {
		//solution set modifiers that must have a variable as object
		Set<String> predicates = new HashSet<String>();
		predicates.add(SaplN3Constants.All);
		predicates.add(SaplN3Constants.Some);
		predicates.add(SaplN3Constants.OrderBy);
		predicates.add(SaplN3Constants.OrderByDescending);
		return Collections.unmodifiableSet(predicates);
	}

	/**
	 * other solution set modifiers
	 */
	private final static Set<String> set2_predicates = UbiwareAgent.create_set2_predicates();

	private static Set<String> create_set2_predicates() {
		////other solution set modifiers
		Set<String> predicates = new HashSet<String>();
		predicates.add(SaplN3Constants.Limit);
		predicates.add(SaplN3Constants.Offset);
		return Collections.unmodifiableSet(predicates);
	}

	/**
	 * the contexts containers of those must not be automatically joined
	 */
	private final static Set<String> log_predicates = UbiwareAgent.create_log_predicates();

	private static Set<String> create_log_predicates() {
		//the contexts containers of those must not be automatically joined
		Set<String> predicates = new HashSet<String>();
		predicates.add(SaplN3Constants.remove);
		predicates.add(SaplN3Constants.doNotBelieve);
		predicates.add(SaplN3Constants.hasMember);
		predicates.add(SaplN3Constants.memberOf);
		predicates.add(SaplN3Constants.forSome);
		predicates.add(SaplN3Constants.forAll);
		predicates.add(SaplN3Constants.configuredAs);
		return Collections.unmodifiableSet(predicates);
	}

	private static final Set<String> inference_predicates = UbiwareAgent.create_inference_predicates();

	private static Set<String> create_inference_predicates() {
		//inference predicates
		Set<String> predicates = new HashSet<String>();
		predicates.add(SaplN3Constants.implies); // =>
		predicates.add(SaplN3Constants.impliesNow); // ->
		predicates.add(SaplN3Constants.achievedBy); // >>
		predicates.add(SaplN3Constants.infers); // ==>
		return Collections.unmodifiableSet(predicates);
	}

	//global data structure for private use in some procedures
	//TODO this sounds like a basic violation of encapsulation in functional programming. Or this should be removed or better commented
	private HashMap<String, String> ids; //needed in copy() and when loading from SaplN3.load();
	private HashMap<String, String> bNodeMap; // in copy()
	private String bnNameBase; // in copy()
	private int bnCount; // in copy()

	//debug control
	private Debugger debugger;
	private long stopAfterLifeCycle = -1;
	//behavior which helps the agent to handle commands issued from outside the agent thread.
	private CommandHandlingBehavior commandHandlingBahavior;
	//object used in the agents RAB execution code to check whether an action is allowed or not.
	private PolicyChecker policyChecker;
	/**
	 * The scripts which should be loaded by the agent when activated. This field will be null after initialisation.
	 */
	private Set<String> initialScripts;
	/**
	 * The roles which should be loaded by the agent when activated. This field will be null after initialisation.
	 */
	private Set<String> initialRoles;

	public UbiwareAgent() {
		this(new HashSet<String>(), new HashSet<String>());
	}

	/**
	 * Create a new Ubiware agent which will load the specified scripts and roles as soon as it is put in use. (Scripts defined bu
	 * setArguments will be added to these sets)
	 * 
	 * @param scripts
	 *            The set of scipts.
	 * @param roles
	 *            The set of roles.
	 */
	public UbiwareAgent(Set<String> scripts, Set<String> roles) {
		this(scripts, roles, null);
	}

	public UbiwareAgent(Set<String> scripts, Set<String> roles, SaplDocument initBeliefs) {
		if (scripts == null) {
			throw new NullPointerException("The parameter scripts to ubiwareAgent must not be null");
		}
		if (roles == null) {
			throw new NullPointerException("The parameter roles to ubiwareAgent must not be null");
		}
		this.initialScripts = scripts;
		this.initialRoles = roles;
		//initialisation of the command handling bahjavior happens in the earliest stage. 
		//This results in the agent being able to handle commands even if it isn't scheduled yet.
		this.commandHandlingBahavior = new CommandHandlingBehavior(this);
		// if the user specified some initial beliefs, use a Command object to feed them into the agent 
		//this must happen after the commandhandlingbehavior is installed!
		if (initBeliefs != null) {
			this.addCommand(new AddBeliefsN3Command(initBeliefs));
		}
		//by default the agent is allowed to do any actions untill another PolicyChecker is installed:
		this.policyChecker = AllowAllPolicyChecker.getInstance();
	}

	//initialization
	@Override
	protected void setup() {
		this.createStandardBeliefsAndContexts();
		this.handleStartupArguments();
		//starting Live
		this.addBehaviour(this.live = new Live());
		//add command handling behavior to the agent
		this.addBehaviour(this.commandHandlingBahavior);
	}

	/**
	 * This method handles the arguments the agent was started with. It is called from within setup.
	 * 
	 * @modified Michael Cochez for UBIWARE project
	 */
	private void handleStartupArguments() {
		//reading agent's startup arguments
		int lists = 0;
		String script_files = null;
		String roles = null;
		boolean startDebugger = false;

		Object[] args = this.getArguments();
		if (args != null) {
			//load roles and scripts from arguments.
			for (int i = 0; i < args.length; i++) {
				String arg = args[i].toString().trim();
				if (arg.startsWith("--")) {
					if (arg.equals("--debug")) {
						startDebugger = true;
					} else {
						this.print("Warning: Unknown startup argument: " + arg);
					}
				} else if (lists == 0) {
					script_files = arg;
					lists = 1;
				} else if (lists == 1) {
					roles = arg;
					lists = 2;
				} else {
					this.print("Warning: Too many lists: " + arg);
				}
			}

			//Add scripts from parameters to initialscripts
			StringTokenizer token = new StringTokenizer(script_files, "*");
			while (token.hasMoreTokens()) {
				String script = token.nextToken();
				this.initialScripts.add(script);
			}

			//add roles from parameters to initialRoles
			if (roles != null) {
				token = new StringTokenizer(roles, "+");
				while (token.hasMoreTokens()) {
					String role = token.nextToken();
					this.initialRoles.add(role);
				}
			}
		}
		//load all scripts:
		for (String script : this.initialScripts) {
			ReusableAtomicBehavior beliefsLoader = new BeliefsLoadBehavior();
			BehaviorStartParameters params = new BehaviorStartParameters();
			params.add(param.inputFromFile, script);
			params.add(param.saveTo, "G");
			beliefsLoader.setStartParameters(params);
			this.addRAB(beliefsLoader);
		}

		//load all roles:
		int i = 1;
		for (String role : this.initialRoles) {
			String id = "_SRole" + String.valueOf(i++);
			this.putBelief(id, new SemanticStatement(SaplN3Constants.I, SaplN3Constants.missingModel, role));
			this.putLink(UbiwareAgent.GENERAL_CONTEXT, id);
		}

		//clean up the initial argument lists
		this.initialRoles = null;
		this.initialScripts = null;

		//starting DebugBehavior if ordered
		if (startDebugger) {
			DebugBehavior deb = new DebugBehavior();
			BehaviorStartParameters params = new BehaviorStartParameters();
			params.add(param.stop, "true");
			deb.setStartParameters(params);
			deb.debugController = new DebugController();
			this.addRAB(deb);
		}
	}

	/**
	 * Creates the beliefs for the agent when it is still fresh and new. This method is used from setup()
	 * 
	 * @modified Michael Cochez for UBIWARE project
	 */
	private void createStandardBeliefsAndContexts() {
		//creating standard beliefs and contexts

		//moved to member initialization.
		//this.beliefs = new HashMap<String, SemanticStatement>();
		//this.contexts = new HashMap<String, ContextContainer>();

		this.putContext(UbiwareAgent.GENERAL_CONTEXT); //general context G
		this.putContext(UbiwareAgent.FALSE_CONTEXT); //container for statements known to be false
		this.putContext(UbiwareAgent.RULES_CONTEXT); //container for rules
		this.putContext(UbiwareAgent.METARULES_CONTEXT); //container for metarules
		this.putContext(UbiwareAgent.GOALS_CONTEXT); //container for goals
		this.putContext(UbiwareAgent.ACHIEVE_CONTEXT); //container for goals that are attempted to be achieved
		this.putContext(UbiwareAgent.DO_CONTEXT); //container for running RABs: actually commitments that led to execution
		this.putBelief(UbiwareAgent.RULES_BELIEF, new SemanticStatement(UbiwareAgent.RULES_CONTEXT, SaplN3Constants.is,
				SaplN3Constants.Rule));
		this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.RULES_BELIEF);
		this.putBelief(UbiwareAgent.METARULES_BELIEF, new SemanticStatement(UbiwareAgent.METARULES_CONTEXT, SaplN3Constants.is,
				SaplN3Constants.MetaRule));
		this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.METARULES_BELIEF);
		this.putBelief(UbiwareAgent.FALSE_BELIEF, new SemanticStatement(UbiwareAgent.FALSE_CONTEXT, SaplN3Constants.is,
				SaplN3Constants.FALSE));
		this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.FALSE_BELIEF);
		this.putBelief(UbiwareAgent.GOALS_BELIEF,
				new SemanticStatement(SaplN3Constants.I, SaplN3Constants.want, UbiwareAgent.GOALS_CONTEXT));
		this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.GOALS_BELIEF);
		this.putBelief(UbiwareAgent.ACHIEVE_BELIEF, new SemanticStatement(SaplN3Constants.I, SaplN3Constants.achieving,
				UbiwareAgent.ACHIEVE_CONTEXT));
		this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.ACHIEVE_BELIEF);
		this.putBelief(UbiwareAgent.DO_BELIEF, new SemanticStatement(SaplN3Constants.I, SaplN3Constants.doing, UbiwareAgent.DO_CONTEXT));
		this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.DO_BELIEF);
	}

	/**
	 * Wrapper class for RAB's. Goal is to decouple RAB's from the fact that Jade is used as underlying agent platform.
	 * {@link ReusableAtomicBehavior} does not implement {@link Behaviour}. This wrapper acts as a proxy. 6.5.2011
	 * 
	 * @author Michael Cochez for UBIWARE project
	 */
	private static class RABWrapper extends Behaviour {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private ReusableAtomicBehavior behavior;

		/**
		 * Create a wrapper for the specified behavior.
		 * 
		 * @param behavior
		 */
		public RABWrapper(ReusableAtomicBehavior behavior) {
			this.behavior = behavior;
		}

		@Override
		public void action() {
			this.behavior.action();
		}

		@Override
		public boolean done() {
			return this.behavior.done();
		}

		@Override
		public void onStart() {
			this.behavior.onStart();
		}

		@Override
		public int onEnd() {
			return this.behavior.onEnd();
		}
	}

	public void addRAB(ReusableAtomicBehavior rab) {
		RABWrapper rabWrapper = new RABWrapper(rab);
		rab.setAgent(this);
		rab.setEncapsulatingBehavior(rabWrapper);
		this.addBehaviour(rabWrapper);
	}

	@Override
	protected void takeDown() {
		this.print("NOOOOOO!!!!!!! I do not want to die!!");
	}

	//close gui and server before move
	@Override
	protected void beforeMove() {
		//The server is no longer part of the Agent, the agent is not able to stop it.
		//TODO check whether it is feasible to restart the server after the agent has moved / same applies to restoring GUI's
		//perhaps it makes sense to make a new method on RAB before() and aftermove()
		//see also comments in afterMove
	}

	//open back gui and server after move
	//TODO see comments in beforemove
	@Override
	protected void afterMove() {
		this.print("Just moved to location " + this.here().getName());
	}

	//prefix a resource using known (defined in any preciously loaded S-APL script) namespaces 
	public String prefixResource(String resource) {
		return this.prefixer.prefixResource(resource);
	}

	//usedNS is a variable to which every ns/prefix pair which is used will be added.
	public String prefixResource(String resource, Map<String, String> usedNS) {
		return this.prefixer.prefixResource(resource, usedNS);
	}

	//TODO sohuld the visibility be different?
	public void setPolicyChecker(PolicyChecker checker) {
		this.policyChecker = checker;
	}

	//TODO should there be a getter for the policy checker as well?

	/** Start of Methods used by Live or onStart and onEnd of ReusableAtomicBehavior **/

	void clearLocalIDs() {
		if (this.ids == null) {
			this.ids = new HashMap<String, String>();
		} else {
			this.ids.clear();
		}
	}

	//Adds a belief to beliefs
	void putBelief(String id, SemanticStatement statement) {
		if (statement.isSubjectContext() && !(statement.predicate.equals(SaplN3Constants.hasMember))) {
			ContextContainer c = this.contexts.get(statement.subject);
			if (c != null) {
				if (c.referencedBy != null) {
					this.print("Warning: Making second statement about a context: " + statement);
				}
				c.referenced++;
				c.referencedBy = id;
			} else {
				this.print("Error: context container does not exist: " + statement.toString());
			}
		}
		if ((statement.objectDatatype == null) && statement.isObjectContext() && !statement.predicate.equals(SaplN3Constants.memberOf)
				&& !statement.predicate.equals(SaplN3Constants.erase)) {
			ContextContainer c = this.contexts.get(statement.object);
			if (c != null) {
				if (c.referencedBy != null) {
					this.print("Warning: Making second statement about a context: " + statement);
				}
				c.referenced++;
				c.referencedBy = id;
			} else {
				this.print("Error: context container does not exist: " + statement.toString());
			}
		}

		this.beliefs.put(id, statement);

		//add a new local ID
		if (statement.predicate.equals(SaplN3Constants.ID) && statement.isSubjectContext() && !statement.object.startsWith("?")
				&& !statement.isObjectContext()) {
			List<String> temp = this.contexts.get(statement.subject).members;
			if (temp.size() > 0) {
				this.ids.put(statement.object, temp.get(0));
			}
		}
	}

	//Removes a belief from beliefs
	void removeBelief(String id) {
		SemanticStatement statement = this.beliefs.get(id);
		if (statement != null) {
			if (statement.isSubjectContext() && !statement.predicate.equals(SaplN3Constants.hasMember)) {
				ContextContainer c = this.contexts.get(statement.subject);
				if (c != null) {
					c.referenced--;
					c.referencedBy = null;
				}
			}
			if ((statement.objectDatatype == null) && statement.isObjectContext() && !statement.predicate.equals(SaplN3Constants.memberOf)
					&& !statement.predicate.equals(SaplN3Constants.erase)) {
				ContextContainer c = this.contexts.get(statement.object);
				if (c != null) {
					c.referenced--;
					c.referencedBy = null;
				}
			}

			Iterator<String> it = statement.memberOf.iterator();
			while (it.hasNext()) {
				ContextContainer c = this.contexts.get(it.next());
				if (c != null) {
					if (c.members.remove(id)) {
						UbiwareAgent.removeFromIndices(statement, id, c);
					}
				}
			}

			this.beliefs.remove(id);
		}
	}

	//Creates a new context and adds to contexts
	void putContext(String id) {
		this.putContext(id, null);
	}

	void putContext(String id, int membersSize) {
		this.putContext(id, null, membersSize);
	}

	void putContext(String id, String toCopy) {
		ContextContainer context = new ContextContainer();
		this.contexts.put(id, context);
		if (toCopy != null) {
			this.copyLinks(toCopy, id);
		}
	}

	void putContext(String id, String toCopy, int membersSize) {
		ContextContainer context = new ContextContainer(membersSize);
		this.contexts.put(id, context);
		if (toCopy != null) {
			this.copyLinks(toCopy, id);
		}
	}

	//Removes a context from contexts and also all the beliefs about this context
	void removeContext(String id) {
		ContextContainer context = this.contexts.get(id);
		if (context != null) {
			if (context.referencedBy != null) {
				this.removeBelief(context.referencedBy);
			}

			Iterator<String> it = context.members.iterator();
			while (it.hasNext()) {
				SemanticStatement st = this.beliefs.get(it.next());
				if (st != null) {
					st.referenced--;
					st.memberOf.remove(id);
				}
			}
			this.contexts.remove(id);
		}
	}

	//Links a context to a belief. Joins contexts if needed
	//In many situation, does not create the specified link, but creates something else
	//leaving the statement hanging -> garbage collector will clean
	void putLink(String contextID, String statementID) {
		this.putLink(contextID, statementID, true);
	}

	void putLink(String contextID, String statementID, boolean doChecks) {
		SemanticStatement ss = this.beliefs.get(statementID);
		if (ss == null) {
			return;
		}
		ContextContainer container = this.contexts.get(contextID);

		if (ss.predicate.startsWith(UbiwareAgent.STATEMENT_PREFIX) || ss.predicate.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
			this.print("Error: Cannot use context or statement ID as a predicate: " + ss);
			return;
		}

		//creating links through hasMember and memberOf
		if (contextID.equals(UbiwareAgent.GENERAL_CONTEXT)
				&& ((ss.predicate.equals(SaplN3Constants.hasMember) && ss.isSubjectContext()) || (ss.predicate
						.equals(SaplN3Constants.memberOf) && ss.isObjectContext()))) {
			String sts;
			String context;
			if (ss.predicate.equals(SaplN3Constants.hasMember)) {
				sts = ss.object;
				context = ss.subject;
			} else {
				sts = ss.subject;
				context = ss.object;
			}
			if (sts.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
				this.copyLinks(sts, context);
				return;
			}
			if (this.beliefs.get(sts) != null) {
				this.putLink(context, sts);
				return;
			}
		}
		//changing resources through rdf:subject, rdf:object and rdf:predicate
		if (contextID.equals(UbiwareAgent.GENERAL_CONTEXT) && ss.subject.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
			if (ss.predicate.equals("<" + UbiwareAgent.RDF_NS + "subject>")) {
				SemanticStatement st = this.beliefs.get(ss.subject);
				if (st != null) {
					UbiwareAgent.removeFromIndices(st, ss.subject, container);
					st.subject = ss.object;
					UbiwareAgent.addToIndices(st, ss.subject, container);
				}
				return;
			}
			if (ss.predicate.equals("<" + UbiwareAgent.RDF_NS + "predicate>")) {
				SemanticStatement st = this.beliefs.get(ss.subject);
				if (st != null) {
					UbiwareAgent.removeFromIndices(st, ss.subject, container);
					st.predicate = ss.object;
					UbiwareAgent.addToIndices(st, ss.subject, container);
				}
				return;
			}
			if (ss.predicate.equals("<" + UbiwareAgent.RDF_NS + "object>")) {
				SemanticStatement st = this.beliefs.get(ss.subject);
				if (st != null) {
					UbiwareAgent.removeFromIndices(st, ss.subject, container);
					st.object = ss.object;
					UbiwareAgent.addToIndices(st, ss.subject, container);
				}
				return;
			}
		}
		//remove sapl:or wrappings if either of the sides is empty
		else if (ss.predicate.equals(SaplN3Constants.or)) {
			ContextContainer sc = this.contexts.get(ss.subject);
			ContextContainer oc = this.contexts.get(ss.object);
			if ((sc != null) && (oc != null)) {
				if ((sc.members.size() == 0) && (oc.members.size() == 0)) {
					return;
				}
				if ((sc.members.size() == 0) && (oc.members.size() != 0)) {
					this.copyLinks(ss.object, contextID);
					return;
				}
				if ((oc.members.size() == 0) && (sc.members.size() != 0)) {
					this.copyLinks(ss.subject, contextID);
					return;
				}
			}
		}
		//remove residual set wrappings like sapl:All (after variables were bound) - this is for inter-agent querying
		else if (UbiwareAgent.set_predicates.contains(ss.predicate) && !ss.object.startsWith("?")) {
			this.copyLinks(ss.subject, contextID);
			return;
		}

		else if (!ss.isSubjectContext() && (!ss.isObjectContext() || (ss.objectDatatype != null))) {
			//reference to a local ID -> substitute with real ID
			if (ss.predicate.equals(SaplN3Constants.is) && ss.object.equals(SaplN3Constants.TRUE) && !ss.subject.startsWith("?")) {
				String id = this.ids.get(ss.subject);
				if (id == null) {
					//reference to a real ID, also allowed
					if (this.beliefs.get(ss.subject) != null) {
						id = ss.subject;
					} else {
						this.print("Error: undefined local ID " + ss.subject + " in " + ss);
						return;
					}
				}
				this.putLink(contextID, id);
				return;
			}

			HashSet<String> sub_set = container.index_sub.get(ss.subject);
			if (sub_set != null) {
				HashSet<String> ob_set = container.index_ob.get(ss.object);
				if (ob_set != null) {
					HashSet<String> pred_set = container.index_pred.get(ss.predicate);
					if (pred_set != null) {
						Set<String> result = SetOperation.intersect3Sets(ob_set, sub_set, pred_set);
						if (result.size() > 0) {
							return; // do not add if already exists
						}
					}
				}
			}
		} else if (ss.isSubjectContext() ^ ss.isObjectContext()) {
			//remove "{} is true" wrappings
			if (ss.predicate.equals(SaplN3Constants.is) && ss.object.equals(SaplN3Constants.TRUE)) {
				this.copyLinks(ss.subject, contextID);
				return;
			}

			//unwrap the statement inside the definition of a local ID, ID itself has already been addded in putBelief()
			if (ss.predicate.equals(SaplN3Constants.ID) && ss.isSubjectContext() && !ss.object.startsWith("?")) {
				this.copyLinks(ss.subject, contextID);
				return;
			}

			if (doChecks && !contextID.equals(UbiwareAgent.GOALS_CONTEXT) && !contextID.equals(UbiwareAgent.ACHIEVE_CONTEXT)
					&& !UbiwareAgent.log_predicates.contains(ss.predicate) && !ss.object.equals(SaplN3Constants.Optional)) {
				String requestedID = null;
				boolean toCheck = true;
				Set<String> hs = null;
				if (ss.isSubjectContext()) {
					HashSet<String> ob_set = container.index_ob.get(ss.object);
					if (ob_set != null) {
						Set<String> pred_set = container.index_pred.get(ss.predicate);
						if (pred_set != null) {
							hs = SetOperation.intersect3Sets(ob_set, pred_set, null);
							if (hs.size() == 0) {
								toCheck = false;
							} else {
								requestedID = ss.subject;
							}
						} else {
							toCheck = false;
						}
					} else {
						toCheck = false;
					}
				} else {
					HashSet<String> sub_set = container.index_sub.get(ss.subject);
					if (sub_set != null) {
						HashSet<String> pred_set = container.index_pred.get(ss.predicate);
						if (pred_set != null) {
							hs = SetOperation.intersect3Sets(sub_set, pred_set, null);
							if (hs.size() == 0) {
								toCheck = false;
							} else {
								requestedID = ss.object;
							}
						} else {
							toCheck = false;
						}
					} else {
						toCheck = false;
					}
				}

				if (toCheck) {
					Iterator<String> it = hs.iterator();
					while (it.hasNext()) {
						String matchID = it.next();
						SemanticStatement match = this.beliefs.get(matchID);
						String existID;
						if (ss.isSubjectContext()) {
							existID = match.subject;
						} else {
							existID = match.object;
						}
						if (existID.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
							ContextContainer cont = this.contexts.get(existID);
							if (cont != null) {
								if (!requestedID.equals(existID)) {
									this.copyLinks(requestedID, existID);
								}
								return;
							}
						}
					}
				}
			}
		}

		if (ss.subject.startsWith(UbiwareAgent.STATEMENT_PREFIX)
				|| (ss.object.startsWith(UbiwareAgent.STATEMENT_PREFIX) && (ss.objectDatatype == null)
						&& !ss.predicate.equals(SaplN3Constants.add) && !ss.predicate.equals(SaplN3Constants.remove) && !ss.predicate
							.equals(SaplN3Constants.erase))) {
			this.print("Error: Statements about statements are not allowed: " + ss);
			return;
		}

		//if nothing else has been triggered, just add the link
		ContextContainer cont = this.contexts.get(contextID);
		cont.members.add(statementID);
		ss.referenced++;
		ss.memberOf.add(contextID);
		UbiwareAgent.addToIndices(ss, statementID, cont);
	}

	private static void addToIndex(String key, String statementID, HashMap<String, HashSet<String>> index) {
		HashSet<String> hs = index.get(key);
		if (hs == null) {
			hs = new HashSet<String>(1);
			hs.add(statementID);
			index.put(key, hs);
		} else {
			hs.add(statementID);
		}
	}

	private static void addToIndices(SemanticStatement ss, String statementID, ContextContainer cont) {
		UbiwareAgent.addToIndex(ss.subject, statementID, cont.index_sub);
		UbiwareAgent.addToIndex(ss.predicate, statementID, cont.index_pred);
		UbiwareAgent.addToIndex(ss.object, statementID, cont.index_ob);
		if (ss.isSubjectAny() || ss.isPredicateAny() || ss.isObjectAny()) {
			if (cont.statementsWithVariables == null) {
				cont.statementsWithVariables = new ArrayList<String>();
			}
			cont.statementsWithVariables.add(statementID);
		}
	}

	private static void removeFromIndex(String key, String statementID, HashMap<String, HashSet<String>> index) {
		HashSet<String> hs = index.get(key);
		if (hs.size() > 1) {
			hs.remove(statementID);
		} else {
			index.remove(key);
		}
	}

	private static void removeFromIndices(SemanticStatement ss, String statementID, ContextContainer cont) {
		UbiwareAgent.removeFromIndex(ss.subject, statementID, cont.index_sub);
		UbiwareAgent.removeFromIndex(ss.predicate, statementID, cont.index_pred);
		UbiwareAgent.removeFromIndex(ss.object, statementID, cont.index_ob);
		if ((cont.statementsWithVariables != null) && cont.statementsWithVariables.contains(statementID)) {
			cont.statementsWithVariables.remove(statementID);
		}
	}

	//Removes a link
	void removeLink(String contextID, String statementID) {
		ContextContainer cont = this.contexts.get(contextID);
		if (cont.members.remove(statementID)) {
			SemanticStatement st = this.beliefs.get(statementID);
			if (st != null) {
				UbiwareAgent.removeFromIndices(st, statementID, cont);
				st.referenced--;
				st.memberOf.remove(contextID);
			}
		}
	}

	//Links beliefs from one context to another context
	void copyLinks(String contextID, String targetID) {
		Iterator<String> it = this.contexts.get(contextID).members.iterator();
		while (it.hasNext()) {
			this.putLink(targetID, it.next());
		}
	}

	//Removes beliefs from G using pattern given by the context or the statement given
	void removeByPattern(String ID) {
		if (!ID.startsWith(UbiwareAgent.CONTEXT_PREFIX) && !ID.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
			this.print("Error in \"remove\": Object must be context container or ID of statement");
			return;
		}
		VariablesBindingManager manager = new VariablesBindingManager();

		if (this.live.evaluate(ID, UbiwareAgent.GENERAL_CONTEXT, manager, true)) {
			Iterator<LinkInfo> it = manager.getForRemove().iterator();
			while (it.hasNext()) {
				LinkInfo li = it.next();
				if (!UbiwareAgent.protected_entities.contains(li.statementID)) {
					this.removeLink(li.contextID, li.statementID);
				}
			}
		}
	}

	//When RAB ends its execution, it calls this
	void removeRABInstance(ReusableAtomicBehavior behavior) {
		this.runningRABs.remove(behavior);
		this.rabInstanceIDs.remove(behavior.ID);
		this.removeLink(UbiwareAgent.DO_CONTEXT, behavior.ID);
		this.restartLive();
	}

	//finds out if the given context shares all its statements with some other context(s)
	//as if e.g. created through { statements } P O ; P O 
	HashSet<String> hasParallelContexts(String contextID) {
		ContextContainer cnt = this.contexts.get(contextID);
		if (cnt == null) {
			return null;
		}
		Iterator<String> it = cnt.members.iterator();
		HashSet<String> otherContexts = new HashSet<String>();
		boolean first = true;
		while (it.hasNext()) {
			SemanticStatement st = this.beliefs.get(it.next());
			if (first) {
				if (st.referenced > 1) {
					Iterator<String> cit = st.memberOf.iterator();
					while (cit.hasNext()) {
						String context = cit.next();
						if (!context.equals(contextID)) {
							otherContexts.add(context);
						}
					}
				}
				first = false;
			} else {
				Iterator<String> cit = otherContexts.iterator();
				while (cit.hasNext()) {
					String context = cit.next();
					if (!st.memberOf.contains(context)) {
						cit.remove();
					}
				}
			}
			if (otherContexts.size() == 0) {
				return null;
			}
		}
		return otherContexts;
	}

	//collects garbage, i.e. non-referenced contexts and statements plus empty contexts
	public void collectGarbage() {
		boolean done = false;
		while (!done) {
			done = true;
			//removing contexts that are not referenced or are empty
			List<String> removeContexts = new ArrayList<String>();
			for (Entry<String, ContextContainer> entry : this.contexts.entrySet()) {
				String contextID = entry.getKey();
				if (UbiwareAgent.protected_entities.contains(contextID)) {
					continue;
				}
				ContextContainer contextContainer = entry.getValue();
				if ((contextContainer.referenced <= 0) || contextContainer.members.isEmpty()) {
					removeContexts.add(contextID);
					done = false;
				}
			}

			for (String context : removeContexts) {
				this.removeContext(context);
			}
			/*
			 * replaced by nicer java for loop, see above
			Iterator it = this.contexts.entrySet().iterator();
			Vector<String> removeContexts = new Vector<String>();
			while (it.hasNext()) {
				Map.Entry ent = (Map.Entry) it.next();

				String contextID = ((String) ent.getKey());
				if (UbiwareAgent.protected_entities.contains(contextID)) {
					continue;
				}

				ContextContainer cont = (ContextContainer) ent.getValue();
				if ((cont.referenced <= 0) || cont.members.isEmpty()) {
					removeContexts.add(contextID);
					done = false;
				}
			}
			
			for (int i = 0; i < removeContexts.size(); i++) {
				this.removeContext(removeContexts.elementAt(i));
			}
			*/
			List<String> removeBeliefs = new ArrayList<String>();
			for (Entry<String, SemanticStatement> entry : this.beliefs.entrySet()) {
				SemanticStatement statement = entry.getValue();
				if (statement.referenced <= 0) {
					removeBeliefs.add(entry.getKey());
					done = false;
				}
			}

			for (String belief : removeBeliefs) {
				this.removeBelief(belief);
			}
		}
	}

	//copies a sub-graph(with binding variables) into another context, e.g. G
	List<LinkInfo> copy(String ID, String targetContextID, HashMap<String, String> vars) {
		if (!ID.startsWith(UbiwareAgent.CONTEXT_PREFIX) && !ID.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
			this.print("Error in \"add\": Object must be context container or ID of statement");
			return new ArrayList<LinkInfo>();
		}
		List<String> tvars = null;
		if (vars != null) {
			tvars = new ArrayList<String>(vars.keySet());
			Collections.sort(tvars);
		}

		if ((this.bnNameBase == null) || !this.bnNameBase.contains(String.valueOf(System.currentTimeMillis()))) {
			this.bnCount = 0;
		}
		this.bNodeMap = null;
		this.bnNameBase = null;

		List<LinkInfo> result = this.copyRecursion(ID, targetContextID, vars, tvars, false);
		return result;
	}

	List<LinkInfo> copy(List<String> vecID, String targetContextID, HashMap<String, String> vars) {
		List<String> tvars = null;
		if (vars != null) {
			tvars = new ArrayList<String>(vars.keySet());
			Collections.sort(tvars);
		}

		List<LinkInfo> result = new ArrayList<LinkInfo>();

		if ((this.bnNameBase == null) || !this.bnNameBase.contains(String.valueOf(System.currentTimeMillis()))) {
			this.bnCount = 0;
		}
		this.bNodeMap = null;
		this.bnNameBase = null;

		//Iterator<String> it = vecID.iterator();
		//while (it.hasNext()) {
		for (String toBeAdded : vecID) {
			result.addAll(this.copyRecursion(toBeAdded, targetContextID, vars, tvars, false));
		}
		return result;
	}

	private List<LinkInfo> copyRecursion(String ID, String targetContextID, HashMap<String, String> vars, List<String> tvars,
			boolean insideImplies) {

		List<LinkInfo> toAdd = new ArrayList<LinkInfo>();
		Iterator<String> it;
		if (ID.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
			ContextContainer c = this.contexts.get(ID);
			if (c == null) {
				this.print("Error: context container does not exist: " + ID);
				return toAdd;
			}
			it = c.members.iterator();
		} else {
			List<String> tv = new ArrayList<String>();
			tv.add(ID);
			it = tv.iterator();
		}

		while (it.hasNext()) {
			String old_id = it.next();
			String id = this.ids.get(old_id);

			boolean copy = true;

			if (id == null) { //have not been copied yet
				SemanticStatement st = this.beliefs.get(old_id);
				SemanticStatement newSt = new SemanticStatement(st);
				if (vars != null) {
					for (int i = tvars.size() - 1; i >= 0; i--) {
						String variable = tvars.get(i);
						if (!variable.equals("") && !variable.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
							String value = vars.get(variable);
							boolean unbound = value.equals(UbiwareAgent.UNBOUND_VARIABLE);
							if (unbound) {
								value = "";
							}
							String replace = "?" + variable;
							if (newSt.subject.contains(replace)) {
								if (unbound && newSt.subject.equals(replace)) {
									copy = false;
								} else {
									newSt.subject = newSt.subject.replace(replace, value);
								}
							}
							if (copy && newSt.predicate.contains(replace)) {
								if (unbound && newSt.predicate.equals(replace)) {
									copy = false;
								} else {
									newSt.predicate = newSt.predicate.replace(replace, value);
								}
							}
							if (copy && newSt.object.contains(replace)) {
								if (unbound && newSt.object.equals(replace) && !UbiwareAgent.set_predicates.contains(newSt.predicate)) {
									copy = false;
								} else {
									newSt.object = newSt.object.replace(replace, value);
								}
							}
						}
					}
				}

				if (copy) {
					if (!insideImplies && UbiwareAgent.embedded_predicates.contains(newSt.predicate) && !newSt.subject.startsWith("?")
							&& !newSt.object.contains("?")) {
						copy = false;
					}
				}

				if (copy) {
					if (newSt.subject.startsWith("<" + UbiwareAgent.BLANK_NODE_NS)) {
						if (this.bNodeMap == null) {
							this.bNodeMap = new HashMap<String, String>();
							this.bnNameBase = "<" + UbiwareAgent.BLANK_NODE_NS + this.getLocalName()
									+ String.valueOf(System.currentTimeMillis()) + "_";
						}
						String newName = this.bNodeMap.get(newSt.subject);
						if (newName == null) {
							newName = this.bnNameBase + String.valueOf(++this.bnCount) + ">";
							this.bNodeMap.put(newSt.subject, newName);
						}
						newSt.subject = newName;
					}
					if (newSt.object.startsWith("<" + UbiwareAgent.BLANK_NODE_NS)) {
						if (this.bNodeMap == null) {
							this.bNodeMap = new HashMap<String, String>();
							this.bnNameBase = "<" + UbiwareAgent.BLANK_NODE_NS + this.getLocalName()
									+ String.valueOf(System.currentTimeMillis()) + "_";
						}
						String newName = this.bNodeMap.get(newSt.object);
						if (newName == null) {
							newName = this.bnNameBase + String.valueOf(++this.bnCount) + ">";
							this.bNodeMap.put(newSt.object, newName);
						}
						newSt.object = newName;
					}
				}

				if (copy) {
					if (!newSt.predicate.equals(SaplN3Constants.hasMember)) {
						if (st.isSubjectContext()) {
							String newCid = this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++this.contextCount);
							this.putContext(newCid);
							toAdd.addAll(this.copyRecursion(st.subject, newCid, vars, tvars,
									(UbiwareAgent.inference_predicates.contains(newSt.predicate)) ? true : false));
							newSt.subject = newCid;
						} else if (newSt.isSubjectContext()) {
							ContextContainer c = this.contexts.get(newSt.subject);
							if (c.referenced > 0) { //disallow making two statements about the same context
								String newCid = this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++this.contextCount);
								this.putContext(newCid, newSt.subject);
								newSt.subject = newCid;
							}
						}
					}

					if (!newSt.predicate.equals(SaplN3Constants.memberOf) && !newSt.predicate.equals(SaplN3Constants.erase)) {
						if ((newSt.objectDatatype == null) && st.isObjectContext()) {
							String newCid = this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++this.contextCount);
							this.putContext(newCid);
							toAdd.addAll(this.copyRecursion(st.object, newCid, vars, tvars, false));
							newSt.object = newCid;
						} else if ((newSt.objectDatatype == null) && newSt.isObjectContext()) {
							ContextContainer c = this.contexts.get(newSt.object);
							if (c.referenced > 0) {//disallow making two statements about the same context
								String newCid = this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++this.contextCount);
								this.putContext(newCid, newSt.object);
								newSt.object = newCid;
							}
						}
					}

					id = this.statementIDGenerator.getNewStatementID();//UbiwareAgent.STATEMENT_PREFIX + String.valueOf(++this.statementCount);

					this.putBelief(id, newSt);
					this.ids.put(old_id, id);
				}
			}
			if (copy) {
				toAdd.add(new LinkInfo(targetContextID, id));
			}
		}
		return toAdd;
	}

	public ResourcePrefixer getResourcePrefixer() {
		return this.prefixer;
	}

	//called from SaplN3.load so that the agent knows the namespace prefices that were used
	void addNameSpaces(HashMap<String, String> namespaces) {
		this.prefixer.addNameSpaces(namespaces);
	}

	/** End of Methods used by Live, or onEnd of ReusableAtomicBehavior **/

	/** Start of Methods used by Reusable Atomic Behaviors **/

	//wakes the agent
	void restartLive() {
		this.live.restart();
	}

	//prints the message preceeded by the timestamp and the name of the agent
	void print(Object ob) {
		String str;

		if (ob != null) {
			str = ob.toString(); //works for both String and non-String
		} else {
			str = "null";
		}
		Calendar now = Calendar.getInstance();
		int hour = now.get(Calendar.HOUR_OF_DAY);
		int minute = now.get(Calendar.MINUTE);
		int second = now.get(Calendar.SECOND);
		String time = ((hour < 10) ? "0" : "") + String.valueOf(hour) + ":" + ((minute < 10) ? "0" : "") + String.valueOf(minute) + ":"
				+ ((second < 10) ? "0" : "") + String.valueOf(second);

		System.out.println("[" + time + "] " + this.getLocalName() + ": " + str);
	}

	//adds beliefs (copies a sub-graph given with ID can be of a context or a statement) 
	//with binding variables (if given)
	void addBeliefs(String ID, HashMap<String, String> vars) {
		if (!ID.startsWith(UbiwareAgent.CONTEXT_PREFIX) && !ID.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
			this.print("Error in addBeliefs: the argument must be context container or ID of a statement");
			return;
		}
		this.clearLocalIDs();
		List<LinkInfo> addLinks = this.copy(ID, UbiwareAgent.GENERAL_CONTEXT, vars);
		this.clearLocalIDs();
		for (LinkInfo li : addLinks) {
			this.putLink(li.contextID, li.statementID);
		}
	}

	/**
	 * Same add {@link #addBeliefsN3(String, boolean)} with doChecks true
	 * 
	 * @param n3string
	 *            The string in N3 notation
	 * @return whether adding the belief was successful.
	 */
	public boolean addBeliefsN3(String n3string) {
		return this.addBeliefsN3(n3string, true);
	}

	/**
	 * Adds beliefs defined by a N3 string
	 * 
	 * @param n3string
	 *            The String contianing S-APL to be added to the beliefs.
	 * @param doChecks
	 * @return
	 */
	boolean addBeliefsN3(String n3string, boolean doChecks) {
		//SaplN3 sapl = new SaplN3(this);
		List<String> code = SaplN3.compile(n3string, "addBeliefsN3");
		if (code != null) {
			this.clearLocalIDs();
			SaplN3.load(code, this, UbiwareAgent.GENERAL_CONTEXT, "addBeliefsN3", doChecks);
			return true;
		}
		return false;
	}

	//remove beliefs by pattern given (ID can be of a context or a statement)
	void removeBeliefs(String ID) {
		if (!ID.startsWith(UbiwareAgent.CONTEXT_PREFIX) && !ID.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
			this.print("Error in removeBeliefs: the argument must be context container or ID of a statement");
			return;
		}
		this.removeByPattern(ID);
	}

	//remove beliefs specified in a N3 string
	public void removeBeliefsN3(String n3string) {
		//SaplN3 sapl = new SaplN3(this);
		List<String> code = SaplN3.compile(n3string, "removeBeliefsN3");
		if (code != null) {
			String id = this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++this.contextCount);
			this.putContext(id);
			this.clearLocalIDs();
			SaplN3.load(code, this, id, "removeBeliefsN3", true);
			this.removeByPattern(id);
			this.removeContext(id);
		}
	}

	//erase: ID can be of a context or a statement
	void eraseBeliefs(String ID) {
		if (ID.startsWith(UbiwareAgent.STATEMENT_PREFIX) && (this.beliefs.get(ID) != null)) {
			this.removeBelief(ID);
		} else if (ID.startsWith(UbiwareAgent.CONTEXT_PREFIX) && (this.contexts.get(ID) != null)) {
			this.removeContext(ID);
		}
	}

	//returns a vector of possible solutions (sets of bindings of variables) to 
	//beliefs given (with ID can be of a context or a statement)
	BindingsSet hasBeliefs(String ID, String queryContextID) {
		if (!ID.startsWith(UbiwareAgent.CONTEXT_PREFIX) && !ID.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
			this.print("Error in hasBeliefs: the argument must be context container or ID of a statement");
			return null;
		}
		VariablesBindingManager manager = new VariablesBindingManager();
		//boolean result=live.evaluate(ID, GENERAL_CONTEXT, manager);
		boolean result = this.live.evaluate(ID, queryContextID, manager);
		if (result == true) {
			return new BindingsSet(manager.solutions);
		}
		return null;
	}

	BindingsSet hasBeliefs(String ID) {
		return this.hasBeliefs(ID, UbiwareAgent.GENERAL_CONTEXT);
	}

	//returns a vector of possible solutions (sets of bindings of variables) to 
	//beliefs specified in a N3 string
	BindingsSet hasBeliefsN3(String n3string, String queryContextID) {
		//SaplN3 sapl = new SaplN3(this);
		List<String> code = SaplN3.compile(n3string, "hasBeliefsN3");
		if (code != null) {
			String id = this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++this.contextCount);
			this.putContext(id);
			this.clearLocalIDs();
			SaplN3.load(code, this, id, "hasBeliefsN3", true);
			VariablesBindingManager manager = new VariablesBindingManager();
			boolean result = this.live.evaluate(id, queryContextID, manager);
			this.removeContext(id);
			if (result == true) {
				return new BindingsSet(manager.solutions);
			}
			return null;
		}
		return null;
	}

	public BindingsSet hasBeliefsN3(String n3string) {
		return this.hasBeliefsN3(n3string, UbiwareAgent.GENERAL_CONTEXT);
	}

	//moves all links from a context to General Context and removes the former context 
	void moveBeliefs(String contextID) {
		this.copyLinks(contextID, UbiwareAgent.GENERAL_CONTEXT);
		this.removeContext(contextID);
	}

	//produces an N3 string from a sub-graph
	String produceN3(String contextID) {
		return this.produceN3(contextID, ProduceN3Options.defaultOptions);
	}

	String produceN3(String contextID, ProduceN3Options options) {
		this.collectGarbage(); // otherwise, statements may still be referenced several times which will slow down the procedure a lot
		//SaplN3 sapl = new SaplN3(this);
		return SaplN3.produceN3(contextID, options, this);
	}

	//returns the vector of copies of statements in a context container (+ a vector of their original IDs)
	List<SemanticStatement> getStatements(String contextID) {
		return this.getStatements(contextID, null);
	}

	List<SemanticStatement> getStatements(String contextID, List<String> originalIDs) {
		ContextContainer c = this.contexts.get(contextID);
		if (c == null) {
			this.print("Error in getStatements: context container does not exist: " + contextID);
			return null;
		}
		if (originalIDs != null) {
			originalIDs.clear();
		}
		List<SemanticStatement> result = new ArrayList<SemanticStatement>();
		Iterator<String> it = c.members.iterator();
		while (it.hasNext()) {
			String id = it.next();
			result.add(new SemanticStatement(this.beliefs.get(id)));
			if (originalIDs != null) {
				originalIDs.add(id);
			}
		}
		return result;
	}

	//adds an object to blackboard, return the ID of the created object
	public Resource addOnBlackboard(Object o) {
		return this.blackboard.addObject(o);
	}

	//removes an object from blackboard
	public void removeFromBlackboard(Resource id) {
		this.blackboard.removeObject(id);
	}

	//returns an object from blackboard
	Object getFromBlackboard(Resource id) {
		return this.blackboard.getObject(id);
	}

	//adds beliefs to a temporary context, returns the ID of the context
	public String addToTempContext(String ID) {
		String id = this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++this.contextCount);
		this.putContext(id);
		if (ID.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
			this.copyLinks(ID, id);
		} else {
			this.putLink(id, ID);
		}
		return id;
	}

	//adds beliefs defined by a N3 string to a temporary context, returns the ID of the context
	public String addN3ToTempContext(String n3string) {
		List<String> code = SaplN3.compile(n3string, "addN3ToTempContext");
		if (code != null) {
			String id = this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++this.contextCount);
			this.putContext(id);
			this.clearLocalIDs();
			SaplN3.load(code, this, id, "addN3ToTempContext", true);
			return id;
		}
		return null;
	}

	/** End of Methods used by Reusable Atomic Behaviors **/

	//For DebugBehavior
	public class DebugController {
		private DebugController() {
			//No construction from outside the agent.
		}

		public void setDebugger(Debugger d) {
			UbiwareAgent.this.debugger = d;
		}

		public void stopNow() {
			UbiwareAgent.this.stopAfterLifeCycle = UbiwareAgent.this.lifeCycleCount;
		}

		public void stopAfter(int cycles) {
			UbiwareAgent.this.stopAfterLifeCycle = UbiwareAgent.this.lifeCycleCount + cycles;
			UbiwareAgent.this.restartLive();
		}

		public void start() {
			UbiwareAgent.this.stopAfterLifeCycle = -1;
			UbiwareAgent.this.restartLive();
		}

	}

	/*
		//For system RABs
		public class CoreAccess {
			private CoreAccess() {
				//no construction from outside the agent.
			}

			public boolean checkValue(String value1, String predicate, String value2) {
				return UbiwareAgent.this.live.checkValue(value1, predicate, value2);
			}

			public String unwrap(String ID, String queryContextID, boolean add) {
				return UbiwareAgent.this.live.unwrapSetPredicates(ID, queryContextID, add);
			}

			public void execute(String contextID, VariablesBindingManager manager, boolean inference) {
				UbiwareAgent.this.live.executeRule(contextID, manager, inference);
				UbiwareAgent.this.live.handleChanges();
			}
		}
	*/

	private static final URI executionIDNamespace = URITools.createUncheckedURI("http://ubiware.jyu.fi/executionID/");

	//The Deliberation Behavior
	class Live extends CyclicBehaviour {
		private Live() {
			//no construction from outside the agent
		}

		private final UniqueInstanceGenerator executionIDGenerator = new UniqueInstanceGenerator(UbiwareAgent.this.getLocalName(),
				UbiwareAgent.executionIDNamespace);

		private static final long serialVersionUID = 1L;

		private UbiwareAgent myAgent;

		@Override
		public void setAgent(Agent agent) {
			if (!(agent instanceof UbiwareAgent)) {
				throw new Error("Reusable Atomic behavior only makes sense for Ubiware Agents.");
			}
			UbiwareAgent theAgent = (UbiwareAgent) agent;
			super.setAgent(theAgent);
			this.myAgent = (UbiwareAgent) agent;
		}

		//Should this behavior block in the next execution of action() ???
		private boolean block = true;

		private List<LinkInfo> addLinks; //order is important
		private HashSet<LinkInfo> removeLinks;
		private HashSet<String> removeBeliefs;
		private HashSet<String> removeContexts;

		private String goals_var = "";
		private HashSet<String> handledGoals;
		private HashSet<String> handledCommitments;
		private HashSet<String> handledEvents;

		//TODO Can this Vector better be implemented as a SortedSet?
		final private List<Long> waitingFor = new ArrayList<Long>(); //a queue of future time moments that are conditions in some rules or existsWhile
		private Map<String, Class<?>> behaviorClasses = new HashMap<String, Class<?>>(); //Keeps a reference to the RAB's this agent has used before for quicker loading
		private Set<String> nonAvailableBehaviors = new HashSet<String>(); //RAB .class files that are missing

		private List<String> globalForAll; //sapl:All defined on the left side of sapl:implies or similar
		private List<String> globalForSome; //sapl:Some defined on the left side of sapl:implies or similar
		private List<String> globalOrderBy; //sapl:OrderBy defined on the left side of sapl:implies or similar
		private int globalLimit = -1; //sapl:Limit defined on the left side of sapl:implies or similar
		private int globalOffset = 0; //sapl:Offset defined on the left side of sapl:implies or similar

		//used inside evaluate()
		// I think this is used after the object of the statement is evaluated (evaluation is needed if it's some function call like indexOf, sum, avg or if it's some expression (?a+?b))
		// Example: ?x3 sapl:expression ?x1+?x2
		private boolean checkValue(String value1, String predicate, String value2) {

			if (predicate.equals("<" + UbiwareAgent.RDF_NS + "type>") && value2.equals(SaplN3Constants.Container)) {
				if (value1.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
					return true;
				}
				return false;
			}
			if (predicate.equals("<" + UbiwareAgent.RDF_NS + "type>") && value2.equals(SaplN3Constants.Variable)) {
				if (value1.startsWith("?")) {
					return true;
				}
				return false;
			}

			if (predicate.equals(SaplN3Constants.regex)) {
				return value1.matches(value2);
			}

			boolean numberValues = true;
			double v1 = 0, v2 = 0;
			try {
				v1 = Double.valueOf(value1);
				v2 = Double.valueOf(value2);
			} catch (NumberFormatException e) {
				numberValues = false;
			}

			if (predicate.equals(SaplN3Constants.eq)) {
				if (numberValues) {
					return v1 == v2;
				}
				return value2.equals("*") || value1.equals(value2);
			} else if (predicate.equals(SaplN3Constants.neq)) {
				if (numberValues) {
					return v1 != v2;
				}
				return !value2.equals("*") && !value1.equals(value2);
			} else if (predicate.equals(SaplN3Constants.gt)) {
				if (numberValues) {
					return v1 > v2;
				}
				return value1.compareTo(value2) > 0;
			} else if (predicate.equals(SaplN3Constants.lt)) {
				if (numberValues) {
					return v1 < v2;
				}
				return value1.compareTo(value2) < 0;
			} else if (predicate.equals(SaplN3Constants.gte)) {
				if (numberValues) {
					return v1 >= v2;
				}
				return value1.compareTo(value2) >= 0;
			} else if (predicate.equals(SaplN3Constants.lte)) {
				if (numberValues) {
					return v1 <= v2;
				}
				return value1.compareTo(value2) <= 0;
			} else if (predicate.equals(SaplN3Constants.startsWith)) {
				return value1.startsWith(value2);
			} else if (predicate.equals(SaplN3Constants.endsWith)) {
				return value1.endsWith(value2);
			} else if (predicate.equals(SaplN3Constants.contains)) {
				return value1.contains(value2);
			} else if (predicate.equals(SaplN3Constants.doesNotStartWith)) {
				return !value1.startsWith(value2);
			} else if (predicate.equals(SaplN3Constants.doesNotEndWith)) {
				return !value1.endsWith(value2);
			} else if (predicate.equals(SaplN3Constants.doesNotContain)) {
				return !value1.contains(value2);
			} else {
				return false;
			}
		}

		//used inside evaluate(), unwraps statements inside all layers of {} sapl:All ?x , {} sapl:Some ?x, {} sapl:OrderBy ?x, {} sapl:Limit X and {} sapl:Offset X
		//return final inner contextID, records, if add==true, defined values to corresponding global structures 
		private String unwrapSetPredicates(String ID, String queryContextID, boolean add) {
			if (queryContextID.equals(UbiwareAgent.GENERAL_CONTEXT) && ID.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
				List<String> m = UbiwareAgent.this.contexts.get(ID).members;
				if (m.size() == 1) {
					SemanticStatement st = UbiwareAgent.this.beliefs.get(m.get(0));
					if (st.isSubjectContext()
							&& (UbiwareAgent.set_predicates.contains(st.predicate) || UbiwareAgent.set2_predicates.contains(st.predicate))) {
						if (st.predicate.equals(SaplN3Constants.All)) {
							if (add && st.object.startsWith("?")) {
								this.globalForAll.add(st.object.substring(1));
							}
							return this.unwrapSetPredicates(st.subject, queryContextID, add);
						} else if (st.predicate.equals(SaplN3Constants.Some)) {
							if (add && st.object.startsWith("?")) {
								this.globalForSome.add(st.object.substring(1));
							}
							return this.unwrapSetPredicates(st.subject, queryContextID, add);
						} else if (st.predicate.equals(SaplN3Constants.OrderBy)) {
							if (add && st.object.startsWith("?")) {
								this.globalOrderBy.add(st.object.substring(1));
							}
							return this.unwrapSetPredicates(st.subject, queryContextID, add);
						} else if (st.predicate.equals(SaplN3Constants.OrderByDescending)) {
							if (add && st.object.startsWith("?")) {
								this.globalOrderBy.add(" " + st.object.substring(1));
							}
							return this.unwrapSetPredicates(st.subject, queryContextID, add);
						} else if (st.predicate.equals(SaplN3Constants.Limit)) {
							if (add) {
								try {
									int value = Integer.valueOf(st.object);
									if (value < 1) {
										throw new NumberFormatException();
									}
									this.globalLimit = value;
								} catch (NumberFormatException e) {
									UbiwareAgent.this.print("Error in defining Limit: the object must be integer > 0 : Ignored");
								}
							}
							return this.unwrapSetPredicates(st.subject, queryContextID, add);
						} else if (st.predicate.equals(SaplN3Constants.Offset)) {
							if (add) {
								try {
									int value = Integer.valueOf(st.object);
									if (value < 0) {
										throw new NumberFormatException();
									}
									this.globalOffset = value;
								} catch (NumberFormatException e) {
									UbiwareAgent.this.print("Error in defining Offset: the object must be integer >= 0 : Ignored");
								}
							}
							return this.unwrapSetPredicates(st.subject, queryContextID, add);
						}
					}
				}
			}
			return ID;
		}

		//matches a sub-graph with (ID is either of context or a statement), normally, G
		//modes: 0-normal check:all must match, 1-some (used only when removing beliefs)
		private boolean evaluate(String ID, String queryContextID, VariablesBindingManager manager) {
			return this.evaluate(ID, queryContextID, manager, false, null);
		}

		private boolean evaluate(String ID, String queryContextID, VariablesBindingManager manager, boolean removeMode) {
			return this.evaluate(ID, queryContextID, manager, removeMode, null);
		}

		/*
		 * Most likely a very important method used for evaluation of left side of a rule
		 * Inputs probably:
		 * ID - ID of a left side of a rule (e.g. if we have a rule "_C1 sapl:implies _C2", then we use ID = "_C1")
		 * queryContextID - where we should query data from (very often the main G context)
		 * manager - as name says VariablesBindingManager (all possible variable-to-value pairs)
		 * removeMode - not sure right now
		 * idVars - not sure, but probably all the variable IDs
		 */
		boolean evaluate(String ID, String queryContextID, VariablesBindingManager manager, boolean removeMode, List<String> idVars) {

			List<String> time_vars = new ArrayList<String>(); //for catching time-related conditions 

			if (idVars == null) {
				ID = this.unwrapSetPredicates(ID, queryContextID, true);
			} else {
				ID = this.unwrapSetPredicates(ID, queryContextID, false); //it is not an upper-level call, globalForAll, globalForSome, and globalOrderBy should not be affected
			}

			List<String> falseIf = null; //doNotBelieve conditions, checked after the normal ones
			List<String> localIf = null; // conditions on variables, checked locally and after the normal ones
			List<String> calcIf = null; // calculated variables, evaluated after normal conditions
			List<String> statIf = null; // calculated variables defined on a set of matching solutions, rather than inside a solution

			Iterator<String> it;
			if (ID.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
				it = UbiwareAgent.this.contexts.get(ID).members.iterator();
			} else {
				List<String> tv = new ArrayList<String>();
				tv.add(ID);
				it = tv.iterator();
			}

			// let's go through all the statements in the container with the identifier ID (argument of this method) = all the statements in the left side of the rule
			while (it.hasNext()) {

				String id = it.next();
				SemanticStatement st = UbiwareAgent.this.beliefs.get(id);

				String queryContextID1 = queryContextID;

				//exclusive condition -> to treat later
				if (st.subject.equals(SaplN3Constants.I) && st.predicate.equals(SaplN3Constants.doNotBelieve) && st.isObjectContext()) {
					if (falseIf == null) {
						falseIf = new ArrayList<String>();
					}
					falseIf.add(st.object);
					continue;
				}

				//filtering and statistic predicates -> to treat later
				if (queryContextID.equals(UbiwareAgent.GENERAL_CONTEXT) && UbiwareAgent.embedded_predicates.contains(st.predicate)
						&& !st.isObjectContext()) {
					if (st.predicate.equals(SaplN3Constants.gt) || st.predicate.equals(SaplN3Constants.lt)
							|| st.predicate.equals(SaplN3Constants.gte) || st.predicate.equals(SaplN3Constants.lte)
							|| st.predicate.equals(SaplN3Constants.regex) || st.predicate.equals(SaplN3Constants.neq)
							|| st.predicate.equals(SaplN3Constants.startsWith) || st.predicate.equals(SaplN3Constants.endsWith)
							|| st.predicate.equals(SaplN3Constants.contains) || st.predicate.equals(SaplN3Constants.doesNotStartWith)
							|| st.predicate.equals(SaplN3Constants.doesNotEndWith) || st.predicate.equals(SaplN3Constants.doesNotContain)
							|| st.predicate.equals(SaplN3Constants.eq)) {
						if (localIf == null) {
							localIf = new ArrayList<String>();
						}
						localIf.add(id);
						continue;
					} else if (st.subject.startsWith("?")) {
						if (st.predicate.equals(SaplN3Constants.expression)) {
							if (calcIf == null) {
								calcIf = new ArrayList<String>();
							}
							calcIf.add(id);
							continue;
						} else if (st.predicate.equals(SaplN3Constants.count) || st.predicate.equals(SaplN3Constants.countGroupedBy)
								|| st.predicate.equals(SaplN3Constants.max) || st.predicate.equals(SaplN3Constants.min)
								|| st.predicate.equals(SaplN3Constants.sum)) {
							if (statIf == null) {
								statIf = new ArrayList<String>();
							}
							statIf.add(id);
							continue;
						}
					}
				}
				if (st.predicate.equals("<" + UbiwareAgent.RDF_NS + "type>")
						&& (st.object.equals(SaplN3Constants.Container) || st.object.equals("<" + SaplN3Constants.Variable))) {
					if (localIf == null) {
						localIf = new ArrayList<String>();
					}
					localIf.add(id);
					continue;
				}

				//unwrap the statement inside a match for ID; ask later the VariablesBindingManager to find the ID
				List<String> id_vars;
				if (idVars == null) {
					id_vars = new ArrayList<String>();
				} else {
					id_vars = new ArrayList<String>(idVars);
				}
				if (st.predicate.equals(SaplN3Constants.ID) && st.object.startsWith("?") && st.isSubjectContext()) {
					id_vars.add(st.object.substring(1));
					String new_id;
					if (idVars == null) {
						new_id = this.unwrapSetPredicates(st.subject, queryContextID, true);
					} else {
						new_id = this.unwrapSetPredicates(st.subject, queryContextID, false);
					}
					boolean ok = this.evaluate(new_id, queryContextID, manager, removeMode, id_vars);
					if (!ok) {
						return false;
					}
					continue;
				}

				//optional condition
				if (st.predicate.equals(SaplN3Constants.is) && st.object.equals(SaplN3Constants.Optional) && st.isSubjectContext()) {
					VariablesBindingManager sub_manager = new VariablesBindingManager(manager.solutions);
					sub_manager.solutionIndices = manager.solutionIndices;
					if (removeMode) {
						sub_manager.forRemove = new HashMap<String, HashSet<LinkInfo>>(manager.forRemove);
					}
					boolean ok = this.evaluate(st.subject, queryContextID, sub_manager, removeMode, id_vars);
					if (ok) {
						HashSet<String> b_ids = new HashSet<String>();
						for (int i = 0; i < sub_manager.solutions.size(); i++) {
							String bid = sub_manager.solutions.get(i).get("");
							int index = 0;
							b_ids.add(bid);
							while (true) {
								index = bid.indexOf(".", index + 1);
								if (index != -1) {
									b_ids.add(bid.substring(0, index));
								} else {
									break;
								}
							}
						}

						List<HashMap<String, String>> newSolutions = new ArrayList<HashMap<String, String>>();
						HashMap<String, HashSet<LinkInfo>> newForRemove = null;
						if (removeMode) {
							newForRemove = new HashMap<String, HashSet<LinkInfo>>();
						}
						for (int i = 0; i < manager.solutions.size(); i++) {
							HashMap<String, String> solution = manager.solutions.get(i);
							String solId = solution.get("");
							if (!b_ids.contains(solId)) {
								newSolutions.add(solution);
								if (removeMode) {
									newForRemove.put(solId, manager.forRemove.get(solId));
								}
							}
						}
						newSolutions.addAll(sub_manager.solutions);
						if (removeMode) {
							newForRemove.putAll(sub_manager.forRemove);
						}
						manager.solutions = newSolutions;
						if (removeMode) {
							manager.forRemove = newForRemove;
						}
					}
					manager.definedVars.addAll(sub_manager.definedVars);
					continue;
				}

				//OR conditions
				if (st.predicate.equals(SaplN3Constants.or) && st.isSubjectContext() && st.isObjectContext()) {
					List<HashMap<String, String>> newSolutions = new ArrayList<HashMap<String, String>>();
					HashMap<String, HashSet<LinkInfo>> newForRemove = null;
					if (removeMode) {
						newForRemove = new HashMap<String, HashSet<LinkInfo>>();
					}

					VariablesBindingManager sub_manager = new VariablesBindingManager(manager.solutions);
					sub_manager.solutionIndices = manager.solutionIndices;
					if (removeMode) {
						sub_manager.forRemove = new HashMap<String, HashSet<LinkInfo>>(manager.forRemove);
					}
					boolean ok1 = this.evaluate(st.subject, queryContextID, sub_manager, removeMode, id_vars);

					VariablesBindingManager ob_manager = new VariablesBindingManager(manager.solutions);
					ob_manager.solutionIndices = manager.solutionIndices;
					if (removeMode) {
						ob_manager.forRemove = new HashMap<String, HashSet<LinkInfo>>(manager.forRemove);
					}
					boolean ok2 = this.evaluate(st.object, queryContextID, ob_manager, removeMode, id_vars);

					if (!ok1 && !ok2) {
						return false;
					} else if (ok1 && !ok2) {
						newSolutions.addAll(sub_manager.solutions);
						if (removeMode) {
							newForRemove.putAll(sub_manager.forRemove);
						}
					} else if (!ok1 && ok2) {
						newSolutions.addAll(ob_manager.solutions);
						if (removeMode) {
							newForRemove.putAll(ob_manager.forRemove);
						}
					} else {
						newSolutions.addAll(sub_manager.solutions);
						if (removeMode) {
							newForRemove.putAll(sub_manager.forRemove);
						}
						HashMap<String, String> index_newSolutions = new HashMap<String, String>();
						Iterator<HashMap<String, String>> newIt = sub_manager.solutions.iterator();
						while (newIt.hasNext()) {
							HashMap<String, String> temp = new HashMap<String, String>(newIt.next());
							String tempid = temp.remove("");
							index_newSolutions.put(temp.toString(), tempid);
						}

						newIt = ob_manager.solutions.iterator();
						while (newIt.hasNext()) {
							HashMap<String, String> solution = newIt.next();
							String solID = solution.get("");
							HashMap<String, String> temp = new HashMap<String, String>(solution);
							temp.remove("");
							String tempid = index_newSolutions.get(temp.toString());
							if (tempid == null) {
								newSolutions.add(solution);
								if (removeMode) {
									newForRemove.put(solID, ob_manager.forRemove.get(solID));
								}
							} else if (removeMode) {
								HashSet<LinkInfo> t = ob_manager.forRemove.get(solID);
								HashSet<LinkInfo> t2 = newForRemove.get(tempid);
								if ((t != null) && (t2 != null)) {
									t2.addAll(t);
								}
							}
						}
					}

					manager.definedVars.addAll(sub_manager.definedVars);
					manager.definedVars.addAll(ob_manager.definedVars);
					manager.solutions = newSolutions;
					if (removeMode) {
						manager.forRemove = newForRemove;
					}
					continue;
				}

				SemanticStatement st1 = new SemanticStatement(st);
				if (st.subject.startsWith("<" + UbiwareAgent.BLANK_NODE_NS)) {
					st1.subject = "?" + st.subject;
				}
				if (st.object.startsWith("<" + UbiwareAgent.BLANK_NODE_NS)) {
					st1.object = "?" + st.object;
				}

				//check hasMember or memberOf condition
				if (st.predicate.equals(SaplN3Constants.hasMember) || st.predicate.equals(SaplN3Constants.memberOf)) {
					String sts;
					String context;
					if (st.predicate.equals(SaplN3Constants.hasMember)) {
						sts = st1.object;
						context = st1.subject;
					} else {
						sts = st1.subject;
						context = st1.object;
					}

					if (context.startsWith(UbiwareAgent.CONTEXT_PREFIX) || context.startsWith("?")) {
						if (sts.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
							boolean ok = this.evaluate(sts, context, manager, removeMode, id_vars);
							if (!ok) {
								return false;
							}
							continue;
						} else if (sts.startsWith("?")) {
							if (!manager.definedVars.contains(sts.substring(1))) {
								st1 = new SemanticStatement(sts, SaplN3Constants.is, SaplN3Constants.TRUE);
								queryContextID1 = context;
							} else {
								if (context.startsWith("?") && !manager.definedVars.contains(context.substring(1))) {
									UbiwareAgent.this.print("Error: Wrong query: unbound yet context: " + context);
								} else {
									List<Integer> toRemove = new ArrayList<Integer>();
									int n_binds = manager.getNumberOfSolutions();
									for (int i = 0; i < n_binds; i++) {
										HashMap<String, String> solution = manager.getSolution(i);
										String cnt_value;
										if (context.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
											cnt_value = context;
										} else {
											cnt_value = solution.get(context.substring(1));
										}
										String sts_value = solution.get(sts.substring(1));
										if (sts_value.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
											UbiwareAgent.this.print("Error: Wrong query: variable value (" + sts
													+ ") must be a statement ID, but a context ID found: " + sts_value);
											toRemove.add(i);
										} else if (!UbiwareAgent.this.contexts.get(cnt_value).members.contains(sts_value)) {
											toRemove.add(i);
										}
									}
									manager.removeSolutions(toRemove);
									if (manager.getNumberOfSolutions() == 0) {
										return false;
									}
									continue;
								}
							}
						} else {
							UbiwareAgent.this.print("Error: Wrong query: not a context or statement ID: " + sts);
						}
					} else {
						UbiwareAgent.this.print("Error: Wrong query: not a context: " + context);
					}
				}

				SemanticStatement st2 = new SemanticStatement(st1);
				if (st1.isSubjectContext()) {
					st2.subject = "?" + st1.subject;
				}

				if (st1.isObjectContext() && (st1.objectDatatype == null)) {
					st2.object = "?" + st1.object;
					if (queryContextID.equals(UbiwareAgent.GENERAL_CONTEXT) && st.subject.equals(SaplN3Constants.I)
							&& st.predicate.equals(SaplN3Constants.want)) {
						this.goals_var = st2.object;
					}
				}

				if (queryContextID.equals(this.goals_var)) {
					id_vars.add("__handledGoal__");
				}
				if (queryContextID.equals(UbiwareAgent.GENERAL_CONTEXT) && st.predicate.equals(SaplN3Constants.configuredAs)) {
					id_vars.add("__handledCommitment__");
				}
				if (queryContextID.equals(UbiwareAgent.GENERAL_CONTEXT)
						&& (st.predicate.equals(SaplN3Constants.eventFromGUI) || st.predicate.equals(SaplN3Constants.eventFromSocket))) {
					id_vars.add("__handledEvent__");
				}

				if (manager.evaluate(st2, queryContextID1, this.myAgent, removeMode, id_vars)) {
					boolean ok = true;
					if (st1.isSubjectContext()) {
						ok = this.evaluate(st1.subject, st2.subject, manager, removeMode);
						if (!ok) {
							return false;
						}
					}
					if (ok && st1.isObjectContext() && (st.objectDatatype == null)) {
						//commmented next line because the variable was never read
						//HashSet<String> sub_marked = new HashSet<String>();
						ok = this.evaluate(st1.object, st2.object, manager, removeMode);
						if (!ok) {
							return false;
						}
					}
					if (ok) {
						//catching variables used in time-related conditions
						if (queryContextID.equals(UbiwareAgent.GENERAL_CONTEXT)
								&& (st1.subject.equals(SaplN3Constants.Now) || st1.subject.equals(SaplN3Constants.Time) || st1.subject
										.equals(SaplN3Constants.Date)) && st1.predicate.equals(SaplN3Constants.is)
								&& st1.object.startsWith("?")) {
							time_vars.add(st.object.substring(1));
						}
					}
				} else {
					return false;
				}

			}//trueIf, i.e. normal matching conditons

			if (falseIf != null) {
				Iterator<String> fit = falseIf.iterator();
				while (fit.hasNext()) {
					List<Integer> toRemove = new ArrayList<Integer>();
					String cid = fit.next();
					int n_binds = manager.getNumberOfSolutions();
					if (n_binds == 0) {
						VariablesBindingManager temp_manager = new VariablesBindingManager();
						if (this.evaluate(cid, UbiwareAgent.GENERAL_CONTEXT, temp_manager, removeMode)) {
							return false;
						}
					} else {
						for (int i = 0; i < n_binds; i++) {
							List<HashMap<String, String>> vec = new ArrayList<HashMap<String, String>>();
							HashMap<String, String> solution = manager.getSolution(i);
							vec.add(solution);
							VariablesBindingManager temp_manager = new VariablesBindingManager(vec);
							temp_manager.solutionIndices = new HashMap<String, Integer>(manager.solutionIndices);

							if (this.evaluate(cid, queryContextID, temp_manager, removeMode)) {
								toRemove.add(i);
							}
						}
						manager.removeSolutions(toRemove);

						if (manager.getNumberOfSolutions() == 0) {
							return false;
						}
					}
				}
			}//exclusive conditions

			// All below works only at G level

			while (calcIf != null) {
				List<String> remain = null;
				int n_binds = manager.getNumberOfSolutions();
				Iterator<String> lit = calcIf.iterator();
				ExpressionEvaluator evaluator = new ExpressionEvaluator();
				while (lit.hasNext()) {
					String id = lit.next();
					SemanticStatement st = UbiwareAgent.this.beliefs.get(id);
					String var = st.subject.substring(1);
					if (st.predicate.equals(SaplN3Constants.expression)) {
						if (n_binds == 0) {
							manager.solutions.add(new HashMap<String, String>());
							n_binds = manager.getNumberOfSolutions();
						}
						List<Integer> toRemove = new ArrayList<Integer>();
						for (int i = 0; i < n_binds; i++) {
							HashMap<String, String> solution = manager.getSolution(i);
							String value = solution.get(var);
							if (value != null) {
								UbiwareAgent.this.print("Error in \"expression\": Variable is already bound: " + st);
								return false;
							}
							StringBuilder errorMessage = new StringBuilder();
							String expression_value = ((st.objectDatatype != null) || (st.object.length() < 2)) ? st.object : evaluator
									.calculate(st.object, st.object, solution, errorMessage, this.myAgent);
							if (expression_value != null) {
								solution.put(var, expression_value);
							} else {
								String error = errorMessage.toString();
								if (error.startsWith("?") && error.endsWith("not bound")) {
									String variab = error.substring(1, error.indexOf(" "));
									if (manager.definedVars.contains(variab)) {
										if (st.object.equals("exists(?" + variab + ")")) {
											solution.put(var, SaplN3Constants.FALSE);
										} else {
											toRemove.add(i);
										}
									} else {
										if (remain == null) {
											remain = new ArrayList<String>();
										}
										remain.add(id); //postpone checking
										break;
									}
								} else {
									UbiwareAgent.this.print(error);
									toRemove.add(i);
								}
							}
						}
						manager.definedVars.add(var);
						manager.removeSolutions(toRemove);
						if (manager.getNumberOfSolutions() == 0) {
							return false;
						}
						n_binds = manager.getNumberOfSolutions();
					}
				}
				if ((remain != null) && (calcIf.size() == remain.size())) {
					//String sts = "";
					StringBuilder statements = new StringBuilder();
					for (int i = 0; i < remain.size(); i++) {
						//	sts += " " + UbiwareAgent.this.beliefs.get(remain.get(i));
						statements.append(' ');
						statements.append(UbiwareAgent.this.beliefs.get(remain.get(i)));
					}
					String sts = statements.toString();
					UbiwareAgent.this.print("Error in \"expression\": A variable is not defined or a dead loop:" + sts);
					return false;
				}
				calcIf = remain;
			}//calculated variables

			while ((localIf != null) || (statIf != null)) {
				List<String> remain = null;
				if (localIf != null) {
					Iterator<String> lit = localIf.iterator();
					ExpressionEvaluator evaluator = new ExpressionEvaluator();
					while (lit.hasNext()) {
						List<Integer> toRemove = new ArrayList<Integer>();
						String id = lit.next();
						SemanticStatement st = UbiwareAgent.this.beliefs.get(id);
						String var = null;
						boolean constant = false;
						if (st.subject.startsWith("?")) {
							var = st.subject.substring(1);
						} else {
							constant = true;
						}

						int n_binds = manager.getNumberOfSolutions();
						if (n_binds == 0) {
							manager.addEmptySolution();
						}
						n_binds = manager.getNumberOfSolutions();
						for (int i = 0; i < n_binds; i++) {
							HashMap<String, String> solution = manager.getSolution(i);
							String value;
							if (constant) {
								value = st.subject;
							} else {
								value = solution.get(var);
							}
							if (value == null) {
								if (statIf == null) {
									if (!(st.predicate.equals("<" + UbiwareAgent.RDF_NS + "type>") && st.object
											.equals(SaplN3Constants.Variable))) {
										UbiwareAgent.this.print("Warning: variable ?" + var
												+ " is not bound and the condition is considered false: " + st);
										toRemove.add(i);
									}
								} else {
									if (remain == null) {
										remain = new ArrayList<String>();
									}
									remain.add(id); //postpone checking
								}
								break;
							}

							StringBuilder errorMessage = new StringBuilder();
							String expression_value;
							if ((st.objectDatatype == null) && !(st.object.length() < 2)) {
								// here we are trying to get the expression value (as you can see the input is the object of the statement)
								// it is needed, because the object can be some function like indexOf, sum, avg
								expression_value = evaluator.calculate(st.object, st.object, solution, errorMessage, this.myAgent);
							} else {
								expression_value = st.object;
							}

							if (expression_value == null) {
								String error = errorMessage.toString();
								if (error.startsWith("?") && error.endsWith("not bound")) {
									String variab = error.substring(1, error.indexOf(" "));
									if (manager.definedVars.contains(variab)) {
										if (st.object.equals("exists(?" + variab + ")")) {
											expression_value = SaplN3Constants.FALSE;
										} else {
											toRemove.add(i);
										}
									} else {
										if (statIf != null) {
											if (remain == null) {
												remain = new ArrayList<String>();
											}
											remain.add(id); //postpone checking
										} else {
											UbiwareAgent.this.print("Error in filtering predicate: " + error);
											toRemove.add(i);
										}
									}
								} else {
									UbiwareAgent.this.print("Error in filtering predicate: " + error);
									toRemove.add(i);
								}
							}
							if (expression_value != null) {
								boolean cv = this.checkValue(value, st.predicate, expression_value);
								if (!cv) {
									toRemove.add(i);
								}

								//catching blocking time-related conditions
								if (time_vars.contains(var)) {
									if (!((!cv && (st.predicate.equals(SaplN3Constants.lt) || st.predicate.equals(SaplN3Constants.lte))) || (cv && (st.predicate
											.equals(SaplN3Constants.gt) || st.predicate.equals(SaplN3Constants.gte))))) {
										long time;
										if (expression_value.charAt(4) == '-') { //was Date
											int year = Integer.valueOf(expression_value.substring(0, 4));
											int month = Integer.valueOf(expression_value.substring(5, 7)) - 1;
											int day = Integer.valueOf(expression_value.substring(8));
											if (!cv
													&& (st.predicate.equals(SaplN3Constants.gt) || st.predicate.equals(SaplN3Constants.neq))) {
												day++;
											} else if (cv
													&& (st.predicate.equals(SaplN3Constants.lte) || st.predicate.equals(SaplN3Constants.eq))) {
												day++;
											}

											Calendar calendar = Calendar.getInstance();
											calendar.set(year, month, day, 0, 0, 0);
											calendar.set(Calendar.MILLISECOND, 0);
											time = calendar.getTimeInMillis();
										} else if (expression_value.charAt(2) == ':') { //was Time
											int hour = Integer.valueOf(expression_value.substring(0, 2));
											int minute = Integer.valueOf(expression_value.substring(3, 5));
											int second = Integer.valueOf(expression_value.substring(6));
											if (!cv
													&& (st.predicate.equals(SaplN3Constants.gt) || st.predicate.equals(SaplN3Constants.neq))) {
												second++;
											} else if (cv
													&& (st.predicate.equals(SaplN3Constants.lte) || st.predicate.equals(SaplN3Constants.eq))) {
												second++;
											}

											Calendar calendar = Calendar.getInstance();
											calendar.set(Calendar.HOUR_OF_DAY, hour);
											calendar.set(Calendar.MINUTE, minute);
											calendar.set(Calendar.SECOND, second);
											calendar.set(Calendar.MILLISECOND, 0);
											time = calendar.getTimeInMillis();
										} else {
											time = Long.valueOf(expression_value); //was Now in mills
											if (!cv
													&& (st.predicate.equals(SaplN3Constants.gt) || st.predicate.equals(SaplN3Constants.neq))) {
												time++;
											} else if (cv
													&& (st.predicate.equals(SaplN3Constants.lte) || st.predicate.equals(SaplN3Constants.eq))) {
												time++;
											}
										}

										Long lTime = Long.valueOf(time);
										if (!this.waitingFor.contains(lTime)) {
											this.waitingFor.add(lTime);
										}
										Collections.sort(this.waitingFor);
									}
								}
							}
						}
						manager.removeSolutions(toRemove);
						if (manager.getNumberOfSolutions() == 0) {
							return false;
						}
						n_binds = manager.getNumberOfSolutions();
					}
					localIf = remain;
				}//filtering predicates

				if (statIf != null) {
					int n_binds = manager.getNumberOfSolutions();
					Iterator<String> lit = statIf.iterator();
					while (lit.hasNext()) {
						String id = lit.next();
						SemanticStatement st = UbiwareAgent.this.beliefs.get(id);
						HashMap<String, String> b0 = manager.getSolution(0);
						String var = st.subject.substring(1);
						String val = b0.get(var);
						if (val != null) {
							UbiwareAgent.this.print("Error in \"count\": variable is already bound: " + st);
							continue;
						}

						if (st.predicate.equals(SaplN3Constants.count) || st.predicate.equals(SaplN3Constants.countGroupedBy)) {
							List<String> count = new ArrayList<String>();
							String[] result = st.object.split("\\s+");
							for (int x = 0; x < result.length; x++) {
								String var2 = result[x].substring(1);
								if (b0.get(var2) != null) {
									count.add(var2);
								} else {
									UbiwareAgent.this.print("Warning in \"count\": variable ?" + var2 + " is not bound and ignored: " + st);
								}
							}
							if (count.size() > 0) {
								if (st.predicate.equals(SaplN3Constants.count)) {
									int num = manager.getSolutions(count).size();
									for (int i = 0; i < n_binds; i++) {
										HashMap<String, String> solution = manager.getSolution(i);
										solution.put(var, String.valueOf(num));
									}
									manager.definedVars.add(var);
								} else {
									manager.getSolutions(count, var); //manager will count and add variable itself
								}
							}
						} else {
							String var2 = st.object.substring(1);
							if (b0.get(var2) == null) {
								UbiwareAgent.this.print("Warning: variable ?" + var2 + " is not bound and the condition is ignored: " + st);
								continue;
							}

							List<HashMap<String, String>> vec = manager.solutions;
							if (st.predicate.equals(SaplN3Constants.sum)) {
								vec = manager.rearrangeSolutions(new ArrayList<HashMap<String, String>>(vec), this.globalOrderBy, -1, 0);
							}

							String sResult = "";
							double dResult = 0;
							boolean numberValues = true;

							for (int el = 0; el < vec.size(); el++) {
								String value = vec.get(el).get(var2);
								double v = 0;
								if (numberValues) {
									try {
										v = Double.valueOf(value);
									} catch (NumberFormatException e) {
										numberValues = false;
									}
								}
								if (st.predicate.equals(SaplN3Constants.sum)) {
									if (numberValues) {
										dResult += v;
									}
									sResult += value;
								} else if (st.predicate.equals(SaplN3Constants.max)) {
									if (numberValues) {
										if ((el == 0) || (v > dResult)) {
											dResult = v;
										}
									}
									if ((el == 0) || (value.compareTo(sResult) > 0)) {
										sResult = value;
									}
								} else if (st.predicate.equals(SaplN3Constants.min)) {
									if (numberValues) {
										if ((el == 0) || (v < dResult)) {
											dResult = v;
										}
									}
									if ((el == 0) || (value.compareTo(sResult) < 0)) {
										sResult = value;
									}
								}
							}

							if (numberValues) {
								double d = Math.floor(dResult);
								if (d == dResult) {
									sResult = String.valueOf((long) d);
								} else {
									sResult = String.valueOf(dResult);
								}
							}

							for (int i = 0; i < n_binds; i++) {
								HashMap<String, String> solution = manager.getSolution(i);
								solution.put(var, sResult);
							}
							manager.definedVars.add(var);
						}
					}
					statIf = null;
				}//statistic predicates
			}

			return true;
		}

		//used inside action()
		private VariablesBindingManager checkRule(SemanticStatement st) {
			if (!st.isSubjectContext() || !st.isObjectContext()) {
				UbiwareAgent.this.print("Error in rule: Both subject and object must be context containers: " + st);
				return null;
			}
			this.handledGoals.clear();
			this.handledCommitments.clear();
			this.handledEvents.clear();
			this.globalForAll.clear();
			this.globalForSome.clear();
			this.globalOrderBy.clear();
			this.globalOffset = 0;
			this.globalLimit = -1;

			VariablesBindingManager manager = new VariablesBindingManager();
			if (this.evaluate(st.subject, UbiwareAgent.GENERAL_CONTEXT, manager)) {
				if (!st.predicate.equals(SaplN3Constants.infers)) {
					if ((this.globalOffset == 0)
							|| ((this.globalForAll.size() == 0) && (manager.getNumberOfSolutions() > this.globalOffset))
							|| ((this.globalForAll.size() > 0) && (manager.getSolutions(this.globalForAll).size() > this.globalOffset))) {
						return manager;
					}
				} else {
					List<Integer> toRemove = new ArrayList<Integer>();
					for (int i = 0; i < manager.getNumberOfSolutions(); i++) {
						List<HashMap<String, String>> vec = new ArrayList<HashMap<String, String>>();
						HashMap<String, String> solution = manager.getSolution(i);
						vec.add(solution);
						VariablesBindingManager temp_manager = new VariablesBindingManager(vec);
						temp_manager.solutionIndices = new HashMap<String, Integer>(manager.solutionIndices);
						if (this.evaluate(st.object, UbiwareAgent.GENERAL_CONTEXT, temp_manager)) {
							toRemove.add(i);
						}
					}
					manager.removeSolutions(toRemove);
					if (manager.getNumberOfSolutions() == 0) {
						return null;
					}
					return manager;

				}
			}
			return null;
		}

		//used inside executeRule()
		private void generateRuleOutput(String contextID, VariablesBindingManager manager, List<String> forAll, List<String> forSome,
				List<String> orderBy, int limit, int offset, boolean inference) {

			List<String> simple = new ArrayList<String>();
			List<SemanticStatement> toUnwrap = new ArrayList<SemanticStatement>();

			Iterator<String> mit = UbiwareAgent.this.contexts.get(contextID).members.iterator();
			while (mit.hasNext()) {
				String id = mit.next();
				SemanticStatement st = UbiwareAgent.this.beliefs.get(id);
				if (st.isSubjectContext()
						&& (UbiwareAgent.set_predicates.contains(st.predicate) || UbiwareAgent.set2_predicates.contains(st.predicate))) {
					toUnwrap.add(st);
				} else {
					simple.add(id);
				}
			}

			if (simple.size() > 0) {
				List<HashMap<String, String>> vec;
				if (!inference) {
					vec = manager.getSolutions(forAll, forSome, orderBy, limit, offset, null);
				} else {
					vec = manager.solutions;
				}

				Iterator<HashMap<String, String>> it = vec.iterator();
				while (it.hasNext()) {
					HashMap<String, String> vars = it.next();
					this.addLinks.addAll(UbiwareAgent.this.copy(simple, UbiwareAgent.GENERAL_CONTEXT, vars));

					String id_var = vars.get("__handledGoal__");
					if (id_var != null) {
						this.handledGoals.add(id_var);
					}
					id_var = vars.get("__handledCommitment__");
					if (id_var != null) {
						this.handledCommitments.add(id_var);
					}
					id_var = vars.get("__handledEvent__");
					if (id_var != null) {
						this.handledEvents.add(id_var);
					}

					if (it.hasNext()) {
						UbiwareAgent.this.clearLocalIDs();
					}
				}
			}

			Iterator<SemanticStatement> sit = toUnwrap.iterator();
			while (sit.hasNext()) {
				SemanticStatement st = sit.next();
				List<String> newForAll = forAll;
				List<String> newForSome = forSome;
				List<String> newOrderBy = orderBy;
				int newLimit = limit;
				int newOffset = offset;

				if (st.object.startsWith("?")) {
					if (st.predicate.equals(SaplN3Constants.All)) {
						newForAll = new ArrayList<String>(forAll);
						newForAll.add(st.object.substring(1));
					} else if (st.predicate.equals(SaplN3Constants.Some)) {
						newForSome = new ArrayList<String>(forSome);
						newForSome.add(st.object.substring(1));
					} else if (st.predicate.equals(SaplN3Constants.OrderBy)) {
						newOrderBy = new ArrayList<String>(orderBy);
						newOrderBy.add(st.object.substring(1));
					} else if (st.predicate.equals(SaplN3Constants.OrderByDescending)) {
						newOrderBy = new ArrayList<String>(orderBy);
						newOrderBy.add(" " + st.object.substring(1));
					}
				} else if (st.predicate.equals(SaplN3Constants.Limit)) {
					try {
						int value = Integer.valueOf(st.object);
						if (value < 1) {
							throw new NumberFormatException();
						}
						newLimit = value;
					} catch (NumberFormatException e) {
						UbiwareAgent.this.print("Error in defining Limit: the object must be integer > 0 : Ignored");
					}
				} else if (st.predicate.equals(SaplN3Constants.Offset)) {
					try {
						int value = Integer.valueOf(st.object);
						if (value < 0) {
							throw new NumberFormatException();
						}
						newOffset = value;
					} catch (NumberFormatException e) {
						UbiwareAgent.this.print("Error in defining Offset: the object must be integer >= 0 : Ignored");
					}
				}

				this.generateRuleOutput(st.subject, manager, newForAll, newForSome, newOrderBy, newLimit, newOffset, inference);
			}
		}

		//used inside action()
		private void executeRule(String contextID, VariablesBindingManager manager, boolean inference) {
			//print(manager.getNumberOfSolutions()+" solution(s): "+manager.solutions);
			UbiwareAgent.this.clearLocalIDs();
			this.generateRuleOutput(contextID, manager, this.globalForAll, this.globalForSome, this.globalOrderBy, this.globalLimit,
					this.globalOffset, inference);
			UbiwareAgent.this.clearLocalIDs();
		}

		//used inside action()
		private void checkElse(String contextID, boolean execute) {
			HashSet<String> otherContexts = UbiwareAgent.this.hasParallelContexts(contextID);
			if (otherContexts != null) {
				Iterator<String> it = otherContexts.iterator();
				while (it.hasNext()) {
					String id = it.next();
					String statementID = UbiwareAgent.this.contexts.get(id).referencedBy;
					if (statementID != null) {
						SemanticStatement st = UbiwareAgent.this.beliefs.get(statementID);
						if ((st.memberOf.contains(UbiwareAgent.GENERAL_CONTEXT) || st.memberOf.contains(UbiwareAgent.RULES_CONTEXT))
								&& st.predicate.equals(SaplN3Constants.ELSE)) {
							if (execute) {
								UbiwareAgent.this.clearLocalIDs();
								this.addLinks.addAll(UbiwareAgent.this.copy(st.object, UbiwareAgent.GENERAL_CONTEXT, null));
								UbiwareAgent.this.clearLocalIDs();
							}
							if (st.memberOf.contains(UbiwareAgent.GENERAL_CONTEXT)) {
								this.removeLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, statementID));
							}
							if (st.memberOf.contains(UbiwareAgent.RULES_CONTEXT)) {
								this.removeLinks.add(new LinkInfo(UbiwareAgent.RULES_CONTEXT, statementID));
							}
						}
					}
				}
			}
		}

		//used inside action()
		private void handleChanges() {
			Iterator<String> it = this.removeContexts.iterator();
			while (it.hasNext()) {
				String id = it.next();
				if (!UbiwareAgent.protected_entities.contains(id)) {
					UbiwareAgent.this.removeContext(id);
				}
			}
			it = this.removeBeliefs.iterator();
			while (it.hasNext()) {
				String statementID = it.next();
				if (!UbiwareAgent.protected_entities.contains(statementID)) {
					UbiwareAgent.this.removeBelief(statementID);
				}
			}
			Iterator<LinkInfo> lit = this.removeLinks.iterator();
			while (lit.hasNext()) {
				LinkInfo li = lit.next();
				if (!UbiwareAgent.protected_entities.contains(li.statementID)) {
					UbiwareAgent.this.removeLink(li.contextID, li.statementID);
				}
			}
			lit = this.addLinks.iterator();
			while (lit.hasNext()) {
				LinkInfo li = lit.next();
				UbiwareAgent.this.putLink(li.contextID, li.statementID);
			}

			if ((this.addLinks.size() > 0) || (this.removeLinks.size() > 0) || (this.removeBeliefs.size() > 0)
					|| (this.removeContexts.size() > 0)) {
				this.block = false;
			}

			this.addLinks.clear();
			this.removeLinks.clear();
			this.removeBeliefs.clear();
			this.removeContexts.clear();
		}

		//used inside action()
		private void removeHandledAttitudes() {
			Iterator<String> ait = this.handledGoals.iterator();
			while (ait.hasNext()) {
				String aid = ait.next();
				this.addLinks.add(new LinkInfo(UbiwareAgent.ACHIEVE_CONTEXT, aid));
				this.removeLinks.add(new LinkInfo(UbiwareAgent.GOALS_CONTEXT, aid));
			}
			ait = this.handledCommitments.iterator();
			while (ait.hasNext()) {
				String aid = ait.next();
				this.removeLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, aid));
			}
			ait = this.handledEvents.iterator();
			while (ait.hasNext()) {
				String aid = ait.next();
				this.removeLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, aid));
			}
		}

		//used inside action()
		//Updating beliefs that the engine itself maintains (time and name)
		private void updateMaintainedBeliefs() {
			//name of agent
			SemanticStatement st = UbiwareAgent.this.beliefs.get(UbiwareAgent.NAME_BELIEF);
			if (st == null) {
				st = new SemanticStatement(SaplN3Constants.I, SaplN3Constants.haveName, UbiwareAgent.this.getLocalName());
				UbiwareAgent.this.putBelief(UbiwareAgent.NAME_BELIEF, st);
				UbiwareAgent.this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.NAME_BELIEF);
			}

			//current system time in milliseconds
			st = UbiwareAgent.this.beliefs.get(UbiwareAgent.NOW_BELIEF);
			if (st == null) {
				st = new SemanticStatement(SaplN3Constants.Now, SaplN3Constants.is, "");
				UbiwareAgent.this.putBelief(UbiwareAgent.NOW_BELIEF, st);
				UbiwareAgent.this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.NOW_BELIEF);
			}
			UbiwareAgent.removeFromIndex(st.object, UbiwareAgent.NOW_BELIEF,
					UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT).index_ob);
			st.object = String.valueOf(System.currentTimeMillis());
			UbiwareAgent.addToIndex(st.object, UbiwareAgent.NOW_BELIEF,
					UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT).index_ob);

			//current time of the day
			st = UbiwareAgent.this.beliefs.get(UbiwareAgent.TIME_BELIEF);
			if (st == null) {
				st = new SemanticStatement(SaplN3Constants.Time, SaplN3Constants.is, "");
				UbiwareAgent.this.putBelief(UbiwareAgent.TIME_BELIEF, st);
				UbiwareAgent.this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.TIME_BELIEF);
			}
			Calendar now = Calendar.getInstance();
			int hour = now.get(Calendar.HOUR_OF_DAY);
			int minute = now.get(Calendar.MINUTE);
			int second = now.get(Calendar.SECOND);
			UbiwareAgent.removeFromIndex(st.object, UbiwareAgent.TIME_BELIEF,
					UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT).index_ob);
			st.object = ((hour < 10) ? "0" : "") + String.valueOf(hour) + ":" + ((minute < 10) ? "0" : "") + String.valueOf(minute) + ":"
					+ ((second < 10) ? "0" : "") + String.valueOf(second);
			UbiwareAgent.addToIndex(st.object, UbiwareAgent.TIME_BELIEF,
					UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT).index_ob);

			//current date
			st = UbiwareAgent.this.beliefs.get(UbiwareAgent.DATE_BELIEF);
			if (st == null) {
				st = new SemanticStatement(SaplN3Constants.Date, SaplN3Constants.is, "");
				UbiwareAgent.this.putBelief(UbiwareAgent.DATE_BELIEF, st);
				UbiwareAgent.this.putLink(UbiwareAgent.GENERAL_CONTEXT, UbiwareAgent.DATE_BELIEF);
			}
			int day = now.get(Calendar.DAY_OF_MONTH);
			int month = now.get(Calendar.MONTH) + 1;
			int year = now.get(Calendar.YEAR);
			UbiwareAgent.removeFromIndex(st.object, UbiwareAgent.DATE_BELIEF,
					UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT).index_ob);
			st.object = String.valueOf(year) + "-" + ((month < 10) ? "0" : "") + String.valueOf(month) + "-" + ((day < 10) ? "0" : "")
					+ String.valueOf(day);
			UbiwareAgent.addToIndex(st.object, UbiwareAgent.DATE_BELIEF,
					UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT).index_ob);
		}

		//The Deliberation Loop of the Agent
		@Override
		public void action() {
			//to overrule JADE's scheduling mechanism
			//if there are RABs that have not been yet executed, do not start next cycle of Live			
			for (ReusableAtomicBehavior behavior : UbiwareAgent.this.runningRABs) {
				if (!behavior.hasBeenExecuted()) {
					return;
				}
			}
			this.actionNoRABWaiting();
		}

		//The action method used to take a boolean this.interfaceHandling into account to unconditionally run the action if the action() was called from inside handleInterfaceEvent from Ubiware agent,
		//This method assumes that the old boolean value is true
		//It contains all th ol code from action() except that it doesn't do an initial check whichc still resides in the original action method.
		//This is the deliberation loop of the agent!
		//TODO find a better name for this function, it can be called ahen there is no other RAB which should be executed first.
		private void actionNoRABWaiting() {
			if ((UbiwareAgent.this.stopAfterLifeCycle != -1) && (UbiwareAgent.this.stopAfterLifeCycle <= UbiwareAgent.this.lifeCycleCount)) {
				UbiwareAgent.this.collectGarbage();
				this.block();
				UbiwareAgent.this.print("[Execution stopped after Live iteration " + UbiwareAgent.this.lifeCycleCount + "...]");
				if (UbiwareAgent.this.debugger != null) {
					UbiwareAgent.this.debugger.notifyLiveIterationEnded(UbiwareAgent.this.lifeCycleCount);
				}
				return;
			} else if (UbiwareAgent.this.debugger != null) {
				UbiwareAgent.this.debugger.notifyLiveIterationEnded(UbiwareAgent.this.lifeCycleCount);
			}

			UbiwareAgent.this.lifeCycleCount++;

			//print("Live iteration: "+lifeCycleCount);

			this.block = true; //unless will change to false, the Live will block itself at the end of action()

			this.addLinks = new ArrayList<LinkInfo>();
			this.removeLinks = new HashSet<LinkInfo>();
			this.removeBeliefs = new HashSet<String>();
			this.removeContexts = new HashSet<String>();
			this.handledGoals = new HashSet<String>();
			this.handledCommitments = new HashSet<String>();
			this.handledEvents = new HashSet<String>();
			this.globalForAll = new ArrayList<String>();
			this.globalForSome = new ArrayList<String>();
			this.globalOrderBy = new ArrayList<String>();

			//RESTORING RUN ATOMIC BEHAVIORS
			this.restoreRunAtomicBehaviors();

			//CLEANING GOALS
			this.cleanGoals();

			//Updating beliefs that the engine itself maintains (time and name)
			this.updateMaintainedBeliefs();

			//Cleaning in the queue of future time moments that are blocking conditions in some rules
			this.cleanWaitingForQueue();

			//check existsWhile conditions
			this.existenceConditions();

			//Iterator<String> it;
			//List<String> members;

			//Execute possible rules.
			this.executeRules();

			//EXTERNAL ACTIONS
			this.externalActions();

			if (this.block) {
				UbiwareAgent.this.collectGarbage();

				if (UbiwareAgent.this.debugger != null) {
					UbiwareAgent.this.debugger.notifyGoingSleep();
				}

				if (this.waitingFor.size() == 0) {
					this.block();
				} else {
					final long now = System.currentTimeMillis();
					long time = this.waitingFor.get(0).longValue();
					if (time > now) {
						this.block(time - now);
					}
				}

				//print("...going to sleep...");

				/*Runtime rt=Runtime.getRuntime();
				print("Used memory: "+String.valueOf(rt.totalMemory()-rt.freeMemory()));
				long before=doGC();
				print("Used memory after GC: "+String.valueOf(before));*/
			}
		}

		/**
		 * Execute external actions keeping policies in mind. This method searches the G context of the agent for Unconditional commitment
		 * to an action When this is found, the corresponding RAB class is loaded and when the policychecker allos its execution, the
		 * behavior is scheduled in the agent.
		 */
		private void externalActions() {
			//get all actions which should get executed
			ContextContainer G = UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT);
			Set<String> actionStatementIDs = G.index_pred.get(SaplN3Constants.configuredAs);
			if (actionStatementIDs == null) {
				//no actions to be executed.
				return;
			}
			for (String actionStatementID : actionStatementIDs) {
				SemanticStatement actionStatement = UbiwareAgent.this.beliefs.get(actionStatementID);
				//check whether it has the form {} sapl:configuredAs sapl:null / {}
				if (!actionStatement.isSubjectContext()
						|| (!actionStatement.isObjectContext() && !actionStatement.object.equals(SaplN3Constants.NULL))) {
					UbiwareAgent.this.print("Error in \"configuredAs\": The subject must be context container "
							+ "and the object must be either context container or sapl:null '" + actionStatement + "'");
					continue;
				}
				//TODO the following code removes the statement executing PrintBeliefs before doing so. Why?
				//				String rabname = "";
				//				String tempid = UbiwareAgent.this.contexts.get(actionStatement.subject).members.get(0);
				//				if (tempid != null) {
				//					rabname = UbiwareAgent.this.beliefs.get(tempid).object;
				//				}
				//			if (rabname.equals("<#PrintBeliefs>")) {
				//					it.remove();//The old iteratore over the statements
				//					UbiwareAgent.removeFromIndices(st, id, UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT));
				//				}

				//there could be multiple actions with the same parameter configuration! We do an iteration over all members of the subject
				List<String> actionSubjectMemeberIDs = UbiwareAgent.this.contexts.get(actionStatement.subject).members;
				if (actionSubjectMemeberIDs.size() > 1) {
					System.err.println("More as two statements in subject contianer of action, result is unprdictable.");
				}
				for (String actionSubjectMemberID : actionSubjectMemeberIDs) {
					SemanticStatement actionSubjectMember = UbiwareAgent.this.beliefs.get(actionSubjectMemberID);
					String actionName = actionSubjectMember.object;
					//check whether the subject of sapl:configureAs is of the form {sapl:I sapl:do java:actioname/:embeddedAction}
					if (!actionSubjectMember.subject.equals(SaplN3Constants.I) || !actionSubjectMember.predicate.equals(SaplN3Constants.DO)
							|| (!actionName.startsWith("<" + SaplN3Constants.RAB_NS) && !actionName.startsWith("<#"))) {
						//this is not a proper call for an external action!
						UbiwareAgent.this.print("Error in configuredAs : The statements in the subject context must be of the form "
								+ "sapl:I sapl:do java:clasname or sapl:I sapl:do :embeddedActionName. Got '" + actionSubjectMember
								+ "' instead");
						continue;
					}

					Class<ReusableAtomicBehavior> rabClass;
					try {
						rabClass = UbiwareAgent.loader.loadClass(actionName);
						this.behaviorClasses.put(actionName, rabClass);

					} catch (ClassNotFoundException e) {
						if (!this.nonAvailableBehaviors.contains(actionName)) {
							UbiwareAgent.this.print("Atomic behavior " + actionName + " is not found : " + e.getMessage());
							this.nonAvailableBehaviors.add(actionName);
							//add belief about missing RAB to the agent.
							String id = UbiwareAgent.this.statementIDGenerator.getNewStatementID();//UbiwareAgent.STATEMENT_PREFIX + String.valueOf(++UbiwareAgent.this.statementCount);
							UbiwareAgent.this.putBelief(id,
									new SemanticStatement(SaplN3Constants.I, SaplN3Constants.missingRAB, actionName.replace('.', '/')));
							this.addLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, id));
						}
						continue;
					}

					//the action looks well formed, now check policies:
					this.executeIfAllowed(rabClass, actionStatement.object, actionStatementID);
					//after execution (or NotAllowed by policy checker. remove the links to the belief that this action should get executed.
					this.removeLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, actionStatementID));
				}
			}
			//handle all changes done by externalActions.
			this.handleChanges();
		}

		/**
		 * Schedules an instance of the rabClass in the agent is the policy checker allows it. Otherwise adds the content of sapl:Denied
		 * sapl:add to G and removes the content od sapl:Denied sapl:remove.
		 * 
		 * @param rabClass
		 *            The class from which an instance should be scheduled in the agent.
		 * @param parameterContext
		 *            the ID of the context from which parameters for execution should be found.
		 * @param commitmentStatementID
		 *            the ID of the statement declaring the unconditional commitment.
		 */
		private void executeIfAllowed(Class<ReusableAtomicBehavior> rabClass, String parameterContext, String commitmentStatementID) {
			Set<String> referencedContexts = new HashSet<String>();
			//generate the s-apl code for the policy check:
			BehaviorStartParameters parameters = this.collectRABParameters(parameterContext, referencedContexts);
			//TODO shortcut for policy chckers which allow all actions to be executed.
			Status status = UbiwareAgent.this.policyChecker.performPolicyCheck(UbiwareAgent.this, rabClass.getName(), parameters,
					new UWAParameterGenerator(parameters));
			switch (status) {
			case Allowed:
				this.doRABExecution(rabClass, parameters, referencedContexts, commitmentStatementID);
				break;
			case DisAllowed:
				this.addAndRemoveDenied(parameterContext);
				break;
			default:
				throw new Error("Unknown policy check state");
			}

		}

		private class UWAParameterGenerator implements ParameterGenerator {

			private BehaviorStartParameters parameters;

			public UWAParameterGenerator(BehaviorStartParameters parameters) {
				this.parameters = parameters;
			}

			@Override
			public void generateParametersN3(StringBuilder checkSaplBuilder) {
				Multimap<Resource, String> parameterMap = this.parameters.getParameters();
				Collection<Entry<Resource, String>> rabParameters = parameterMap.entries();
				//				List<BehaviorStartParameter> parameterList = this.parameters.getParameters();
				if (rabParameters.size() == 0) {
					checkSaplBuilder.append(SaplN3Constants.NULL);
				} else {
					//there is 1+ elements, use context:
					checkSaplBuilder.append('{');
					boolean generateDot = false;
					for (Entry<Resource, String> parameter : rabParameters) {
						if (generateDot) {
							checkSaplBuilder.append(' ');
							checkSaplBuilder.append('.');
							checkSaplBuilder.append(' ');
						}
						parameter.getKey().generateSapl(checkSaplBuilder);
						checkSaplBuilder.append(' ');
						checkSaplBuilder.append(SaplN3Constants.is);
						checkSaplBuilder.append(' ');
						String parameterValue = parameter.getValue();
						if (parameterValue.startsWith("<") && parameterValue.endsWith(">")) {
							//it is normal uri
							checkSaplBuilder.append(parameterValue);
						} else if (parameterValue.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
							checkSaplBuilder.append('{');
							//generate sapl for context
							SaplN3.produceN3(parameterValue, PolicyChecker.optionsForPolicyCheckN3Generation, UbiwareAgent.this,
									checkSaplBuilder);
							checkSaplBuilder.append('}');
						} else {
							//it is a literal
							checkSaplBuilder.append("\"");
							checkSaplBuilder.append(StringEscape.escape(parameterValue));
							checkSaplBuilder.append("\"");
						}
						//the next statement will not be the first on any more.
						generateDot = true;
					}
					checkSaplBuilder.append('}');
				}
			}
		}

		/**
		 * Adds the content of sapl:Denied sapl:add to G and removes the content od sapl:Denied sapl:remove.
		 * 
		 * @param parameterContext
		 */
		private void addAndRemoveDenied(String parameterContext) {
			if (parameterContext.equals(SaplN3Constants.NULL)) {
				//empty parameter list => nothing to be done.
				return;
			}
			List<String> parameterIDs = UbiwareAgent.this.contexts.get(parameterContext).members;
			for (String parameterID : parameterIDs) {
				SemanticStatement parameterStatement = UbiwareAgent.this.beliefs.get(parameterID);
				if (parameterStatement.subject.equals(SaplN3Constants.Denied)) {
					String predicate = parameterStatement.predicate;
					if (parameterStatement.isObjectContext()) {
						String object = parameterStatement.object;
						if (predicate.equals(SaplN3Constants.add)) {
							//sapl:denied sapl:add {}
							List<String> memberIDs = UbiwareAgent.this.contexts.get(object).members;
							for (String memberID : memberIDs) { //for all members of the object container, add a link.
								System.out.println("Adding link to member of context " + parameterStatement.object);
								this.addLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, memberID));
							}
						} else if (predicate.equals(SaplN3Constants.remove)) {
							//TODO remove
							System.out.println("Removing all from context " + parameterStatement.object);
							throw new UnsupportedOperationException("removing beliefs trough sapl:Denied sapl:remove is not yet supported.");
						} else {
							UbiwareAgent.this.print("Warning : Found sapl:Denied '" + predicate + "' instead of sapl:add or sapl:remove");
						}
					} else {
						UbiwareAgent.this.print("Warning : Found sapl:Denied with a object which is not a container, ignored"
								+ parameterStatement);
					}
				}

			}
		}

		/**
		 * This method collects the parameters defined in the parameterContext for RAB execution. Furthermore, it adds to the
		 * referencedContexts set; all ID's of contexts rerenced from inside the parameters.
		 * 
		 * @param parameterContext
		 *            The context from which the parameters must be extracted.
		 * @param referencedContexts
		 *            The set to which all context ID's of contects referenced from the parameters must be added.
		 * @param parameterBuilder
		 *            A StringBuilder in which this method will produce the S-APL representation of the parameters.
		 * @return The paramters fro RAB execution.
		 */
		private BehaviorStartParameters collectRABParameters(String parameterContext, Set<String> referencedContexts) {
			BehaviorStartParameters params = new BehaviorStartParameters();
			if (parameterContext.equals(SaplN3Constants.NULL)) {
				return params;
			}
			List<String> parameterIDs = UbiwareAgent.this.contexts.get(parameterContext).members;
			//Iterator<String> it1 = members1.iterator();
			//while (it1.hasNext()) {
			//	SemanticStatement st1 = UbiwareAgent.this.beliefs.get(it1.next());

			for (String parameterID : parameterIDs) {
				SemanticStatement parameterStatement = UbiwareAgent.this.beliefs.get(parameterID);
				if (parameterStatement.predicate.equals(SaplN3Constants.is)) {
					String parameter = parameterStatement.subject;
					if (!parameter.startsWith("<") || !parameter.endsWith(">")) {
						throw new Error("every parameter must be a resource");
					}
					//parameter = parameter.substring(1, parameter.length() - 1);
					Resource parameterResource = Resource.createUncheckedResource(parameter);
					/*
					int ind1 = parameter.indexOf("#");
					if (ind1 == -1) {
						ind1 = parameter.lastIndexOf("/");
					}
					String parameterNamespace = parameter.substring(0, ind1 + 1);
					String parameterName = parameter.substring(ind1 + 1);
					*/

					String parameterValue = parameterStatement.object;
					params.add(parameterResource, parameterValue);
					if (parameterStatement.isObjectContext()) {
						referencedContexts.add(parameterValue);
					}
				} else if (parameterStatement.predicate.equals(SaplN3Constants.add) && parameterStatement.isObjectContext()) {
					//add
					String contextToBeAdded = parameterStatement.object;
					String condition = parameterStatement.subject;
					if (condition.equals(SaplN3Constants.Start)) {
						params.addOnStart.add(contextToBeAdded);
						referencedContexts.add(contextToBeAdded);
					} else if (condition.equals(SaplN3Constants.End)) {
						params.addOnEnd.add(contextToBeAdded);
						referencedContexts.add(contextToBeAdded);
					} else if (condition.equals(SaplN3Constants.Success)) {
						params.addOnSuccess.add(contextToBeAdded);
						referencedContexts.add(contextToBeAdded);
					} else if (condition.equals(SaplN3Constants.Fail)) {
						params.addOnFail.add(contextToBeAdded);
						referencedContexts.add(contextToBeAdded);
					} else if (condition.equals(SaplN3Constants.Denied)) {
						//ignore
					} else {
						UbiwareAgent.this
								.print("Error in conditinal adding parameters : subject must be sapl:Start sapl:End sapl:Success or sapl:Fail"
										+ parameterStatement);
					}

				} else if (parameterStatement.predicate.equals(SaplN3Constants.remove) && parameterStatement.isObjectContext()) {
					String contextToBeAdded = parameterStatement.object;
					String condition = parameterStatement.subject;
					if (condition.equals(SaplN3Constants.Start)) {
						params.removeOnStart.add(contextToBeAdded);
						referencedContexts.add(contextToBeAdded);
					} else if (condition.equals(SaplN3Constants.End)) {
						params.removeOnEnd.add(contextToBeAdded);
						referencedContexts.add(contextToBeAdded);
					} else if (condition.equals(SaplN3Constants.Success)) {
						params.removeOnSuccess.add(contextToBeAdded);
						referencedContexts.add(contextToBeAdded);
					} else if (condition.equals(SaplN3Constants.Fail)) {
						params.removeOnFail.add(contextToBeAdded);
						referencedContexts.add(contextToBeAdded);
					} else if (condition.equals(SaplN3Constants.Denied)) {
						//ignore
					} else {
						UbiwareAgent.this
								.print("Error in conditinal remove parameters : subject must be sapl:Start sapl:End sapl:Success or sapl:Fail"
										+ parameterStatement);
					}
				} else {
					UbiwareAgent.this
							.print("Error in action parameter: the predicate must be sapl:is or for conditional adding and removing, the Object must be context container: '"
									+ parameterStatement + "'");
				}
			}
			return params;
		}

		//execute the RAB
		private void doRABExecution(Class<ReusableAtomicBehavior> rabClass, BehaviorStartParameters parameters,
				Set<String> referencedContexts, String commitmentStatementID) {
			//actual execution
			ReusableAtomicBehavior rab = null;
			try {
				rab = rabClass.newInstance();
				//TODO : what with this???
				//				this.nonAvailableBehaviors.remove(b_class);
			} catch (InstantiationException e) {
				throw new Error("A ReusableAtomicBehavior must have a non throwing default constructor.", e);
			} catch (IllegalAccessException e) {
				throw new Error("A ReusableAtomicBehavior must have a public default constructor.", e);
			}

			//set parameters... on RAB
			rab.setStartParameters(parameters);
			for (String contextID : referencedContexts) {
				//increase reference counts 
				UbiwareAgent.this.contexts.get(contextID).referenced++;
			}

			if (rab instanceof DebugBehavior) {
				((DebugBehavior) rab).debugController = new DebugController();
			}
			/*
			if ((rab instanceof CoreAccessingBehavior) && rab.getClass().getName().equals("ubiware.fingrid.OntonutBehavior")) {
				((CoreAccessingBehavior) rab).setCoreAccess(new CoreAccess());
			}
			 */
			//TODO for now, the RAB holds its own linked contexts, in the future this might be kept by UW agent.
			rab.linkedContexts = referencedContexts;

			rab.ID = commitmentStatementID;
			UbiwareAgent.this.runningRABs.add(rab);
			//this.debugList.add(rab.getClass());
			UbiwareAgent.this.addRAB(rab);
			//Add beliefs about execution:
			// TODO what with this? if (!rabname.endsWith("BeliefsBackupBehavior>")) {
			this.addLinks.add(new LinkInfo(UbiwareAgent.DO_CONTEXT, commitmentStatementID));
			String cid = UbiwareAgent.this.contextIDGenerator.getNewContextID();//UbiwareAgent.CONTEXT_PREFIX + String.valueOf(++UbiwareAgent.this.contextCount);
			UbiwareAgent.this.putContext(cid);
			UbiwareAgent.this.putLink(cid, commitmentStatementID);
			String sid = UbiwareAgent.this.statementIDGenerator.getNewStatementID();
			String exID = this.executionIDGenerator.getNewID();
			UbiwareAgent.this.putBelief(sid, new SemanticStatement(cid, SaplN3Constants.executionID, exID));
			this.addLinks.add(new LinkInfo(UbiwareAgent.DO_CONTEXT, sid));
			UbiwareAgent.this.rabInstanceIDs.put(commitmentStatementID, exID);
			//}

		}

		/**
		 * extracted form actionNoRABActive executes the rules in the agent's beliefs
		 */
		private void executeRules() {
			/*
			RULES
			We are checking for the following types of "rules" (in this order):
			0: Metarules ({} sapl:is sapl:MetaRule)
			1: Rule context ({} sapl:is sapl:Rule)
			2: General context (just being directly in the main G context)
			3: Metarules again
			*/
			for (int iteration = 0; iteration < 4; iteration++) {
				String curr_context;
				if (iteration == 0) {
					curr_context = UbiwareAgent.METARULES_CONTEXT;
				} else if (iteration == 1) {
					curr_context = UbiwareAgent.RULES_CONTEXT;
				} else if (iteration == 2) {
					curr_context = UbiwareAgent.GENERAL_CONTEXT;
				} else {
					curr_context = UbiwareAgent.METARULES_CONTEXT;
				}
				List<String> members = UbiwareAgent.this.contexts.get(curr_context).members;
				Iterator<String> it = members.iterator();
				// let's go through all the statements in this context 
				while (it.hasNext()) {
					String id = it.next();
					SemanticStatement st = UbiwareAgent.this.beliefs.get(id);
					// I am interested in "rules" (implies, impliesNow, etc.) now
					if (UbiwareAgent.inference_predicates.contains(st.predicate)) {
						// now we will check the rule and get the binding for the variables used
						VariablesBindingManager manager = this.checkRule(st);
						if (manager != null) {
							if (st.predicate.equals(SaplN3Constants.infers)) {
								this.executeRule(st.object, manager, true);
							} else {
								this.executeRule(st.object, manager, false);
							}
							if (curr_context.equals(UbiwareAgent.GENERAL_CONTEXT)) {
								this.removeLinks.add(new LinkInfo(curr_context, id));
								if (st.predicate.equals(SaplN3Constants.impliesNow)) {
									this.checkElse(st.subject, false);
								}
							}
							if (st.predicate.equals(SaplN3Constants.achievedBy)) {
								this.removeHandledAttitudes();
							}
						} else if (st.predicate.equals(SaplN3Constants.impliesNow)) {
							this.removeLinks.add(new LinkInfo(curr_context, id));
							this.checkElse(st.subject, true);
						}
					} else if (st.predicate.equals(SaplN3Constants.requires)) {
						if (!st.isSubjectContext() || !st.isObjectContext()) {
							UbiwareAgent.this.print("Error in \"requires\" condition: Both subject and object must be context containers: "
									+ st);
							continue;
						}
						Iterator<String> mit = UbiwareAgent.this.contexts.get(st.subject).members.iterator();
						while (mit.hasNext()) {
							String mid = mit.next();
							SemanticStatement mst = UbiwareAgent.this.beliefs.get(mid);
							if (UbiwareAgent.inference_predicates.contains(mst.predicate)
									&& !mst.predicate.equals(SaplN3Constants.requires)) {
								VariablesBindingManager manager = this.checkRule(mst);
								if (manager != null) { //rule is applicable
									VariablesBindingManager ob_manager = new VariablesBindingManager(manager.solutions);

									if (this.evaluate(st.object, UbiwareAgent.GENERAL_CONTEXT, ob_manager)) {//rule is executable
										manager.solutions = ob_manager.solutions;
										manager.definedVars = ob_manager.definedVars;

										if (mst.predicate.equals(SaplN3Constants.infers)) {
											this.executeRule(mst.object, manager, true);
										} else {
											this.executeRule(mst.object, manager, false);
										}

										if (curr_context.equals(UbiwareAgent.GENERAL_CONTEXT)) {
											this.removeLinks.add(new LinkInfo(curr_context, id));
										}
										if (mst.predicate.equals(SaplN3Constants.achievedBy)) {
											this.removeHandledAttitudes();
										}
									} else { //add required conditions as goals
										List<HashMap<String, String>> allBinds = manager.getSolutions(this.globalForAll,
												this.globalForSome, this.globalOrderBy);
										Iterator<HashMap<String, String>> itt = allBinds.iterator();
										while (itt.hasNext()) {
											HashMap<String, String> solution = itt.next();
											List<HashMap<String, String>> vec = new ArrayList<HashMap<String, String>>();
											vec.add(solution);
											Iterator<String> git = UbiwareAgent.this.contexts.get(st.object).members.iterator();
											while (git.hasNext()) {
												String st_id = git.next();
												VariablesBindingManager temp_manager = new VariablesBindingManager(vec);
												if (!this.evaluate(st_id, UbiwareAgent.GENERAL_CONTEXT, temp_manager)) {
													temp_manager = new VariablesBindingManager(vec);
													if (!this.evaluate(st_id, UbiwareAgent.GOALS_CONTEXT, temp_manager)) {
														temp_manager = new VariablesBindingManager(vec);
														if (!this.evaluate(st_id, UbiwareAgent.ACHIEVE_CONTEXT, temp_manager)) {
															UbiwareAgent.this.clearLocalIDs();
															this.addLinks.addAll(UbiwareAgent.this.copy(st_id, UbiwareAgent.GOALS_CONTEXT,
																	solution));
															UbiwareAgent.this.clearLocalIDs();
														}
													}
												}
											}
										}
									}//else
								} else if (mst.predicate.equals(SaplN3Constants.impliesNow)) {
									this.removeLinks.add(new LinkInfo(curr_context, id));
								}
							}
						}
					}//requires
				}
				if ((iteration == 0) || (iteration == 2) || (iteration == 3)) { //after rules + commitments, and then after metarules
					boolean doMental = false;
					if ((iteration == 0) || (this.addLinks.size() > 0) || (this.removeLinks.size() > 0) || (this.removeBeliefs.size() > 0)
							|| (this.removeContexts.size() > 0)) {
						doMental = true;
					}
					this.handleChanges();
					//MENTAL ACTIONS
					if (doMental) {
						members = UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT).members;
						it = members.iterator();
						while (it.hasNext()) {
							String id = it.next();
							SemanticStatement st = UbiwareAgent.this.beliefs.get(id);

							//unconditional commitment to belief removal
							if (st.subject.equals(SaplN3Constants.I) && st.predicate.equals(SaplN3Constants.remove)) {
								if ((!st.isObjectContext() || (UbiwareAgent.this.contexts.get(st.object) == null))
										&& (!st.object.startsWith(UbiwareAgent.STATEMENT_PREFIX) || (UbiwareAgent.this.beliefs
												.get(st.object) == null))) {
									UbiwareAgent.this.print("Error in \"remove\": Object must be context container or ID of statement: "
											+ st);
									continue;
								}
								VariablesBindingManager manager = new VariablesBindingManager();
								if (this.evaluate(st.object, UbiwareAgent.GENERAL_CONTEXT, manager, true)) {
									this.removeLinks.addAll(manager.getForRemove());
								}
								this.removeLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, id));
							}
							//unconditional commitment to belief addition
							else if (st.subject.equals(SaplN3Constants.I) && st.predicate.equals(SaplN3Constants.add)) {
								if ((!st.isObjectContext() || (UbiwareAgent.this.contexts.get(st.object) == null))
										&& (!st.object.startsWith(UbiwareAgent.STATEMENT_PREFIX) || (UbiwareAgent.this.beliefs
												.get(st.object) == null))) {
									UbiwareAgent.this.print("Error in \"add\": Object must be context container or ID of statement: " + st);
									continue;
								}
								UbiwareAgent.this.clearLocalIDs();
								this.addLinks.addAll(UbiwareAgent.this.copy(st.object, UbiwareAgent.GENERAL_CONTEXT, null));
								UbiwareAgent.this.clearLocalIDs();
								this.removeLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, id));
							}
							//unconditional commitment to erasing a belief or a context through ID
							else if (st.subject.equals(SaplN3Constants.I) && st.predicate.equals(SaplN3Constants.erase)) {
								if (st.object.startsWith(UbiwareAgent.STATEMENT_PREFIX)
										&& (UbiwareAgent.this.beliefs.get(st.object) != null)) {
									this.removeBeliefs.add(st.object);
								} else if (st.isObjectContext() && (UbiwareAgent.this.contexts.get(st.object) != null)) {
									this.removeContexts.add(st.object);
								} else {
									UbiwareAgent.this.print("Error in \"erase\": Object must be ID of statement or context container: "
											+ st);
									continue;
								}
								this.removeLinks.add(new LinkInfo(UbiwareAgent.GENERAL_CONTEXT, id));
							}
						}
						this.handleChanges();
					}
				}
			}
		}

		/**
		 * extracted form actionNoRABActive
		 * 
		 */
		private void cleanWaitingForQueue() {
			final long now1 = System.currentTimeMillis();
			while (this.waitingFor.size() > 0) {
				long time = this.waitingFor.get(0).longValue();
				if (time <= now1) {
					this.waitingFor.remove(0);
				} else {
					break;
				}
			}
		}

		/**
		 * extracted from actionNoRABActive Cleaning in the queue of future time moments that are blocking conditions in some rules
		 */
		private void existenceConditions() {
			Iterator<String> it;
			//EXISTENCE CONDITIONS
			List<String> members = UbiwareAgent.this.contexts.get(UbiwareAgent.GENERAL_CONTEXT).members;
			it = members.iterator();
			while (it.hasNext()) {
				String id = it.next();
				SemanticStatement st = UbiwareAgent.this.beliefs.get(id);

				if (st.predicate.equals(SaplN3Constants.existsWhile)) {
					if (!st.isSubjectContext() || !st.isObjectContext()) {
						UbiwareAgent.this.print("Error in \"existsWhile\" condition: Both subject and object must be context containers: "
								+ st);
						continue;
					}

					VariablesBindingManager manager = new VariablesBindingManager();
					if (!this.evaluate(st.object, UbiwareAgent.GENERAL_CONTEXT, manager)) {
						Iterator<String> it1 = UbiwareAgent.this.contexts.get(st.subject).members.iterator();
						while (it1.hasNext()) {
							this.removeBeliefs.add(it1.next());
						}
						this.removeBeliefs.add(id);
					}
				}
			}

			this.handleChanges();
		}

		/**
		 * Method extracted from actionNoRABActive
		 */
		private void cleanGoals() {
			Iterator<String> it;
			//CLEANING GOALS
			it = UbiwareAgent.this.contexts.get(UbiwareAgent.GOALS_CONTEXT).members.iterator();
			while (it.hasNext()) {
				String id = it.next();
				VariablesBindingManager manager = new VariablesBindingManager();
				if (this.evaluate(id, UbiwareAgent.GENERAL_CONTEXT, manager)) {
					this.removeLinks.add(new LinkInfo(UbiwareAgent.GOALS_CONTEXT, id));
				}
			}
			it = UbiwareAgent.this.contexts.get(UbiwareAgent.ACHIEVE_CONTEXT).members.iterator();
			while (it.hasNext()) {
				String id = it.next();
				VariablesBindingManager manager = new VariablesBindingManager();
				if (this.evaluate(id, UbiwareAgent.GENERAL_CONTEXT, manager)) {
					this.removeLinks.add(new LinkInfo(UbiwareAgent.ACHIEVE_CONTEXT, id));
				}
			}

			this.handleChanges();
		}

		/**
		 * Method extracted form actionNoRABActive.
		 */
		private void restoreRunAtomicBehaviors() {
			//RESTORING RUN ATOMIC BEHAVIORS
			for (String doContextMemberID : UbiwareAgent.this.contexts.get(UbiwareAgent.DO_CONTEXT).members) {
				SemanticStatement doContextMemberStatement = UbiwareAgent.this.beliefs.get(doContextMemberID);
				if (doContextMemberStatement.predicate.equals(SaplN3Constants.executionID)) {
					//for all sapl:I sapl:do {?_ sapl:executionID ?_}
					String executionID = doContextMemberStatement.object;
					if (!UbiwareAgent.this.rabInstanceIDs.containsValue(executionID)) {
						//which has been finished!
						for (String actionID : UbiwareAgent.this.contexts.get(doContextMemberStatement.subject).members) {
							//take the action(s) from in there
							//if the actions are not in sapl:I sapl:do
							if (!UbiwareAgent.this.contexts.get(UbiwareAgent.DO_CONTEXT).members.contains(actionID)) {
								this.removeBeliefs.add(doContextMemberID); //RAB finished already
							} else {
								throw new AssertionError("The action" + actionID
										+ "is reported to be finished but still resides in the agents beliefs.");
							}
						}
					}
				}
			}
		}
	}

	//////////////start of code for CommandHAndlingBahavior
	/**
	 * Add a {@link UbiwareAgentCommand} to the agent, this Command will be executed in the agent's thread and is thus be guaranteed to not
	 * introduce concurrency inconsistencies.
	 * 
	 * @param command
	 *            the command to be added to the agent
	 */
	public void addCommand(UbiwareAgentCommand<?> command) {
		this.commandHandlingBahavior.addCommand(command);
	}

	public PolicyChecker getPolicyChecker() {
		return this.policyChecker;
	}

	////////////end of code for CommandHandlingBehavior
	/*
		//TODO check whether this method is still needed.
		private static long doGC() {
			Runtime rt = Runtime.getRuntime();
			long usedMem1 = rt.totalMemory() - rt.freeMemory();
			long usedMem2 = Long.MAX_VALUE;
			for (int r = 0; r < 10; r++) {
				for (int i = 0; (usedMem1 < usedMem2) && (i < 5000); i++) {
					rt.runFinalization();
					rt.gc();
					Thread.currentThread();
					Thread.yield();
					usedMem2 = usedMem1;
					usedMem1 = rt.totalMemory() - rt.freeMemory();
				}
			}
			return usedMem1;
		}
	*/

}
