package ubiware.core.behaviors;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.UbiwareAgent;
import ubiware.core.visualizer.VisualizerPrototype;
import ubiware.ontology.param;

/**
 * <p>
 * A RAB to start debugging behaviour for an {@link UbiwareAgent}.
 * </p>
 * 
 * <p>
 * In addition to invoking the GUI and passing the instances required, this RAB also operates as a scheduler in order to deliver events from
 * the Swing-thread(s) to the agent thread in a synchronized and safe manner.
 * </p>
 * 
 * <p>
 * It is guaranteed that events/tasks {@link #invokeLater delivered} to this RAB will be forwarded to the agent (when the time comes) in the
 * order that they were delivered to this RAB.
 * </p>
 * 
 * @author Nikos Mouchtaris
 */
public class DebugBehavior extends ReusableAtomicBehavior implements Runnable {
	private boolean guiInvoked, initialStop;
	private static final long serialVersionUID = 3l;
	private final Collection<Runnable> tasks;
	private final Collection<Runnable> tasks_unsync;

	/**
	 * Will be set by UbiwareAgent.
	 */
	public UbiwareAgent.DebugController debugController;

	/**
	 * Creates a new instance of this RAB.
	 * 
	 * <p>
	 * The RAB parameters that this RAB accepts and understands are:
	 * </p>
	 * <ul>
	 * <li>{@code stop} : if "{@code true}" then the agent will be stopped as soon as this RAB runs.</li>
	 * </ul>
	 * 
	 */
	public DebugBehavior() {
		this.guiInvoked = false;
		this.tasks_unsync = new LinkedList<Runnable>();
		this.tasks = Collections.synchronizedCollection(this.tasks_unsync);
	}

	// Implementation of Runnable //////////////////////////////////////////////
	/**
	 * Creates and makes visible the GUI.
	 */
	@Override
	public void run() {
		new VisualizerPrototype(this.myAgent, this).setVisible(true);
	}

	////////////////////////////////////////////////////////////////////////////

	// Customization of RAB ////////////////////////////////////////////////////
	/**
	 * <p>
	 * Invokes {@link VisualizerPrototype} in the Swing event despatch thread.
	 * </p>
	 */
	@Override
	public void doAction() {
		//this is a cyclic behavior.
		this.finished = false;
		// if we have not started yet
		if (!this.guiInvoked) {
			// run initializations
			this.checkedInvokeGUI();
		} else { // running normally
			// dispatch tasks (we are on the agent thread)
			synchronized (this.tasks_unsync) {
				for (final Runnable task : this.tasks_unsync) {
					task.run();
				}
				this.tasks_unsync.clear();
			}

			// there is not point to run with no tasks to dispatch.
			// behaviour will be restart()ed when a new task is added.
			this.block();
		}
	}

	/**
	 * Checks the parameters given to this RAB and extracts information for the execution of this RAB.
	 */
	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		// Check parameters
		// p:stop sapl:is
		final String stop = parameters.getParameterValue(param.stop);
		this.initialStop = (stop != null) && stop.equals("true");
		this.checkedInvokeGUI();
	}

	////////////////////////////////////////////////////////////////////////////
	/**
	 * Adds the given {@link Runnable} in the queue for execution. It will be executed the next time this RAB will be {@link #doAction()
	 * invoked}.
	 * 
	 * @param r
	 *            a task to be executed next time this RAB is executed (probably on the Agent's thread)
	 */
	public void invokeLater(final Runnable r) {
		// list of tasks is synchronized
		this.tasks.add(r);
		// wake up this behaviour
		this.restart();
	}

	/**
	 * Returns whether this behaviour will stop the agent as soon as it is started and able to.
	 * 
	 * @return whether this behaviour will stop the agent as soon as it is started and able to.
	 */
	public boolean isInitialStop() {
		return this.initialStop;
	}

	private void checkedInvokeGUI() {
		// if debugController has been set, we are ready to go
		if (this.debugController != null) {
			if (this.initialStop) {
				this.debugController.stopNow();
			}
			javax.swing.SwingUtilities.invokeLater(this);
			this.guiInvoked = true;
		}
	}
}
