package ubiware.core.commands;

import java.util.concurrent.CountDownLatch;

import ubiware.core.UbiwareAgent;

/**
 * Implementation of a command that has an extra method {@link BlockingCommand#waitForExecution()} which only returns once the command has
 * been executed.
 * 
 * 15.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class BlockingCommand<E> implements UbiwareAgentCommand<E> {
	private UbiwareAgentCommand<E> command;
	private CountDownLatch latch = new CountDownLatch(1);
	
	/**
	 * Create the command with the wrapped command.
	 * 
	 * @param command
	 *            The wrapped command
	 */
	public BlockingCommand(UbiwareAgentCommand<E> command) {
		this.command = command;
	}

	@Override
	public void performCommand(UbiwareAgent agent) {
		//perform the command 
		this.command.performCommand(agent);
		// notify the waiting parties about finish.
		this.latch.countDown();
	}

	/**
	 * This method only returns once the wrapped command has been executed It returns the same value as getResult returns after this call
	 * has been made.
	 */
	public E waitForExecution() {
		try {
			this.latch.await();
		} catch (InterruptedException e) {
			throw new Error(e);
		}
		return this.getResult();
	}

	@Override
	public E getResult() {
		return this.command.getResult();
	}
}
