package ubiware.core.visualizer;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.SemanticStatement;

/**
 * <p>
 * Maps a belief container/context to a tree-like structure.
 * </p>
 * 
 * <p>
 * This is a tree structure whose nodes are complete subtrees. For more information, read {@link SubtreesTreeModel}.
 * </p>
 * 
 * <p>
 * A {@link BeliefContainerTreeModel} has one or more {@link BeliefTreeModel}s as children of its. It is initialised only <strong
 * >once</strong> and after that it only uses its own, local, final and unmutable storage of beliefs. It never asks the Agent about beliefs
 * ever again.
 * </p>
 * 
 * @author Nikos Mouchtaris
 */
public class BeliefContainerTreeModel implements SubtreesTreeModel {
	/** Local storage of beliefs. */
	private final List<BeliefTreeModel> iBeefs;
	/** The actual root of the tree. */
	private final TreeModel root;
	/** The context/container ID as given by the agent. */
	private final String ID;

	/**
	 * <p>
	 * Initialises a new instance of {@link BeliefContainerTreeModel} which will represent the context/container specified by this ID.
	 * </p>
	 * 
	 * <p>
	 * The given {@link ReusableAtomicBehavior}, {@code rabi}, is used in order to acquire all beliefs in the belief context specified by
	 * the given {@code contextID}. {@code rabi} is never used again, throughout the whole life cycle of this object. It is not even stored
	 * in a field.
	 * </p>
	 * 
	 * <p>
	 * This model will create a whole tree-like substructure of the Agent's beliefs (as acquired by {@code rabi}). The children of this
	 * model will be {@link BeliefTreeModel}s, created according to the actual beliefs of the Agent. This locally re-created structure of
	 * beliefs is used whenever queries are made to this model. The Agent's actual belief storage is never re-read.
	 * </p>
	 * 
	 * <p>
	 * {@code root} is the <em>actual</em> root element of this model. This could be any {@link TreeModel} which might cooperate with this
	 * one. If {@code root} is {@code null} then the root of the tree is believed to be <em>{@code this}</em>! and this belief is being
	 * passed accordingly to the children {@link TreeModel}s created as the result of this model's initialisation.
	 * </p>
	 * 
	 * @param root
	 *            the actual root of the whole tree (if null then root = this)
	 * @param rabi
	 *            a RAB to be used for accessing the Agent's beliefs
	 * @param contextID
	 *            the context id to be represented by this model
	 */
	public BeliefContainerTreeModel(final TreeModel root, final ReusableAtomicBehavior rabi, final String contextID) {
		this.root = root == null ? this : root;
		this.ID = contextID;

		List<SemanticStatement> sss = rabi.getStatements(contextID);
		if (sss == null) {
			// it happens...
			sss = new ArrayList<SemanticStatement>(0);
		}
		this.iBeefs = new ArrayList<BeliefTreeModel>(sss.size());
		for (SemanticStatement ss : sss) {
			this.iBeefs.add(new BeliefTreeModel(this.getRoot(), rabi, ss));
		}
	}

	@Override
	public TreeModel getRoot() {
		return this.root;
	}

	@Override
	public Object getChild(final Object parent, final int index) {
		if (parent == this) {
			return this.iBeefs.get(index);
		}
		return TreeModel.class.cast(parent).getChild(parent, index);
	}

	@Override
	public int getChildCount(final Object parent) {
		if (parent == this) {
			return this.iBeefs.size();
		}
		return TreeModel.class.cast(parent).getChildCount(parent);
	}

	@Override
	public boolean isLeaf(final Object node) {
		if (node == this) {
			//			return false;
			// :'(
			return this.getChildCount(this) == 0;
		}
		return TreeModel.class.cast(node).isLeaf(node);
	}

	/**
	 * <p>
	 * This method has no effect in this implementation.
	 * </p>
	 * 
	 * @param path
	 *            ignored
	 * @param newValue
	 *            ignored
	 */
	@Override
	public void valueForPathChanged(final TreePath path, final Object newValue) {
		// ignored because we don't allow changes yet
	}

	@Override
	public int getIndexOfChild(final Object parent, final Object child) {
		if (parent == this) {
			// return -1 for null child is assured because there are no null
			// elements in Ibeefs
			return this.iBeefs.indexOf(child);
		}
		if (parent == null) {
			return -1;
		}
		// return -1 for child == null is assured by children nodes
		return TreeModel.class.cast(parent).getIndexOfChild(parent, child);
	}

	/**
	 * <p>
	 * This method has no effect in this implementation.
	 * </p>
	 * 
	 * @param l
	 */
	@Override
	public void addTreeModelListener(final TreeModelListener l) {
		// sorry, ignoring :(
	}

	@Override
	public void removeTreeModelListener(final TreeModelListener l) {
		// sorry, ignoring :(
	}

	/**
	 * <p>
	 * Returns a textual representation of this object, which is the ID of the belief context/container, surrounded by braces ({})
	 * </p>
	 * 
	 * @return {_contextID}
	 */
	@Override
	public String toString() {
		return "{" + this.ID + "} (" + this.getChildCount(this) + ")";
	}

	/**
	 * Returns the actual ID of the container that this node came from.
	 * 
	 * @return the actual ID of the container that this node came from.
	 */
	public String getContextID() {
		return this.ID;
	}
}
