package ubiware.core.visualizer;

import java.util.Arrays;
import java.util.Formatter;
import java.util.List;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.SemanticStatement;
import ubiware.core.UbiwareAgent;

/**
 * <p>
 * Maps a belief to a tree like structure.
 * </p>
 * 
 * <p>
 * This is a tree structure whose nodes are complete subtrees. For more information, read {@link SubtreesTreeModel}.
 * </p>
 * 
 * <p>
 * A belief always has three children, with the following indexes:
 * </p>
 * 
 * <pre>
 * 0 : subject
 * 1 : predicate
 * 2 : object
 * </pre>
 * <p>
 * Subject and Object can be either a string literal or a {@link BeliefContainerTreeModel}. A Predicate is always a string literal. A string
 * literal has 0 children.
 * </p>
 * 
 * <p>
 * String literals are now officially always instances of {@link StringLiteralTreeModel} or subclasses.
 * </p>
 * 
 * @author Nikos Mouchtaris
 */
public class BeliefTreeModel implements SubtreesTreeModel {

	/**
	 * <p>
	 * A private reference to the {@link SemanticStatement} that this instance has been created from, just for reference. Since it is never
	 * used, it is now disabled. (michael)
	 * </p>
	 */
	//	private final SemanticStatement SemanticStatement;
	/**
	 * <p>
	 * An array of the children of this belief. "String literals" are implementations of TreeModel.
	 * </p>
	 */
	private final List<TreeModel> children;
	/**
	 * The actual root of the whole tree model.
	 */
	private final TreeModel Root;

	/**
	 * <p>
	 * Initialises a new instance of {@link BeliefTreeModel} which will represent the semantic statement specified by the given semantic
	 * statement.
	 * </p>
	 * 
	 * <p>
	 * The given {@link ReusableAtomicBehavior}, {@code rabi}, is used in order to acquire all beliefs in the belief contexts found as
	 * subject, predicate or object of this semantic statement. {@code rabi} is never used again, throughout the whole life cycle of this
	 * object. It is not even stored in a field.
	 * </p>
	 * 
	 * <p>
	 * This model will create a whole tree-like substructure of the Agent's beliefs (as acquired by {@code rabi}). The children of this
	 * model will be either {@link BeliefContainerTreeModel}s or string literals, created according to the actual beliefs found in
	 * {@code ss}'s {@link SemanticStatement#subject} and {@link SemanticStatement#object}. {@link SemanticStatement#predicate} is expected
	 * always to be a string literal. The locally re-created structure of beliefs is used whenever queries are made to this model. The
	 * Agent's actual belief storage is never re-read.
	 * </p>
	 * 
	 * <p>
	 * {@code root} is the <em>actual</em> root element of this model. This could be any {@link javax.swing.tree.TreeModel} which might
	 * cooperate with this one. If {@code root} is {@code null} then the root of the tree is believed to be <em>{@code this}</em>! and this
	 * belief is being passed accordingly to the children {@link javax.swing.tree.TreeModel}s created as the result of this model's
	 * initialisation.
	 * </p>
	 * 
	 * @param root
	 *            the actual root of the whole tree (if null then root = this)
	 * @param rabi
	 *            a RAB to be used for accessing the Agent's beliefs
	 * @param ss
	 *            the semantic statement to be represented by this model
	 */
	public BeliefTreeModel(final TreeModel root, final ReusableAtomicBehavior rabi, final SemanticStatement ss) {
		this.Root = root == null ? this : root;

		//this.SemanticStatement = ss;
		this.children = Arrays.asList(new TreeModel[] {
				this.createTreeModel(this.getRoot(), rabi, ss.subject, SemanticStatementElementTreeModel.Type.Subject),
				new SemanticStatementElementTreeModel(this.getRoot(), ss.predicate, SemanticStatementElementTreeModel.Type.Predicate),
				this.createTreeModel(this.getRoot(), rabi, ss.object, SemanticStatementElementTreeModel.Type.Object) });
	}

	@Override
	public TreeModel getRoot() {
		return this.Root;
	}

	/**
	 * <p>
	 * If {@code parent == this} then it returns the semantic statement parts as follows:
	 * </p>
	 * <dl style="font-family: monospace">
	 * <dt>index == 0</dt>
	 * <dd>subject</dd>
	 * <dt>index == 1</dt>
	 * <dd>predicate</dd>
	 * <dt>index == 2</dt>
	 * <dd>object</dd>
	 * </dl>
	 * 
	 * <p>
	 * Otherwise it calls {@link javax.swing.tree.TreeModel#getChild} for {@code parent}, after casting it to a
	 * {@link javax.swing.tree.TreeModel}.
	 * </p>
	 * 
	 * @param parent
	 *            the parent of the child
	 * @param index
	 *            the index of the child (0, 1, 2)
	 * @return a child according to parent and index
	 */
	@Override
	public Object getChild(Object parent, int index) {
		if (parent == this) {
			return this.children.get(index);
		}
		return TreeModel.class.cast(parent).getChild(parent, index);
	}

	@Override
	public int getChildCount(Object parent) {
		if (parent == this) {
			return this.children.size();
		}
		return TreeModel.class.cast(parent).getChildCount(parent);
	}

	@Override
	public boolean isLeaf(Object node) {
		if (node == this) {
			return false;
		}
		// :)
		//		{
		//			for (TreeModel tm : children)
		//				if (BeliefContainerTreeModel.class.isAssignableFrom(
		//						tm.getClass()))
		//					return false;
		//			return true;
		//		}
		return TreeModel.class.cast(node).isLeaf(node);
	}

	/**
	 * <p>
	 * This method has no effect in this implementation.
	 * </p>
	 * 
	 * @param path
	 *            ignored
	 * @param newValue
	 *            ignored
	 */
	@Override
	public void valueForPathChanged(TreePath path, Object newValue) {
		// ignored because we don't allow changes yet
	}

	@Override
	public int getIndexOfChild(Object parent, Object child) {
		if (parent == this) {
			// return -1 for null child is assured because there are no null
			// elements in children
			return this.children.indexOf(child);
		}
		if (parent == null) {
			return -1;
		}
		// return -1 for child == null is assured by children nodes
		return TreeModel.class.cast(parent).getIndexOfChild(parent, child);
	}

	/**
	 * <p>
	 * This method has no effect in this implementation.
	 * </p>
	 * 
	 * @param l
	 *            ignored
	 */
	@Override
	public void addTreeModelListener(TreeModelListener l) {
		// sorry, ignoring :(
	}

	/**
	 * <p>
	 * This method has no effect in this implementation.
	 * </p>
	 * 
	 * @param l
	 *            ignored
	 */
	@Override
	public void removeTreeModelListener(TreeModelListener l) {
		// sorry, ignoring :(
	}

	/**
	 * <p>
	 * Equivalent to {@code getChild(this, 0)}.
	 * </p>
	 * 
	 * @return the subtree of the subject of this semantic statement.
	 */
	public TreeModel getSubject() {
		return TreeModel.class.cast(this.getChild(this, 0));
	}

	/**
	 * <p>
	 * Equivalent to {@code getChild(this, 1)}.
	 * </p>
	 * 
	 * @return the subtree of the predicate of this semantic statement.
	 */
	public TreeModel getPredicate() {
		return TreeModel.class.cast(this.getChild(this, 1));
	}

	/**
	 * <p>
	 * Equivalent to {@code getChild(this, 2)}.
	 * </p>
	 * 
	 * @return the subtree of the object of this semantic statement.
	 */
	public TreeModel getObject() {
		return TreeModel.class.cast(this.getChild(this, 2));
	}

	/**
	 * <p>
	 * Returns a simple textual representation of this semantic statement tree model representation which consists of the subject's,
	 * predicate's and object's textual representations ({@code toString()}), separated by a space character.
	 * </p>
	 * 
	 * @return "subject predicate object"
	 */
	@Override
	public String toString() {
		ubiware.core.visualizer.Formatter F = ubiware.core.visualizer.Formatter.getSingleton();
		String result = null;
		synchronized (F) {
			Formatter f = F.getFormatter();
			F.getOut().reset();
			f.format("%s %s %s", this.getSubject(), this.getPredicate(), this.getObject());
			result = f.toString();
		}
		return result;
	}

	// private and alchemical //////////////////////////////////////////////////
	private TreeModel createTreeModel(final TreeModel root, final ReusableAtomicBehavior rabi, final String maybeContextID,
			final SemanticStatementElementTreeModel.Type type) {
		// if it is indeed a context id
		if (maybeContextID.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
			return new BeliefContainerTreeModel(root, rabi, maybeContextID);
		}
		return new SemanticStatementElementTreeModel(root, maybeContextID, type);
	}
}
