package ubiware.core.visualizer;

import java.io.CharArrayWriter;

/**
 * <p>
 * Default formatter used by the package in order to avoid creating many instances.
 * </p>
 * 
 * <p>
 * <em><strong>Warning:</strong></em> Methods making use of this class/instance should synchronize it, using the singletong instance as a
 * monitor, for example:
 * 
 * <pre>
 * ...
 * Formatter f = Formatter.getSingleton();
 * synchronized(f) {
 * // do stuff...
 * // ...
 * }
 * </pre>
 * 
 * </p>
 * 
 * <p>
 * Also this class provides a convenience method, {@link #format}.
 * </p>
 * 
 * @author Nikos Mouchtaris
 */
final class Formatter {
	// Convenience method, used as a layer above Formatter
	/**
	 * <p>
	 * Convenience method to produce a formatted string on the fly.
	 * </p>
	 * 
	 * <p>
	 * This method will delegate {@code format} and {@code args} to the underlying {@link java.util.Formatter#format(String,Object...)}
	 * method. It will also do that in the adviced synchronized manner.
	 * </p>
	 * 
	 * <p>
	 * This method is completely independent from the rest of the class, it's build as a layer above it, and its effects can be reproduced
	 * by the following code:
	 * </p>
	 * 
	 * <pre>
	 * final Formatter F = getSingleton();
	 * synchronized (F) {
	 * 	F.getOut().reset();
	 * 	return F.getFormatter().format(format, args).toString();
	 * }
	 * </pre>
	 * 
	 * @param format
	 *            delegated to {@link java.util.Formatter#format(String,Object...)}
	 * @param args
	 * @return
	 */
	public static String format(final String format, final Object... args) {
		final Formatter F = Formatter.getSingleton();
		synchronized (F) {
			F.getOut().reset();
			return F.getFormatter().format(format, args).toString();
		}
	}

	private static final Formatter me = new Formatter();

	/**
	 * <p>
	 * Provides access to the only instance allowed to be of this class.
	 * </p>
	 * <p>
	 * <em><strong>Warning:</strong></em>Methods making use of this class/instance should synchronize it, using the singletong instance as a
	 * monitor, for example:
	 * 
	 * <pre>
	 * ...
	 * Formatter f = Formatter.getSingleton();
	 * synchronized(f) {
	 * // do stuff...
	 * // ...
	 * }
	 * </pre>
	 * 
	 * @return the only and only instance allowed to be of this class.
	 */
	static Formatter getSingleton() {
		return Formatter.me;
	}

	/**
	 * <p>
	 * Provided access to the actual {@link java.util.Formatter}.
	 * </p>
	 * 
	 * <p>
	 * The destination to which the formatter writes is a {@link CharArrayWriter} and should be cleared by invoking
	 * {@link CharArrayWriter#reset}. It is not done automatically but it is not ensured that the contents of the array will remain
	 * untouched between invokations.
	 * </p>
	 * 
	 * <p>
	 * To obtain the destination char array without the need to cast anything, one can use {@link #getOut}.
	 * </p>
	 * 
	 * @return a shared instace of a {@link java.util.Formatter}..
	 */
	public java.util.Formatter getFormatter() {
		return this.javaFormatter;
	}

	/**
	 * <p>
	 * Provides access to the destination {@link CharArrayWriter} so that it can be {@link CharArrayWriter#reset reset}.
	 * </p>
	 * 
	 * @return the destination {@link CharArrayWriter}
	 */
	public CharArrayWriter getOut() {
		return this.out;
	}

	private final java.util.Formatter javaFormatter;
	private final CharArrayWriter out;

	// no instances allowed
	private Formatter() {
		this.out = new CharArrayWriter(10240);
		this.javaFormatter = new java.util.Formatter(this.out);
	}
}
