package ubiware.core.visualizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.tree.TreeCellRenderer;

import ubiware.core.UbiwareAgent;

/**
 * <p>
 * Custom cell renderer class for the Belief Tree.
 * </p>
 * 
 * <p>
 * The special things that this renderer does are:
 * </p>
 * <ul>
 * <li>Replace common namespaces with their shorter, alias prefices.</li>
 * <li>Make sure that there is enough space for the whole beliefs to be displayed</li>
 * <li>Colour and format beliefs for easier reading.</li>
 * </ul>
 * 
 * @author Nikos Mouchtaris
 */
public class SAPLTreeCellRenderer extends JLabel implements TreeCellRenderer {

	private static final long serialVersionUID = 2l;

	/**
	 * Interface to specify the minimun knowledge about user that should be provided to this Cell Renderer in order to render cells
	 * according to user preferences.
	 * 
	 * @author Nikos Mouchtaris
	 */
	public interface UserPreferencies {
		/**
		 * Returns true if the user has given the notion that she would like namespaces to be printed as short aliases.
		 * 
		 * @return {@code true} if the user has given the notion that she would like namespaces to be printed as short aliases.
		 */
		boolean isAliasNamespaces();

		/**
		 * Indicates the user's preference about whether a node which are belief containers <em>and</em> expanded should also have its text
		 * displayed. If this method returns {@code false} then expanded belief container nodes should show no text.
		 * 
		 * @return {@code true} if the user wants expanded belief container nodes to display their text
		 */
		boolean isShowExpandedBeliefContainerText();

		/**
		 * Indicates the user's preference about whether a node which are belief containers <em>and</em> expanded should also have its icon
		 * displayed. If this method returns {@code false} then expanded belief container nodes should show no icon.
		 * 
		 * @return {@code true} if the user wants expanded belief container nodes to display their icon
		 */
		boolean isShowExpandedBeliefContainerIcon();

		/**
		 * Indicates the user's preference about whether a node which are belief containers <em>and</em> expanded should also have its icon
		 * displayed. If this method returns {@code false} then expanded belief container nodes should show no icon.
		 * 
		 * @return {@code true} if the user wants icons, {@code false} otherwise
		 */
		boolean isEmptyBeliefContainersExpandable();

		/**
		 * Provides access to the {@link Font} which is used to display information to the user.
		 * 
		 * @return the {@link Font} which is used to display information to the user.
		 */
		Font getDisplayFont();

		/**
		 * Provides access to the {@link Graphics} toolkit which is used at rendering time.
		 * 
		 * @return the {@link Graphics} toolkit which is used at rendering time.
		 */
		Graphics getGraphics();
	}

	private final UserPreferencies userPreferencies;
	private final UbiwareAgent agi;
	private final Dimension dimensionGnome; // used to set dimensions
	private final ImageIcon asundio, ferundio, container, belief, subject, predicate, object, selectedSubject, selectedPredicate,
			selectedObject, selectedContainer, selectedBelief;
	private final int widthPadding, heightPadding;
	private final Logger logger;
	private final String expandedNodeTextValue;
	private final Color selectedForeground, selectedBackground;

	/**
	 * <p>
	 * Creates a new instance which will use the given {@link UserPreferencies} to render beliefs for viewing on screen.
	 * </p>
	 * 
	 * @param UserPreferencies
	 *            user preferences about how to view beliefs
	 * @param agi
	 *            the agent whose beliefs are being rendered - used to do a correct namespace/prefix replacement
	 */
	public SAPLTreeCellRenderer(final UserPreferencies UserPreferencies, final UbiwareAgent agi) {
		this.asundio = new ImageIcon(ResourceManager.getAsundio(this.getToolkit()), "O ASKIRYFIS!!!1");
		this.ferundio = new ImageIcon(ResourceManager.getFerundio(this.getToolkit()));
		this.container = new ImageIcon(ResourceManager.getContainer(this.getToolkit()), "Belief container icon");
		this.belief = new ImageIcon(ResourceManager.getBelief(this.getToolkit()));
		this.subject = new ImageIcon(ResourceManager.getSubject(this.getToolkit()));
		this.predicate = new ImageIcon(ResourceManager.getPredicate(this.getToolkit()));
		this.object = new ImageIcon(ResourceManager.getObject(this.getToolkit()));
		this.selectedContainer = new ImageIcon(ResourceManager.getSelectedCotainer(this.getToolkit()));
		this.selectedBelief = new ImageIcon(ResourceManager.getSelectedBelief(this.getToolkit()));
		this.selectedSubject = new ImageIcon(ResourceManager.getSelectedSubject(this.getToolkit()));
		this.selectedPredicate = new ImageIcon(ResourceManager.getSelectedPredicate(this.getToolkit()));
		this.selectedObject = new ImageIcon(ResourceManager.getSelectedObject(this.getToolkit()));

		this.userPreferencies = UserPreferencies;
		this.agi = agi;
		this.dimensionGnome = new Dimension();
		this.widthPadding = 0x0;
		this.heightPadding = 0x0;

		this.expandedNodeTextValue = "";

		this.selectedForeground = UIManager.getColor("Tree.selectionForeground");
		this.selectedBackground = UIManager.getColor("Tree.selectionBackground");

		// Initialise self settings that do not change
		// -- Set text alignment
		this.setHorizontalAlignment(SwingConstants.LEFT);
		this.setVerticalAlignment(SwingConstants.CENTER);

		// our logger <3
		this.logger = java.util.logging.Logger.getLogger(this.getClass().getCanonicalName());
	}

	@Override
	public Component getTreeCellRendererComponent(final JTree tree, final Object value, final boolean selected, final boolean expanded,
			final boolean leaf, final int row, final boolean hasFocus) {
		return this.getCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
	}

	/**
	 * Same semantics as {@link #getTreeCellRendererComponent} but with more general arguments.
	 * 
	 * @param tree
	 * @param value
	 * @param selected
	 * @param expanded
	 * @param leaf
	 * @param row
	 * @param hasFocus
	 * @return a component to render a cell
	 */
	public Component getCellRendererComponent(final JComponent tree, final Object value, final boolean selected, final boolean expanded,
			final boolean leaf, final int row, final boolean hasFocus) {
		// get result component
		final JLabel result = this;

		// do appropriate changes on resulting component (which is this)
		// -- set parent so that we may get some graphics toolkit
		tree.add(this);
		// -- Set text
		this.setAppropriateText(result, tree, value, selected, expanded, leaf, row, hasFocus, this.userPreferencies.isAliasNamespaces(),
				this.userPreferencies.isShowExpandedBeliefContainerText());
		this.logger.fine(Formatter.format("%s(%s) -> %s", value, value.getClass(), result.getText()));
		// -- set appropriate icon
		this.setAppropriateIcon(result, tree, value, selected, expanded, leaf, hasFocus,
				this.userPreferencies.isShowExpandedBeliefContainerIcon());

		// copy properties from parent tree
		this.copyAttributes(result, tree);

		// -- set appropriate colours
		this.setAppropriateColours(result, tree, value, selected, expanded, leaf, hasFocus);

		// set size ourselves
		this.setAppropriateSize(result, tree, value);
		return result;
	}

	private String getMaxPossibleStringFrom(Object value) {
		return this.getTextFromValue(value, false);
	}

	/**
	 * Method which is used by {@link #getCellRendererComponent} in order to transform the given {@code value} to text.
	 * 
	 * @param value
	 *            the value to transform to text
	 * @param short_namespaces
	 *            alias namespaces by {@link UbiwareAgent#prefixResource(String)}
	 * @return a textual representation of the given {@code value}
	 */
	public String getTextFromValue(final Object value, final boolean short_namespaces) {
		if (value instanceof BeliefContainerTreeModel) {
			return this.getTextFromBeliefContainer(short_namespaces, (BeliefContainerTreeModel) value);
		}
		if (value instanceof BeliefTreeModel) {
			return this.getTextFromBelief(short_namespaces, (BeliefTreeModel) value);
		}
		if (value instanceof SemanticStatementElementTreeModel) {
			return this.getTextFromSemanticStatement(short_namespaces, (SemanticStatementElementTreeModel) value);
		}
		return value.toString();
	}

	// Private and confusing, probably /////////////////////////////////////////
	private void copyAttributes(final JLabel result, final JComponent tree) {
		result.setFont(tree.getFont());
		result.setForeground(tree.getForeground());
		result.setBackground(tree.getBackground());
		result.setCursor(tree.getCursor());
	}

	private void setAppropriateSize(final JLabel result, final JComponent tree, final Object value) {
		// Get the "max" version of the value (we want to display things
		// always according to the max space they could possibly
		// occupy)
		final String maxVal = this.getMaxPossibleStringFrom(value);
		final Component font_component = result;
		final Font my_font = font_component.getFont();

		// Try to get graphics from somewhere
		Graphics graphics = this.getGraphics();
		if (graphics == null) {
			graphics = font_component.getGraphics();
		}
		if (graphics == null) {
			graphics = this.userPreferencies.getGraphics();
		}
		// Get Font stuff
		final FontMetrics font_metrics = // shit happens
		graphics == null ? font_component.getFontMetrics(my_font) : graphics.getFontMetrics();
		final FontRenderContext font_render_context = font_metrics.getFontRenderContext();

		// Get the GlyphVector for rendering the string on display.
		// This should indicate how much space it takes.
		final GlyphVector glyph_vector = my_font.layoutGlyphVector(font_render_context, maxVal.toCharArray(), 0, maxVal.length(),
				Font.LAYOUT_LEFT_TO_RIGHT);

		// Get the rendering dimensions
		final Rectangle textRenderingBounds = glyph_vector.getLogicalBounds().getBounds();

		// use the largest dimensions of all the possible icons
		final Dimension askiryf_dimensions = this.getLargestAskiryfDimensions();
		this.dimensionGnome.setSize(
		// Set total width
				textRenderingBounds.width + // text rendering width
						askiryf_dimensions.width + // icon width
						result.getIconTextGap() + // distance between label icon & text
						this.widthPadding // width padding for nice looks!
				, // Set total height
				textRenderingBounds.height + // text rendering height
						askiryf_dimensions.height + // icon height
						this.heightPadding // height padding for good looks
		);
		result.setMinimumSize(this.dimensionGnome);
		result.setPreferredSize(this.dimensionGnome);
	}

	private Dimension getLargestAskiryfDimensions() {
		// if something is not loaded, it counts as zero dimensions
		if (this.asundio == null) {
			if (this.ferundio == null) {
				return new Dimension(0, 0);
			} else {
				return new Dimension(this.ferundio.getIconWidth(), this.ferundio.getIconHeight());
			}
		} else // Asundio non null
		if (this.ferundio == null) {
			return new Dimension(this.asundio.getIconWidth(), this.asundio.getIconHeight());
		} else {
			return new Dimension(Math.max(this.asundio.getIconWidth(), this.ferundio.getIconWidth()), Math.max(
					this.asundio.getIconHeight(), this.ferundio.getIconHeight()));
		}
	}

	private void setAppropriateIcon(final JLabel result, final JComponent tree, final Object value, final boolean selected,
			final boolean expanded, final boolean leaf, final boolean hasFocus, final boolean showExpandedBeliefContainerIcon) {
		// maybe it is a belief container tree
		if (value instanceof BeliefContainerTreeModel) {
			this.setIcon((expanded && showExpandedBeliefContainerIcon) || !expanded ? selected ? this.selectedContainer : this.container
					: null);
		} else if (value instanceof BeliefTreeModel) {
			this.setIcon(selected ? this.selectedBelief : this.belief);
		} else {
			try {
				final SemanticStatementElementTreeModel ssetm = SemanticStatementElementTreeModel.class.cast(value);
				switch (ssetm.getType()) {
				case Subject:
					this.setIcon(selected ? this.selectedSubject : this.subject);
					return;
				case Predicate:
					this.setIcon(selected ? this.selectedPredicate : this.predicate);
					return;
				case Object:
					this.setIcon(selected ? this.selectedObject : this.object);
					return;
				default:
					this.logger.severe("ssetm.type == " + ssetm.getType());
					throw new AssertionError("Illegal program state");
				}
			} catch (ClassCastException classCastException) {
			}

			this.logger.warning("unknown treee to render: " + value.getClass().toString() + " = " + value);
			result.setIcon(this.asundio);
		}
	}

	private String getTextFromBeliefContainer(final boolean short_namespaces, final BeliefContainerTreeModel bftm) {
		String result = bftm.toString();

		// if user wants to replace namespaces, then do it
		if (short_namespaces) {
			result = this.agi.prefixResource(result);
		}

		result = "<html><em>" + StringEscapingUtils.forHTML(result) + "</em></html>";

		return result;
	}

	private String getTextFromBelief(final boolean short_namespaces, final BeliefTreeModel btm) {
		String result = null;

		// if user wants to replace namespaces, then do it
		if (short_namespaces) {
			final String theSubject = btm.getSubject().toString(), thePredicate = btm.getPredicate().toString(), theObject = btm
					.getObject().toString();
			result = Formatter.format("%s %s %s", this.agi.prefixResource(theSubject), this.agi.prefixResource(thePredicate),
					this.agi.prefixResource(theObject));
		} else {
			result = btm.toString();
		}

		return result;
	}

	private String getTextFromSemanticStatement(final boolean short_namespaces, final SemanticStatementElementTreeModel sstm) {
		String result = sstm.toString();

		if (short_namespaces) {
			result = this.agi.prefixResource(result);
		}

		return result;
	}

	private void setAppropriateText(final JLabel result, final JComponent tree, final Object value, final boolean selected,
			final boolean expanded, final boolean leaf, final int row, final boolean hasFocus, final boolean aliasNamespacesEnabled,
			final boolean showExpandedBeliefContainerText) {
		if ((value instanceof BeliefContainerTreeModel) && expanded) {
			if (showExpandedBeliefContainerText) {
				result.setText(this.getTextFromValue(value, aliasNamespacesEnabled));
			} else {
				result.setText(this.expandedNodeTextValue);
			}
		} else if (expanded) {
			result.setText(this.expandedNodeTextValue);
		} else {
			result.setText(this.getTextFromValue(value, aliasNamespacesEnabled));
		}
	}

	private void setAppropriateColours(final Component result, final JComponent tree, final Object value, final boolean selected,
			final boolean expanded, final boolean leaf, final boolean hasFocus) {
		if (selected) {
			result.setForeground(this.selectedForeground);
			result.setBackground(this.selectedBackground);
		}
	}

}
