package ubiware.core.visualizer;

import java.awt.Font;
import java.awt.Graphics;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import ubiware.core.Debugger;
import ubiware.core.UbiwareAgent;
import ubiware.core.behaviors.DebugBehavior;

/**
 * <p>
 * This class implements the "logical" functionality of the visualizer.
 * </p>
 * 
 * <p>
 * There are the concepts of "properties" which are values of significance and they represent higher-level options for the user. For each
 * one of those properties, there are {@link java.beans.PropertyChangeEvent}s fired whenever there is a vlaue change. Those values are the
 * following:
 * </p>
 * 
 * <ul>
 * <li>aliasNamespaces</li>
 * <li>showSettings</li>
 * <li>agentLifeCycles</li>
 * <li>waitUntilAgentLifeCycle</li>
 * <li>waitingUntilAgentLifeCycle</li>
 * <li>expandableEmptyBeliefContainers</li>
 * <li>showExpandedBeliefContainerText</li>
 * <li>showExpandedBeliefContainerIcon</li>
 * </ul>
 * 
 * <p>
 * The string representing those properties in the fired events is the same as the strings above. They are also available as public fields.
 * The class also provides accessors for those properties. The setters fire the change events if there is an actual value change.
 * </p>
 * 
 * @author Nikos Mouchtaris
 */
public class VisualizerPrototype extends VisualizerPrototypeForm implements SAPLTreeCellRenderer.UserPreferencies, Debugger {
	private static final long serialVersionUID = 4l;

	/**
	 * The name of the property change for the property change event that is thrown when property "aliasNamespaces" is changed.
	 */
	public static final String aliasNamespacesPropertyName = "aliasNamespaces";
	/**
	 * The name of the property change for the property change event that is thrown when property "showSettings" is changed.
	 */
	public static final String showSettingsPropertyName = "showSettings";
	/**
	 * The name of the property change for the property change event that is thrown when property "agentLifeCycles" is changed.
	 */
	public static final String agentLifeCyclesPropertyName = "agentLifeCycles";
	/**
	 * The name of the property change for the property change event that is thrown when property "waitUntilAgentLifeCycle" is changed.
	 */
	public static final String waitUntilAgentLifeCyclePropertyName = "waitUntilAgentLifeCycle";
	/**
	 * The name of the property change for the property change event that is thrown when property "waitingUntilAgentLifeCycle" is changed.
	 */
	public static final String waitingUntilAgentLifeCyclePropertyName = "waitingUntilAgentLifeCycle";
	/**
	 * The name of the property change for the property change event that is thrown when property "expandableEmptyBeliefContainers" is
	 * changed.
	 */
	public static final String expandableEmptyBeliefContainersPropertyName = "expandableEmptyBeliefContainers";
	/**
	 * The name of the property change for the property change event that is thrown when property "showExpandedBeliefContainerIcon" is
	 * changed.
	 */
	public static final String showExpandedBeliefContainerIconPropertyName = "showExpandedBeliefContainerIcon";
	/**
	 * The name of the property change for the property change event that is thrown when property "showExpandedBeliefContainerText" is
	 * changed.
	 */
	public static final String showExpandedBeliefContainerTextPropertyName = "showExpandedBeliefContainerText";

	/**
	 * <p>
	 * Constructs a new Visualizer instance for the given agent, using the given behavior to access beliefs.
	 * </p>
	 * 
	 * @param agi
	 *            the Agent's whose beliefs we are messing with
	 * @param rabi
	 *            a RAB whose parent agent <strong>must</strong> be {@code agi}
	 */
	public VisualizerPrototype(UbiwareAgent agi, DebugBehavior rabi) {
		super(agi, rabi);
		// initialize messages from some components
		this.initAllowedShowMessages();

		// Initialize my fields
		// Finals:
		this.agi = agi;
		this.rabi = rabi;
		this.debugController = this.rabi.debugController;
		this.debugController.setDebugger(this);
		this.initMyFields(rabi.isInitialStop());
		this.logger = java.util.logging.Logger.getLogger(this.getClass().getCanonicalName());
		this.callsLogger = java.util.logging.Logger.getLogger(this.logger.getName() + ".calls");
		this.searchesLogger = java.util.logging.Logger.getLogger(this.logger.getName() + ".searches");
		this.r = new BeliefListCellRenderer(this, this.agi);

		// initialize default user-settings
		this.initUserSettings();

		// perform post-default-initialization iniitialization on
		// components
		this.initComponenets();
	}

	/**
	 * Classes implementing this interface can act as predicates, accepting or rejecting elements of type {@code T}.
	 * 
	 * @param T
	 *            the type of elements this predicate can filtre.
	 */
	public interface Predicate<T> {
		/**
		 * Accepts or rejects the given element.
		 * 
		 * @param element
		 *            the element to test
		 * @return {@code true} is the element is accepted, {@code false} otherwise
		 */
		boolean isAcceptable(final T element);
	}

	/**
	 * <p>
	 * Traverses the subtree of the given tree model whose root is indicated by the given tree path.
	 * </p>
	 * 
	 * <p>
	 * Each node (not only leaves) are tested with the given predicate and if accepted, they are included in the resulting Set.
	 * </p>
	 * 
	 * <p>
	 * If {@code parent} is {@code null}, then the whole tree is checked.
	 * </p>
	 * 
	 * @param tm
	 *            the tree model to traverse
	 * @param parent
	 *            the parent of the subtree to check
	 * @param predicate
	 *            a predicate for TreePaths of objects in the tree model
	 * @param logger
	 *            an optional logger to log actions to
	 * @return a set of accepted tree paths
	 */
	public static Set<TreePath> searchTree(final TreeModel tm, final Predicate<TreePath> predicate, TreePath parent, final Logger logger) {
		// if logger == null, yield to the defaulter
		if (logger == null) {
			return VisualizerPrototype.searchTree(tm, predicate, parent);
		}

		logger.entering(VisualizerPrototype.class.getCanonicalName(), "searchTree", new Object[] { tm, predicate, parent, logger });
		// make sure parent and logger are not null
		if (parent == null) {
			parent = new TreePath(tm.getRoot());
		}
		logger.fine("new parent = " + parent);

		Set<TreePath> result = null, ducky = null;

		// Do kids first
		final int child_len = tm.getChildCount(parent.getLastPathComponent());
		logger.fine(child_len + " kids to check");
		// if there are any
		if (child_len > 0) {
			int i = 0;
			// do kids while there are no results
			while ((result == null) && (i < child_len)) {
				result = VisualizerPrototype.searchTree(tm, predicate,
						parent.pathByAddingChild(tm.getChild(parent.getLastPathComponent(), i++)), logger);
			}
			// do the rest (if any)
			while (i < child_len) {
				if ((ducky = VisualizerPrototype.searchTree(tm, predicate,
						parent.pathByAddingChild(tm.getChild(parent.getLastPathComponent(), i++)), logger)) != null) {
					result.addAll(ducky);
				}
			}
		}

		// do self
		if (predicate.isAcceptable(parent)) {
			logger.fine("I :: accepted");
			// if there is a result already
			if (result != null) {
				// add myself to the result
				result.add(parent);
			} else {
				// create one and add myself
				(result = new HashSet<TreePath>()).add(parent);
				//else // even I am not acceptable
				// leave the result as it is
			}
		} else {
			logger.fine("I :: rejected");
		}

		logger.exiting(VisualizerPrototype.class.getCanonicalName(), "searchTree", result);
		return result;
	}

	/**
	 * Calls {@link #searchTree} with a logger that writes to nowhere.
	 * 
	 * @param tm
	 *            see {@link #searchTree}
	 * @param predicate
	 *            see {@link #searchTree}
	 * @param parent
	 *            see {@link #searchTree}
	 * @return see {@link #searchTree}
	 */
	public static Set<TreePath> searchTree(final TreeModel tm, final Predicate<TreePath> predicate, final TreePath parent) {
		// no logger, create dugru logger
		// layer the logger
		Logger logger = new java.util.logging.Logger("", null) {
			@Override
			public void log(final LogRecord record) {
				// nada
			}
		};
		return VisualizerPrototype.searchTree(tm, predicate, parent, logger);
	}

	// Implementation of interface SAPLTreeCellRenderer.UserPreferencies ///////
	/**
	 * Returns true if the user has given the notion that she would like namespaces to be printed as short aliases.
	 * 
	 * @return {@code true} if the user has given the notion that she would like namespaces to be printed as short aliases.
	 */
	@Override
	public boolean isAliasNamespaces() {
		return this.aliasNamespaces;
	}

	/**
	 * <p>
	 * Provides access to the {@link Font} which is used to display information to the user. This method is a "best effort" method to
	 * determine which font is used to display text. It is possible (especially if this method is invoked before the initialization of the
	 * component) that it returns {@code null} as a result. The sources that it looks for a font are the following, in the order they are
	 * presented:
	 * </p>
	 * 
	 * <ol>
	 * <li>{@link java.awt.Component#getFont this.getFont()}</li>
	 * <li>{@link javax.swing.JFrame#getGraphics this.getGraphics}.{@link java.awt.Graphics#getFont getFont()} unless {@code getGraphics()}
	 * returns {@code null}.</li>
	 * </ol>
	 * 
	 * <p>
	 * If all of the above return {@code null}, then this method will return {@code null} as well.
	 * </p>
	 * 
	 * @return the {@link Font} which is used to display information to the user or {@code null}.
	 */
	@Override
	public Font getDisplayFont() {
		Font result = this.getFont();
		if (result == null) {
			final Graphics graphics = this.getGraphics();
			result = graphics != null ? graphics.getFont() : null;
		}
		return result;
	}

	@Override
	public boolean isEmptyBeliefContainersExpandable() {
		return this.isExpandableEmptyBeliefContainers();
	}

	////////////////////////////////////////////////////////////////////////////

	// Implementation of the UbiwareAgent.Debugger interface ///////////////////
	/**
	 * Method specified by the Ubiware agent {@link Debugger}. When the agent notifies the visualizer instance through this method, the
	 * visualizer will update information about in which cycle the Agent is currently in. It will also rebuild the belief tree if this is a
	 * cycle that the visualizer is {@link #getWaitUntilAgentLifeCycle waiting for}.
	 * 
	 * @param cycle
	 *            number of cycles the agent has lived
	 */
	@Override
	public void notifyLiveIterationEnded(final long cycle) {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				VisualizerPrototype.this.setAgentLifeCycles(cycle);
				if (VisualizerPrototype.this.isWaitingUntilAgentLifeCycle()
						&& (cycle >= VisualizerPrototype.this.getWaitUntilAgentLifeCycle())) {
					VisualizerPrototype.this.rebuildBeliefTree();
					VisualizerPrototype.this.enableAgentProgressControls();
					VisualizerPrototype.this.setWaitingUntilAgentLifeCycle(false);
				}
			}
		});
	}

	/**
	 * Method specified by the Ubiware agent {@link Debugger}. When the agent notifies the visualizer instance through this method, the
	 * visualizer will re-read all the belief storage of the agent and will update the displayed belief tree.
	 */
	@Override
	public void notifyGoingSleep() {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				VisualizerPrototype.this.showMessage("agent mom", "Agent schlafen");
				VisualizerPrototype.this.enableAgentProgressControls();
				VisualizerPrototype.this.rebuildBeliefTree();
				// we are not longer waiting for any life cycle
				VisualizerPrototype.this.setWaitingUntilAgentLifeCycle(false);
			}
		});
	}

	////////////////////////////////////////////////////////////////////////////

	/**
	 * Sets whether namespaces should aliased as prefixes or not. If there is an actual value change, it fires a property change for
	 * property "aliasNamespaces".
	 * 
	 * @param aliasNamespaces
	 *            alias namespaces?
	 */
	protected void setAliasNamespaces(final boolean aliasNamespaces) {
		final boolean old = this.isAliasNamespaces();
		if (aliasNamespaces != old) {
			this.aliasNamespaces = aliasNamespaces;
			this.firePropertyChange("aliasNamespaces", old, aliasNamespaces);
		}
	}

	/**
	 * Sets whether settings panel should be displayed or not. If there is an actual value change, it fires a property change for property
	 * "showSettings".
	 * 
	 * @param showSettings
	 *            show settings panel?
	 */
	protected void setShowSettings(final boolean showSettings) {
		final boolean old = this.isShowSettings();
		if (old != showSettings) {
			this.showSettings = showSettings;
			this.firePropertyChange("showSettings", old, showSettings);
		}
	}

	/**
	 * Sets the internal belief about in which life cycle the agent is right now. If there is an actual value change, it fires a property
	 * change for property "agentLifeCycles".
	 * 
	 * @param agentLifeCycles
	 *            the current agent life cycle
	 */
	protected void setAgentLifeCycles(final long agentLifeCycles) {
		final long old = this.getAgentLifeCycles();
		if (old != agentLifeCycles) {
			this.agentLifeCycles = agentLifeCycles;
			this.firePropertyChange("agentLifeCycles", old, agentLifeCycles);
		}
	}

	/**
	 * Sets whether empty belief containers should be expandable on the tree. If there is an actual value change, it fires a property change
	 * for property "expandableEmptyBeliefContainers".
	 * 
	 * @param expandableEmptyBeliefContainers
	 *            should empty belief containers be expandable?
	 */
	protected void setExpandableEmptyBeliefContainers(final boolean expandableEmptyBeliefContainers) {
		final boolean old = this.isExpandableEmptyBeliefContainers();
		if (old != expandableEmptyBeliefContainers) {
			this.expandableEmptyBeliefContainers = expandableEmptyBeliefContainers;
			this.firePropertyChange("expandableEmptyBeliefContainers", old, expandableEmptyBeliefContainers);
		}
	}

	/**
	 * Sets whether expanded belief container nodes should display their text or not. If there is an actual value change, it fires a
	 * property change for property "showExpandedBeliefContainerText".
	 * 
	 * @param showExpandedBeliefContainerText
	 *            should expanded belief containers display their text?
	 */
	protected void setShowExpandedBeliefContainerText(final boolean showExpandedBeliefContainerText) {
		final boolean old = this.isShowExpandedBeliefContainerText();
		if (old != showExpandedBeliefContainerText) {
			this.showExpandedBeliefContainerText = showExpandedBeliefContainerText;
			this.firePropertyChange("showExpandedBeliefContainerText", old, showExpandedBeliefContainerText);
		}
	}

	/**
	 * Sets whether expanded belief container nodes should display their icon or not. If there is an actual value change, it fires a
	 * property change for property "showExpandedBeliefContainerIcon".
	 * 
	 * @param showExpandedBeliefContainerIcon
	 *            should expanded belief containers display their icon?
	 */
	protected void setShowExpandedBeliefContainerIcon(final boolean showExpandedBeliefContainerIcon) {
		final boolean old = this.isShowExpandedBeliefContainerIcon();
		if (old != showExpandedBeliefContainerIcon) {
			this.showExpandedBeliefContainerIcon = showExpandedBeliefContainerIcon;
			this.firePropertyChange("showExpandedBeliefContainerIcon", old, showExpandedBeliefContainerIcon);
		}
	}

	/**
	 * <p>
	 * Sets the agent life cycle number that the visualizer is waiting for.
	 * </p>
	 * 
	 * <p>
	 * If parameter {@code lifeCycle} is a negative number, an exception is thrown.
	 * </p>
	 * 
	 * <p>
	 * If this method is called with a valid argument, {@link #setWaitingUntilAgentLifeCycle setWaitingUntilAgentLifeCycle(true)} is
	 * automatically called.
	 * </p>
	 * 
	 * <p>
	 * If there is an actual value change, a property change event is fired for property "waitUntilAgentLifeCycle".
	 * </p>
	 * 
	 * @param lifeCycle
	 *            the life cycle to wait for
	 * @throws java.lang.IllegalArgumentException
	 *             if parameter {@code lifeCycle} is a negative number
	 */
	protected void setWaitUntilAgentLifeCycle(final long lifeCycle) throws IllegalArgumentException {
		if (Long.signum(lifeCycle) == -1) {
			throw new IllegalArgumentException("life cycle number has " + "negative value");
		}
		// value is ok
		final long old = this.getWaitUntilAgentLifeCycle();
		if (old != lifeCycle) {
			this.waitUntilAgentLifeCycle = lifeCycle;
			this.firePropertyChange("waitUntilAgentLifeCycle", old, lifeCycle);
			this.setWaitingUntilAgentLifeCycle(true);
		}
	}

	/**
	 * <p>
	 * Sets whether the visualizer is expecting for a specific agent life cycle to be completed or not. If {@code false} is passed as a
	 * parameter to this method, then the value of property "{@link #getWaitUntilAgentLifeCycle waitUntilAgentLifeCycle}" is ignored.
	 * </p>
	 * 
	 * <p>
	 * This method is automatically called from {@link #setWaitUntilAgentLifeCycle} with argument {@code true} and there is no need to call
	 * is separately after calling {@code setWaitUntilAgentLifeCycle}.
	 * </p>
	 * 
	 * <p>
	 * If there is an actual value change, this method fires a property change event for property "waitingUntilAgentLifeCycle".
	 * </p>
	 * 
	 * @param waitingUntilAgentLifeCycle
	 *            wait until agent life cycle {@code x} ?
	 */
	protected void setWaitingUntilAgentLifeCycle(final boolean waitingUntilAgentLifeCycle) {
		final boolean old = this.isWaitingUntilAgentLifeCycle();
		if (old != waitingUntilAgentLifeCycle) {
			this.waitingUntilAgentLifeCycle = waitingUntilAgentLifeCycle;
			this.firePropertyChange("waitingUntilAgentLifeCycle", old, waitingUntilAgentLifeCycle);
		}
	}

	/**
	 * Returns whether the visualizer is waiting for the completion of some agent life cycle.
	 * 
	 * @return whether the visualizer is waiting for the completion of some agent life cycle
	 */
	public boolean isWaitingUntilAgentLifeCycle() {
		return this.waitingUntilAgentLifeCycle;
	}

	/**
	 * Returns the life cycle number of the agent that the visualizer is waiting for. This is implies that <code>{@link
	 * #isWaitingUntilAgentLifeCycle isWaitingUntilAgentLifeCycle()
	 * } == true</code>.
	 * 
	 * @return the life cycle number of the agent that the visualizer is waiting for
	 */
	public long getWaitUntilAgentLifeCycle() {
		return this.waitUntilAgentLifeCycle;
	}

	/**
	 * Returns whether the user prefers that expanded belief container nodes display their icon or not.
	 * 
	 * @return whether expanded belief container nodes display their icon
	 */
	@Override
	public boolean isShowExpandedBeliefContainerIcon() {
		return this.showExpandedBeliefContainerIcon;
	}

	/**
	 * Returns whether the user prefers that expanded belief container nodes display their text or not.
	 * 
	 * @return whether expanded belief container nodes display their text
	 */
	@Override
	public boolean isShowExpandedBeliefContainerText() {
		return this.showExpandedBeliefContainerText;
	}

	/**
	 * Returns whether the user prefers that empty belief containers are expandable or not.
	 * 
	 * @return whether empty belief containers are expandable
	 */
	public boolean isExpandableEmptyBeliefContainers() {
		return this.expandableEmptyBeliefContainers;
	}

	/**
	 * Returns whether the user has made some gesture which implies that she whould like the settings pane to be shown.
	 * 
	 * @return whether the user has made some gesture which implies that she whould like the settings pane to be shown
	 */
	public boolean isShowSettings() {
		return this.showSettings;
	}

	/**
	 * Returns the life cycles the agent has lived up to now.
	 * 
	 * @return the life cycles the agent has lived up to now
	 */
	public long getAgentLifeCycles() {
		return this.agentLifeCycles;
	}

	// Semantic Event handling /////////////////////////////////////////////////
	@Override
	protected void alias_namespaces_changed(final boolean alias) {
		// update our internal memory
		this.setAliasNamespaces(alias);
		// repaint belief tree
		this.repaintBeliefTree();
	}

	@Override
	protected void show_settings_panel_changed(final boolean show) {
		// hide settings panel
		this.showSettingsPanel(show);
		// update internal memory
		this.setShowSettings(show);
	}

	/**
	 * Will arrange everything so that the visualizer will wait for a number of specified life cycles. If {@code lifeCycle} is a negative
	 * number or zero an exception is thrown.
	 * 
	 * @param wait
	 *            {@inheritDoc}
	 * @param cycles
	 *            {@inheritDoc}
	 * @throws IllegalArgumentException
	 *             when argument {@code lifeCycle} is {@code &lt;=} 0
	 */
	@Override
	protected void wait_until_agent_life_cycle_changed(final boolean wait, final int cycles) {
		if (cycles < 0l) {
			throw new IllegalArgumentException("lifeCycle argument is <=0");
		}

		// values are ok here
		if (wait) {
			final long currentCycle = this.getAgentLifeCycles();
			final long waitUntil = currentCycle + cycles;
			this.rabi.invokeLater(new Runnable() {
				@Override
				public void run() {
					VisualizerPrototype.this.rabi.debugController.stopAfter(cycles);
				}
			});
			this.disableAgentProgressControls();
			this.setWaitUntilAgentLifeCycle(waitUntil);
		} else {
			this.setWaitingUntilAgentLifeCycle(false);
		}
	}

	@Override
	protected void stop_agent_changed(boolean stopped) {
		if (stopped) {
			this.rabi.invokeLater(new Runnable() {
				@Override
				public void run() {
					VisualizerPrototype.this.rabi.debugController.stopNow();
				}
			});
			this.setWaitUntilAgentLifeCycle(this.getAgentLifeCycles());
		} else {
			this.rabi.invokeLater(new Runnable() {
				@Override
				public void run() {
					VisualizerPrototype.this.rabi.debugController.start();
				}
			});
			this.setWaitingUntilAgentLifeCycle(false);
		}
	}

	@Override
	protected void expandable_belief_containers_changed(boolean expandable) {
		this.setExpandableEmptyBeliefContainers(expandable);
		this.repaintBeliefTree();
	}

	@Override
	protected void show_expanded_belief_containers_icon_changed(boolean show) {
		this.setShowExpandedBeliefContainerIcon(show);
		this.repaintBeliefTree();
	}

	@Override
	protected void show_expanded_belief_containers_text_changed(boolean show) {
		this.setShowExpandedBeliefContainerText(show);
		this.repaintBeliefTree();
	}

	@Override
	protected void show_in_N3_requested(Object node) {
		try {
			this.showN3(this.treeNodeToN3(TreeModel.class.cast(node)));
			this.request_focus(0x0001);
		} catch (ClassCastException classCastException) {
			this.logger.warning("Unknown node type: " + node.getClass().getCanonicalName());
		} catch (IllegalArgumentException illegalArgumentException) {
			this.showMessage("Error", illegalArgumentException.getMessage(), true);
		}
	}

	@Override
	protected void search_requested(final BeliefSearchArguments arguments, final BeliefSearchType search_type) {
		switch (search_type) {
		case SimpleTextualMatch:
			this.handleSimpleTextualMatchSearch(arguments);
			break;
		case Advanced:
			this.handleAdvancedSearch(arguments);
			break;
		default:
			throw new AssertionError("Illegal program state");
		}
	}

	@Override
	protected void search_result_selected(final TreePath path) {
		// just show it
		this.make_displayed(path);
	}

	////////////////////////////////////////////////////////////////////////////

	/**
	 * <p>
	 * Convenience method which equals calling {@link #rebuildBeliefTree(TreeModel) rebuildBeliefTree}(
	 * {@link BeliefContainerTreeModel#BeliefContainerTreeModel new BeliefContainerTreeModel}({@code null}, {@link #rabi rabi},
	 * {@link UbiwareAgent#GENERAL_CONTEXT}).
	 * </p>
	 */
	protected void rebuildBeliefTree() {
		this.rebuildBeliefTree(this.lastTreeModel = new BeliefContainerTreeModel(null, this.rabi, UbiwareAgent.GENERAL_CONTEXT));
	}

	/**
	 * Produces an N3 string from the given node, which must be either {@link BeliefTreeModel} or {@link BeliefContainerTreeModel}.
	 * 
	 * @param node
	 *            the node
	 * @return an N3 representation of the given node
	 * @throws java.lang.IllegalArgumentException
	 *             when {@code node} is a {@link BeliefTreeModel}
	 */
	protected String treeNodeToN3(final TreeModel node) throws IllegalArgumentException {
		if (node instanceof BeliefTreeModel) {
			final StringBuilder strbob = new StringBuilder();
			strbob.append('"').append(node).append("\" is a belief. Select a belief container to ").append("show in N3");
			throw new IllegalArgumentException(strbob.toString());
		}
		if (node instanceof BeliefContainerTreeModel) {
			final BeliefContainerTreeModel bctm = (BeliefContainerTreeModel) node;
			return this.rabi.produceN3(bctm.getContextID());
		}
		if (node instanceof SemanticStatementElementTreeModel) {
			final SemanticStatementElementTreeModel ssetm = (SemanticStatementElementTreeModel) node;
			return this.agi.prefixResource(ssetm.toString());
		}
		this.logger.warning("Unknown tree model type: " + node + " (" + node.getClass().getCanonicalName() + ")");
		return null;
	}

	// Private and weird ///////////////////////////////////////////////////////
	// Communication with the agent
	private final UbiwareAgent agi;
	private final DebugBehavior rabi;
	private final UbiwareAgent.DebugController debugController;
	// Utilities
	private final Logger logger;
	private final Logger callsLogger;
	private final Logger searchesLogger;
	// Agent state
	private long agentLifeCycles;
	private long waitUntilAgentLifeCycle;
	private boolean waitingUntilAgentLifeCycle;
	// Internal state
	private BeliefContainerTreeModel lastTreeModel;
	// Components for swing
	// this is also a SAPLTreeCellRenderer
	final private BeliefListCellRenderer r;

	private void initMyFields(final boolean stopped) {
		this.agentLifeCycles = this.agi.getLifeCycleCount();
		this.waitUntilAgentLifeCycle = -1l;
		this.waitingUntilAgentLifeCycle = false;
	}

	// User preferencies
	private boolean aliasNamespaces;
	private boolean showSettings;
	private boolean showExpandedBeliefContainerText;
	private boolean showExpandedBeliefContainerIcon;
	private boolean expandableEmptyBeliefContainers;

	private void initUserSettings() {
		this.aliasNamespaces = true;
		this.showSettings = true;
		this.showExpandedBeliefContainerText = false;
		this.showExpandedBeliefContainerIcon = true;
		this.expandableEmptyBeliefContainers = false;
	}

	/**
	 * <p>
	 * Initializes components with custom values.
	 * </p>
	 * 
	 * <p>
	 * This initialization junk depends on:
	 * </p>
	 * 
	 * <ul>
	 * <li>aliasNamespaces</li>
	 * <li>showSettings</li>
	 * <li>showExpandedBeliefContainerText</li>
	 * <li>showExpandedBeliefContainerIcon</li>
	 * <li>Agi</li>
	 * <li>Rabi</li>
	 * </ul>
	 */
	private void initComponenets() {
		// Fire initial events to synchronize gui with state...
		// ... for user settings ...
		this.firePropertyChange(VisualizerPrototype.aliasNamespacesPropertyName, !this.isAliasNamespaces(), this.isAliasNamespaces());
		this.firePropertyChange(VisualizerPrototype.showSettingsPropertyName, !this.isShowSettings(), this.isShowSettings());
		this.firePropertyChange(VisualizerPrototype.showExpandedBeliefContainerTextPropertyName, !this.isShowExpandedBeliefContainerText(),
				this.isShowExpandedBeliefContainerText());
		this.firePropertyChange(VisualizerPrototype.showExpandedBeliefContainerIconPropertyName, !this.isShowExpandedBeliefContainerIcon(),
				this.isShowExpandedBeliefContainerIcon());
		// ... for agent state ...
		this.firePropertyChange(VisualizerPrototype.agentLifeCyclesPropertyName, -2l, -1l);
		this.firePropertyChange(VisualizerPrototype.waitingUntilAgentLifeCyclePropertyName, !this.isWaitingUntilAgentLifeCycle(),
				this.isWaitingUntilAgentLifeCycle());

		// Set tree model
		this.rebuildBeliefTree();
		// home made cell renderer
		this.setBeliefTreeCellRenderer(this.r);

		// home made cell renderer for lists!
		this.setSearcResultsListCellRenderer(this.r);

		// pack again
		this.pack();
	}

	private void initAllowedShowMessages() {
		this.setMessageFromShowable("quick searcher", true);
		this.setMessageFromShowable("form", false);
	}

	private void handleSimpleTextualMatchSearch(final BeliefSearchArguments a) {
		this.logEntry(a);
		final String search_string = SimpleTextualMatchBeliefSearchArguments.class.cast(a).SearchString;

		// get results of search
		final Set<TreePath> result = VisualizerPrototype.searchTree(this.lastTreeModel, new Predicate<TreePath>() {
			@Override
			public boolean isAcceptable(TreePath element) {
				boolean acceptable = false;
				try {
					final BeliefTreeModel btm = BeliefTreeModel.class.cast(element.getLastPathComponent());
					final String string_elem = VisualizerPrototype.this.r.getTextFromValue(btm,
							VisualizerPrototype.this.isAliasNamespaces());
					VisualizerPrototype.this.searchesLogger.fine("\"" + string_elem + "\".contains(\"" + search_string + "\") = "
							+ (acceptable = string_elem.contains(search_string)));
				} catch (ClassCastException classCastException) {
					VisualizerPrototype.this.searchesLogger.fine(element + " :: not a belief");
				}
				return acceptable;
			}
		}, null, this.searchesLogger);

		// if there are results, tell the form to show them
		if (result != null) {
			this.show_treepath_list(result.toArray(new TreePath[result.size()]), "Search results");
			this.request_focus(0x0100);
		} else {
			this.showMessage("searcher", "No results for search \"" + search_string + '"', JOptionPane.WARNING_MESSAGE);
		}

		this.logExit(result);
	}

	private void handleAdvancedSearch(BeliefSearchArguments bsargs) {
		this.logEntry(bsargs);
		final AdvancedBeliefSearchArguments args = AdvancedBeliefSearchArguments.class.cast(bsargs);

		// get results of search
		final Set<TreePath> result = VisualizerPrototype.searchTree(this.lastTreeModel, new Predicate<TreePath>() {
			@Override
			public boolean isAcceptable(TreePath element) {
				// we are looking only for beliefs and not S, P or O-s
				boolean acceptable = false;
				try {
					final BeliefTreeModel btm = BeliefTreeModel.class.cast(element.getLastPathComponent());
					boolean subject = this.calculateCriterion(
							VisualizerPrototype.this.r.getTextFromValue(btm.getChild(btm, 0), VisualizerPrototype.this.isAliasNamespaces()),
							args.Subjects, args.SubjectCaseSensitive);
					VisualizerPrototype.this.searchesLogger.fine("criterion0 = " + subject);
					boolean predicate = this.calculateCriterion(
							VisualizerPrototype.this.r.getTextFromValue(btm.getChild(btm, 1), VisualizerPrototype.this.isAliasNamespaces()),
							args.Predicates, args.PredicateCaseSensitive);
					VisualizerPrototype.this.searchesLogger.fine("criterion1 = " + predicate);
					boolean object = this.calculateCriterion(
							VisualizerPrototype.this.r.getTextFromValue(btm.getChild(btm, 2), VisualizerPrototype.this.isAliasNamespaces()),
							args.Objects, args.ObjectCaseSensitive);
					VisualizerPrototype.this.searchesLogger.fine("criterion2 = " + object);

					switch (args.Combination) {
					case 1:
						acceptable = subject && predicate && object;
						break;
					case 2:
						acceptable = (subject && predicate) || object;
						break;
					case 3:
						acceptable = (subject && object) || predicate;
						break;
					case 4:
						acceptable = subject || (predicate && object);
						break;
					case 5:
						acceptable = subject || predicate || object;
						break;
					default:
						throw new AssertionError("Illegal Program State");
					}
				} catch (ClassCastException classCastException) {
					VisualizerPrototype.this.searchesLogger.fine("wrong class: " + classCastException.toString());
				}
				VisualizerPrototype.this.searchesLogger.fine("combination-" + args.Combination + "result = " + acceptable);
				return acceptable;
			}

			private boolean calculateCriterion(String testee, final List<String> testers, final boolean cs) // case sensitive
			{
				if (!cs) {
					testee = testee.toLowerCase();
				}

				boolean theResult = false;
				for (String tester : testers) {
					if (!cs) {
						tester = tester.toLowerCase();
					}
					boolean contains;
					VisualizerPrototype.this.searchesLogger.fine("Advanced calculation criterion (" + (cs ? "" : "not ")
							+ "case sensitive): \"" + tester + "\".contains(" + testee + ") --> " + (contains = tester.contains(testee)));
					if (contains) {
						VisualizerPrototype.this.searchesLogger.fine("YES");
						theResult = true;
						break;
					}
				}
				if (!theResult) {
					VisualizerPrototype.this.searchesLogger.fine("Criterion failed");
				}
				return theResult;
			}
		}, null, this.searchesLogger);

		// if there are results, tell the form to show them
		if (result != null) {
			this.show_treepath_list(result.toArray(new TreePath[result.size()]), "Search results");
			this.request_focus(0x0100);
		} else {
			this.showMessage("searcher", "No results for advanced search \"" + args + '"', JOptionPane.WARNING_MESSAGE);
		}

		this.logExit(result);
	}

	private void logEntry(final Object... args) {
		final StackTraceElement frame = Thread.currentThread().getStackTrace()[2];
		this.callsLogger.entering(frame.getClassName(), frame.getMethodName(), args);
	}

	private void logExit(final Object result) {
		final StackTraceElement frame = Thread.currentThread().getStackTrace()[2];
		this.callsLogger.exiting(frame.getClassName(), frame.getMethodName(), result);
	}
}
