/*
 * VisualizerPrototypeForm.java
 *
 * Created on February 8, 2008, 6:25 PM
 */

package ubiware.core.visualizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyListener;
import java.beans.PropertyChangeListener;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.reflect.Field;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import ubiware.core.UbiwareAgent;
import ubiware.core.behaviors.DebugBehavior;

/**
 * <p>
 * The GUI part of the visualizer.
 * </p>
 * 
 * <p>
 * This part of the visualizer translates GUI events to semantic events, which are then supposed to be understood by implementing
 * subclasses. Those semantic events are signaled by {@code protected abstract} methods of this class.
 * </p>
 * 
 * <p>
 * This class is also listening for property change events which are thrown by {@code this} (effectively, by subclasses). When it catches a
 * recognizable property change, it will make sure that all visual components that represent or indicate the value of the changed property
 * will be updated and synchronized in order to display the actual value of the property. Recognizable properties are the following:
 * </p>
 * 
 * <ul>
 * <li>"aliasNamespaces"</li>
 * <li>"showSettingsPanel"</li>
 * <li>"agentLifeCycles"</li>
 * <li>"waitUntilAgentLifeCycle"</li>
 * <li>"expandableEmptyBeliefContainers"</li>
 * <li>"showExpandedBeliefContainerText"</li>
 * <li>"showExpandedBeliefContainerIcon"</li>
 * </ul>
 * 
 * @author Nikos Mouchtaris
 */
public abstract class VisualizerPrototypeForm extends javax.swing.JFrame implements ItemListener, ActionListener, PropertyChangeListener,
		TreeSelectionListener, ListSelectionListener, KeyListener, FocusListener {
	private static final long serialVersionUID = 1L;

	// High level collaboration classes
	private final UbiwareAgent Agi;
	// Loggers
	private final Logger Logger;
	private final Logger ColourLogger;
	private final Logger SemanticEventLogger;
	private final Logger ShowSettingsActionLogger;
	// Display information
	private String agentLifeCyclesDescription;
	private String waitUntilAgentLifeCycleDescription;
	private final Collection<Component> agentProgressControls;
	private final Collection<Component> waitingForLifeCycleLabels;
	private final Map<String, Boolean> messagesFromShowable;
	// More components
	private JPanel contentPane;
	// Colours
	private final Color titledBorderColour;
	// Models
	private final ArrayListModel searchResultsListModel;
	private final SpinnerNumberModel progress_agent_cycles_spinner_model;
	// Actions
	private final ShowSettingsAction showSettingsAction;
	private final ExpandCollapseAllAction expandCollapseAllAction;
	private final QuickSearchRequestedAction quickSearchRequestedAction;

	/**
	 * Creates new form VisualizerPrototypeForm.
	 * 
	 * @param agi
	 *            the agent for which this visualizer will be
	 * @param rabi
	 *            the debug behavior which invoked this visualizer and which <strong>must</strong> be associated with the given agent
	 */
	public VisualizerPrototypeForm(final UbiwareAgent agi, final DebugBehavior rabi) {
		// Utils
		this.Logger = java.util.logging.Logger.getLogger(this.getClass().getCanonicalName());
		this.ColourLogger = java.util.logging.Logger.getLogger(this.Logger.getName() + ".colours");
		this.SemanticEventLogger = java.util.logging.Logger.getLogger(this.Logger.getName() + ".semantic_events");
		this.ShowSettingsActionLogger = java.util.logging.Logger.getLogger(this.Logger.getName() + ".show_settings_action");
		this.Agi = agi;
		// default display information
		this.agentProgressControls = new LinkedList<Component>();
		this.waitingForLifeCycleLabels = new LinkedList<Component>();
		this.agentLifeCyclesDescription = "Agent life cycle: ";
		this.waitUntilAgentLifeCycleDescription = "Waiting until life cycle: ";
		this.messagesFromShowable = new HashMap<String, Boolean>();

		// final stuff have to be assigned here
		// --
		// the spinner number model in order to be able to get the values back
		// without type castings
		this.progress_agent_cycles_spinner_model = new SpinnerNumberModel(1, 1, Integer.MAX_VALUE, 1);
		// Custom list model to update efficiently for search results
		this.searchResultsListModel = new ArrayListModel();
		// Actions
		this.showSettingsAction = new ShowSettingsAction(this.ShowSettingsActionLogger);
		this.expandCollapseAllAction = new ExpandCollapseAllAction();
		this.quickSearchRequestedAction = new QuickSearchRequestedAction();

		// Colours -- BEFORE initComponents!
		this.titledBorderColour = new javax.swing.JLabel().getForeground().darker();
		this.ColourLogger.fine("titledBorderColour = " + this.titledBorderColour);

		// Init components
		this.initComponentsPre();
		this.initComponents();
		this.initComponentsPost();

		// now that the componenets have been initialized, we can 
		// create their groups as well:
		this.agentProgressControls.add(this.progress_agent_button);
		//
		this.waitingForLifeCycleLabels.add(this.waiting_for_agent_life_cycle_description_label);
		this.waitingForLifeCycleLabels.add(this.waiting_for_agent_life_cycle_label);
		this.setTitle(this.Agi.getName());
	}

	/**
	 * Passes each treepath in tm (ALL treepaths, not just leafs) to f.
	 * 
	 * @param tm
	 *            the treemodel to be traversed
	 * @param root
	 *            the root from which to begin traversal
	 * @param f
	 *            the function to apply on all treepaths
	 */
	public static void for_all_treepaths(final TreeModel tm, final TreePath root, final Lambda<?> f) {
		final int kids_num = tm.getChildCount(root.getLastPathComponent());
		for (int i = 0; i < kids_num; ++i) {
			VisualizerPrototypeForm.for_all_treepaths(tm, root.pathByAddingChild(tm.getChild(root.getLastPathComponent(), i)), f);
		}
		f.call(root);
	}

	/**
	 * A collection of possible search types that a user can make. The are used for the semantic event {@link #search_requested}. Each
	 * search type is related to a {@link BeliefSearchArguments} structure which is specific to the arguments each search type
	 * accepts/requires.
	 */
	public enum BeliefSearchType {
		/**
		 * Simple check for the occurence of a string. Arguments structure: {@link SimpleTextualMatchBeliefSearchArguments}.
		 */
		SimpleTextualMatch("Simple textual", SimpleTextualMatchBeliefSearchArguments.class),
		/**
		 * A set of various criteria which can be combined in many ways. Arguments structure: {@link AdvancedBeliefSearchArguments}.
		 */
		Advanced("Advanced", AdvancedBeliefSearchArguments.class);

		private BeliefSearchType(final String short_title, final Class<? extends BeliefSearchArguments> arguments_structure) {
			this.ShortTitle = short_title;
		}

		private final String ShortTitle;

		/**
		 * Returns the result of {@link #getShortTitle}.
		 * 
		 * @return {@link #getShortTitle getShortTitle();}
		 */
		@Override
		public String toString() {
			return this.getShortTitle();
		}

		/**
		 * <ul>
		 * <li>{@code SimpleTextualMatch -> "Simple Textual"}</li>
		 * </ul>
		 * 
		 * @return the short title for this matching method
		 */
		public String getShortTitle() {
			return this.ShortTitle;
		}
	}

	/**
	 * The base class for a belief search arguments structure. All argument structures are immutable once created.
	 */
	public static abstract class BeliefSearchArguments {
		/**
		 * <p>
		 * {@link BeliefSearchArguments}' {@link #toString} returns a string as:
		 * </p>
		 * 
		 * <pre>
		 * '[' [<em>Property</em>'='<em>Value</em> [, ...] ']'
		 * </pre>
		 * <p>
		 * where '[', ']' and '=' are literal characters, <em>Property</em> is the name of the property (which is the same as the public
		 * fields of the arguments structure) and <em>Value</em> is the string representation of the value.
		 * </p>
		 * 
		 * @return a string representation of this argument structure with all arguments and their values included
		 */
		@Override
		public String toString() {
			final Field[] my_fields = this.getClass().getFields();
			StringBuilder result = new StringBuilder("[");
			for (Field field : my_fields) {
				try {
					if (field.isAnnotationPresent(Argument.class)) {
						result.append(field.getName()).append("=\"").append(field.get(this)).append("\" ");
					}

					// in case of exception, just ignore the field
				} catch (IllegalArgumentException illegalArgumentException) {
				} catch (IllegalAccessException illegalAccessException) {
				}
			}
			return result.append("]").toString();
		}

		/**
		 * Indicated that a public field of this class is an argument.
		 */
		@Retention(RetentionPolicy.RUNTIME)
		@Target(ElementType.FIELD)
		public @interface Argument {
		}
	}

	/**
	 * <p>
	 * The arguments structure for the simple textual matching search method.
	 * </p>
	 * <p>
	 * This structure has only one field: the search string the user entered.
	 * </p>
	 */
	public class SimpleTextualMatchBeliefSearchArguments extends BeliefSearchArguments {
		/**
		 * The search string the user entered
		 */
		@Argument
		public final String SearchString;

		/**
		 * Initialises a new instance with the given {@code search_str}. The given {@code search_str} will be accessible through
		 * {@link #SearchString} public field.
		 * 
		 * @param search_str
		 *            the search string the user entered
		 */
		public SimpleTextualMatchBeliefSearchArguments(final String search_str) {
			this.SearchString = search_str;
		}
	}

	/**
	 * <p>
	 * The argument structure for the advanced belief search.
	 * </p>
	 */
	public class AdvancedBeliefSearchArguments extends BeliefSearchArguments {
		/** The subject strings to be searched for. */
		@Argument
		public final List<String> Subjects;
		/** The predicate strings to be searched for. */
		@Argument
		public final List<String> Predicates;
		/** The object strings to be searched for. */
		@Argument
		public final List<String> Objects;
		/** Make a case sensitive search for subject? */
		@Argument
		public final boolean SubjectCaseSensitive;
		/** Make a case sensitive search for predicate? */
		@Argument
		public final boolean PredicateCaseSensitive;
		/** Make a case sensitive search for object? */
		@Argument
		public final boolean ObjectCaseSensitive;
		/**
		 * In which way should the criteria be combined? Any wrong argument causes an {@link IllegalArgumentException} to be thrown.
		 * <ol>
		 * <li>subject AND predicate AND object</li>
		 * <li>subject AND predicate OR object</li>
		 * <li>subject AND object OR predicate</li>
		 * <li>subject OR predicate AND object</li>
		 * <li>subject OR predicate OR object</li>
		 * </ol>
		 */
		@Argument
		public final int Combination;

		/**
		 * Creates and initialised an instance of advanced search arguments with the given arguments.
		 * 
		 * @param subject_string
		 *            whitespace separated list of subjects for {@link #Subjects}
		 * @param predicate_string
		 *            whitespace separated list of predicates for {@link #Predicates}
		 * @param object_string
		 *            whitespace separated list of objects for {@link #Objects}
		 * @param subject_case_sensitive
		 *            for SubjectCaseSensitive
		 * @param predicate_case_sensitive
		 *            for PredicateCaseSensitive
		 * @param object_case_sensitive
		 *            for ObjectCaseSensitive
		 * @param combination
		 *            for Combination
		 * @throws java.lang.IllegalArgumentException
		 *             if {@code combination} is <1 or >5
		 */
		public AdvancedBeliefSearchArguments(final String subject_string, final String predicate_string, final String object_string,
				final boolean subject_case_sensitive, final boolean predicate_case_sensitive, final boolean object_case_sensitive,
				final int combination) throws IllegalArgumentException {
			// error checking
			if ((combination < 1) || (combination > 5)) {
				throw new IllegalArgumentException("combination=" + combination + " - not 1<combination<5");
			}
			this.Subjects = Arrays.asList(subject_string.split("\\s+"));
			this.Predicates = Arrays.asList(predicate_string.split("\\s+"));
			this.Objects = Arrays.asList(object_string.split("\\s+"));
			this.SubjectCaseSensitive = subject_case_sensitive;
			this.PredicateCaseSensitive = predicate_case_sensitive;
			this.ObjectCaseSensitive = object_case_sensitive;
			this.Combination = combination;
		}
	}

	/**
	 * Creates the appropriate label text to represent the given agent life cycles.
	 * 
	 * @param cycles
	 *            the agent life cycles
	 * @return a string representing the given agent life cycles
	 */
	public static String getAgentLifeCycleText(final long cycles) {
		String result;

		if (cycles == -1l) {
			result = "<html><i>(unknown yet)</i></html>";
		} else {
			result = Long.toString(cycles);
		}

		return result;
	}

	/**
	 * <p>
	 * Creates a an appropriate string representation to display in the label about which agent life cycle the visualizer is waiting for.
	 * </p>
	 * 
	 * <p>
	 * This does not set the actual text, it just returns the string that would actually be set as the label's text.
	 * </p>
	 * 
	 * @param cycle
	 *            the life cycle to generate text for
	 * @return a string representation for the life cycle being waited for
	 */
	public static String getWaitingForLifeCycleText(final long cycle) {
		if (cycle == -1) {
			return "(n/a)";
		}
		return Long.toString(cycle);
	}

	// Protected ---------------------------------------------------------------
	/**
	 * Shows the given message on some message area on the gui. {@code msg} does not need to include new lines and other such separation
	 * characters. In fact, it should not.
	 * 
	 * @param from
	 *            "who" is this message from?
	 * @param msg
	 *            the message text
	 */
	protected void showMessage(final String from, final String msg) {
		this.showMessage(from, msg, false);
	}

	/**
	 * <p>
	 * Shows the given message on some message area on the gui. {@code msg} does not need to include new lines and other such separation
	 * characters. In fact, it should not.
	 * </p>
	 * 
	 * <p>
	 * If {@code error} is true, then the message is displayed in a more observable, notable and direct way.
	 * </p>
	 * 
	 * @param from
	 *            "who" is this message from?
	 * @param msg
	 *            the message text
	 * @param error
	 *            is this message an error message?
	 */
	protected void showMessage(final String from, final String msg, final boolean error) {
		this.showMessage(from, msg, error ? JOptionPane.ERROR_MESSAGE : JOptionPane.PLAIN_MESSAGE);
	}

	/**
	 * <p>
	 * Shows the given message on some message area on the gui. {@code msg} does not need to include new lines and other such separation
	 * characters. In fact, it should not.
	 * </p>
	 * 
	 * <p>
	 * {@code message_type} is the same as the messag type for {@link JOptionPane#showMessageDialog(Component,Object,String,int)}. If it is
	 * anything else other than {@link JOptionPane#PLAIN_MESSAGE} then the message will be also displayed in a dialogue window of the
	 * corresponding level.
	 * </p>
	 * 
	 * @param from
	 *            "who" is this message from?
	 * @param msg
	 *            the message text
	 * @param message_type
	 *            is this message an error message?
	 */
	protected void showMessage(final String from, final String msg, final int message_type) {
		if (message_type != JOptionPane.PLAIN_MESSAGE) {
			JOptionPane.showMessageDialog(this.getContentPane(), msg, from, message_type);
		}
		if (this.isMessageFromShowable(from)) {
			this.message_area.append("<" + from + "> " + msg + "\n");
		}
	}

	/**
	 * <p>
	 * Returns {@code true} if messages from the given source are shown on the message area of the interface, {@code false} otherwise.
	 * </p>
	 * 
	 * <p>
	 * Messages are shown in the message area if they have been enabled by calling {@link #showMessage showMessage("sender", true)}.
	 * </p>
	 * 
	 * @param from
	 *            the source of messages
	 * @return {{@code true} if messages from the given source are shown on the interface, {@code false} otherwise.
	 */
	protected boolean isMessageFromShowable(final String from) {
		final Boolean result = this.messagesFromShowable.get(from);
		return (result != null) && result.booleanValue();
	}

	/**
	 * <p>
	 * Causes messages comming from {@code from} to be shown if argument {@code s} is {@code true}, otherwise not.
	 * </p>
	 * 
	 * @param from
	 *            the source to filtre
	 * @param s
	 *            show messages from the secified source or not?
	 */
	protected void setMessageFromShowable(final String from, final boolean s) {
		this.messagesFromShowable.put(from, Boolean.valueOf(s));
	}

	// Chunky changes in the gui ///////////////////////////////////////////////
	/**
	 * <p>
	 * Instructs {@code this} to show or hide the settings panel.
	 * </p>
	 * 
	 * @param show_settings
	 *            show settings?
	 */
	protected void showSettingsPanel(Boolean show_settings) {
		// add-remove panel
		if (show_settings) {
			this.add(this.settingsPanel);
		} else {
			this.remove(this.settingsPanel);
		}

		// repaint/redraw/validate (??) this container
		//		repaint();
		this.validate();
	}

	/**
	 * Rebuilds {@link #belief_tree} after setting the new {@code tree_model} as its new tree model.
	 * 
	 * @param tree_model
	 */
	protected final void rebuildBeliefTree(final TreeModel tree_model) {
		this.belief_tree.setModel(tree_model);
		this.belief_tree.repaint();
	}

	/**
	 * Disables all the buttons that are related to controlling the agent. Usually this happens because the user issued a "progress #cycles"
	 * command and has to wait for that many cycles to pass.
	 */
	protected void disableAgentProgressControls() {
		for (Component agentControl : this.agentProgressControls) {
			agentControl.setEnabled(false);
		}
	}

	/**
	 * Enables all the buttons that are related to controlling the agent. Usually this happens after the agent has reached the life cycle
	 * limit set by the user.
	 */
	protected void enableAgentProgressControls() {
		for (Component agentControl : this.agentProgressControls) {
			agentControl.setEnabled(true);
		}
	}

	/**
	 * Disables all labels that are related to displaying information about waiting for a life cycle.
	 */
	protected void disableWaitingForLifeCycleLabels() {
		for (Component label : this.waitingForLifeCycleLabels) {
			label.setEnabled(false);
		}
	}

	/**
	 * Enables all labels that are related to displaying information about waiting for a life cycle.
	 */
	protected void enableWaitingForLifeCycleLabels() {
		for (Component label : this.waitingForLifeCycleLabels) {
			label.setEnabled(true);
		}
	}

	/**
	 * Creates an internal frame which displays the given string.
	 * 
	 * @param n3
	 *            the string (supposedly in N3) to display
	 */
	protected void showN3(final String n3) {
		this.n3_text_area.setText(n3);
		this.gnome_tabs_pain.setSelectedComponent(this.n3_pane);
		this.gnome_frame.setTitle("N3 result for " + this.Agi.getName());
		this.gnome_frame.setSize(800, 400);
		this.gnome_frame.setVisible(true);
	}

	/**
	 * Makes the specific tree path displayed.
	 * 
	 * @param tp
	 *            the tree path to become displayed.
	 */
	protected void make_displayed(final TreePath tp) {
		this.belief_tree.scrollPathToVisible(tp);
		this.belief_tree.setSelectionPath(tp);
	}

	/**
	 * Displays the given array of treepaths in a window with title {@code ttl} and allows the user to select one of them. When the user
	 * does so, the selected path will be displayed on the belief tree (if there is such a path).
	 * 
	 * @param tps
	 *            the tree paths to list
	 * @param ttl
	 *            the title to be displayed for this array of treepaths
	 */
	protected void show_treepath_list(final TreePath[] tps, final String ttl) {
		this.searchResultsListModel.setCore(tps);
	}

	/**
	 * <p>
	 * Tries to give focus to the window indicated by the argument. The argument is an integer which indicates which of the windows should
	 * gain focus. The semantics of this argument are not final and can change between versions.
	 * </p>
	 * 
	 * <p>
	 * <ul>
	 * <li>0x0000 -> Main window</li>
	 * <li>0x0001 -> Assistive window (N3, search results, etc)</li>
	 * <li>0x0100 -> Search results window</li>
	 * </ul>
	 * </p>
	 * 
	 * @param window
	 *            one of the values specified above
	 * @throws java.lang.IllegalArgumentException
	 *             when argument {@code window} is not in one of the specified values above.
	 */
	protected void request_focus(final int window) throws IllegalArgumentException {
		switch (window) {
		case 0x0000:
			this.toFront();
			this.requestFocusInWindow();
			break;
		case 0x0001:
		case 0x0100:
			this.gnome_frame.toFront();
			this.gnome_frame.requestFocusInWindow();
			break;
		default:
			throw new IllegalArgumentException("window == " + window + " -- illegal value");
		}
	}

	////////////////////////////////////////////////////////////////////////////
	/**
	 * Sets this {@link #belief_tree}'s cell renderer. Does not repaint anything.
	 * 
	 * @param r
	 *            the new cell renderer for the belief tree
	 */
	protected final void setBeliefTreeCellRenderer(final TreeCellRenderer r) {
		this.belief_tree.setCellRenderer(r);
	}

	/**
	 * Calls {@link JTree#repaint repaint} on {@link #belief_tree}.
	 */
	protected final void repaintBeliefTree() {
		this.belief_tree.repaint();
	}

	/**
	 * Sets this {@link #search_results_list}'s cell renderer. Does not repaint anything.
	 * 
	 * @param r
	 *            the new cell renderer for the belief tree
	 */
	protected final void setSearcResultsListCellRenderer(final ListCellRenderer r) {
		this.search_results_list.setCellRenderer(r);
	}

	// Semantic Event Abstract Methods /////////////////////////////////////////
	/**
	 * Indicates that the user preference about aliasing namespaces has changed.
	 * 
	 * @param alias
	 *            alias namespaces?
	 */
	protected abstract void alias_namespaces_changed(final boolean alias);

	/**
	 * Indicates that the user preference about showing or hiding the settings panel has changed.
	 * 
	 * @param show
	 *            show settings panel?
	 */
	protected abstract void show_settings_panel_changed(final boolean show);

	/**
	 * Indicates that the user has changed her preference about waiting until a certain life cycle. If argument {@code wait} is
	 * {@code false}, then argument {@code lifeCycle} should be ignored. Parameter {@code cycles} is the number of cycles to wait for to
	 * pass and is <em>not</em> an absolute life cycle number.
	 * 
	 * @param wait
	 *            should we wait until a certain life cycle?
	 * @param cycles
	 *            the life cycles to wait for (if {@code wait} is {@code true})
	 */
	protected abstract void wait_until_agent_life_cycle_changed(final boolean wait, final int cycles);

	/**
	 * Indicates that the user has changed her preference about stopping the agent. If argument {@code stopped} is {@code true} then the
	 * user would like to stop the agent, resume it otherwise.
	 * 
	 * @param stopped
	 *            stop the agent? or resume?
	 */
	protected abstract void stop_agent_changed(final boolean stopped);

	/**
	 * Indicates that the user has changed her preference about showing expanded belief container nodes' text.
	 * 
	 * @param show
	 *            show them?
	 */
	protected abstract void show_expanded_belief_containers_text_changed(final boolean show);

	/**
	 * Indicates that the user has changed her preference about showing expanded belief container nodes' icon.
	 * 
	 * @param show
	 *            show them?
	 */
	protected abstract void show_expanded_belief_containers_icon_changed(final boolean show);

	/**
	 * Indicates that the user has changed her preferenece about empty belief container nodes to be expandable.
	 * 
	 * @param expandable
	 *            should they be expandable?
	 */
	protected abstract void expandable_belief_containers_changed(final boolean expandable);

	/**
	 * Called when the user has requested a specific node of the tree to be rendered in N3 notation.
	 * 
	 * @param node
	 *            the node to be rendered in N3
	 */
	protected abstract void show_in_N3_requested(final Object node);

	/**
	 * <p>
	 * Called when the user has requested a search on the belief tree to be made.
	 * </p>
	 * <p>
	 * The {@code args} argument is depended on the {@code search_type}. For more information about the specific arguments-structure for
	 * each search-type, refere to the {@link BeliefSearchType} documentation.
	 * </p>
	 * 
	 * @param args
	 *            the arguments the user has entered - this is search-type depended.
	 * @param search_type
	 *            what type of search is it, what does the given string signify
	 */
	protected abstract void search_requested(final BeliefSearchArguments args, final BeliefSearchType search_type);

	/**
	 * Called when the user has selected a specific tree path from the list of search results.
	 * 
	 * @param path
	 *            the path the user selected
	 */
	protected abstract void search_result_selected(final TreePath path);

	////////////////////////////////////////////////////////////////////////////

	// Privates ----------------------------------------------------------------

	// Methods called as result of property change events //////////////////////
	/**
	 * An indication that the aliasNamespaces property of this compomenet has changed. This method should make sure that everything is
	 * synchronized and consistent in displaying the new state.
	 * 
	 * @param alias_namespaces
	 *            aliasNamespaces property new value
	 */
	private void aliasNamespacesChanged(final Boolean alias_namespaces) {
		// synchronize check box
		this.alias_namespaces_menucheckbox.setSelected(alias_namespaces);
	}

	/**
	 * An indication that the showSettings property of this compomenet has changed. This method should make sure that everything is
	 * synchronized and consistent in displaying the new state.
	 * 
	 * @param alias_namespaces
	 *            aliasNamespaces property new value
	 */
	private void showSettingsChanged(final Boolean show_settings) {
		// synchronize menu check box
		this.show_settings_menucheckbox.setSelected(show_settings);
	}

	/**
	 * <p>
	 * An indication that the agentLifeCycles property of this compomenet has changed. This method will update the display of current agent
	 * life cycle on the gui.
	 * </p>
	 * 
	 * @param cycles
	 *            agentLifeCycles property new value
	 */
	private void agentLifeCyclesChanged(final long cycles) {
		this.agent_life_cycle_label.setText(VisualizerPrototypeForm.getAgentLifeCycleText(cycles));
	}

	/**
	 * <p>
	 * An indication that the waitUntilAgentLifeCycle property has changed. This method will update the display of which life cycle is being
	 * waiting for on the gui.
	 * </p>
	 * <
	 * 
	 * @param cycle
	 *            the cycle that is being waited for
	 */
	private void waitUntilAgentLifeCycleChanged(final long cycle) {
		this.waiting_for_agent_life_cycle_label.setText(VisualizerPrototypeForm.getWaitingForLifeCycleText(cycle));
	}

	/**
	 * <p>
	 * An indication that the waitingUntilAgentLifeCycle property has changed. This method will enable or disable displays related to
	 * information about waiting for a specific agent life cycle.
	 * </p>
	 * 
	 * @param wait
	 *            waiting for a cycle?
	 */
	private void waitingUntilAgentLifeCycleChanged(final boolean wait) {
		if (wait) {
			this.enableWaitingForLifeCycleLabels();
		} else {
			this.disableWaitingForLifeCycleLabels();
		}
	}

	/**
	 * <p>
	 * An indication that the expandableEmptyBeliefContainers property has changed. This method will set
	 * {@link #expandable_belief_containers_checkbox} to selected or not according to the argument passed.
	 * 
	 * @param expdbl
	 *            expandable?
	 */
	private void expandableEmptyBeliefContainersChanged(final boolean expdbl) {
		this.expandable_belief_containers_menucheckbox.setSelected(expdbl);
	}

	/**
	 * <p>
	 * An indication that the showExpandedBeliefContainerText property has changed. This method will set
	 * {@link #show_expanded_belief_containers_text_checkbox} to selected or not according to the argument passed.
	 * 
	 * @param show
	 *            show them?
	 */
	private void showExpandedBeliefContainerTextChanged(final boolean show) {
		this.show_expanded_belief_containers_text_menucheckbox.setSelected(show);
	}

	/**
	 * <p>
	 * An indication that the showExpandedBeliefContainerIcon property has changed. This method will set
	 * {@link #show_expanded_belief_containers_icon_checkbox} to selected or not according to the argument passed.
	 * 
	 * @param show
	 *            show them?
	 */
	private void showExpandedBeliefContainerIconChanged(final boolean show) {
		this.show_expanded_belief_containers_icon_menucheckbox.setSelected(show);
	}

	// Pre/Post init components tweakings //////////////////////////////////////
	private void initComponentsPre() {
		// custom content pane (for type safety)
		this.contentPane = new JPanel();
		this.setContentPane(this.contentPane);
	}

	private void initComponentsPost() {
		this.belief_tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		this.belief_search_criteria_prompt.pack();

		// keyboard actions
		this.initKeyboardActions();

		this.gnome_frame.pack();
	}

	/**
	 * Returns the color bor titled borders lines and text. It is based on the default foreground of some label.
	 */
	private Color getTitledBorderColour() {
		return this.titledBorderColour;
	}

	private void initKeyboardActions() {
		//http://java.sun.com/products/jfc/tsc/special_report/kestrel/keybindings.html

		// Ctrl-F globally takes to quick search
		final JComponent handler = this.contentPane;
		final int condition = JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT;
		handler.getInputMap(condition).put(this.quickSearchRequestedAction.getAcceleratorKey(), this.quickSearchRequestedAction.getName());
		handler.getActionMap().put(this.quickSearchRequestedAction.getName(), this.quickSearchRequestedAction);
	}

	////////////////////////////////////////////////////////////////////////////

	// Private internal events /////////////////////////////////////////////////
	/**
	 * <p>
	 * Internal event which signifies that a seach has begun. As part of this we must make some appropriate changes on the GUI so that user
	 * does not feel that it is not responding.
	 * </p>
	 * 
	 * @param search_string
	 *            the string that is searched for
	 */
	private void searchBegin(final String search_string) {
		final String searching_for = "Searching for \"" + search_string + '"';
		this.quick_search_textfield.setText(searching_for);
		this.quick_search_textfield.setEnabled(false);
		this.showMessage("quick searcher", searching_for);
	}

	/**
	 * <p>
	 * Internal event which signifies that a seach has ended. As part of this we must make some appropriate changes on the GUI so that user
	 * does not feel that it is not responding.
	 * </p>
	 * 
	 * @param search_string
	 *            the string that is searched for
	 */
	private void searchEnd(final String search_string) {
		this.quick_search_textfield.setEnabled(true);
		this.quick_search_textfield.setText(search_string);
		this.showMessage("quick searcher", "search is complete");
	}

	////////////////////////////////////////////////////////////////////////////

	/**
	 * Private implementation of the {@link ListModel} interface. It simply delegates everything to an internal instance of an array. It is
	 * supposedly more efficient than copying an entrire array into a {@link DefaultListModel}.
	 * 
	 * @author Nikos Mouchtaris
	 */
	private static class ArrayListModel implements ListModel {
		private Object[] core;
		private Collection<ListDataListener> listeners;

		public ArrayListModel() {
			this(new Object[0]);
		}

		public ArrayListModel(final Object[] core) {
			this.core = core;
			this.listeners = new HashSet<ListDataListener>();
		}

		public void setCore(final Object[] core) {
			this.core = core;
			this.notifyListeners();
		}

		public Object[] getCore() {
			return this.core;
		}

		@Override
		public int getSize() {
			return this.core.length;
		}

		@Override
		public Object getElementAt(int index) {
			return this.core[index];
		}

		@Override
		public void addListDataListener(ListDataListener l) {
			this.listeners.add(l);
		}

		@Override
		public void removeListDataListener(ListDataListener l) {
			this.listeners.remove(l);
		}

		private void notifyListeners() {
			final ListDataEvent e = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, this.core.length - 1);
			for (ListDataListener l : this.listeners) {
				l.contentsChanged(e);
			}
		}
	}

	/**
	 * An action related to showing and hiding the settings panel. It is accessed but at least two components, so there has to be one.
	 * 
	 * @author Nikos Mouchtaris
	 */
	private class ShowSettingsAction extends AbstractAction {
		//private static final String from = "settings_action";
		private static final long serialVersionUID = 12l;
		private final Logger logger;
		private final StringBuilder builder;
		private final String endl;

		public ShowSettingsAction(final Logger logger) {
			super("Show/hide settings panel");
			this.logger = logger;
			this.builder = new StringBuilder(300);
			this.endl = System.getProperty("line.separator", "\n");
			this.putValue(Action.SHORT_DESCRIPTION, "Shows or hides the settings panel");
			this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl pressed S"));
			this.putValue(Action.DISPLAYED_MNEMONIC_INDEX_KEY, Integer.valueOf(0));
			this.putValue(Action.SELECTED_KEY, Boolean.TRUE);
		}

		@Override
		public void actionPerformed(final ActionEvent e) {
			this.logEvent(e);
			VisualizerPrototypeForm.this.show_settings_panel_changed(Boolean.class.cast(this.getValue(Action.SELECTED_KEY)));
		}

		private void logEvent(final ActionEvent e) {
			this.builder.delete(0, this.builder.length());
			this.builder.append(e.toString()).append(this.endl);
			for (final Object o_key : this.getKeys()) {
				final String key = o_key.toString();
				final Object val = this.getValue(key);
				this.builder.append("    ").append(key).append(" = ").append(val).append('(').append(val.getClass()).append(')')
						.append(this.endl);
			}
			this.logger.fine(this.builder.toString());
		}
	}

	/**
	 * An action related to expanding or collapsing all the nodes of {@link belief_tree}. It is accessed but at least two components, so
	 * there has to be one.
	 * 
	 * @author Nikos Mouchtaris
	 */
	private class ExpandCollapseAllAction extends AbstractAction {
		private static final long serialVersionUID = 10l;
		private final Lambda<Void> Expand = new Lambda<Void>() {
			@Override
			public Void call(final Object... args) {
				// Args
				final TreePath path = TreePath.class.cast(args[0]);
				VisualizerPrototypeForm.this.belief_tree.expandPath(path);
				return null;
			}
		};
		private final Lambda<Void> Collapse = new Lambda<Void>() {
			@Override
			public Void call(final Object... args) {// Args
				final TreePath path = TreePath.class.cast(args[0]);
				VisualizerPrototypeForm.this.belief_tree.collapsePath(path);
				return null;
			}
		};

		@Override
		public void actionPerformed(final ActionEvent e) {
			final TreeModel tm = VisualizerPrototypeForm.this.belief_tree.getModel();
			final TreePath root = new TreePath(tm.getRoot());
			final String astr = e.getActionCommand();
			final Lambda<Void> action = (astr.contains("Collapse") || astr.contains("collapse")) ? this.Collapse
					: (astr.contains("Expand") || astr.contains("expand")) ? this.Expand : null;

			if (action != null) {
				VisualizerPrototypeForm.for_all_treepaths(tm, root, action);
			}
		}
	}

	/**
	 * <p>
	 * An action for the event when the user requests to make a quick search. <strong>Note:</strong> this is not when the user issues a
	 * quick search, but when she requests to make one. In other words, this action should transfer the focus, or pop up the appropriate
	 * windows or whatever in order for the user to be able to issue the search.
	 * </p>
	 * 
	 * <p>
	 * Currently it moves the focus to the quicksearch bar.
	 * </p>
	 */
	private class QuickSearchRequestedAction extends AbstractAction {
		private static final long serialVersionUID = 15l;
		private final KeyStroke acceleratorKey;
		private final String name;

		public QuickSearchRequestedAction() {
			super("Quick search");
			this.acceleratorKey = KeyStroke.getKeyStroke("control pressed F");
			this.name = "quick_search_request";
			this.putValue(Action.ACCELERATOR_KEY, this.acceleratorKey);
			this.putValue(Action.NAME, this.name);
		}

		public KeyStroke getAcceleratorKey() {
			return this.acceleratorKey;
		}

		public String getName() {
			return this.name;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			VisualizerPrototypeForm.this.quick_search_textfield.requestFocusInWindow();
		}
	}

	////////////////////////////////////////////////////////////////////////////

	// =========================================================================
	// || Everything below this are generated by NetBeans. Better not touch.  ||
	// =========================================================================
	// <editor-fold defaultstate="collapsed" desc="All Generated Code">

	/**
	 * This method is called from witohin the constructor to initialize the form. WARNING: Do NOT modify this code. The content of this
	 * method is always regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		this.belief_search_advanced_search_search_criteria_combinations_radiobuttons_buttongroup = new javax.swing.ButtonGroup();
		this.belief_tree_popup_menu = new javax.swing.JPopupMenu();
		this.show_in_N3_menu_item = new javax.swing.JMenuItem();
		this.belief_search_criteria_prompt = new javax.swing.JDialog();
		this.belief_search_criteria_prompt_message_label = new javax.swing.JLabel();
		this.belief_search_submit_button = new javax.swing.JButton();
		this.belief_search_criteria_pane = new javax.swing.JTabbedPane();
		this.belief_search_simple_textual_match_criterion_settings_tab = new javax.swing.JPanel();
		this.belief_search_simple_textual_match_criterion_settings_description_label = new javax.swing.JLabel();
		this.belief_search_simple_textual_match_criterion_settings = new javax.swing.JPanel();
		this.belief_search_simple_textual_match_criterion_message_label = new javax.swing.JLabel();
		this.belief_search_simple_textual_match_criterion_input_text_field = new javax.swing.JTextField();
		this.belief_search_advanced_search_criteria_settings_tab = new javax.swing.JPanel();
		this.belief_search_advanced_search_general_description_label = new javax.swing.JLabel();
		this.belief_search_advanced_search_criteria_spo_description_label = new javax.swing.JLabel();
		this.belief_search_advanced_search_spo_crieria_pane = new javax.swing.JPanel();
		this.belief_search_advnaced_search_subject_textfield = new javax.swing.JTextField();
		this.belief_search_advnaced_search_predicate_textfield = new javax.swing.JTextField();
		this.belief_search_advnaced_search_object_textfield = new javax.swing.JTextField();
		this.belief_search_advnaced_search_subject_label = new javax.swing.JLabel();
		this.belief_search_advnaced_search_predicate_label = new javax.swing.JLabel();
		this.belief_search_advnaced_search_object_label = new javax.swing.JLabel();
		this.belief_search_advanced_search_criteria_case_sensitivity_description_label = new javax.swing.JLabel();
		this.belief_search_advanced_search_case_sensitivity_criteria_pane = new javax.swing.JPanel();
		this.belief_search_advanced_search_subject_case_sensitivity_checkbox = new javax.swing.JCheckBox();
		this.belief_search_advanced_search_predicate_case_sensitivity_checkbox = new javax.swing.JCheckBox();
		this.belief_search_advanced_search_object_case_sensitivity_checkbox = new javax.swing.JCheckBox();
		this.belief_search_advanced_search_criteria_combinations_description_label = new javax.swing.JLabel();
		this.belief_search_advanced_search_criteria_combinations_pane = new javax.swing.JPanel();
		this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton = new javax.swing.JRadioButton();
		this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton = new javax.swing.JRadioButton();
		this.belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton = new javax.swing.JRadioButton();
		this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton = new javax.swing.JRadioButton();
		this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton = new javax.swing.JRadioButton();
		this.gnome_frame = new javax.swing.JFrame();
		this.gnome_tabs_pain = new javax.swing.JTabbedPane();
		this.n3_pane = new javax.swing.JPanel();
		this.n3_text_area_scroll_pane = new javax.swing.JScrollPane();
		this.n3_text_area = new javax.swing.JTextArea();
		this.belief_tree_pane = new javax.swing.JPanel();
		this.belief_tree_scroll_pane = new javax.swing.JScrollPane();
		this.belief_tree = new javax.swing.JTree();
		this.quick_search_pane = new javax.swing.JPanel();
		this.quick_search_label = new javax.swing.JLabel();
		this.quick_search_textfield = new javax.swing.JTextField();
		this.settingsPanel = new javax.swing.JPanel();
		this.grid_display_panel = new javax.swing.JPanel();
		this.agent_life_cycle_description_label = new javax.swing.JLabel();
		this.agent_life_cycle_label = new javax.swing.JLabel();
		this.waiting_for_agent_life_cycle_description_label = new javax.swing.JLabel();
		this.waiting_for_agent_life_cycle_label = new javax.swing.JLabel();
		this.progress_agent_button = new javax.swing.JButton();
		this.progress_agent_cycles_spinner = new javax.swing.JSpinner();
		this.stop_agent_button = new javax.swing.JButton();
		this.restart_agent_button = new javax.swing.JButton();
		this.search_results_label = new javax.swing.JLabel();
		this.search_results_pane = new javax.swing.JPanel();
		this.search_results_list_scroll_pane = new javax.swing.JScrollPane();
		this.search_results_list = new javax.swing.JList();
		this.message_area_label = new javax.swing.JLabel();
		this.message_area_scroll_pane = new javax.swing.JScrollPane();
		this.message_area = new javax.swing.JTextArea();
		this.menu_bar = new javax.swing.JMenuBar();
		this.view_menu = new javax.swing.JMenu();
		this.show_settings_menucheckbox = new javax.swing.JCheckBoxMenuItem();
		this.tools_menu = new javax.swing.JMenu();
		this.search_belief_menu_item = new javax.swing.JMenuItem();
		this.tools_menu_separator_1 = new javax.swing.JSeparator();
		this.expand_all_menu_item = new javax.swing.JMenuItem();
		this.collapse_all_menu_item = new javax.swing.JMenuItem();
		this.settings_menu = new javax.swing.JMenu();
		this.alias_namespaces_menucheckbox = new javax.swing.JCheckBoxMenuItem();
		this.show_expanded_belief_containers_text_menucheckbox = new javax.swing.JCheckBoxMenuItem();
		this.show_expanded_belief_containers_icon_menucheckbox = new javax.swing.JCheckBoxMenuItem();
		this.expandable_belief_containers_menucheckbox = new javax.swing.JCheckBoxMenuItem();

		this.belief_tree_popup_menu.setName("belief_tree_popup_menu"); // NOI18N

		this.show_in_N3_menu_item.setText("Show in N3 notation");
		this.show_in_N3_menu_item.setEnabled(false);
		this.show_in_N3_menu_item.setName("show_in_N3_menu_item"); // NOI18N
		this.show_in_N3_menu_item.addActionListener(this);
		this.belief_tree_popup_menu.add(this.show_in_N3_menu_item);

		this.belief_search_criteria_prompt.setTitle("Search criteria");
		this.belief_search_criteria_prompt.setName("belief_search_criteria_prompt"); // NOI18N

		this.belief_search_criteria_prompt_message_label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.belief_search_criteria_prompt_message_label.setText("Select and insert the belief search criteria below.");
		this.belief_search_criteria_prompt_message_label.setName("belief_search_criteria_prompt_message_label"); // NOI18N

		this.belief_search_submit_button.setText("Search...");
		this.belief_search_submit_button.setToolTipText("Perform a search with the selected criterion");
		this.belief_search_submit_button.setName("belief_search_submit_button"); // NOI18N
		this.belief_search_submit_button.addActionListener(this);

		this.belief_search_criteria_pane.setName("belief_search_criteria_pane"); // NOI18N

		this.belief_search_simple_textual_match_criterion_settings_tab.setName("belief_search_simple_textual_match_criterion_settings_tab"); // NOI18N

		this.belief_search_simple_textual_match_criterion_settings_description_label
				.setText("<html>A simple textual search will search for \nthe appearance of the given string in the textual\nrepresentation of each belief and belief container.\nPrefixes are fully supported, so one can search for \"sapl:I\", for example.\nWhitespace, letter case, punctuation and everything\nelse that appears in the search string is taken\nabsolutely literally and will match exactly what it is\non a belief textual representation.</html>");
		this.belief_search_simple_textual_match_criterion_settings_description_label
				.setName("belief_search_simple_textual_match_criterion_settings_description_label"); // NOI18N
		this.belief_search_simple_textual_match_criterion_settings_description_label.setPreferredSize(new java.awt.Dimension(200, 200));

		this.belief_search_simple_textual_match_criterion_settings.setName("belief_search_simple_textual_match_criterion_settings"); // NOI18N

		this.belief_search_simple_textual_match_criterion_message_label.setText("Search string:");
		this.belief_search_simple_textual_match_criterion_message_label
				.setToolTipText("Simple textual matching makes a simple search among all the beliefs, looking for any exact occurence of the specified string. The given string is not interepreted or parsed in any way, it is considered absolutely literal.");
		this.belief_search_simple_textual_match_criterion_message_label
				.setName("belief_search_simple_textual_match_criterion_message_label"); // NOI18N

		this.belief_search_simple_textual_match_criterion_input_text_field.setText(":popcorn gb:is :salty");
		this.belief_search_simple_textual_match_criterion_input_text_field
				.setName("belief_search_simple_textual_match_criterion_input_text_field"); // NOI18N

		javax.swing.GroupLayout belief_search_simple_textual_match_criterion_settingsLayout = new javax.swing.GroupLayout(
				this.belief_search_simple_textual_match_criterion_settings);
		this.belief_search_simple_textual_match_criterion_settings.setLayout(belief_search_simple_textual_match_criterion_settingsLayout);
		belief_search_simple_textual_match_criterion_settingsLayout
				.setHorizontalGroup(belief_search_simple_textual_match_criterion_settingsLayout.createParallelGroup(
						javax.swing.GroupLayout.Alignment.LEADING).addGroup(
						belief_search_simple_textual_match_criterion_settingsLayout
								.createSequentialGroup()
								.addContainerGap()
								.addComponent(this.belief_search_simple_textual_match_criterion_message_label)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.belief_search_simple_textual_match_criterion_input_text_field,
										javax.swing.GroupLayout.DEFAULT_SIZE, 513, Short.MAX_VALUE)));
		belief_search_simple_textual_match_criterion_settingsLayout
				.setVerticalGroup(belief_search_simple_textual_match_criterion_settingsLayout
						.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
						.addComponent(this.belief_search_simple_textual_match_criterion_input_text_field,
								javax.swing.GroupLayout.PREFERRED_SIZE, 30, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								belief_search_simple_textual_match_criterion_settingsLayout.createSequentialGroup().addContainerGap()
										.addComponent(this.belief_search_simple_textual_match_criterion_message_label).addContainerGap()));

		javax.swing.GroupLayout belief_search_simple_textual_match_criterion_settings_tabLayout = new javax.swing.GroupLayout(
				this.belief_search_simple_textual_match_criterion_settings_tab);
		this.belief_search_simple_textual_match_criterion_settings_tab
				.setLayout(belief_search_simple_textual_match_criterion_settings_tabLayout);
		belief_search_simple_textual_match_criterion_settings_tabLayout
				.setHorizontalGroup(belief_search_simple_textual_match_criterion_settings_tabLayout.createParallelGroup(
						javax.swing.GroupLayout.Alignment.LEADING).addGroup(
						belief_search_simple_textual_match_criterion_settings_tabLayout
								.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										belief_search_simple_textual_match_criterion_settings_tabLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(this.belief_search_simple_textual_match_criterion_settings_description_label,
														javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE,
														618, Short.MAX_VALUE)
												.addComponent(this.belief_search_simple_textual_match_criterion_settings,
														javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)).addContainerGap()));
		belief_search_simple_textual_match_criterion_settings_tabLayout
				.setVerticalGroup(belief_search_simple_textual_match_criterion_settings_tabLayout.createParallelGroup(
						javax.swing.GroupLayout.Alignment.LEADING).addGroup(
						javax.swing.GroupLayout.Alignment.TRAILING,
						belief_search_simple_textual_match_criterion_settings_tabLayout
								.createSequentialGroup()
								.addContainerGap()
								.addComponent(this.belief_search_simple_textual_match_criterion_settings_description_label,
										javax.swing.GroupLayout.DEFAULT_SIZE, 332, Short.MAX_VALUE)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.belief_search_simple_textual_match_criterion_settings,
										javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE).addContainerGap()));

		this.belief_search_criteria_pane.addTab("Simple textual", this.belief_search_simple_textual_match_criterion_settings_tab);

		this.belief_search_advanced_search_criteria_settings_tab.setName("belief_search_advanced_search_criteria_settings_tab"); // NOI18N

		this.belief_search_advanced_search_general_description_label
				.setText("<html> Advanced search combines preferneces about case sensitivity, textual matching in specific fields (subject, object, predicate) and the various combinations of those criteria. </html> ");
		this.belief_search_advanced_search_general_description_label.setName("belief_search_advanced_search_general_description_label"); // NOI18N

		this.belief_search_advanced_search_criteria_spo_description_label
				.setText("<html> <b>Subject, predicate, object:</b> <br> The text specified in those fields will be tested for occurance in the corresponding field (subject field of the triplet for subject field, etc). Multiple options for each field can be given by separating them with space.</html> ");
		this.belief_search_advanced_search_criteria_spo_description_label
				.setName("belief_search_advanced_search_criteria_spo_description_label"); // NOI18N

		this.belief_search_advanced_search_spo_crieria_pane.setName("belief_search_advanced_search_spo_crieria_pane"); // NOI18N

		this.belief_search_advnaced_search_subject_textfield.setName("belief_search_advnaced_search_subject_textfield"); // NOI18N

		this.belief_search_advnaced_search_predicate_textfield.setName("belief_search_advnaced_search_predicate_textfield"); // NOI18N

		this.belief_search_advnaced_search_object_textfield.setName("belief_search_advnaced_search_object_textfield"); // NOI18N

		this.belief_search_advnaced_search_subject_label.setText("Subject:");
		this.belief_search_advnaced_search_subject_label.setName("belief_search_advnaced_search_subject_label"); // NOI18N

		this.belief_search_advnaced_search_predicate_label.setText("Predicate:");
		this.belief_search_advnaced_search_predicate_label.setName("belief_search_advnaced_search_predicate_label"); // NOI18N

		this.belief_search_advnaced_search_object_label.setText("Object:");
		this.belief_search_advnaced_search_object_label.setName("belief_search_advnaced_search_object_label"); // NOI18N

		javax.swing.GroupLayout belief_search_advanced_search_spo_crieria_paneLayout = new javax.swing.GroupLayout(
				this.belief_search_advanced_search_spo_crieria_pane);
		this.belief_search_advanced_search_spo_crieria_pane.setLayout(belief_search_advanced_search_spo_crieria_paneLayout);
		belief_search_advanced_search_spo_crieria_paneLayout.setHorizontalGroup(belief_search_advanced_search_spo_crieria_paneLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(
						belief_search_advanced_search_spo_crieria_paneLayout
								.createSequentialGroup()
								.addGroup(
										belief_search_advanced_search_spo_crieria_paneLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(this.belief_search_advnaced_search_object_label)
												.addComponent(this.belief_search_advnaced_search_predicate_label)
												.addComponent(this.belief_search_advnaced_search_subject_label))
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(
										belief_search_advanced_search_spo_crieria_paneLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(this.belief_search_advnaced_search_subject_textfield,
														javax.swing.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)
												.addComponent(this.belief_search_advnaced_search_object_textfield,
														javax.swing.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)
												.addComponent(this.belief_search_advnaced_search_predicate_textfield,
														javax.swing.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)).addContainerGap()));
		belief_search_advanced_search_spo_crieria_paneLayout.setVerticalGroup(belief_search_advanced_search_spo_crieria_paneLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(
						belief_search_advanced_search_spo_crieria_paneLayout
								.createSequentialGroup()
								.addGroup(
										belief_search_advanced_search_spo_crieria_paneLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(this.belief_search_advnaced_search_subject_label)
												.addComponent(this.belief_search_advnaced_search_subject_textfield,
														javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(
										belief_search_advanced_search_spo_crieria_paneLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(this.belief_search_advnaced_search_predicate_label)
												.addComponent(this.belief_search_advnaced_search_predicate_textfield,
														javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(
										belief_search_advanced_search_spo_crieria_paneLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(this.belief_search_advnaced_search_object_label)
												.addComponent(this.belief_search_advnaced_search_object_textfield,
														javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE)).addContainerGap()));

		this.belief_search_advanced_search_criteria_case_sensitivity_description_label
				.setText("<html> <b>Case sensitivity:</b> <br> If checked, text occurence will be checked with strict letter case matching. </html>\n");
		this.belief_search_advanced_search_criteria_case_sensitivity_description_label
				.setName("belief_search_advanced_search_criteria_case_sensitivity_description_label"); // NOI18N

		this.belief_search_advanced_search_case_sensitivity_criteria_pane
				.setName("belief_search_advanced_search_case_sensitivity_criteria_pane"); // NOI18N

		this.belief_search_advanced_search_subject_case_sensitivity_checkbox.setText("<html><i>Subject</i> case sensitive</html>");
		this.belief_search_advanced_search_subject_case_sensitivity_checkbox
				.setName("belief_search_advanced_search_subject_case_sensitivity_checkbox"); // NOI18N

		this.belief_search_advanced_search_predicate_case_sensitivity_checkbox.setText("<html><i>Predicate</i> case sensitive</html>");
		this.belief_search_advanced_search_predicate_case_sensitivity_checkbox
				.setName("belief_search_advanced_search_predicate_case_sensitivity_checkbox"); // NOI18N

		this.belief_search_advanced_search_object_case_sensitivity_checkbox.setText("<html> <i>Object</i> case sensitive</html>");
		this.belief_search_advanced_search_object_case_sensitivity_checkbox
				.setName("belief_search_advanced_search_object_case_sensitivity_checkbox"); // NOI18N

		javax.swing.GroupLayout belief_search_advanced_search_case_sensitivity_criteria_paneLayout = new javax.swing.GroupLayout(
				this.belief_search_advanced_search_case_sensitivity_criteria_pane);
		this.belief_search_advanced_search_case_sensitivity_criteria_pane
				.setLayout(belief_search_advanced_search_case_sensitivity_criteria_paneLayout);
		belief_search_advanced_search_case_sensitivity_criteria_paneLayout
				.setHorizontalGroup(belief_search_advanced_search_case_sensitivity_criteria_paneLayout.createParallelGroup(
						javax.swing.GroupLayout.Alignment.LEADING).addGroup(
						belief_search_advanced_search_case_sensitivity_criteria_paneLayout
								.createSequentialGroup()
								.addGroup(
										belief_search_advanced_search_case_sensitivity_criteria_paneLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(this.belief_search_advanced_search_subject_case_sensitivity_checkbox)
												.addComponent(this.belief_search_advanced_search_predicate_case_sensitivity_checkbox)
												.addComponent(this.belief_search_advanced_search_object_case_sensitivity_checkbox))
								.addContainerGap(143, Short.MAX_VALUE)));
		belief_search_advanced_search_case_sensitivity_criteria_paneLayout
				.setVerticalGroup(belief_search_advanced_search_case_sensitivity_criteria_paneLayout.createParallelGroup(
						javax.swing.GroupLayout.Alignment.LEADING).addGroup(
						belief_search_advanced_search_case_sensitivity_criteria_paneLayout.createSequentialGroup()
								.addComponent(this.belief_search_advanced_search_subject_case_sensitivity_checkbox)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.belief_search_advanced_search_predicate_case_sensitivity_checkbox)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.belief_search_advanced_search_object_case_sensitivity_checkbox)));

		this.belief_search_advanced_search_criteria_combinations_description_label
				.setText("<html> <b>Combinations of criteria:</b> <br>  The criteria above (matching of subject, predicate or object, with or without case sensitivity) can be combinad logically in the five ways displayed here. Note that <i>AND</i> has a higher priority than <i>OR</i> in the combinations shown to the side.</html>\n");
		this.belief_search_advanced_search_criteria_combinations_description_label
				.setName("belief_search_advanced_search_criteria_combinations_description_label"); // NOI18N

		this.belief_search_advanced_search_criteria_combinations_pane.setName("belief_search_advanced_search_criteria_combinations_pane"); // NOI18N

		this.belief_search_advanced_search_search_criteria_combinations_radiobuttons_buttongroup
				.add(this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton);
		this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton
				.setText("subject AND predicate AND object");
		this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton
				.setName("belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton"); // NOI18N

		this.belief_search_advanced_search_search_criteria_combinations_radiobuttons_buttongroup
				.add(this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton);
		this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton
				.setText("subject AND predicate OR object");
		this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton
				.setName("belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton"); // NOI18N

		this.belief_search_advanced_search_search_criteria_combinations_radiobuttons_buttongroup
				.add(this.belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton);
		this.belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton
				.setText("subject AND object OR predicate");
		this.belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton
				.setName("belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton"); // NOI18N

		this.belief_search_advanced_search_search_criteria_combinations_radiobuttons_buttongroup
				.add(this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton);
		this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton
				.setText("subject OR predicate AND object");
		this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton
				.setName("belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton"); // NOI18N

		this.belief_search_advanced_search_search_criteria_combinations_radiobuttons_buttongroup
				.add(this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton);
		this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton
				.setText("subject OR predicate OR object");
		this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton
				.setName("belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton"); // NOI18N

		javax.swing.GroupLayout belief_search_advanced_search_criteria_combinations_paneLayout = new javax.swing.GroupLayout(
				this.belief_search_advanced_search_criteria_combinations_pane);
		this.belief_search_advanced_search_criteria_combinations_pane
				.setLayout(belief_search_advanced_search_criteria_combinations_paneLayout);
		belief_search_advanced_search_criteria_combinations_paneLayout
				.setHorizontalGroup(belief_search_advanced_search_criteria_combinations_paneLayout
						.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
						.addComponent(
								this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton)
						.addComponent(
								this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton)
						.addComponent(
								this.belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton)
						.addComponent(
								this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton)
						.addComponent(
								this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton));
		belief_search_advanced_search_criteria_combinations_paneLayout
				.setVerticalGroup(belief_search_advanced_search_criteria_combinations_paneLayout
						.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								belief_search_advanced_search_criteria_combinations_paneLayout
										.createSequentialGroup()
										.addComponent(
												this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton)
										.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton)
										.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton)
										.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton)
										.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton)));

		javax.swing.GroupLayout belief_search_advanced_search_criteria_settings_tabLayout = new javax.swing.GroupLayout(
				this.belief_search_advanced_search_criteria_settings_tab);
		this.belief_search_advanced_search_criteria_settings_tab.setLayout(belief_search_advanced_search_criteria_settings_tabLayout);
		belief_search_advanced_search_criteria_settings_tabLayout
				.setHorizontalGroup(belief_search_advanced_search_criteria_settings_tabLayout
						.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								belief_search_advanced_search_criteria_settings_tabLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												belief_search_advanced_search_criteria_settings_tabLayout
														.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(this.belief_search_advanced_search_general_description_label,
																javax.swing.GroupLayout.DEFAULT_SIZE, 618, Short.MAX_VALUE)
														.addGroup(
																belief_search_advanced_search_criteria_settings_tabLayout
																		.createSequentialGroup()
																		.addGroup(
																				belief_search_advanced_search_criteria_settings_tabLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING,
																								false)
																						.addComponent(
																								this.belief_search_advanced_search_criteria_spo_description_label,
																								javax.swing.GroupLayout.DEFAULT_SIZE, 277,
																								Short.MAX_VALUE)
																						.addComponent(
																								this.belief_search_advanced_search_criteria_case_sensitivity_description_label,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								271, javax.swing.GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								this.belief_search_advanced_search_criteria_combinations_description_label,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								302, javax.swing.GroupLayout.PREFERRED_SIZE))
																		.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				belief_search_advanced_search_criteria_settings_tabLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addComponent(
																								this.belief_search_advanced_search_case_sensitivity_criteria_pane,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)
																						.addComponent(
																								this.belief_search_advanced_search_spo_crieria_pane,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)
																						.addComponent(
																								this.belief_search_advanced_search_criteria_combinations_pane,
																								javax.swing.GroupLayout.DEFAULT_SIZE, 304,
																								Short.MAX_VALUE)))).addContainerGap()));
		belief_search_advanced_search_criteria_settings_tabLayout
				.setVerticalGroup(belief_search_advanced_search_criteria_settings_tabLayout.createParallelGroup(
						javax.swing.GroupLayout.Alignment.LEADING).addGroup(
						belief_search_advanced_search_criteria_settings_tabLayout
								.createSequentialGroup()
								.addContainerGap()
								.addComponent(this.belief_search_advanced_search_general_description_label)
								.addGap(18, 18, 18)
								.addGroup(
										belief_search_advanced_search_criteria_settings_tabLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(this.belief_search_advanced_search_criteria_spo_description_label)
												.addComponent(this.belief_search_advanced_search_spo_crieria_pane,
														javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE))
								.addGap(18, 18, 18)
								.addGroup(
										belief_search_advanced_search_criteria_settings_tabLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(
														this.belief_search_advanced_search_criteria_case_sensitivity_description_label)
												.addComponent(this.belief_search_advanced_search_case_sensitivity_criteria_pane,
														javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE))
								.addGap(18, 18, 18)
								.addGroup(
										belief_search_advanced_search_criteria_settings_tabLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(this.belief_search_advanced_search_criteria_combinations_pane,
														javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addComponent(this.belief_search_advanced_search_criteria_combinations_description_label))
								.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)));

		this.belief_search_criteria_pane.addTab("Advanced", this.belief_search_advanced_search_criteria_settings_tab);

		javax.swing.GroupLayout belief_search_criteria_promptLayout = new javax.swing.GroupLayout(
				this.belief_search_criteria_prompt.getContentPane());
		this.belief_search_criteria_prompt.getContentPane().setLayout(belief_search_criteria_promptLayout);
		belief_search_criteria_promptLayout.setHorizontalGroup(belief_search_criteria_promptLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				belief_search_criteria_promptLayout
						.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								belief_search_criteria_promptLayout
										.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
										.addComponent(this.belief_search_criteria_pane, javax.swing.GroupLayout.Alignment.TRAILING)
										.addComponent(this.belief_search_criteria_prompt_message_label,
												javax.swing.GroupLayout.DEFAULT_SIZE, 650, Short.MAX_VALUE)
										.addComponent(this.belief_search_submit_button, javax.swing.GroupLayout.Alignment.TRAILING,
												javax.swing.GroupLayout.DEFAULT_SIZE, 650, Short.MAX_VALUE)).addContainerGap()));
		belief_search_criteria_promptLayout.setVerticalGroup(belief_search_criteria_promptLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				belief_search_criteria_promptLayout.createSequentialGroup().addContainerGap()
						.addComponent(this.belief_search_criteria_prompt_message_label)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(this.belief_search_criteria_pane)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(this.belief_search_submit_button)
						.addContainerGap()));

		this.gnome_frame.setName("gnome_frame"); // NOI18N

		this.gnome_tabs_pain.setName("gnome_tabs_pain"); // NOI18N

		this.n3_pane.setName("n3_pane"); // NOI18N

		this.n3_text_area_scroll_pane.setName("n3_text_area_scroll_pane"); // NOI18N

		this.n3_text_area.setColumns(20);
		this.n3_text_area.setEditable(false);
		this.n3_text_area.setFont(new java.awt.Font("Monospaced", 0, 12));
		this.n3_text_area.setRows(5);
		this.n3_text_area.setName("n3_text_area"); // NOI18N
		this.n3_text_area_scroll_pane.setViewportView(this.n3_text_area);

		javax.swing.GroupLayout n3_paneLayout = new javax.swing.GroupLayout(this.n3_pane);
		this.n3_pane.setLayout(n3_paneLayout);
		n3_paneLayout.setHorizontalGroup(n3_paneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				this.n3_text_area_scroll_pane, javax.swing.GroupLayout.DEFAULT_SIZE, 27, Short.MAX_VALUE));
		n3_paneLayout.setVerticalGroup(n3_paneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				this.n3_text_area_scroll_pane, javax.swing.GroupLayout.DEFAULT_SIZE, 27, Short.MAX_VALUE));

		this.gnome_tabs_pain.addTab("N3", this.n3_pane);

		javax.swing.GroupLayout gnome_frameLayout = new javax.swing.GroupLayout(this.gnome_frame.getContentPane());
		this.gnome_frame.getContentPane().setLayout(gnome_frameLayout);
		gnome_frameLayout.setHorizontalGroup(gnome_frameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				this.gnome_tabs_pain, javax.swing.GroupLayout.DEFAULT_SIZE, 268, Short.MAX_VALUE));
		gnome_frameLayout.setVerticalGroup(gnome_frameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				this.gnome_tabs_pain, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 238,
				Short.MAX_VALUE));

		this.setDefaultCloseOperation(javax.swing.WindowConstants.HIDE_ON_CLOSE);
		this.setFont(new java.awt.Font("SansSerif", 0, 12));
		this.setLocationByPlatform(true);
		this.setName("mainForm"); // NOI18N
		this.addPropertyChangeListener(this);
		this.addKeyListener(this);
		this.getContentPane().setLayout(new javax.swing.BoxLayout(this.getContentPane(), javax.swing.BoxLayout.LINE_AXIS));

		this.belief_tree_pane.setName("belief_tree_pane"); // NOI18N

		this.belief_tree_scroll_pane.setName("belief_tree_scroll_pane"); // NOI18N

		this.belief_tree.setComponentPopupMenu(this.belief_tree_popup_menu);
		this.belief_tree.setFont(this.getFont());
		this.belief_tree.setName("belief_tree"); // NOI18N
		this.belief_tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		this.belief_tree.addTreeSelectionListener(this);
		this.belief_tree_scroll_pane.setViewportView(this.belief_tree);

		this.quick_search_pane.setMinimumSize(new java.awt.Dimension(68, 22));
		this.quick_search_pane.setName("quick_search_pane"); // NOI18N
		this.quick_search_pane.setPreferredSize(new java.awt.Dimension(68, 22));

		this.quick_search_label.setLabelFor(this.quick_search_textfield);
		this.quick_search_label.setText("Quick search:");
		this.quick_search_label.setName("quick_search_label"); // NOI18N

		this.quick_search_textfield.setText(":Lila :dreads :icecream");
		this.quick_search_textfield.setToolTipText("Performs a simple textual search with the given string");
		this.quick_search_textfield.setName("quick_search_textfield"); // NOI18N
		this.quick_search_textfield.addFocusListener(this);
		this.quick_search_textfield.addKeyListener(this);

		javax.swing.GroupLayout quick_search_paneLayout = new javax.swing.GroupLayout(this.quick_search_pane);
		this.quick_search_pane.setLayout(quick_search_paneLayout);
		quick_search_paneLayout.setHorizontalGroup(quick_search_paneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						quick_search_paneLayout.createSequentialGroup().addComponent(this.quick_search_label)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.quick_search_textfield, javax.swing.GroupLayout.DEFAULT_SIZE, 433, Short.MAX_VALUE)));
		quick_search_paneLayout.setVerticalGroup(quick_search_paneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						quick_search_paneLayout
								.createSequentialGroup()
								.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addGroup(
										quick_search_paneLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(this.quick_search_label)
												.addComponent(this.quick_search_textfield, javax.swing.GroupLayout.PREFERRED_SIZE,
														javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))));

		javax.swing.GroupLayout belief_tree_paneLayout = new javax.swing.GroupLayout(this.belief_tree_pane);
		this.belief_tree_pane.setLayout(belief_tree_paneLayout);
		belief_tree_paneLayout.setHorizontalGroup(belief_tree_paneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.quick_search_pane, javax.swing.GroupLayout.DEFAULT_SIZE, 523, Short.MAX_VALUE)
				.addComponent(this.belief_tree_scroll_pane, javax.swing.GroupLayout.DEFAULT_SIZE, 523, Short.MAX_VALUE));
		belief_tree_paneLayout.setVerticalGroup(belief_tree_paneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						javax.swing.GroupLayout.Alignment.TRAILING,
						belief_tree_paneLayout
								.createSequentialGroup()
								.addComponent(this.belief_tree_scroll_pane, javax.swing.GroupLayout.DEFAULT_SIZE, 324, Short.MAX_VALUE)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.quick_search_pane, javax.swing.GroupLayout.PREFERRED_SIZE, 34,
										javax.swing.GroupLayout.PREFERRED_SIZE).addContainerGap()));

		this.getContentPane().add(this.belief_tree_pane);

		this.settingsPanel.setName("settingsPanel"); // NOI18N

		this.grid_display_panel.setBorder(javax.swing.BorderFactory.createTitledBorder(
				new javax.swing.border.LineBorder(this.getTitledBorderColour(), 1, true), "Agent Controls",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font(
						"DejaVu Sans", 0, 11), this.getTitledBorderColour()));
		this.grid_display_panel.setName("grid_display_panel"); // NOI18N
		this.grid_display_panel.setLayout(new java.awt.GridLayout(4, 2, 5, 2));

		this.agent_life_cycle_description_label.setFont(this.getFont());
		this.agent_life_cycle_description_label.setText(this.agentLifeCyclesDescription);
		this.agent_life_cycle_description_label.setToolTipText("Displays the number of life cycles the agent has lived");
		this.agent_life_cycle_description_label.setName("agent_life_cycle_description_label"); // NOI18N
		this.grid_display_panel.add(this.agent_life_cycle_description_label);

		this.agent_life_cycle_label.setFont(this.getFont().deriveFont(Font.BOLD));
		this.agent_life_cycle_label.setToolTipText("Displays the number of life cycles the agent has lived");
		this.agent_life_cycle_label.setName("agent_life_cycle_label"); // NOI18N
		this.grid_display_panel.add(this.agent_life_cycle_label);

		this.waiting_for_agent_life_cycle_description_label.setFont(this.getFont());
		this.waiting_for_agent_life_cycle_description_label.setText(this.waitUntilAgentLifeCycleDescription);
		this.waiting_for_agent_life_cycle_description_label
				.setToolTipText("Waiting for the agent to complete his life cycles up to this number");
		this.waiting_for_agent_life_cycle_description_label.setName("waiting_for_agent_life_cycle_description_label"); // NOI18N
		this.grid_display_panel.add(this.waiting_for_agent_life_cycle_description_label);

		this.waiting_for_agent_life_cycle_label.setFont(this.getFont().deriveFont(Font.BOLD));
		this.waiting_for_agent_life_cycle_label.setToolTipText("Waiting for the agent to complete his life cycles up to this number");
		this.waiting_for_agent_life_cycle_label.setName("waiting_for_agent_life_cycle_label"); // NOI18N
		this.grid_display_panel.add(this.waiting_for_agent_life_cycle_label);

		this.progress_agent_button.setMnemonic('p');
		this.progress_agent_button.setText("Progress Agent");
		this.progress_agent_button
				.setToolTipText("<html>Allow the agent to progress <i>N</i> steps, where <i>N</i> is the number specified next to this button</html>");
		this.progress_agent_button.setName("progress_agent_button"); // NOI18N
		this.progress_agent_button.addActionListener(this);
		this.grid_display_panel.add(this.progress_agent_button);

		this.progress_agent_cycles_spinner.setModel(this.progress_agent_cycles_spinner_model);
		this.progress_agent_cycles_spinner.setToolTipText("Specify the number of life cycles to allow the agent to advance");
		this.progress_agent_cycles_spinner.setName("progress_agent_cycles_spinner"); // NOI18N
		this.grid_display_panel.add(this.progress_agent_cycles_spinner);

		this.stop_agent_button.setMnemonic('t');
		this.stop_agent_button.setText("Stop Agent");
		this.stop_agent_button
				.setToolTipText("<html>Stop the agent; will not execute any life cycles unless started again or by step <u>p</u>rogressing</html>");
		this.stop_agent_button.setName("stop_agent_button"); // NOI18N
		this.stop_agent_button.addActionListener(this);
		this.grid_display_panel.add(this.stop_agent_button);

		this.restart_agent_button.setMnemonic('r');
		this.restart_agent_button.setText("Restart Agent");
		this.restart_agent_button.setToolTipText("Restarts the agent, that is, it will keep executing life cycles without any constraint.");
		this.restart_agent_button.setName("restart_agent_button"); // NOI18N
		this.restart_agent_button.addActionListener(this);
		this.grid_display_panel.add(this.restart_agent_button);

		this.search_results_label.setText("Search Results:");
		this.search_results_label.setName("search_results_label"); // NOI18N

		this.search_results_pane.setName("search_results_pane"); // NOI18N

		this.search_results_list_scroll_pane.setName("search_results_list_scroll_pane"); // NOI18N

		this.search_results_list.setModel(this.searchResultsListModel);
		this.search_results_list.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		this.search_results_list.setToolTipText("Select a result to see it on the beliefs tree");
		this.search_results_list.setName("search_results_list"); // NOI18N
		this.search_results_list.addListSelectionListener(this);
		this.search_results_list_scroll_pane.setViewportView(this.search_results_list);

		javax.swing.GroupLayout search_results_paneLayout = new javax.swing.GroupLayout(this.search_results_pane);
		this.search_results_pane.setLayout(search_results_paneLayout);
		search_results_paneLayout.setHorizontalGroup(search_results_paneLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(this.search_results_list_scroll_pane,
				javax.swing.GroupLayout.DEFAULT_SIZE, 262, Short.MAX_VALUE));
		search_results_paneLayout.setVerticalGroup(search_results_paneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.search_results_list_scroll_pane, javax.swing.GroupLayout.DEFAULT_SIZE, 165, Short.MAX_VALUE));

		this.message_area_label.setText("Message Area:");
		this.message_area_label.setName("message_area_label"); // NOI18N

		this.message_area_scroll_pane.setName("message_area_scroll_pane"); // NOI18N

		this.message_area.setColumns(20);
		this.message_area.setFont(new java.awt.Font("Monospaced", 0, 10));
		this.message_area.setRows(5);
		this.message_area.setMinimumSize(new java.awt.Dimension(80, 50));
		this.message_area.setName("message_area"); // NOI18N
		this.message_area_scroll_pane.setViewportView(this.message_area);

		javax.swing.GroupLayout settingsPanelLayout = new javax.swing.GroupLayout(this.settingsPanel);
		this.settingsPanel.setLayout(settingsPanelLayout);
		settingsPanelLayout.setHorizontalGroup(settingsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						javax.swing.GroupLayout.Alignment.TRAILING,
						settingsPanelLayout
								.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										settingsPanelLayout
												.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
												.addComponent(this.message_area_scroll_pane, javax.swing.GroupLayout.Alignment.LEADING,
														javax.swing.GroupLayout.DEFAULT_SIZE, 262, Short.MAX_VALUE)
												.addComponent(this.search_results_pane, javax.swing.GroupLayout.Alignment.LEADING,
														javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
														Short.MAX_VALUE)
												.addComponent(this.grid_display_panel, javax.swing.GroupLayout.DEFAULT_SIZE, 262,
														Short.MAX_VALUE)
												.addComponent(this.search_results_label, javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(this.message_area_label, javax.swing.GroupLayout.Alignment.LEADING))
								.addContainerGap()));
		settingsPanelLayout.setVerticalGroup(settingsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				settingsPanelLayout
						.createSequentialGroup()
						.addComponent(this.grid_display_panel, javax.swing.GroupLayout.PREFERRED_SIZE,
								javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addComponent(this.search_results_label)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(this.search_results_pane, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE)
						.addGap(18, 18, 18)
						.addComponent(this.message_area_label)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(this.message_area_scroll_pane, javax.swing.GroupLayout.PREFERRED_SIZE,
								javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE).addContainerGap()));

		this.getContentPane().add(this.settingsPanel);

		this.menu_bar.setName("menu_bar"); // NOI18N

		this.view_menu.setMnemonic('v');
		this.view_menu.setText("View");
		this.view_menu.setName("view_menu"); // NOI18N

		this.show_settings_menucheckbox.setAction(this.showSettingsAction);
		this.show_settings_menucheckbox.setName("show_settings_menucheckbox"); // NOI18N
		this.view_menu.add(this.show_settings_menucheckbox);

		this.menu_bar.add(this.view_menu);

		this.tools_menu.setMnemonic('o');
		this.tools_menu.setText("Tools");
		this.tools_menu.setToolTipText("Tools for manipulating or finding beliefs");
		this.tools_menu.setName("tools_menu"); // NOI18N

		this.search_belief_menu_item.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_B,
				java.awt.event.InputEvent.CTRL_MASK));
		this.search_belief_menu_item.setMnemonic('b');
		this.search_belief_menu_item.setText("Search for belief...");
		this.search_belief_menu_item.setToolTipText("Perform a search for a specific belief according to various criteria");
		this.search_belief_menu_item.setName("search_belief_menu_item"); // NOI18N
		this.search_belief_menu_item.addActionListener(this);
		this.tools_menu.add(this.search_belief_menu_item);

		this.tools_menu_separator_1.setName("tools_menu_separator_1"); // NOI18N
		this.tools_menu.add(this.tools_menu_separator_1);

		this.expand_all_menu_item.setAction(this.expandCollapseAllAction);
		this.expand_all_menu_item.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E,
				java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
		this.expand_all_menu_item.setMnemonic('e');
		this.expand_all_menu_item.setText("Expand all beliefs");
		this.expand_all_menu_item.setName("expand_all_menu_item"); // NOI18N
		this.tools_menu.add(this.expand_all_menu_item);

		this.collapse_all_menu_item.setAction(this.expandCollapseAllAction);
		this.collapse_all_menu_item.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C,
				java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
		this.collapse_all_menu_item.setMnemonic('c');
		this.collapse_all_menu_item.setText("Collapse all beliefs");
		this.collapse_all_menu_item.setName("collapse_all_menu_item"); // NOI18N
		this.tools_menu.add(this.collapse_all_menu_item);

		this.menu_bar.add(this.tools_menu);

		this.settings_menu.setMnemonic('s');
		this.settings_menu.setText("Settings");
		this.settings_menu.setName("settings_menu"); // NOI18N

		this.alias_namespaces_menucheckbox.setMnemonic('n');
		this.alias_namespaces_menucheckbox.setText("Alias namespaces");
		this.alias_namespaces_menucheckbox
				.setToolTipText("When enabled: namespaces are not displayed complete but their short aliases are displayed instead");
		this.alias_namespaces_menucheckbox.setName("alias_namespaces_menucheckbox"); // NOI18N
		this.alias_namespaces_menucheckbox.addItemListener(this);
		this.settings_menu.add(this.alias_namespaces_menucheckbox);

		this.show_expanded_belief_containers_text_menucheckbox.setMnemonic('x');
		this.show_expanded_belief_containers_text_menucheckbox.setText("Show expanded belief containers text");
		this.show_expanded_belief_containers_text_menucheckbox
				.setToolTipText("<html>When this checkbox is checked, belief container node which are expanded will <em>still</em> display text</html>");
		this.show_expanded_belief_containers_text_menucheckbox.setName("show_expanded_belief_containers_text_menucheckbox"); // NOI18N
		this.show_expanded_belief_containers_text_menucheckbox.addItemListener(this);
		this.settings_menu.add(this.show_expanded_belief_containers_text_menucheckbox);

		this.show_expanded_belief_containers_icon_menucheckbox.setMnemonic('c');
		this.show_expanded_belief_containers_icon_menucheckbox.setText("Show expanded belief containers icons");
		this.show_expanded_belief_containers_icon_menucheckbox
				.setToolTipText("<html>When this checkbox is checked, belief container node which are expanded will <em>still</em> display icons</html>");
		this.show_expanded_belief_containers_icon_menucheckbox.setName("show_expanded_belief_containers_icon_menucheckbox"); // NOI18N
		this.show_expanded_belief_containers_icon_menucheckbox.addItemListener(this);
		this.settings_menu.add(this.show_expanded_belief_containers_icon_menucheckbox);

		this.expandable_belief_containers_menucheckbox.setMnemonic('e');
		this.expandable_belief_containers_menucheckbox.setText("Expandable empty belief containers");
		this.expandable_belief_containers_menucheckbox
				.setToolTipText("<html>When this checkbox is checked, belief containers which contain 0 beliefs will still be expandable as trees</html>");
		this.expandable_belief_containers_menucheckbox.setEnabled(false);
		this.expandable_belief_containers_menucheckbox.setName("expandable_belief_containers_menucheckbox"); // NOI18N
		this.expandable_belief_containers_menucheckbox.addItemListener(this);
		this.settings_menu.add(this.expandable_belief_containers_menucheckbox);

		this.menu_bar.add(this.settings_menu);

		this.setJMenuBar(this.menu_bar);

		this.pack();
	}

	// Code for dispatching events from components to event handlers.

	@Override
	public void actionPerformed(java.awt.event.ActionEvent evt) {
		if (evt.getSource() == this.progress_agent_button) {
			VisualizerPrototypeForm.this.progress_agent_buttonActionPerformed(evt);
		} else if (evt.getSource() == this.stop_agent_button) {
			VisualizerPrototypeForm.this.stop_agent_buttonActionPerformed(evt);
		} else if (evt.getSource() == this.restart_agent_button) {
			VisualizerPrototypeForm.this.restart_agent_buttonActionPerformed(evt);
		} else if (evt.getSource() == this.show_in_N3_menu_item) {
			VisualizerPrototypeForm.this.show_in_N3_menu_itemActionPerformed(evt);
		} else if (evt.getSource() == this.belief_search_submit_button) {
			VisualizerPrototypeForm.this.belief_search_submit_buttonActionPerformed(evt);
		} else if (evt.getSource() == this.search_belief_menu_item) {
			VisualizerPrototypeForm.this.search_belief_menu_itemActionPerformed(evt);
		}
	}

	@Override
	public void focusGained(java.awt.event.FocusEvent evt) {
		if (evt.getSource() == this.quick_search_textfield) {
			VisualizerPrototypeForm.this.quick_search_textfieldFocusGained(evt);
		}
	}

	@Override
	public void focusLost(java.awt.event.FocusEvent evt) {
	}

	@Override
	public void itemStateChanged(java.awt.event.ItemEvent evt) {
		if (evt.getSource() == this.alias_namespaces_menucheckbox) {
			VisualizerPrototypeForm.this.alias_namespaces_menucheckboxItemStateChanged(evt);
		} else if (evt.getSource() == this.show_expanded_belief_containers_text_menucheckbox) {
			VisualizerPrototypeForm.this.show_expanded_belief_containers_text_menucheckboxItemStateChanged(evt);
		} else if (evt.getSource() == this.show_expanded_belief_containers_icon_menucheckbox) {
			VisualizerPrototypeForm.this.show_expanded_belief_containers_icon_menucheckboxItemStateChanged(evt);
		} else if (evt.getSource() == this.expandable_belief_containers_menucheckbox) {
			VisualizerPrototypeForm.this.expandable_belief_containers_menucheckboxItemStateChanged(evt);
		}
	}

	@Override
	public void keyPressed(java.awt.event.KeyEvent evt) {
		if (evt.getSource() == VisualizerPrototypeForm.this) {
			VisualizerPrototypeForm.this.formKeyPressed(evt);
		}
	}

	@Override
	public void keyReleased(java.awt.event.KeyEvent evt) {
	}

	@Override
	public void keyTyped(java.awt.event.KeyEvent evt) {
		if (evt.getSource() == this.quick_search_textfield) {
			VisualizerPrototypeForm.this.quick_search_textfieldKeyTyped(evt);
		}
	}

	@Override
	public void propertyChange(java.beans.PropertyChangeEvent evt) {
		if (evt.getSource() == VisualizerPrototypeForm.this) {
			VisualizerPrototypeForm.this.formPropertyChange(evt);
		}
	}

	@Override
	public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
		if (evt.getSource() == this.search_results_list) {
			VisualizerPrototypeForm.this.search_results_listValueChanged(evt);
		}
	}

	@Override
	public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
		if (evt.getSource() == this.belief_tree) {
			VisualizerPrototypeForm.this.belief_treeValueChanged(evt);
		}
	}// </editor-fold>//GEN-END:initComponents

	private void progress_agent_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_progress_agent_buttonActionPerformed
		try {
			this.progress_agent_cycles_spinner.commitEdit();
			final int cycles = this.progress_agent_cycles_spinner_model.getNumber().intValue();
			//			Logger.entering(
			//					getClass().getName(),
			//					"progress_agent_buttonActionPerformed",
			//					new Object[]{cycles, waitUntil});
			this.wait_until_agent_life_cycle_changed(true, cycles);
			//			Logger.exiting(getClass().getName(),
			//					"progress_agent_buttonActionPerformed");
		} catch (ParseException ex) {
			this.Logger.log(Level.SEVERE, null, ex);
		}
	}//GEN-LAST:event_progress_agent_buttonActionPerformed

	private void stop_agent_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stop_agent_buttonActionPerformed
		this.stop_agent_changed(true);
	}//GEN-LAST:event_stop_agent_buttonActionPerformed

	private void restart_agent_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_restart_agent_buttonActionPerformed
		this.stop_agent_changed(false);
	}//GEN-LAST:event_restart_agent_buttonActionPerformed

	private void formPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_formPropertyChange
		final String pname = evt.getPropertyName();
		final Object nval = evt.getNewValue();
		final Object oval = evt.getOldValue();
		this.showMessage("form", pname + " = " + nval + "\n");
		if (pname.equals("aliasNamespaces")) {
			this.aliasNamespacesChanged(Boolean.class.cast(nval));
		} else if (pname.equals("showSettings")) {
			this.showSettingsChanged(Boolean.class.cast(nval));
		} else if (pname.equals("agentLifeCycles")) {
			this.agentLifeCyclesChanged(Long.class.cast(nval));
		} else if (pname.equals("waitUntilAgentLifeCycle")) {
			this.waitUntilAgentLifeCycleChanged(Long.class.cast(nval));
		} else if (pname.equals("waitingUntilAgentLifeCycle")) {
			this.waitingUntilAgentLifeCycleChanged(Boolean.class.cast(nval));
		} else if (pname.equals("expandableEmptyBeliefContainers")) {
			this.expandableEmptyBeliefContainersChanged(Boolean.class.cast(nval));
		} else if (pname.equals("showExpandedBeliefContainerText")) {
			this.showExpandedBeliefContainerTextChanged(Boolean.class.cast(nval));
		} else if (pname.equals("showExpandedBeliefContainerIcon")) {
			this.showExpandedBeliefContainerIconChanged(Boolean.class.cast(nval));
		}
	}//GEN-LAST:event_formPropertyChange

	private void belief_treeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_belief_treeValueChanged
		this.show_in_N3_menu_item.setEnabled(evt.getNewLeadSelectionPath() != null);
	}//GEN-LAST:event_belief_treeValueChanged

	private void show_in_N3_menu_itemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_show_in_N3_menu_itemActionPerformed
		this.show_in_N3_requested(this.belief_tree.getSelectionPath().getLastPathComponent());
	}//GEN-LAST:event_show_in_N3_menu_itemActionPerformed

	private void search_belief_menu_itemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_search_belief_menu_itemActionPerformed
		// Show the string search input dialog
		this.belief_search_criteria_prompt.setVisible(true);
	}//GEN-LAST:event_search_belief_menu_itemActionPerformed

	private void belief_search_submit_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_belief_search_submit_buttonActionPerformed
		final int index = this.belief_search_criteria_pane.getSelectedIndex();
		final BeliefSearchType type = BeliefSearchType.values()[index];
		BeliefSearchArguments args = null;
		switch (index) {
		case 0: // SimpleTextual
			final String msg = this.belief_search_simple_textual_match_criterion_input_text_field.getText();
			args = new SimpleTextualMatchBeliefSearchArguments(msg);
			break;
		case 1: // Advacned
			args = new AdvancedBeliefSearchArguments(
					this.belief_search_advnaced_search_subject_textfield.getText(),
					this.belief_search_advnaced_search_predicate_textfield.getText(),
					this.belief_search_advnaced_search_object_textfield.getText(),
					this.belief_search_advanced_search_subject_case_sensitivity_checkbox.isSelected(),
					this.belief_search_advanced_search_predicate_case_sensitivity_checkbox.isSelected(),
					this.belief_search_advanced_search_object_case_sensitivity_checkbox.isSelected(),
					this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton
							.isSelected() ? 1
							: this.belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton
									.isSelected() ? 2
									: this.belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton
											.isSelected() ? 3
											: this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton
													.isSelected() ? 4
													: this.belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton
															.isSelected() ? 5 : 0);
			break;
		default:
			throw new AssertionError("Illegal program state");
		}
		this.SemanticEventLogger.fine("search_requested[args => \"" + args + "\", " + "type => " + type.name());
		this.search_requested(args, type);
	}//GEN-LAST:event_belief_search_submit_buttonActionPerformed

	private void search_results_listValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_search_results_listValueChanged
		final TreePath selection = TreePath.class.cast(this.search_results_list.getSelectedValue());
		// some search result is selected - post semantic event
		this.SemanticEventLogger.fine("search_result_selected( " + selection + ')');
		this.search_result_selected(selection);
	}//GEN-LAST:event_search_results_listValueChanged

	private void alias_namespaces_menucheckboxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_alias_namespaces_menucheckboxItemStateChanged
		this.alias_namespaces_changed(evt.getStateChange() == ItemEvent.SELECTED);
	}//GEN-LAST:event_alias_namespaces_menucheckboxItemStateChanged

	private void show_expanded_belief_containers_text_menucheckboxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_show_expanded_belief_containers_text_menucheckboxItemStateChanged
		this.show_expanded_belief_containers_text_changed(evt.getStateChange() == ItemEvent.SELECTED);
	}//GEN-LAST:event_show_expanded_belief_containers_text_menucheckboxItemStateChanged

	private void show_expanded_belief_containers_icon_menucheckboxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_show_expanded_belief_containers_icon_menucheckboxItemStateChanged
		this.show_expanded_belief_containers_icon_changed(evt.getStateChange() == ItemEvent.SELECTED);
	}//GEN-LAST:event_show_expanded_belief_containers_icon_menucheckboxItemStateChanged

	private void expandable_belief_containers_menucheckboxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_expandable_belief_containers_menucheckboxItemStateChanged
		this.expandable_belief_containers_changed(evt.getStateChange() == ItemEvent.SELECTED);
	}//GEN-LAST:event_expandable_belief_containers_menucheckboxItemStateChanged

	private void quick_search_textfieldKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_quick_search_textfieldKeyTyped
		// if key is enter, issue search
		if (evt.getKeyChar() == '\n') {
			final String search_text = this.quick_search_textfield.getText();
			this.searchBegin(search_text);
			javax.swing.SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					VisualizerPrototypeForm.this.search_requested(new SimpleTextualMatchBeliefSearchArguments(search_text),
							BeliefSearchType.SimpleTextualMatch);
					VisualizerPrototypeForm.this.searchEnd(search_text);
				}
			});
		}
	}//GEN-LAST:event_quick_search_textfieldKeyTyped

	private void quick_search_textfieldFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_quick_search_textfieldFocusGained
		if (!evt.isTemporary()) {
			this.quick_search_textfield.setSelectionStart(0);
			this.quick_search_textfield.setSelectionEnd(this.quick_search_textfield.getText().length());
		}
	}//GEN-LAST:event_quick_search_textfieldFocusGained

	private void formKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyPressed
		this.showMessage("quick searcher", "key: " + evt.getKeyChar());
		if (evt.getKeyChar() == 'f') {
			this.quick_search_textfield.requestFocusInWindow();
		}
	}//GEN-LAST:event_formKeyPressed

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JLabel agent_life_cycle_description_label;
	private javax.swing.JLabel agent_life_cycle_label;
	private javax.swing.JCheckBoxMenuItem alias_namespaces_menucheckbox;
	private javax.swing.JPanel belief_search_advanced_search_case_sensitivity_criteria_pane;
	private javax.swing.JLabel belief_search_advanced_search_criteria_case_sensitivity_description_label;
	private javax.swing.JLabel belief_search_advanced_search_criteria_combinations_description_label;
	private javax.swing.JPanel belief_search_advanced_search_criteria_combinations_pane;
	private javax.swing.JPanel belief_search_advanced_search_criteria_settings_tab;
	private javax.swing.JLabel belief_search_advanced_search_criteria_spo_description_label;
	private javax.swing.JRadioButton belief_search_advanced_search_critreia_combination_subject_AND_object_OR_predicate_combination_radiobutton;
	private javax.swing.JRadioButton belief_search_advanced_search_critreia_combination_subject_AND_predicate_AND_object_combination_radiobutton;
	private javax.swing.JRadioButton belief_search_advanced_search_critreia_combination_subject_AND_predicate_OR_object_combination_radiobutton;
	private javax.swing.JRadioButton belief_search_advanced_search_critreia_combination_subject_OR_predicate_AND_object_combination_radiobutton;
	private javax.swing.JRadioButton belief_search_advanced_search_critreia_combination_subject_OR_predicate_OR_object_combination_radiobutton;
	private javax.swing.JLabel belief_search_advanced_search_general_description_label;
	private javax.swing.JCheckBox belief_search_advanced_search_object_case_sensitivity_checkbox;
	private javax.swing.JCheckBox belief_search_advanced_search_predicate_case_sensitivity_checkbox;
	private javax.swing.ButtonGroup belief_search_advanced_search_search_criteria_combinations_radiobuttons_buttongroup;
	private javax.swing.JPanel belief_search_advanced_search_spo_crieria_pane;
	private javax.swing.JCheckBox belief_search_advanced_search_subject_case_sensitivity_checkbox;
	private javax.swing.JLabel belief_search_advnaced_search_object_label;
	private javax.swing.JTextField belief_search_advnaced_search_object_textfield;
	private javax.swing.JLabel belief_search_advnaced_search_predicate_label;
	private javax.swing.JTextField belief_search_advnaced_search_predicate_textfield;
	private javax.swing.JLabel belief_search_advnaced_search_subject_label;
	private javax.swing.JTextField belief_search_advnaced_search_subject_textfield;
	private javax.swing.JTabbedPane belief_search_criteria_pane;
	private javax.swing.JDialog belief_search_criteria_prompt;
	private javax.swing.JLabel belief_search_criteria_prompt_message_label;
	private javax.swing.JTextField belief_search_simple_textual_match_criterion_input_text_field;
	private javax.swing.JLabel belief_search_simple_textual_match_criterion_message_label;
	private javax.swing.JPanel belief_search_simple_textual_match_criterion_settings;
	private javax.swing.JLabel belief_search_simple_textual_match_criterion_settings_description_label;
	private javax.swing.JPanel belief_search_simple_textual_match_criterion_settings_tab;
	private javax.swing.JButton belief_search_submit_button;
	private javax.swing.JTree belief_tree;
	private javax.swing.JPanel belief_tree_pane;
	private javax.swing.JPopupMenu belief_tree_popup_menu;
	private javax.swing.JScrollPane belief_tree_scroll_pane;
	private javax.swing.JMenuItem collapse_all_menu_item;
	private javax.swing.JMenuItem expand_all_menu_item;
	private javax.swing.JCheckBoxMenuItem expandable_belief_containers_menucheckbox;
	private javax.swing.JFrame gnome_frame;
	private javax.swing.JTabbedPane gnome_tabs_pain;
	private javax.swing.JPanel grid_display_panel;
	private javax.swing.JMenuBar menu_bar;
	private javax.swing.JTextArea message_area;
	private javax.swing.JLabel message_area_label;
	private javax.swing.JScrollPane message_area_scroll_pane;
	private javax.swing.JPanel n3_pane;
	private javax.swing.JTextArea n3_text_area;
	private javax.swing.JScrollPane n3_text_area_scroll_pane;
	private javax.swing.JButton progress_agent_button;
	private javax.swing.JSpinner progress_agent_cycles_spinner;
	private javax.swing.JLabel quick_search_label;
	private javax.swing.JPanel quick_search_pane;
	private javax.swing.JTextField quick_search_textfield;
	private javax.swing.JButton restart_agent_button;
	private javax.swing.JMenuItem search_belief_menu_item;
	private javax.swing.JLabel search_results_label;
	private javax.swing.JList search_results_list;
	private javax.swing.JScrollPane search_results_list_scroll_pane;
	private javax.swing.JPanel search_results_pane;
	private javax.swing.JPanel settingsPanel;
	private javax.swing.JMenu settings_menu;
	private javax.swing.JCheckBoxMenuItem show_expanded_belief_containers_icon_menucheckbox;
	private javax.swing.JCheckBoxMenuItem show_expanded_belief_containers_text_menucheckbox;
	private javax.swing.JMenuItem show_in_N3_menu_item;
	private javax.swing.JCheckBoxMenuItem show_settings_menucheckbox;
	private javax.swing.JButton stop_agent_button;
	private javax.swing.JMenu tools_menu;
	private javax.swing.JSeparator tools_menu_separator_1;
	private javax.swing.JMenu view_menu;
	private javax.swing.JLabel waiting_for_agent_life_cycle_description_label;
	private javax.swing.JLabel waiting_for_agent_life_cycle_label;
	// End of variables declaration//GEN-END:variables

	// </editor-fold>
}
