package ubiware.infrastructure.packageManager;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import com.google.common.io.Files;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.commands.AddBeliefsN3Command;
import ubiware.core.commands.UbiwareAgentCommand;
import ubiware.ontology.param;
import ubiware.util.StringEscape;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;

public class DeployerBehavior extends ReusableAtomicBehavior {

	private static final long serialVersionUID = 1L;
	private static final String fileGenericProperties = "description/applicationdescription.sapl";
	private static final String folderRoles = "roles/";
	private static final String folderWar = "webcontent/";
	private static final String PREFIXES = "@prefix ubi: <http://www.ubiware.jyu.fi/ubiPackage#> . @prefix sapl: <http://www.ubiware.jyu.fi/sapl#> . @prefix java: <http://www.ubiware.jyu.fi/rab#> . @prefix p: <http://www.ubiware.jyu.fi/rab_parameters#> . @prefix owl:     <http://www.w3.org/2002/07/owl#> . @prefix xsd:     <http://www.w3.org/2001/XMLSchema#> . @prefix rdfs:    <http://www.w3.org/2000/01/rdf-schema#> . @prefix rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#> ";
	private static UniqueInstanceGenerator idGenerator = new UniqueInstanceGenerator("UbiPackage", URITools
			.createUncheckedURI("http://www.ubiware.jyu.fi/ubiPackage#"));
	private File packageName;
	private String tempFolderName;
	private StringBuilder sb = new StringBuilder(DeployerBehavior.PREFIXES);
	private StringBuilder sbMain = new StringBuilder(DeployerBehavior.PREFIXES);

	@Override
	protected void doAction() {
		System.out.println("DeployerBehavior - START");
		File tmpFolder = new File(this.tempFolderName);
		System.err.println("tmpFolder for unpacking ubipack " + this.packageName.getName()+ " -> " + tmpFolder.getAbsolutePath());
		if (!tmpFolder.exists()) {
			System.out.print("Creating temp folder ... ");
			if (!tmpFolder.mkdirs()) {
				System.out.println("FAIL");
				System.err.println("Temp folder cannot be created");
				this.setFailed();
				return;
			} else {
				System.out.println("CREATED");
			}
		}

		this.sb.append(". \n" + DeployerBehavior.idGenerator.getNewID() + " rdf:type ubi:UbiPackage ; ubi:hasState \"1\" \n");
		

		ZipFile zf;
		try {
			zf = new ZipFile(this.packageName);
			Enumeration<? extends ZipEntry> zipFileEntries = zf.entries();
			//zis = new ZipInputStream(new BufferedInputStream(new FileInputStream(this.packageName)));
			while (zipFileEntries.hasMoreElements()) {
				ZipEntry entry = zipFileEntries.nextElement();
				if (entry.isDirectory()) {
					System.out.println("[d] " + entry);
				} else {
					String entryName = entry.getName();
					String outputName = this.tempFolderName + entryName;
					if (entryName.equals(DeployerBehavior.fileGenericProperties)) {
						//System.out.println("Generic properties: " + entryName);
						System.out.println("Generic properties file: " + this.extractEntry(zf, entry, outputName));
						this.handleDescriptor(outputName);
					} else if (entryName.startsWith(DeployerBehavior.folderRoles)) {
						//System.out.println("Roles: " + entryName);
						String fullPath = this.extractEntry(zf, entry, outputName);
						System.out.println("Role file: " + fullPath);
						this.handleRoleFile(entryName.substring(DeployerBehavior.folderRoles.length()), fullPath);
					} else if (entryName.startsWith(DeployerBehavior.folderWar)) {
						//System.out.println("War: " + entryName);
						String fullPath = this.extractEntry(zf, entry, outputName);
						System.out.println("War file: " + fullPath);
						this.handleWarFile(fullPath);
					} else {
						System.out.println("[f] " + entry);
					}
				}
			}
		} catch (ZipException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		this.sbMain.append(" . \n");
		this.sbMain.append(this.sb);
		//System.out.println("++++++++++" + sbMain.toString());
		//System.exit(2323);
		//UbiwareAgentCommand<Boolean> command = new AddBeliefsN3Command(this.sbMain.toString());
		//this.myAgent.addCommand(command);
		String sapl = this.sbMain.toString();
		this.addBeliefsN3(sapl);
	}

	private void handleRoleFile(String roleName, String path) {
		String roleID = DeployerBehavior.idGenerator.getNewID();
		// TODO: maybe it would be wiser to add not just the full path to the role file, but also the role name
		this.sb.append("\n ; ubi:hasRoleFile ");
		this.sb.append(roleID);

		this.sbMain.append(" . \n");
		this.sbMain.append(roleID);
		this.sbMain.append(" ubi:hasRoleName ");
		this.sbMain.append('\"');
		this.sbMain.append(roleName);
		this.sbMain.append("\"\n ; ubi:hasRoleFilePath ");
		this.sbMain.append('\"');
		this.sbMain.append(StringEscape.escape(path));
		this.sbMain.append('\"');
	}

	private void handleDescriptor(String outputName) throws IOException {
		this.sb.append(" ; ubi:hasDescription { \n");
		BufferedReader br = new BufferedReader(new FileReader(outputName));
		String line = null;
		while ((line = br.readLine()) != null) {
			this.sb.append(line);
			this.sb.append('\n');
		}
		this.sb.append("\n}");
		//System.out.println(sb.toString());
		//System.exit(1111);

	}

	private void handleWarFile(String outputName) throws IOException {
		this.sb.append("\n ; ubi:warFileLocation ");
		this.sb.append('\"');
		this.sb.append(StringEscape.escape(outputName));
		this.sb.append('\"');
	}

	private String extractEntry(ZipFile zf, ZipEntry entry, String outputName) throws IOException {
		System.out.print("Going to extract " + entry + " ... ");
		// create directories within temp folder till the depth of the file specified in outputName
		// otherwise new FileOutputStream() will throw an exception
		new File(outputName).getParentFile().mkdirs();
		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(outputName));
		BufferedInputStream bis = new BufferedInputStream(zf.getInputStream(entry));
		byte[] buffer = new byte[5000];
		int length = -1;
		while ((length = bis.read(buffer)) > 0) {
			bos.write(buffer, 0, length);
		}
		bos.close();
		System.out.println("DONE");
		return new File(outputName).getPath();
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.packageName = parameters.getObligatedFileParameter(param.packageName);
		File tempFolder = Files.createTempDir();
//		this.tempFolderName = "temp" + File.separator + "deployment" + File.separator + this.packageName.getName() + File.separator;
		this.tempFolderName = tempFolder.getPath();
	}

	public class DeploymentException extends Exception {

		public DeploymentException(String s) {
			super(s);
		}

		private static final long serialVersionUID = 1L;

	}
}
