package ubiware.infrastructure.ubiwareDF;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import ubiware.util.saplbuilder.Resource;

public class ServiceRepository {
	//maps handlernames to the set of services which need this handler
	private final Map<Handler, Set<Service>> handlerNeededByService = new HashMap<Handler, Set<Service>>();
	//maps Agents to the set of handlers it implements
	private final Map<Agent, Set<Handler>> agentImplementingHandler = new HashMap<Agent, Set<Handler>>();
	//maps services to the set of agents which implement that service
	private final Map<Service, Set<Agent>> serviceProvidedByAgent = new HashMap<Service, Set<Agent>>();
	//Maps services to all listeners registerred for updates of that service
	private final Map<Service, List<ServiceProviderListener>> subscribers = new HashMap<Service, List<ServiceProviderListener>>();

	private static final ServiceRepository serviceRepository = new ServiceRepository();

	/**
	 * Private constructor so that you can't instantiate this (singleton pattern).
	 */
	private ServiceRepository() {
		//empty > needed for singleton.
		//		Timer t = new Timer();
		//		t.schedule(new TimerTask() {
		//			
		//			@Override
		//			public void run() {
		//				System.out.println("handler needed by services defined:");
		//				for(Entry<Handler, Set<Service>> a : serviceRepository.handlerNeededByService.entrySet()){
		//					System.out.println("     " + a.getKey() + " - " + a.getValue());
		//				}
		//				System.out.println();
		//				System.out.println("agents providding services");
		//				for (Entry<Service, Set<Agent>> b : serviceRepository.serviceProvidedByAgent.entrySet()){
		//					System.out.println("     " + b.getKey() + " - " + b.getValue());
		//				}
		//				
		//			}
		//		}, 0, 10000L);

	}

	public static ServiceRepository getServiceRepository() {
		return ServiceRepository.serviceRepository;
	}

	/**
	 * Define a service for this repository
	 * 
	 * @param s
	 */
	//this is a test for annotations
	@HandlerAccessible(handlesResourceName = "<http://www.ubiware.jyu.fi/communication#DefineService>")
	public void defineService(Service s) {
		if (this.serviceProvidedByAgent.containsKey(s)) {
			//re-registration
			return;
		}
		//		this.nameToService.put(s.getName(), s);
		//this service does not exist yet, add its existence to the service map 
		final Set<Agent> agentsProvidingService = new HashSet<Agent>();
		this.serviceProvidedByAgent.put(s, agentsProvidingService);

		//add the service to the handlerNeededByService
		final Set<Handler> serviceHandlers = s.getHandlers();
		for (Handler handler : serviceHandlers) {
			Set<Service> servicesNeedingThisHandler = this.handlerNeededByService.get(handler);
			if (servicesNeedingThisHandler == null) {
				servicesNeedingThisHandler = new HashSet<Service>();
				this.handlerNeededByService.put(handler, servicesNeedingThisHandler);
			}
			servicesNeedingThisHandler.add(s);
		}
		//update the list of agents providing certain interfaces, this is possible if they implement all handlers
		//updateneeded is only true when thre, because of this change is an agent providing this interface
		boolean updateNeeded = false;
		for (Map.Entry<Agent, Set<Handler>> agentAndHandlers : this.agentImplementingHandler.entrySet()) {
			if (agentAndHandlers.getValue().containsAll(serviceHandlers)) {
				agentsProvidingService.add(agentAndHandlers.getKey());
				updateNeeded = true;
			}

		}

		//check whether updates are needed
		//updates are needed whenever there was a subscriber for this service
		if (updateNeeded && this.subscribers.containsKey(s)) {
			//the is a subscriber
			this.fireUpdate(s);
		}
	}

	/**
	 * Register the fact that a certain agent provides a certain handler
	 * 
	 * @param agent
	 * @param handler
	 */
	public void registerHandler(final Agent agent, final Handler handler) {
		Set<Handler> handlers = this.agentImplementingHandler.get(agent);
		if (handlers == null) {
			//this agent does not have any handlers registered yet
			handlers = new HashSet<Handler>();
			this.agentImplementingHandler.put(agent, handlers);
		} else if (handlers.contains(handler)) {
			//this is a re-registration.
			return;
		}
		//add the handler to the agents handler set
		handlers.add(handler);
		//check whether updates are needed:
		if (!this.handlerNeededByService.containsKey(handler)) {
			//no one is interested since there are no services using this handler
			return;
		}
		for (Service service : this.handlerNeededByService.get(handler)) {
			//for these services, it is possible that this agent now has all handlers for this service:
			if (handlers.containsAll(service.getHandlers())) {
				//the agent implements this service and since it was not a re-registration, this is a new service for this agent
				Set<Agent> agentsProvidingService = this.serviceProvidedByAgent.get(service);
				agentsProvidingService.add(agent);
				this.fireUpdate(service);
			}
		}
	}

	/**
	 * Register that a certain agent does not provide a certain handler any longer.
	 * 
	 * @param agent
	 * @param handler
	 */
	public void unRegisterHandler(final Agent agent, final Handler handler) {
		Set<Handler> handlersOfThisAgent = this.agentImplementingHandler.get(agent);
		if (!handlersOfThisAgent.remove(handler)) {
			//not removed, so it was not there; thus the agent did not have this handler registerred anyway => nothing to do
			return;
		}
		Set<Service> servicesNeedingThisHandler = this.handlerNeededByService.get(handler);
		if (servicesNeedingThisHandler != null) {
			for (Service service : servicesNeedingThisHandler) {
				Set<Agent> agentsImplementingService = this.serviceProvidedByAgent.get(service);
				if (agentsImplementingService != null) {
					agentsImplementingService.remove(agent);
					this.fireUpdate(service);
				}
			}
		}
	}

	private void fireUpdate(Service service) {
		Set<Agent> agentsProvidingThisService = new HashSet<Agent>(this.serviceProvidedByAgent.get(service));
		List<ServiceProviderListener> subscriberList = this.subscribers.get(service);
		if (subscriberList != null) {
			for (ServiceProviderListener listener : subscriberList) {
				listener.serviceProviderListChanged(service, agentsProvidingThisService);
			}
		}
	}

	/**
	 * After this subscription the listener will be informed if there are updates for the specified service
	 * 
	 * @param l
	 * @param service
	 */
	private void subscribeToService(ServiceProviderListener l, Service service) {
		//Service s = this.nameToService.get(serviceName);
		List<ServiceProviderListener> listeners = this.subscribers.get(service);
		if (listeners == null) {
			listeners = new ArrayList<ServiceProviderListener>();
			this.subscribers.put(service, listeners);
		}
		listeners.add(l);
		//at least give one update to this listener(even if it is empty)
		//System.out.println(" ");
		Set<Agent> agentsProvidingService = this.serviceProvidedByAgent.get(service);
		if (agentsProvidingService == null) {
			agentsProvidingService = new HashSet<Agent>();
		}
		l.serviceProviderListChanged(service, agentsProvidingService);
	}

	private final Map<Agent, Map<Service, List<ServiceProviderListener>>> registrationMap = new HashMap<Agent, Map<Service, List<ServiceProviderListener>>>();

	/**
	 * Subscribe the listener l for the service with name serviceName for the agent agent.
	 * 
	 * @param l
	 * @param serviceName
	 * @param agent
	 */
	public void subscribeForService(ServiceProviderListener l, Resource serviceName, Agent agent) {
		Service service = new Service(serviceName, null);
		this.subscribeToService(l, service);
		Map<Service, List<ServiceProviderListener>> registrations = this.registrationMap.get(agent);
		if (registrations == null) {
			registrations = new HashMap<Service, List<ServiceProviderListener>>();
			this.registrationMap.put(agent, registrations);
		}
		List<ServiceProviderListener> listenerList = registrations.get(service);
		if (listenerList == null) {
			listenerList = new ArrayList<ServiceProviderListener>();
			registrations.put(service, listenerList);
		}
		listenerList.add(l);
	}

	/**
	 * Unsubscribe the agent from listening to updates about that service
	 * 
	 * @param serviceName
	 * @param agent
	 */
	public void unSubscribeForService(Resource serviceName, Agent agent) {
		Service service = new Service(serviceName, null);
		// this should also work with Service-object with a null handler list, right?
		Map<Service, List<ServiceProviderListener>> registrations = this.registrationMap.get(agent);
		if (registrations == null) {
			//this agent does not have registrations
			return;
		}

		List<ServiceProviderListener> listeners = registrations.get(service);
		if (listeners == null) {
			return;
		}
		List<ServiceProviderListener> allSubscribersForService = this.subscribers.get(service);
		for (ServiceProviderListener listener : listeners) {
			allSubscribersForService.remove(listener);
		}
		listeners.clear();
		//throw new UnsupportedOperationException("Not implemented");
	}

	private Map<Agent, Set<Agent>> puaToWorker = new HashMap<Agent, Set<Agent>>();

	/**
	 * Register a worker to be worker of a certain PUA
	 * 
	 * @param workerAgent
	 * @param PUA
	 */
	public void registerWorkerAgent(Agent workerAgent, Agent PUA) {
		//just add to map
		Set<Agent> workers = this.puaToWorker.get(PUA);
		if (workers == null) {
			workers = new HashSet<Agent>();
			this.puaToWorker.put(PUA, workers);
		}
		workers.add(workerAgent);
		//check which service this agent provides and update them.
		//the point is that there might be a listener waiting for updates about it PUA
		for (Entry<Service, Set<Agent>> entry : this.serviceProvidedByAgent.entrySet()) {
			if (entry.getValue().contains(workerAgent)) {
				this.fireUpdate(entry.getKey());
			}
		}
	}

	/**
	 * Sends to the listener once which agents (woking for this PUA) are now implementing this service
	 * 
	 * @param PUA
	 * @param serviceName
	 * @param l
	 */
	public void getWorkerForService(Agent PUA, Resource serviceName, ServiceProviderListener l) {
		Service s = new Service(serviceName, null);
		Set<Agent> agentsProvidingService = this.serviceProvidedByAgent.get(s);
		Set<Agent> workerAgentsOfPUA = this.puaToWorker.get(PUA);
		Set<Agent> result = null;
		if ((workerAgentsOfPUA == null) || (agentsProvidingService == null)) {
			result = Collections.emptySet();
		} else {
			HashSet<Agent> selection = new HashSet<Agent>(workerAgentsOfPUA);
			selection.retainAll(agentsProvidingService);
			result = new HashSet<Agent>();
			if (selection.size() > 0) {
				result.add(selection.iterator().next());
			}
		}
		l.serviceProviderListChanged(s, result);
	}

	/**
	 * subscribe the given agent for updates of workers of agent PUA providing the given service. Unsubscription is currently handled by
	 * completely unsubscribing the listeneingAgent from this service.
	 * 
	 * @param l
	 * @param serviceName
	 * @param PUA
	 * @param listeningAgent
	 */
	public void subscribeForServiceByWorker(ServiceProviderListener l, Resource serviceName, Agent PUA, Agent listeningAgent) {
		ServiceProviderListener listener = new listenerFilter(l, PUA);
		this.subscribeForService(listener, serviceName, listeningAgent);
	}

	private class listenerFilter implements ServiceProviderListener {

		private final ServiceProviderListener listener;
		private Set<Agent> latestUpdate;
		private Agent PUA;

		public listenerFilter(ServiceProviderListener l, Agent PUA) {
			this.listener = l;
			this.PUA = PUA;
		}

		@Override
		public void serviceProviderListChanged(Service service, Set<Agent> agentsProvidingThisService) {
			//check whether this update is important, this is only the case if the set contains a worker of the PUA!
			//and if it is different from the latest update
			Set<Agent> workerList = ServiceRepository.this.puaToWorker.get(this.PUA);
			Set<Agent> mergedResult = null;
			if ((workerList == null) || agentsProvidingThisService.isEmpty()) {
				//there are no workers => no effort needed
				mergedResult = Collections.emptySet();
			} else {
				//there are workers, effort is needed
				mergedResult = new HashSet<Agent>(agentsProvidingThisService);
				mergedResult.retainAll(workerList);
			}

			if ((this.latestUpdate == null) || !this.latestUpdate.equals(mergedResult)) {
				this.listener.serviceProviderListChanged(service, mergedResult);
				this.latestUpdate = mergedResult;
			}
		}
	}
}
