package ubiware.infrastructure.web;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import ubiware.ontology.com;
import ubiware.ontology.rdf;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.ActionArgumentContainer;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

public class AgentWrapper {

	private final Agent agent;
	private final String recipient;
	private Resource senderResource;

	public AgentWrapper(Agent agent, Resource recipient) {
		this.agent = agent;
		this.recipient = recipient.toString();
		this.senderResource = Resource.createUncheckedResource(this.agent.getAID().getLocalName());
	}

	public String syncAction(Resource action, ActionArgumentContainer params) {
		Resource convID = AgentWrapper.convIDGenerator.getNewResource();
		return this.syncActionConvID(action, params, convID);
	}

	private String syncActionConvID(Resource action, ActionArgumentContainer params, Resource convID) {
		String ontology = "SAPL";

		params.addArgument(com.convID, convID);
		//agents are resources
		params.addArgument(com.sender, this.senderResource);

		SaplDocument content = new SaplDocument();
		Resource anoNode = Resource.creatAnonymousNode();
		content.addStatement(anoNode, rdf.type, com.Action);
		content.addStatement(anoNode, com.hasActionName, action);
		content.addStatement(anoNode, com.hasActionParams, params);

		//System.out.println("AgentWrapper: sending message \n" + content.generateSapl() + "\nto agent " + this.recipient);
		String sapl = content.generateSapl();
		//		String conversationID = convID.toString();
		return this.requestResponse(sapl, ontology, convID);
	}

	private String requestResponse(String content, String ontology, Resource convID) {
		//System.err.println("Jetty: Message sending\n content=" + content +" \n convID="+convID+" \n receiver="+this.recipient);
		this.sendMsg(this.recipient, content, ontology, convID);

		WaitForResponseBehavior b = new WaitForResponseBehavior(this.agent, convID);
		this.agent.addBehaviour(b);
		try {
			return b.get(200, TimeUnit.SECONDS);
		} catch (InterruptedException e) {
			// TODO What to do with this exception? the result is not desired any longer,
			//e.printStackTrace();
			this.agent.removeBehaviour(b);//at least the behaviour will not be scheduled again.
			Thread.currentThread().interrupt();//set interrupt flag
			return null;//TODO null pointer return???/empty string/exception???
		} catch (TimeoutException e) {
			throw new Error(e);
		}
	}

	private void sendMsg(String actualRecipient, String content, String ontology, Resource conversationID) {
		if (conversationID == null) {
			//			conversationID = actualRecipient + "_" + System.currentTimeMillis();
			throw new Error("conversation ID cannot be null!");
		}
		ACLMessage message = new ACLMessage(ACLMessage.REQUEST);
		message.addReceiver(new AID(actualRecipient, AID.ISLOCALNAME));
		message.setContent(content);
		message.setOntology(ontology);
		message.setConversationId(conversationID.toString());
		this.agent.send(message);
		//return conversationID;
	}

	private static final class WaitForResponseBehavior extends SimpleBehaviour {

		private static final long serialVersionUID = 1L;
		private String response;
		private MessageTemplate template;
		private CountDownLatch countDownLatch;

		public WaitForResponseBehavior(Agent a, Resource convID) {
			super(a);
			this.template = MessageTemplate.MatchConversationId(convID.toString());
			this.countDownLatch = new CountDownLatch(1);//when passed the latch, all calls to get() return.
		}

		@Override
		public void action() {
			ACLMessage msg = this.myAgent.receive(this.template);
			if (msg != null) {
				this.response = msg.getContent();
				//System.err.println("Jetty: Message received\n response=" + this.response +" \n convID="+msg.getConversationId()+" \n sender="+msg.getSender().getLocalName());
				//this.finished = true;
				//inform.
				this.countDownLatch.countDown();
			} else {
				this.block();
			}
		}

		@Override
		public boolean done() {
			return this.countDownLatch.getCount() == 0;
		}

		//blocking call, returns when the result is ready.
		public String get(long timeOut, TimeUnit unit) throws InterruptedException, TimeoutException {
			if (this.countDownLatch.await(timeOut, unit)) {
				return this.response;
			}
			throw new TimeoutException("Receiving the message from the agent timed out after waiting for " + timeOut + " " + unit.toString());//TODO is this the desired behaviour?
		}

		//TODO move to version with timeout!
		//blocking call, returns when the result is ready.
		public String get() throws InterruptedException {
			this.countDownLatch.await();
			return this.response;
		}
	}

	@Override
	public String toString() {
		return "wrapper for : " + this.recipient.toString();
	}

	private static UniqueInstanceGenerator convIDGenerator = new UniqueInstanceGenerator("convID",
			URITools.createUncheckedURI("http://ubiware.jyu.fi/agentWrapper/"));

	public MessageContent subscribe(Resource action, ActionArgumentContainer params, UpdateHandler handler) {
		Resource convID = AgentWrapper.convIDGenerator.getNewResource();
		String result = this.syncActionConvID(action, params, convID);
		SubscribedMessagesListener behavior = new SubscribedMessagesListener(convID, handler);
		handler.setBehavior(behavior, convID);
		this.agent.addBehaviour(behavior);
		return new MessageContent(result);
	}

	private class SubscribedMessagesListener extends SimpleBehaviour {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private MessageTemplate template;
		private UpdateHandler handler;
		private boolean done = false;

		public SubscribedMessagesListener(Resource convID, UpdateHandler handler) {
			this.template = MessageTemplate.MatchConversationId(convID.toString());
			this.handler = handler;
		}

		@Override
		public void action() {
			ACLMessage msg = this.myAgent.receive(this.template);
			if ((msg != null) && !this.done) {
				MessageContent content = new MessageContent(msg.getContent());
				this.handler.putMessage(content);
			} else {
				this.block();
			}

		}

		public void stopReceiving() {
			this.done = true;
		}

		@Override
		public boolean done() {
			return this.done;
		}

	}

	public abstract class UpdateHandler {
		
//		private LinkedList<MessageContent> buffer;
		final private LinkedBlockingQueue<MessageContent> myBuffer = new LinkedBlockingQueue<MessageContent>();
		private State state;
		private Object lock = new Object();
		private SubscribedMessagesListener behavior;
		private Resource convID;

		public UpdateHandler() {
			this.state = State.NOT_INITIALISED;
		}

		private void setBehavior(SubscribedMessagesListener behavior, Resource convID) {
			this.behavior = behavior;
			this.state = State.NOT_STARTED;
			this.convID = convID;
		}
		//public abstract void handleUpdate2(MessageContent content);
		public abstract void handleUpdate(MessageContent content);

		public final void start() {
			synchronized (this.lock) {
				switch (this.state) {
				case STARTED:
					throw new IllegalStateException("Start called multiple times.");
				case UNSUBSCRIBED:
					throw new IllegalStateException("Already unsubscribed.");
				case NOT_INITIALISED:
					throw new IllegalStateException("the handler must first be subscribed before it can be started");
				case NOT_STARTED:
					//perform
					//start thread which moves messages from the buffer to the handler
					this.state = State.STARTED;
					new Thread (new Runnable() {
						
						@Override
						public void run() {
							while (state == State.STARTED){
								MessageContent elem;
								try {
									elem = myBuffer.poll(100, TimeUnit.SECONDS);
								} catch (InterruptedException e) {
									throw new Error("this thread is nowhere interuptible!", e);
								}
								if (elem != null && state == State.STARTED){
									UpdateHandler.this.handleUpdate(elem);
								}
							}
						}
					}).start();
					break;
				
				default:
					throw new IllegalStateException("Unknown state");
				}
			}
		}

		private final void putMessage(MessageContent content) {
			synchronized (this.lock) {
				switch (this.state) {
				case UNSUBSCRIBED:
					System.err
							.println("No messages should be added any more after unsubscribing, ignoring since this is not a mayor problem.");
					System.err.println("Ignored message : " + content);
					break;
				case NOT_INITIALISED:
					throw new IllegalStateException("the handler must first be subscribed before it can be started");
				case NOT_STARTED:
				case STARTED:	
					try {
						this.myBuffer.put(content);
					} catch (InterruptedException e) {
						throw new Error("this thread is nowhere interuptible!", e);
					}
					break;
				default:
					throw new IllegalStateException("Unknown state");
				}

			}
		}

		public final MessageContent unsubscribe(Resource action, ActionArgumentContainer params) {
			synchronized (this.lock) {
				switch (this.state) {
				case UNSUBSCRIBED:
					throw new IllegalStateException("Already unsubscribed");
				case NOT_INITIALISED:
					throw new IllegalStateException("the handler must first be subscribed before it can be unsubscribed");
				case NOT_STARTED:
				case STARTED:
					this.state = State.UNSUBSCRIBED;
					this.behavior.stopReceiving();
					this.myBuffer.clear();
					return new MessageContent(AgentWrapper.this.syncActionConvID(action, params, this.convID));
					//break;
				
				default:
					throw new IllegalStateException("Unknown state");
				}
			}
		}

	}

	private static enum State {
		NOT_STARTED, STARTED, UNSUBSCRIBED, NOT_INITIALISED;
	}

}
