package ubiware.rdf2bean.core;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.openrdf.query.BindingSet;
import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.QueryEvaluationException;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.TupleQuery;
import org.openrdf.query.TupleQueryResult;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.repository.sail.SailRepositoryConnection;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFParseException;
import org.openrdf.sail.inferencer.fc.ForwardChainingRDFSInferencer;
import org.openrdf.sail.memory.MemoryStore;

public class Storage {

	private Set<RDFResource> resources = new LinkedHashSet<RDFResource>();
	private Map<Class<? extends RDFResource>, String> registeredClasses = new HashMap<Class<? extends RDFResource>, String>();
	private SailRepositoryConnection con;
	private SailRepository myRepository;
	private Set<String> objectProperties;

	public Storage(Set<String> op) {
		this.objectProperties = op;
	}

	/**
	 * Helper method to fix S-APL code which is not RDF complaint.
	 * 
	 * @param rdfString
	 *            the String to be sanitized
	 * @return the sanitized String
	 */
	public static String sanitize(String rdfString) {
		if (rdfString.equals(" .\n")) {
			rdfString = "";
		} else if (rdfString.equals(" . \n")) {
			rdfString = "";
		}
		//rdfString = StringEscape.unEscape(rdfString);
		//FIXME this is a small hack to change the escaping which the sapl code has which is not RDF complaint.
		//this should obviousely happen in a more structured way.
		rdfString = rdfString.replace("\\'", "'");
		return rdfString;
	}

	/**
	 * Initializes the storage from given RDF string.
	 * 
	 * @param rdfString
	 *            the RDF added to the storage
	 * 
	 * @throws RepositoryException
	 * @throws IllegalArgumentException
	 * @throws MalformedQueryException
	 * @throws QueryEvaluationException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	public void read(String rdfString) throws RepositoryException, MalformedQueryException, QueryEvaluationException, IllegalArgumentException, InstantiationException, IllegalAccessException,
			InvocationTargetException {
		this.myRepository = new SailRepository(new ForwardChainingRDFSInferencer(new MemoryStore()));
		this.myRepository.initialize();
		this.con = this.myRepository.getConnection();

		//FIXME this should not happen here but in the callers code. (?)
		rdfString = Storage.sanitize(rdfString);

		this.readToRepository(rdfString);
		this.loadAllResources();
	}

	public void read2(String rdfString) throws StorageException {
		try {
			this.read(rdfString);
		} catch (Exception e) {
			throw new StorageException("", e);
		}
	}

	/**
	 * Adds the given RDF string to the storage.
	 * 
	 * @param rdfString
	 *            the RDF added to the storage
	 * 
	 * @throws RepositoryException
	 * @throws MalformedQueryException
	 * @throws QueryEvaluationException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	public void add(String rdfString) throws RepositoryException, MalformedQueryException, QueryEvaluationException, InstantiationException, IllegalAccessException, InvocationTargetException {

		rdfString = Storage.sanitize(rdfString);
		this.readToRepository(rdfString);
		this.loadAllResources();
	}

	private void readToRepository(String rdfString) {
		InputStream stringStream;
		try {
			stringStream = new ByteArrayInputStream(rdfString.getBytes("UTF-8"));
		} catch (UnsupportedEncodingException e1) {
			e1.printStackTrace();
			System.out.println("Initializing the repository input stream failed.");
			return;
		}
		try {
			this.con.add(stringStream, "http://something/", RDFFormat.N3);
		} catch (RDFParseException e) {
			Exception ex = new Exception("Violating code : [" + rdfString + "]", e);
			ex.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public <T extends RDFResource> void register(Class<T> cl, String resourceURI) {
		this.registeredClasses.put(cl, resourceURI);
	}

	private void loadAllResources() throws MalformedQueryException, RepositoryException, QueryEvaluationException, IllegalArgumentException, InstantiationException, IllegalAccessException,
			InvocationTargetException {
		Set<Class<? extends RDFResource>> keys = this.registeredClasses.keySet();
		for (Class<? extends RDFResource> k : keys) {
			Set<? extends RDFResource> resourcesOfTypeK = this.giveMeResourcesOfThisType(k);
			System.out.println("Adding " + resourcesOfTypeK.size() + " resources of type " + k);
			this.resources.addAll(resourcesOfTypeK);
		}
	}

	private <T extends RDFResource> Set<T> giveMeResourcesOfThisType(Class<T> x) throws MalformedQueryException, RepositoryException, QueryEvaluationException, IllegalArgumentException,
			InstantiationException, IllegalAccessException, InvocationTargetException {
		Set<T> returnValues = new HashSet<T>();
		List<String> resourceURIs = new ArrayList<String>();
		String type = this.registeredClasses.get(x);
		String queryString = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> select ?res WHERE { ?res rdf:type <" + type + "> } ";
		//System.out.println(queryString);
		TupleQuery tupleQuery = this.con.prepareTupleQuery(QueryLanguage.SPARQL, queryString);
		TupleQueryResult tupleResult = tupleQuery.evaluate();
		while (tupleResult.hasNext()) {
			BindingSet bs = tupleResult.next();
			String res = bs.getValue("res").stringValue();
			resourceURIs.add(res);
		}
		for (String res : resourceURIs) {
			T resource = this.giveMeResource(res, x);
			//System.out.println("Resource " + resource + " fully retrieved");
			returnValues.add(resource);
		}
		return returnValues;
	}

	private <T extends RDFResource> T giveMeResource(String resourceURI, Class<T> x) throws MalformedQueryException, RepositoryException, QueryEvaluationException, IllegalArgumentException,
			InstantiationException, IllegalAccessException, InvocationTargetException {
		Map<String, Set<String>> map = this.giveMeAllPairs(resourceURI);
		//RDFResource res = new RDFResource(resourceURI);
		//System.out.println("Resource " + resourceURI + " has pairs: " + map);
		//System.out.println(map);
		//BeanInfo info = Introspector.getBeanInfo(x);
		return RDFResource.giveMeResourceInstance(resourceURI, x, map, this);
	}

	private Map<String, Set<String>> giveMeAllPairs(String sub) throws MalformedQueryException, RepositoryException, QueryEvaluationException {
		Map<String, Set<String>> returnValues = new HashMap<String, Set<String>>();
		String queryString = "PREFIX conf: <http://www.ubiware.jyu.fi/configuration#> select ?pred ?obj WHERE { <" + sub + ">  ?pred ?obj } ";
		//System.out.println(queryString);
		TupleQuery tupleQuery = this.con.prepareTupleQuery(QueryLanguage.SPARQL, queryString);
		TupleQueryResult tupleResult = tupleQuery.evaluate();
		while (tupleResult.hasNext()) {
			BindingSet bs = tupleResult.next();
			String pred = bs.getValue("pred").stringValue();
			String obj = bs.getValue("obj").stringValue();

			Set<String> set = returnValues.get(pred);
			if (set == null) {
				Set<String> newSet = new HashSet<String>();
				returnValues.put(pred, newSet);
				set = newSet;
			}
			set.add(obj);
		}
		return returnValues;
	}

	public <T extends RDFResource> Set<T> getAllResourcesOfThisType(Class<T> cl) {
		Set<T> results = new HashSet<T>();
		for (RDFResource rdfResource : this.resources) {
			if (rdfResource.getClass().equals(cl)) {
				//System.out.println("OK");
				results.add((T) rdfResource);
			}
		}
		return results;
	}

	public <T extends RDFResource> T getResourceOfThisTypeWithThisURI(String uri, Class<T> cl) {
		T result;
		for (RDFResource rdfResource : this.resources) {
			if (rdfResource.getUri().equals(uri)) {
				//System.out.println("OK");
				try {
					result = (T) rdfResource;
					return result;
				} catch (Exception e) {
					return null;
				}
			}
		}
		return null;
	}

	public String getStringRepresentation() throws QueryEvaluationException, RepositoryException, MalformedQueryException {
		//this.con.commit();
		StringBuilder sb = new StringBuilder();
		for (RDFResource res : this.resources) {
			String resURI = res.getUri();
			Map<String, Set<String>> props = res.properties;
			Set<String> keys = props.keySet();
			for (String k : keys) {
				Set<String> vals = props.get(k);
				if (vals != null) {
					if (!vals.isEmpty()) {
						for (String v : vals) {
							sb.append("<");
							sb.append(resURI);
							sb.append("> <");
							sb.append(k);
							// FIXME: very very stupid way of doing it - change needed!!!
							// if it is in the list of object properties, then it will get <> brackets
							if (this.objectProperties.contains(k)) {
								sb.append("> <");
								sb.append(v);
								sb.append("> . \n");
							} else {
								sb.append("> \"");
								//sb.append(StringEscape.escape(v));
								sb.append(v);
								sb.append("\" . \n");
							}
						}
					}
				}
			}
		}
		return sb.toString();
	}

	private String getStringRepresentationOld() throws QueryEvaluationException, RepositoryException, MalformedQueryException {
		this.con.commit();
		String queryString = "select ?sub ?pred ?obj WHERE { ?sub  ?pred ?obj } ";
		TupleQuery tupleQuery = this.con.prepareTupleQuery(QueryLanguage.SPARQL, queryString);
		TupleQueryResult tupleResult = tupleQuery.evaluate();
		StringBuilder sb = new StringBuilder();
		while (tupleResult.hasNext()) {
			BindingSet bs = tupleResult.next();
			String sub = bs.getValue("sub").stringValue();
			String pred = bs.getValue("pred").stringValue();
			String obj = bs.getValue("obj").stringValue();
			sb.append(sub + " " + pred + " " + obj + " . \n");
		}
		return sb.toString();
	}

	public boolean existsResourceWithThisURI(String uri) {
		for (RDFResource res : this.resources) {
			if (res.getUri().equals(uri)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Adds the given RDFResource object to the storage.
	 * 
	 * @param res
	 *            to be added
	 */
	public void addResource(RDFResource res) {
		this.resources.add(res);
	}

	/**
	 * Adds the given collection of RDFResource objects to the storage.
	 * 
	 * @param resourcesToBeAdded
	 *            to be added
	 */
	public void addResources(Collection<? extends RDFResource> resourcesToBeAdded) {
		for (RDFResource res : resourcesToBeAdded) {
			this.resources.add(res);
		}
	}

	public void debugMe() {
		if (this.resources == null) {
			System.out.println("resources = null");
			return;
		}
		if (this.resources.isEmpty()) {
			System.out.println("resources are empty");
			return;
		}
		for (RDFResource res : this.resources) {
			System.out.println(res);
		}
	}

	public static Storage createStorageWithoutObjectProperties() {
		return new Storage(new HashSet<String>());
	}
}
