package ubiware.shared;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ProduceN3Options;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.UbiwareAgent;
import ubiware.ontology.param;

/**
 * from Saplguide.pdf<br>
 * <h3>Full name: ubiware.shared.BeliefsBackupBehavior</h3>
 * <P>
 * Action: Saves a sub-graph of agents' beliefs as an S-APL Notation3 string into a file or as a literal into a belief.
 * </p>
 * <P>
 * Inputs:
 * <TABLE border = "1">
 * <TR>
 * <TH>Name</TH>
 * <TH>Meaning</TH>
 * <TH>Mandatory</TH>
 * <TH>Default</TH>
 * </TR>
 * <TR>
 * <TD>
 * p:input</td>
 * <td>ID of context container that is the root of the sub-graph. If 'G' is given, the whole beliefs storage is saved.</td>
 * <td>
 * No</td>
 * <td>'G'</td>
 * </tr>
 * <tr>
 * <td>p:saveTo</td>
 * <td>Either the name of the file to save the result to, or a context container specifying the belief structure to be added (should use
 * ?result variable that will be substituted with the result).</td>
 * <td>
 * Yes</td>
 * <td/>
 * </tr>
 * <tr>
 * <td>p:indents</td>
 * <td>If is equal to 'true', the line breaks and indentation is used to separate beliefs. Otherwise, a single line is produced.</td>
 * <td>No</td>
 * <td>'false'</td>
 * </tr>
 * </table>
 * </P>
 * <P>
 * Outputs: the file with Notation3 representation of beliefs Example of usage: Save to 'goals.sapl' the contents of
 * <code>sapl:I sapl:want {}</code>container, i.e. all the active goals of the agent.
 * 
 * <PRE>
 * {sapl:I sapl:want ?in} =>
 * {{sapl:I sapl:do java:ubiware.shared.BeliefsBackupBehavior}
 * sapl:configuredAs {p:input sapl:is ?in.
 * p:saveTo sapl:is "goals.sapl"}
 * </PRE>
 * 
 * </P>
 * <P>
 * Libraries needed: none.
 * </P>
 * 
 * 19.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class BeliefsBackupBehavior extends ReusableAtomicBehavior {

	private static final long serialVersionUID = 1L;
	private String input;
	private String output;
	private ProduceN3Options options;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.input = parameters.getOptionalStringParameter(param.input);
		if ((this.input == null) || this.input.equals("G")) {
			this.input = UbiwareAgent.GENERAL_CONTEXT;
		}
		this.output = parameters.getObligatedNonEmptyStringParameter(param.saveTo);
		this.options = new ProduceN3Options(parameters.getOptionalBooleanParameter(param.indents, false), true);
	}

	@Override
	protected void doAction() {
		String result = this.produceN3(this.input, this.options);

		boolean ok = RABUtils.putResult(this.myAgent, result, this.output);
		if (!ok) {
			this.setFailed();
		}
		this.wakeAgent();
	}

}
