package ubiware.shared;

import java.util.Date;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class EmailSenderBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	long waitTill = 0;
	Thread sendIt;

	MimeMessage msg;
	String receiver;

	boolean tried = false;
	private String smtp;
	private int smtpPort;
	private String smtpUser;
	private String smtpPassword;
	private String cc;
	private String bcc;
	private String message;
	private String content;
	private String subject;
	private String sender;
	private String attached;
	private String wait;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.smtp = parameters.getObligatedNonEmptyStringParameter(param.smtp);
		this.smtpPort = parameters.getOptionalNumericParameter(param.smtpPort, 25);
		this.smtpUser = parameters.getParameterValue(param.smtpUser);
		this.smtpPassword = parameters.getParameterValue(param.smtpPassword);
		this.receiver = parameters.getObligatedNonEmptyStringParameter(param.to);
		this.sender = parameters.getOptionalStringParameter(param.from, "no@addr.ess");
		this.cc = parameters.getParameterValue(param.cc);
		this.bcc = parameters.getParameterValue(param.bcc);
		this.subject = parameters.getOptionalStringParameter(param.subject, "");
		this.message = parameters.getOptionalStringParameter(param.message, "");
		this.content = parameters.getOptionalStringParameter(param.contentType, "text/plain");
		this.attached = parameters.getParameterValue(param.attach);
		this.wait = parameters.getParameterValue(param.waitOnFail);
	}

	@Override
	public void doAction() {

		if (this.sendIt != null) { // control is in the sending thread
			this.block();
			return;
		}

		if (this.tried && this.finished) {
			if (this.hasSucceededTillNow()) {
				this.print("Email to " + this.receiver + " is sent");
			} else {
				this.print("Sending email to " + this.receiver + " is failed!");
			}
			return;
		}

		if (this.waitTill != 0) {
			long now = System.currentTimeMillis();
			if (now < this.waitTill) { // waiting till next attempt of resending
				this.block(this.waitTill - now);
				return;
			}
		}

		this.finished = false;
		this.tried = true;

		try {
			Properties props = System.getProperties();
			props.put("mail.smtp.host", this.smtp);
			//if (smtpPort != null) { redundant null check, smtpPort cannot be null
			props.put("mail.smtp.port", this.smtpPort);
			//}

			Session session;
			if (this.smtpUser == null) {
				session = Session.getDefaultInstance(props, null);
			} else {
				props.put("mail.smtp.auth", "true");

				props.put("mail.smtp.socketFactory.port", this.smtpPort);
				props.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
				props.put("mail.smtp.socketFactory.fallback", "false");

				session = Session.getDefaultInstance(props, new EmailSenderAuthenticator(this.smtpUser, this.smtpPassword));
			}

			this.msg = new MimeMessage(session);
			this.msg.setFrom(new InternetAddress(this.sender));

			InternetAddress[] address = InternetAddress.parse(this.receiver, false);
			this.msg.setRecipients(Message.RecipientType.TO, address);
			if (this.cc != null) {
				address = InternetAddress.parse(this.cc, false);
				this.msg.setRecipients(Message.RecipientType.CC, address);
			}
			if (this.bcc != null) {
				address = InternetAddress.parse(this.bcc, false);
				this.msg.setRecipients(Message.RecipientType.BCC, address);
			}

			this.msg.setSubject(this.subject);

			if (this.attached == null) {
				this.msg.setText(this.message);
			} else {
				MimeBodyPart mbp1 = new MimeBodyPart();
				mbp1.setText(this.message);

				Multipart mp = new MimeMultipart();
				mp.addBodyPart(mbp1);

				StringTokenizer st = new StringTokenizer(this.attached, ",");
				while (st.hasMoreTokens()) {
					String f = st.nextToken();
					MimeBodyPart mbpi = new MimeBodyPart();
					String fil = f;
					FileDataSource fds = new FileDataSource(fil);
					mbpi.setDataHandler(new DataHandler(fds));
					mbpi.setFileName(fds.getName());
					mp.addBodyPart(mbpi);
				}

				this.msg.setContent(mp);
			}

			this.msg.setHeader("X-Mailer", "Ubiware Platform EmailSenderBehavior");
			this.msg.setHeader("Content-Type", this.content);

			this.msg.setSentDate(new Date());

			this.print("Sending email to " + this.receiver + "..");
			Runnable runIt = new SendEmail();
			this.sendIt = new Thread(runIt);
			this.sendIt.start();

			this.block();
		} catch (MessagingException ex) {
			ex.printStackTrace();
			this.setFailed();
			this.finished = true;
			this.print("Sending email to " + this.receiver + " is failed!");
		}
	}

	private static class EmailSenderAuthenticator extends Authenticator {
		private final String smtpUser;
		private final String smtpPassword;

		public EmailSenderAuthenticator(String smtpUser, String smtpPassword) {
			super();
			this.smtpUser = smtpUser;
			this.smtpPassword = smtpPassword;
		}

		@Override
		protected PasswordAuthentication getPasswordAuthentication() {
			return new PasswordAuthentication(this.smtpUser, this.smtpPassword);
		}
	}

	private class SendEmail implements Runnable {
		@Override
		public void run() {
			try {
				Transport.send(EmailSenderBehavior.this.msg);
				EmailSenderBehavior.this.finished = true;
				EmailSenderBehavior.this.sendIt = null;
				EmailSenderBehavior.this.restart();
			} catch (MessagingException ex) {
				String error = ex.getMessage();

				if ((EmailSenderBehavior.this.wait == null) || !(error.startsWith("451") || error.startsWith("452"))) {
					ex.printStackTrace();
					EmailSenderBehavior.this.setFailed();
					EmailSenderBehavior.this.finished = true;
				} else {
					long toWait = new Long(EmailSenderBehavior.this.wait).longValue();
					long now = System.currentTimeMillis();
					EmailSenderBehavior.this.waitTill = now + toWait;

					EmailSenderBehavior.this.print("Email sending error. Waiting " + toWait / 1000 + " sec");
				}
				EmailSenderBehavior.this.sendIt = null;
				EmailSenderBehavior.this.restart();
			}
		}
	}
}
