package ubiware.shared;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.UbiwareAgent;
import ubiware.core.commands.RabRunnable;
import ubiware.core.commands.UbiwareAgentCommand;
import ubiware.ontology.param;

/**
 * From Saplguide.pdf<br>
 * <H3>Full name: ubiware.shared.HttpDataFetcherBehavior</H3>
 * <P>
 * Action: Downloads a document (e.g. an HTML page) from an Internet server and saves it into either a file or a belief structure. The
 * download process is done in a separate thread, so does not block the agent. If everything is fine, ends in success. If there is a problem
 * with either fetching or saving, ends in failure.
 * </P>
 * <P>
 * Inputs:
 * <TABLE border = "1">
 * <TR>
 * <TH>Name</TH>
 * <TH>Meaning</TH>
 * <TH>Mandatory</TH>
 * <TH>Default</TH>
 * </TR>
 * <TR>
 * <TD>p:url</TD>
 * <TD>URL of the document.</TD>
 * <TD>Yes</TD>
 * <TD/>
 * <TR>
 * <TR>
 * <TD>p:saveTo</TD>
 * <TD>Either the name of the file to save the document to, or a context container specifying the belief structure to be added (should use
 * ?result variable that will be substituted with the result)</TD>
 * <TD>Yes</TD>
 * <TD/>
 * </TR>
 * </TABLE>
 * </P>
 * <P>
 * Outputs: Prints to the screen Failed to retrieve &lt;url&gt; if download failed. Creates a file or specified belief structure.
 * </P>
 * <P>
 * Example of usage: Download from the site of Finnish Meteorological Institute the page with the present weather information and forecast
 * for Jyvskyl.
 * 
 * <PRE>
 * {sapl:I sapl:do java:ubiware.shared.HttpDataFetcherBehavior}
 * sapl:configuredAs
 * {p:url sapl:is "http://www.fmi.fi/saa/paikalli.html?
 * kunta=Jyv%E4skyl%E4".
 * p:saveTo sapl:is "received/weather.htm"
 * }
 * </PRE>
 * 
 * </P>
 * <P>
 * Libraries needed: none.
 * </P>
 * 
 * 19.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class HttpDataFetcherBehavior extends ReusableAtomicBehavior {

	private static final long serialVersionUID = 1L;
	private URL uri;
	private String saveTo;

	private boolean started = false;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.uri = parameters.getObligatedURLParameter(param.url);
		this.saveTo = parameters.getObligatedNonEmptyStringParameter(param.saveTo);
	}

	@Override
	protected void doAction() {
		System.out.println("This is a new version of HttpDataFetcherBehavior which is not intensively tested, errors could occur.");
		if (!this.started) {
			this.finished = false; //we don't have the result immediately, we want to be cyclic
			new Thread(new RabRunnable(this.myAgent) {

				@Override
				public void run() {

					try {
						final String result = HttpDataFetcherBehavior.this.fetch(HttpDataFetcherBehavior.this.uri);

						UbiwareAgentCommand<Void> command = new UbiwareAgentCommand<Void>() {

							@Override
							public void performCommand(UbiwareAgent agent) {
								boolean ok = RABUtils.putResult(agent, result, HttpDataFetcherBehavior.this.saveTo);
								if (!ok) {
									HttpDataFetcherBehavior.this.setFailed();
								}
								HttpDataFetcherBehavior.this.finished = true;
								HttpDataFetcherBehavior.this.restart();
							}

							@Override
							public Void getResult() {
								return null;
							}

						};
						this.addCommand(command);

					} catch (IOException ex) {
						HttpDataFetcherBehavior.this.print("failed to retrieve " + HttpDataFetcherBehavior.this.uri);
						HttpDataFetcherBehavior.this.setFailed();
						HttpDataFetcherBehavior.this.finished = true;
						HttpDataFetcherBehavior.this.restart();
					}
				}
			}).start();
			this.started = true;
		}

		this.block();
	}

	private String fetch(URL aURL) throws IOException {
		final String fNEWLINE = System.getProperty("line.separator");

		StringBuilder result = new StringBuilder();
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(aURL.openStream()));
			String line = null;
			while ((line = reader.readLine()) != null) {
				result.append(line);
				result.append(fNEWLINE);
			}
		} finally {
			if (reader != null) {
				reader.close();
			}
		}
		return result.toString();

	}

}
