package ubiware.shared;

import jade.core.behaviours.Behaviour;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.UbiwareAgent;
import ubiware.core.IllegalParameterConfigurationException.State;
import ubiware.ontology.param;
import ubiware.shared.agentServer.AgentServerBehaviorServerEvent;
import ubiware.util.StreamingUtils;
import ubiware.util.saplbuilder.Resource;

/**
 * <p>
 * Send a response to a {@link AgentServerBehaviorServerEvent} which resides on the blackboard of the {@link UbiwareAgent} The behavior
 * tries to respond to the event, if an {@link IOException} occurs the event will still be removed after execution! The actual responding to
 * the event happens in a separate thread to allow the {@link UbiwareAgent} to continue it's work.
 * </p>
 * 
 * 18.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class HttpResponseSenderBehavior extends ReusableAtomicBehavior {

	private static final long serialVersionUID = 1L;
	/**
	 * From which size should the sender start to use a thread for the actual sending of data?
	 */
	private static final long THREADSTARTLIMIT = 5000;
	/**
	 * The message which will be send back, if null, fileName isn't null
	 */
	private String message;
	/**
	 * The file which will be send back, if null, message isn't null
	 */
	private File file;
	/**
	 * The event on which to reply
	 */
	private AgentServerBehaviorServerEvent event;
	/**
	 * the ID of the event on the blackboard of the agent.
	 */
	private Resource eventID;
	/**
	 * Has this {@link Behaviour} been started before?
	 */
	private boolean started = false;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		if (this.started) {
			//the behavior has been started before, no need to reinitialize parameters.
			return;
		}
		this.message = parameters.getOptionalStringParameter(param.message);
		this.file = parameters.getOptionalFileParameter(param.messageFromFile);
		if ((this.message == null) && (this.file == null)) {
			throw new IllegalParameterConfigurationException(param.message, this.message, State.missing);
		}
		if ((this.message != null) && (this.file != null)) {
			throw new IllegalParameterConfigurationException(param.messageFromFile, this.file.getName(), State.conflicting);
		}
		//this.eventID = parameters.getObligatedNonEmptyStringParameter(param.socket);
		this.eventID = parameters.getObligatedResourceParameter(param.socket);
		Object object = this.getFromBlackboard(this.eventID);
		if (object == null) {
			throw new IllegalParameterConfigurationException(param.socket, this.eventID.toString(), State.notOnBlackboard);
		}
		//System.out.println("response started for " + this.file.toString() + " ::::" + this.eventID);
		//System.out.println("#######################################################");
	}

	@Override
	protected void doAction() {
		this.block();//don't call this behavior a second time.
		if (this.started) {
			//the behavior has been started before, no need to do anything.
			return;
		}
		this.started = true;
		this.finished = false;//become cyclic, this is because the agent then believes that this behavior is active till its inner thead has finished sending the reply.

		//reply to the request in a separate thread. if the file or the message is longer as HttpResponseSenderBehavior.THREADSTARTLIMIT

		Runnable replyingRunnable = new Runnable() {

			@Override
			public void run() {
				//try sending the response
				this.send();

				//after trying to send the response, we complete the continuation and remove the event from the Blackboard.
				HttpResponseSenderBehavior.this.event.getContinuation().complete();
				HttpResponseSenderBehavior.this.removeFromBlackboard(HttpResponseSenderBehavior.this.eventID);
				//The behavior which started this thread is now finished
				HttpResponseSenderBehavior.this.finished = true;
				//And should be restarted so to make sure that it's status gets checke again
				HttpResponseSenderBehavior.this.restart();
			}

			private void send() {
				if (HttpResponseSenderBehavior.this.message != null) {
					//there is a message
					try {
						//send message to stream
						OutputStream out = HttpResponseSenderBehavior.this.event.getResponse().getOutputStream();
						PrintWriter pw = new PrintWriter(new OutputStreamWriter(out));
						pw.print(HttpResponseSenderBehavior.this.message);
						pw.flush();
						pw.close();

					} catch (IOException e) {
						HttpResponseSenderBehavior.this.setFailed();
						e.printStackTrace();
						return;
					}
				} else {
					//message from file
					//send file as a result to the request:
					OutputStream os;

					try {
						os = HttpResponseSenderBehavior.this.event.getResponse().getOutputStream();
					} catch (IOException e1) {
						HttpResponseSenderBehavior.this.setFailed();
						e1.printStackTrace();
						return;
					}
					try {
						StreamingUtils.sendFileToOutputStream(HttpResponseSenderBehavior.this.file, os);
					} catch (FileNotFoundException e) {
						e.printStackTrace();
						HttpResponseSenderBehavior.this.setFailed();
						return;
					} catch (IOException e) {
						e.printStackTrace();
						HttpResponseSenderBehavior.this.setFailed();
						return;
					}
				}
				//System.out.println("response ended for " + HttpResponseSenderBehavior.this.file.toString() + " ::::" + HttpResponseSenderBehavior.this.eventID);
				//System.out.println("--------------------------------------------------------");
			}

		};
		if (((this.message != null) && (this.message.length() > HttpResponseSenderBehavior.THREADSTARTLIMIT))
				|| ((this.file != null) && (this.file.length() > HttpResponseSenderBehavior.THREADSTARTLIMIT))) {
			//big enough to start a thread
			new Thread(replyingRunnable).start();
		} else {
			//it shouldn't take long, do immediately
			replyingRunnable.run();
		}
	}

}
