package ubiware.shared;

import jade.lang.acl.ACLMessage;

import java.net.URI;
import java.util.HashMap;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.IllegalParameterConfigurationException.State;
import ubiware.core.ProduceN3Options;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.SaplN3Constants;
import ubiware.core.UbiwareAgent;
import ubiware.ontology.param;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.Resource;

public class MessageSenderBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final static URI messageNamespace = URITools.createUncheckedURI("http://ubiware.jyu.fi/messages/sent/");

	//package private to make other message behaviors use the same
	final static UniqueInstanceGenerator messageInstanceGenerator = new UniqueInstanceGenerator("", MessageSenderBehavior.messageNamespace);
	private Resource receiver;
	//private boolean cleanContent;
	private boolean doPrint;
	private boolean doAddBeliefs;
	private boolean contentIsSapl;
	private String messageContent;
	private String messageId;
	private String conversationID;
	private int performative;
	private String ontology;
	private boolean addDefaultBeliefs;
	private String nonDefaultBeliefContainerID;
	private String performativeString;
	private String contentSpecifier;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.receiver = parameters.getObligatedResourceParameter(param.receiver);
		boolean cleanContent = parameters.getOptionalBooleanParameter(param.cleanContent, false);
		this.doPrint = parameters.getOptionalBooleanParameter(param.print, false);

		String addBeliefs = parameters.getOptionalStringParameter(param.addBeliefs);
		if (addBeliefs == null) {
			this.doAddBeliefs = true;
			this.addDefaultBeliefs = true;
		} else {
			if (addBeliefs.equals("true")) {
				this.doAddBeliefs = true;
				this.addDefaultBeliefs = true;
			} else if (addBeliefs.equals("false")) {
				this.doAddBeliefs = false;
				this.addDefaultBeliefs = false;
			} else if (addBeliefs.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
				this.doAddBeliefs = true;
				this.addDefaultBeliefs = false;
				this.nonDefaultBeliefContainerID = addBeliefs;
			}
		}

		Boolean issapl = parameters.getOptionalBooleanParameter(param.contentIsSAPL);
		if (issapl != null) {
			if (issapl) {
				//check needed
				this.contentSpecifier = parameters.getObligatedContainerID(param.content);
			} else {
				//user takes responsibility that this is a literal
				this.contentSpecifier = parameters.getObligatedStringParameter(param.content);
			}
		} else {
			//automatic detection needed
			this.contentSpecifier = parameters.getObligatedStringParameter(param.content);
			if (this.contentSpecifier.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
				issapl = Boolean.TRUE;
			} else {
				issapl = Boolean.FALSE;
			}
		}
		//at this stage, contentSpecifier indicates the literal if issapl is false or the containerID if issapl is true
		this.contentIsSapl = issapl;
		if (!issapl) {
			//FIXME
			this.messageContent = this.contentSpecifier;
		} else {
			//generate s-apl from the context ID
			ProduceN3Options options = new ProduceN3Options(false, true, true, cleanContent);
			this.messageContent = this.produceN3(this.contentSpecifier, options); //parameter content must be a context ID
			if (this.messageContent.equals("")) {
				throw new IllegalParameterConfigurationException(param.content, this.contentSpecifier, State.notInRange,
						"The generated s-apl code is the empty string.");
			}
		}
		this.messageId = MessageSenderBehavior.messageInstanceGenerator.getNewID();
		//if no conversationID is specified by the agent, we use the ID from the message instead.
		this.conversationID = parameters.getOptionalStringParameter(param.conversationID, this.messageId);
		this.performativeString = parameters.getOptionalStringParameter(param.performative, "request");
		this.performative = MessageSenderBehavior.getPerformative(this.performativeString);
		this.ontology = parameters.getOptionalStringParameter(param.ontology, "unspecified");
	}

	@Override
	public void doAction() {

		this.sendMessage();

		if (this.doPrint) {
			this.print("\"" + this.messageContent + "\"" + " to " + this.receiver.toString() + "is sent.");
		}

		if (this.doAddBeliefs) {
			if (this.addDefaultBeliefs) {
				this.addDefaultBeliefs();
			} else {
				this.addNonDefaultBeliefs();
			}
		}
	}

	/**
	 * 
	 */
	private void addNonDefaultBeliefs() {
		HashMap<String, String> vars = new HashMap<String, String>();
		vars.put("messageID", this.messageId);
		vars.put("conversationID", this.conversationID);
		vars.put("performative", this.performativeString);
		vars.put("receiver", this.receiver.toString());
		vars.put("ontology", this.ontology);

		String temp_context = null;
		if (!this.contentIsSapl) {
			vars.put("content", this.messageContent);
		} else {
			vars.put("content", this.contentSpecifier);
		}

		this.addBeliefs(this.nonDefaultBeliefContainerID, vars);
	}

	/**
	 * 
	 */
	private void addDefaultBeliefs() {
		StringBuilder messageReceivedBelief = new StringBuilder();
		messageReceivedBelief.append(this.messageId);
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('<');
		messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
		messageReceivedBelief.append("sent> {");
		//begin conversationID
		messageReceivedBelief.append('<');
		messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
		messageReceivedBelief.append("conversationID> ");
		messageReceivedBelief.append(SaplN3Constants.is);
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('\"');
		messageReceivedBelief.append(this.conversationID);
		messageReceivedBelief.append('\"');
		//end conversationID
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('.');
		messageReceivedBelief.append(' ');
		//begin performative
		messageReceivedBelief.append('<');
		messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
		messageReceivedBelief.append("performative> ");
		messageReceivedBelief.append(SaplN3Constants.is);
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('\"');
		messageReceivedBelief.append(ACLMessage.getPerformative(this.performative).toLowerCase());
		messageReceivedBelief.append('\"');
		//end performative
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('.');
		messageReceivedBelief.append(' ');
		//begin sender
		messageReceivedBelief.append('<');
		messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
		messageReceivedBelief.append("receiver> ");
		messageReceivedBelief.append(SaplN3Constants.is);
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('\"');
		messageReceivedBelief.append(this.receiver.toString());//TODO : should this remain the local name?
		messageReceivedBelief.append('\"');
		//end sender
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('.');
		messageReceivedBelief.append(' ');
		//begin ontology
		messageReceivedBelief.append('<');
		messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
		messageReceivedBelief.append("ontology> ");
		messageReceivedBelief.append(SaplN3Constants.is);
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('\"');
		messageReceivedBelief.append(this.ontology);
		messageReceivedBelief.append('\"');
		//end ontology
		messageReceivedBelief.append(' ');
		messageReceivedBelief.append('.');
		messageReceivedBelief.append(' ');
		//begin content
		messageReceivedBelief.append('<');
		messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
		messageReceivedBelief.append("content> ");
		messageReceivedBelief.append(SaplN3Constants.is);

		if (!this.contentIsSapl) {
			messageReceivedBelief.append(' ');
			messageReceivedBelief.append('\"');
			messageReceivedBelief.append(RABUtils.escapeString(this.messageContent));
			messageReceivedBelief.append('\"');
		} else {
			messageReceivedBelief.append(' ');
			messageReceivedBelief.append('{');
			messageReceivedBelief.append(this.messageContent);
			messageReceivedBelief.append('}');
		}
		messageReceivedBelief.append('}');
		this.addBeliefsN3(messageReceivedBelief.toString());

		this.wakeAgent();
	}

	/**
	 * 
	 */
	private void sendMessage() {
		super.sendMessage(this.performative, this.receiver, this.messageContent, this.ontology, this.conversationID);
	/*	
		ACLMessage message = new ACLMessage(this.performative);
		message.addReceiver(new AID(this.receiver, AID.ISLOCALNAME));
		message.setContent(this.messageContent);
		message.setOntology(this.ontology);
		message.setConversationId(this.conversationID);
		this.myAgent.send(message);
		*/
	}

	private static int getPerformative(String performative) {
		return ACLMessage.getInteger(performative.toUpperCase());
	}
}
