package ubiware.shared;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import org.openrdf.model.BNode;
import org.openrdf.model.Literal;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.rio.ParseErrorListener;
import org.openrdf.rio.RDFHandler;
import org.openrdf.rio.rdfxml.RDFXMLParser;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.SaplN3;
import ubiware.core.UbiwareAgent;
import ubiware.ontology.param;

public class RdfXmlReaderBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private StringBuilder beliefs;
	private String bnNameBase; // for blank nodes
	private HashMap<String, String> namespaces;
	private File datafile;
	private String output;

	private class RDFErrorHandler implements ParseErrorListener {
		@Override
		public void error(String msg, int lineNo, int colNo) {
			RdfXmlReaderBehavior.this.print("Error in parsing RDF/XML: " + msg);
		}

		@Override
		public void fatalError(String msg, int lineNo, int colNo) {
			RdfXmlReaderBehavior.this.print("Fatal error in parsing RDF/XML: " + msg);
		}

		@Override
		public void warning(String msg, int lineNo, int colNo) {
			RdfXmlReaderBehavior.this.print("Warning in parsing RDF/XML: " + msg);
		}
	}

	private class RDFStatementHandler implements RDFHandler {

		@Override
		public void handleNamespace(String prefix, String uri) {
			RdfXmlReaderBehavior.this.namespaces.put(prefix + ":", uri);
		}

		@Override
		public void handleStatement(Statement statement) {
			Resource subject = statement.getSubject();
			URI predicate = statement.getPredicate();
			Value object = statement.getObject();

			String subj = null;
			if (subject instanceof URI) {
				subj = "<" + ((URI) subject).toString() + ">";
			} else if (subject instanceof BNode) {
				subj = RdfXmlReaderBehavior.this.bnNameBase + ((BNode) subject).getID();
			}

			String pred = "<" + predicate.toString() + ">";

			String obj = null;
			if (object instanceof URI) {
				obj = "<" + ((URI) object).toString() + ">";
			} else if (object instanceof BNode) {
				obj = RdfXmlReaderBehavior.this.bnNameBase + ((BNode) object).getID();
			} else if (object instanceof Literal) {
				obj = "\"" + RABUtils.escapeString(((Literal) object).getLabel()) + "\"";
			}
			final StringBuilder beliefsString = RdfXmlReaderBehavior.this.beliefs;
			if (beliefsString.length() > 0) {
				beliefsString.append(". ");
			}
			beliefsString.append(subj);
			beliefsString.append(' ');
			beliefsString.append(pred);
			beliefsString.append(' ');
			beliefsString.append(obj);
		}

		@Override
		public void handleComment(String comment) {
		}

		@Override
		public void startRDF() {
		}

		@Override
		public void endRDF() {
		}
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.datafile = parameters.getObligatedFileParameter(param.input);
		this.output = parameters.getParameterValue(param.saveTo);
		if (this.output == null) {
			this.output = "G";
		}
		if (this.output.equals("G")) {
			this.output = UbiwareAgent.GENERAL_CONTEXT;
		}
	}

	@Override
	public void doAction() {

		int len = (int) this.datafile.length();

		this.beliefs = new StringBuilder(len);
		this.bnNameBase = UbiwareAgent.BLANK_NODE_NS + this.myAgent.getLocalName() + String.valueOf(System.currentTimeMillis()) + "_";
		this.namespaces = new HashMap<String, String>();

		RDFStatementHandler myStatementHandler = new RDFStatementHandler();
		RDFErrorHandler myParseErrorListener = new RDFErrorHandler();

		RDFXMLParser parser = new RDFXMLParser();
		parser.setRDFHandler(myStatementHandler);
		parser.setParseErrorListener(myParseErrorListener);
		parser.setVerifyData(true);
		parser.setStopAtFirstError(false);

		try {
			InputStreamReader iread = new InputStreamReader(new FileInputStream(this.datafile));
			BufferedReader ff = new BufferedReader(iread);
			parser.parse(ff, "");
		} catch (Exception ex) {
			this.print(this.getClass().getName() + " error: Operation failed: " + ex.getClass().getName() + " " + ex.getMessage());
			this.setFailed();
			return;
		}

		//String str = "";
		StringBuilder resultSapl = new StringBuilder();
		for (Entry<String, String> entry : this.namespaces.entrySet()) {
			String prefix = entry.getKey();
			String ns = entry.getValue();
			//str = str + "@prefix " + prefix + " <" + ns + ">. ";
			resultSapl.append("@pefix ");
			resultSapl.append(prefix);
			resultSapl.append(' ');
			resultSapl.append('<');
			resultSapl.append(ns);
			resultSapl.append(">. ");
		}
		/*
		 * replaced by new style for loop
		Iterator it = this.namespaces.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry ent = (Map.Entry) it.next();
			String prefix = ((String) ent.getKey());
			String ns = ((String) ent.getValue());
			str = str + "@prefix " + prefix + " <" + ns + ">. ";
		}
		*/
		resultSapl.append(this.beliefs);
		//String str = resultSapl.toString();

		//SaplN3 sapl = new SaplN3(this.myAgent);
		List<String> code = SaplN3.compile(resultSapl.toString(), "addBeliefs");
		if (code != null) {
			SaplN3.load(code, this.myAgent, this.output, "addBeliefs", true);
		} else {
			this.setFailed();
			return;
		}

		this.wakeAgent();
	}
}
