package ubiware.shared;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.IllegalParameterConfigurationException.State;
import ubiware.ontology.param;

/**
 * from Saplguide.pdf <h3>Full name: ubiware.shared.TextTableReaderBehavior</h3>
 * <p>
 * Action: Reads a text file containing a data-table, for example a CSV file (comma-separatedvalues). Generates a hierarchical belief
 * structure (see below).
 * </p>
 * <p>
 * Inputs:
 * <table border="1">
 * <tr>
 * <th>Name</th>
 * <th>Meaning</th>
 * <th>Mandatory</th>
 * <th>Default</th>
 * </tr>
 * <tr>
 * <td>p:input</td>
 * <td>Name of the file.</td>
 * <td>Yes</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>p:encoding</td>
 * <td>The name of the file encoding. 'default' implies using the default system encoding. For information on encoding names, consult the
 * Java API docs about the class java.nio.charset.Charset. For example, use 'UTF-16' for Unicode.</td>
 * <td>No.</td>
 * <td>'default'</td>
 * </tr>
 * <tr>
 * <td>p:rowSeparator</td>
 * <td>A regular expression to use as separator for data rows. For information on regular expressions, consult the Java API docs about the
 * class java.util.regex.Pattern.</td>
 * <td>No</td>
 * <td>'\\n'</td>
 * </tr>
 * <tr>
 * <td>p:columnSeparator</td>
 * <td>A regular expression to use as separator for data columns.</td>
 * <td>No</td>
 * <td>','</td>
 * </tr>
 * <tr>
 * <td>p:selectColumns</td>
 * <td>A whitespace-separated list of the column indexes (one-based) to include. 'all' implies including all the columns.</td>
 * <td>No</td>
 * <td>'all'</td>
 * </tr>
 * </table>
 * </p>
 * <p>
 * Outputs: Adds the following belief structure:
 * 
 * <pre>
 * &lt;input&gt; :table {&lt;rowNum&gt; :row { &lt;columnNum&gt; :column &lt;value&gt; '} ' }
 * </pre>
 * 
 * </p>
 * <p>
 * Example of usage: Read the first and the second columns from Unicode file some.txt where rows are separated by either comma or a
 * whitespace, then do something based on that.
 * 
 * <pre>
 * {sapl:I sapl:do java:ubiware.shared.TextTableReaderBehavior}
 * sapl:configuredAs {p:input sapl:is some.txt .
 * 34
 * p:columnSeparator sapl:is ",|\\s" .
 * p:encoding sapl:is "UTF-16" .
 * p:selectColumns sapl:is "1 2"}.
 * {some.txt :table {?rid :row {1 :column ?firstname.
 * 2 :column ?lastname}
 * } } =&gt; {...}
 * </pre>
 * 
 * </p>
 * <p>
 * Libraries needed: none.
 * </p>
 * 
 * 24.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */

public class TextTableReaderBehavior extends ReusableAtomicBehavior {

	private File dataFile;
	private String rSeparator;
	private String cSeparator;
	private String encoding;
	private List<Integer> select;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.dataFile = parameters.getObligatedFileParameter(param.input);
		this.rSeparator = parameters.getOptionalStringParameter(param.rowSeparator, "\n");
		this.cSeparator = parameters.getOptionalStringParameter(param.columnSeparator, ",");

		this.encoding = parameters.getOptionalStringParameter(param.encoding, "default");

		String selectCols = parameters.getOptionalStringParameter(param.selectColumns, "all");
		//boolean selective = false;
		this.select = null;
		if (!selectCols.equals("all")) {
			//selective = true;
			this.select = new ArrayList<Integer>();
			String[] sel = selectCols.split("\\s");
			for (String column : sel) {
				int columnNr = 0;
				try {
					columnNr = Integer.parseInt(column) - 1;

				} catch (NumberFormatException e) {
					throw new IllegalParameterConfigurationException(param.selectColumns, selectCols, State.notInRange);
				}
				if (columnNr < 0) {
					throw new IllegalParameterConfigurationException(param.selectColumns, selectCols, State.notInRange);
				}
				this.select.add(Integer.valueOf(columnNr));
			}
		}

	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Override
	public void doAction() {
		//	try {
		InputStreamReader iread = null;
		BufferedReader ff = null;
		try {
			int len = (int) this.dataFile.length();
			char mess[] = new char[len];
			//InputStreamReader iread;
			if (this.encoding.equals("default")) {
				iread = new FileReader(this.dataFile);
			} else {
				iread = new InputStreamReader(new FileInputStream(this.dataFile), this.encoding);
			}
			ff = new BufferedReader(iread);
			int read = ff.read(mess, 0, len);
			String content = new String(mess, 0, read);

			String[] rows = content.split(this.rSeparator);

			StringBuilder beliefs = new StringBuilder(len);
			beliefs.append(this.dataFile.getName() + " :table {");

			boolean foundRow = false;
			int numCols = 1;
			for (int iRow = 0; iRow < rows.length; iRow++) {
				String[] cols = rows[iRow].split(this.cSeparator);
				if (iRow == 0) {
					numCols = cols.length;
					//if(cols.length<numCols) continue;
				}

				String rid = String.valueOf(iRow + 1);
				beliefs.append(((foundRow) ? ". " : "") + rid + " :row {");
				foundRow = true;
				boolean foundColumn = false;
				for (Integer iCol = 0; iCol < cols.length; iCol++) {
					if ((this.select == null) || this.select.contains(iCol)) {
						String cid = String.valueOf(iCol + 1);
						beliefs.append(((foundColumn) ? ". " : "") + cid + " :column \"" + RABUtils.escapeString(cols[iCol].trim()) + "\"");
						foundColumn = true;
					}
				}
				beliefs.append("}");
			}
			beliefs.append("}");

			this.addBeliefsN3(beliefs.toString());
			this.wakeAgent();
		} catch (Exception ex) {
			this.print(this.getClass().getName() + " error: Operation failed");
			ex.printStackTrace();
			this.setFailed();
		} finally {
			try {
				if (ff != null) {
					ff.close();
				} else if (iread != null) {
					iread.close();
				}
			} catch (IOException e) {
				// not much we can do anymore...
				System.err.println("Error while closing streams.");
			}
			//			}
		}

	}
}
