package ubiware.shared;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.SemanticStatement;
import ubiware.ontology.param;

public class XmlWriterBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static class ElementInfo implements Comparable<ElementInfo> {
		String id;
		String info;

		public ElementInfo(String id, String info) {
			this.id = id;
			this.info = info;
		}

		@Override
		public int compareTo(ElementInfo o) {
			return this.id.compareTo(o.id);
		}

		@Override
		public boolean equals(Object obj) {
			if (!(obj instanceof ElementInfo)) {
				return false;
			}
			ElementInfo elementInfo = (ElementInfo) obj;
			return this.id.equals(elementInfo.id);
		}

		@Override
		public int hashCode() {
			return this.id.hashCode();
		}
	}

	private String contextID;
	private String output;
	private boolean indents;
	private boolean sort;
	private String encoding;

	private StringBuilder getAttributes(String forContextID) {
		StringBuilder result = new StringBuilder();
		List<SemanticStatement> vec = this.getStatements(forContextID);
		if (vec != null) {
			Iterator<SemanticStatement> it = vec.iterator();
			while (it.hasNext()) {
				SemanticStatement st = it.next();
				if (st.predicate.equals("<#leaf>") && !st.object.equals("<#empty>") && st.isSubjectContext()) {
					List<SemanticStatement> subvec = this.getStatements(st.subject);
					if ((subvec != null) && (subvec.size() > 0)) {
						for (int i = 0; i < subvec.size(); i++) {
							SemanticStatement attr = subvec.get(i);
							if (attr.predicate.equals("<#attribute>")) {
								result.append(attr.object + "=\"" + st.object + "\" ");
							}
						}
					}
				}
			}
		}
		return result;
	}

	private StringBuilder recursion(String forContextID, int recursionLevel, boolean saplIndents, boolean sortStatements) {
		int indent_size = 4;
		String white = "                                                                                                                        ";
		StringBuilder result = new StringBuilder();
		List<SemanticStatement> statements = this.getStatements(forContextID);
		if (statements != null) {
			List<ElementInfo> elements = new ArrayList<ElementInfo>();
			Iterator<SemanticStatement> it = statements.iterator();
			while (it.hasNext()) {
				SemanticStatement st = it.next();
				if (st.predicate.equals("<#branch>")) {
					List<SemanticStatement> subvec = this.getStatements(st.subject);
					if ((subvec != null) && (subvec.size() > 0)) {
						SemanticStatement tag = subvec.get(0);
						if (tag.predicate.equals("<#tag>")) {
							String attrs = this.getAttributes(st.object).toString().trim();
							String rec_str = this.recursion(st.object, recursionLevel + 1, saplIndents, sortStatements).toString();
							String info = ((saplIndents) ? white.substring(0, (recursionLevel) * indent_size) : "") + "<" + tag.object
									+ ((attrs.length() > 0) ? " " + attrs : "") + ">" + ((saplIndents) ? "\n" : "") + rec_str
									+ ((saplIndents) ? "\n" + white.substring(0, (recursionLevel) * indent_size) : "") + "</" + tag.object
									+ ">";
							elements.add(new ElementInfo(tag.subject, info));
						}
					}
				} else if (st.predicate.equals("<#leaf>")) {
					if (st.object.equals("<#empty>")) {
						List<SemanticStatement> subvec = this.getStatements(st.subject);
						if ((subvec != null) && (subvec.size() > 0)) {
							for (int i = 0; i < subvec.size(); i++) {
								SemanticStatement tag = subvec.get(i);
								if (tag.predicate.equals("<#tag>")) {
									String info = ((saplIndents) ? white.substring(0, (recursionLevel) * indent_size) : "") + "<" + tag.object
											+ "/>";
									elements.add(new ElementInfo(tag.subject, info));
								}
							}
						}
					} else if (!st.isSubjectContext()) {
						String info = ((saplIndents) ? white.substring(0, (recursionLevel) * indent_size) : "") + st.object;
						elements.add(new ElementInfo(st.subject, info));
					}
				}
			}
			if (sortStatements) {
				Collections.sort(elements);
			}
			for (int i = 0; i < elements.size(); i++) {
				result.append(((i > 0) ? "\n" : "") + elements.get(i).info);
			}
		}
		return result;
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.contextID = parameters.getObligatedContainerID(param.input);
		this.output = parameters.getObligatedNonEmptyStringParameter(param.saveTo);
		this.indents = parameters.getOptionalBooleanParameter(param.indents, false);
		this.sort = parameters.getOptionalBooleanParameter(param.sort, true);
		this.encoding = parameters.getParameterValue(param.encoding);
	}

	@Override
	public void doAction() {
		String result = "<?xml version=\"1.0\"" + ((this.encoding == null) ? "" : " encoding=\"" + this.encoding + "\"") + "?>\n"
				+ this.recursion(this.contextID, 0, this.indents, this.sort).toString();

		boolean ok = RABUtils.putResult(this.myAgent, result, this.output);
		if (!ok) {
			this.setFailed();
		}
		this.wakeAgent();
	}
}
