package ubiware.shared.agentServer;

import javax.servlet.Servlet;
import javax.servlet.http.HttpServletRequest;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

/**
 * <P>
 * Implementation of a http server for agents. It's functionality is based on the old ubiware.core.AgentServer but completely re-implemented
 * using <a href="http://jetty.codehaus.org/jetty/">Jetty</a>  It's counterpart is ubiware.shared.HttpResponseSenderBehavior
 * </p>
 * 
 * <p>
 * Running this server results in beliefs added to the agent each time a new request arrives.<br>
 * The belief looks as follows: <CODE>
 * <PRE>requestID "sapl:hasBlackboardID" blackboardID; "sapl:requestsPath" path
 * </PRE></CODE>
 * Where:
 * <UL>
 * <LI>requestID is a random ID in namespace "http://www.ubiware.jyu.fi/id#"</LI>
 * <LI>blackboardID is a random id with which the event's associated object can be found from the blackboard (of type {@link AgentServerBehaviorServerEvent})</LI>
 * <LI>path is as defined in {@link HttpServletRequest#getPathInfo()} but with the first '/' stripped of!</LI>
 * </UL>
 * </P>
 * <P>
 * The server can be started in obsoletemode, which causes the server to behave as close as possible to the old AgentServer implementation.
 * The main difference is that after the belief has been added to the agent, it doesn't get removed if the agent doesn't handle it
 * immediately. The reasoning behind is that every agent starting the server must take the responsibility of handling it's events. The
 * obsoleteMode will probably be removed in the future.
 * </P>
 * 18.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class AgentServerBehavior extends ReusableAtomicBehavior {

	//TODO There is a problem with mobility of this class, it should do something with the server before and after serialization happens
	private static final long serialVersionUID = 1L;
	/**
	 * How long is a request allowed to be in progress by default? (in miliseconds)
	 */
	public static final int REQUEST_TIMEOUT = 300000;//5 minutes
	/**
	 * Has the server been started before? If this is the case, it doesn't have to be started over again.
	 */
	private boolean started = false;
	/**
	 * The port on which the server is listening.
	 */
	private int port;
	/**
	 * Is the server running in Obsolete mode?
	 */
	private boolean obsoleteMode;
	/**
	 * What is the timeout for the requests on the Http server?
	 */
	private int timeout;

	@Override
	protected void doAction() {
		if (!this.started) {
			this.finished = false; // make this behavior cyclic, so it doesn't disappear.
			//start the server
			Server s = new Server(this.port);
			ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
			context.setContextPath("/");
			s.setHandler(context);
			Servlet agentServerBehaviorServlet = null;
			//depending on whether the server is running in obsoleteMode, a different kind of behavior is expected. thus an other servlet is installed.
			if (this.obsoleteMode) {
				//agentServerBehaviorServlet = new AgentServerBehaviorObsoleteServlet(this.myAgent);
				throw new Error("Obsolete mode for AgServerBehavior is not supported any longer.");
			} else {
				agentServerBehaviorServlet = new AgentServerBehaviorServlet(this.myAgent, this.timeout);
			}

			ServletHolder sh = new ServletHolder(agentServerBehaviorServlet);
			context.addServlet(sh, "/*");

			try {
				s.start();
				//this is needed for preventing the VM to exit, do we need this? No we don't 
				//s.join();
			} catch (Exception e) {
				e.printStackTrace();
				//in case an error occurs, the agent has to know it.
				this.finished = true;
				this.setFailed();
				//s.stop(); not needed, it has not been succesfully started.
			}

			this.started = true;
		}
		//Blok this RAB frmo being executed over and over, there is nothing to be done here. The only reason this must still be alive is so the agent knows that its server is running.
		this.block();
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.port = parameters.getObligatedNumericParameter(param.port);
		//the default value for obsoletemode is false
		this.obsoleteMode = parameters.getOptionalBooleanParameter(param.obsoleteMode, false);
		this.timeout = parameters.getOptionalNumericParameter(param.timeOut, AgentServerBehavior.REQUEST_TIMEOUT);
		if (this.obsoleteMode) {
			this.print("AgentServerBehavior is running in obsolete mode, this mode will be removed in future versions of the platform!");
		}
	}

}