package ubiware.startup;

import jade.core.Agent;
import jade.core.Profile;
import jade.core.ProfileImpl;
import jade.core.Runtime;
import jade.util.leap.Properties;
import jade.wrapper.AgentContainer;
import jade.wrapper.AgentController;
import jade.wrapper.StaleProxyException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import org.openrdf.query.BindingSet;
import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.QueryEvaluationException;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.TupleQuery;
import org.openrdf.query.TupleQueryResult;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFParseException;
import org.openrdf.sail.inferencer.fc.ForwardChainingRDFSInferencer;
import org.openrdf.sail.memory.MemoryStore;

import com.google.common.io.Files;

import ubiware.core.UbiwareAgent;
import ubiware.ontology.conf;
import ubiware.util.saplbuilder.Resource;

/*
 Before you read the code:
 1. For each configuration ontology class there is a bean (Java class) that wraps it - look at the end of this file

 2. Each bean instance handles actions related to that particular ontology class instance - examples:
 - PlatformBean has start() method to start the platform
 - ContainerBean has getAgentContainer() method that returns an instance of JADE AgentContainer
 - AgentBean has getUbiwareAgent() method returns an instance of UbiwareAgent class

 3. These bean methods are called in a cascade way:
 - if you call PlatformBean.start(), then within start() method ContainerBean.getAgentContainer() is called and within that method AgentBean.getUbiwareAgent() is called

 4. Configuration ontology classes create a tree structure -> therefore:
 - Inside of PlatformBean constructor all instances of ContainerBean are stored (because containers are children of platform)
 - Similar applies to the relationship ContainerBean-AgentBean, etc.
 - This also means that if you create a PlatformBean object, within the constructor it will create container objects, etc. 
 */

public class StarterRDF {

	// location of startup script for role loading
	public static final String SCRIPT_DB_STARTUP = "DB/startup.sapl";
	//private static final String T_WEB_INTERFACE_AGENT = "http://www.ubiware.jyu.fi/configuration#WebInterfaceAgent";

	// RDF repository and its connection object
	private Repository myRepository;
	private RepositoryConnection con;

	// path to configuration file and ontology file
	private String confFilePath;
	private InputStream ontologyInputStream;
	private boolean startGUI;

	// private String ontologyPath;

	public StarterRDF(String confPath, InputStream ontologyFileInputStream, boolean startGUI) throws RepositoryException {
		// just create an empty repository (with forward chaining reasoner) and
		// get connection to it
		this.confFilePath = confPath;
		this.ontologyInputStream = ontologyFileInputStream;
		this.myRepository = new SailRepository(new ForwardChainingRDFSInferencer(new MemoryStore()));
		this.myRepository.initialize();
		// System.out.println(this.myRepository.isWritable());
		this.con = this.myRepository.getConnection();
		this.startGUI = startGUI;
	}

	private void initialize() throws StartupException {
		// load the ontology and configuration
		File confFile = new File(this.confFilePath);
		try {
			InputStream fis = new FileInputStream(confFile);
			// TODO: let's see if http://something/ makes any difference
			this.con.add(this.ontologyInputStream, "http://something/", RDFFormat.RDFXML);
			this.con.add(fis, "http://something/", RDFFormat.N3);
		} catch (FileNotFoundException e) {
			System.err.println("Cannot find config file: " + confFile.getAbsolutePath());
			throw new StartupException(e);
		} catch (RDFParseException e) {
			throw new StartupException(e);
		} catch (RepositoryException e) {
			throw new StartupException(e);
		} catch (IOException e) {
			throw new StartupException(e);
		}

	}

	private void startPlatform() throws StartupException {
		try {
			// get all platform instances from the configuration file
			Collection<String> platforms = this.giveMeAllPlatforms();
			// we expect exactly one platform instance in the configuration
			if (platforms.size() == 1) {
				PlatformBean pb = new PlatformBean(platforms.iterator().next());
				// start the whole platform
				pb.start();
			} else {
				throw new StartupException("You specified " + platforms.size() + " platforms. You have to specify exactly one platform, not less, not more");
			}
			this.con.close();
		} catch (Exception e) {
			throw new StartupException(e);
		}
	}

	private List<String> giveMeAllPlatforms() throws RepositoryException, MalformedQueryException, QueryEvaluationException {
		TupleQuery tupleQuery = this.con.prepareTupleQuery(QueryLanguage.SPARQL, "PREFIX conf: <http://www.ubiware.jyu.fi/configuration#> select ?p WHERE { ?p a conf:Platform } ");
		TupleQueryResult tupleResult = tupleQuery.evaluate();

		List<String> returnValues = new ArrayList<String>();
		while (tupleResult.hasNext()) {
			BindingSet bs = tupleResult.next();
			returnValues.add(bs.getValue("p").stringValue());
		}
		return returnValues;
	}

	// This is a universal method that gives you a list of ?x URIs that are
	// result of a query "subject predicate ?x"
	private List<String> giveMeAllValues(String subject, Resource predicate) throws RepositoryException, MalformedQueryException, QueryEvaluationException {
		String stringQuery = "PREFIX conf: <http://www.ubiware.jyu.fi/configuration#> select ?val WHERE { <" + subject + "> " + predicate + " ?val} ";
		TupleQuery tupleQuery = this.con.prepareTupleQuery(QueryLanguage.SPARQL, stringQuery);
		TupleQueryResult tupleResult = tupleQuery.evaluate();
		List<String> returnValues = new ArrayList<String>();
		while (tupleResult.hasNext()) {
			BindingSet bs = tupleResult.next();
			returnValues.add(bs.getValue("val").stringValue());
		}
		return returnValues;
	}

	// similar to giveMeAllValues(), just that only one return value is expected
	// if there are more than one value, IllegalStateException is thrown
	private String giveMeSingleValue(String subject, Resource predicate) throws RepositoryException, MalformedQueryException, QueryEvaluationException {
		List<String> results = this.giveMeAllValues(subject, predicate);
		if (results.size() == 1) {
			return results.get(0);
		}
		// TODO: maybe this is not the best exception to be thrown, maybe we
		// should make our own
		throw new IllegalStateException("There are " + results.size() + " values for ?x in: <" + subject + "> <" + predicate + "> ?x. We need exactly one value.");
	}

	private class PlatformBean {
		private List<ContainerBean> containerList;

		public PlatformBean(String uri) throws StartupException {
			List<String> containerNames;
			try {
				containerNames = StarterRDF.this.giveMeAllValues(uri, conf.hasContainer);
				this.containerList = new ArrayList<ContainerBean>(containerNames.size());
				for (String cn : containerNames) {
					ContainerBean contBean = new ContainerBean(cn);
					this.containerList.add(contBean);
				}
			} catch (Exception e) {
				throw new StartupException(e);
			}

		}

		public void start() throws StartupException {
			for (ContainerBean cBean : this.containerList) {
				AgentContainer ac = cBean.getAgentContainer();
				try {
					for (AgentBean aBean : cBean.agentList) {
						AgentController actl;
						actl = ac.acceptNewAgent(aBean.agentName, aBean.getUbiwareAgent());
						actl.start();
					}
				} catch (StaleProxyException e) {
					throw new StartupException(e);
				}
			}
		}

		@Override
		public String toString() {
			return "Platform " + this.hashCode() + " with " + this.containerList.size() + " containers";
		}
	}

	private class ContainerBean {
		private List<AgentBean> agentList = new ArrayList<AgentBean>();
		private boolean isMain = false;
		private int port = -1;

		public ContainerBean(String uri) throws StartupException {
			try {
				// step 1
				this.isMain = Boolean.parseBoolean(StarterRDF.this.giveMeSingleValue(uri, conf.isMain));

				// step 2
				List<String> agentURIs = StarterRDF.this.giveMeAllValues(uri, conf.hasAgent);
				for (String agURI : agentURIs) {
					/*
					 * if (StarterRDF.this.isWebInterfaceAgent(agURI)) {
					 * this.agentList.add(new WebInterfaceAgentBean(agURI)); }
					 * else {
					 */
					this.agentList.add(new AgentBean(agURI));
					// }
				}

				// step 3
				List<String> runsOnPortResult = StarterRDF.this.giveMeAllValues(uri, conf.runsOnPort);
				// i will check the port number only for the main container
				if (this.isMain == true) {
					if (runsOnPortResult.isEmpty()) {
						// TODO: I am not sure if this is correct, maybe there
						// shouldn't be any information about the port
						this.port = 80;
					} else if (runsOnPortResult.size() == 1) {
						this.port = Integer.parseInt(runsOnPortResult.get(0));
					} else {
						throw new StartupException("The runsOnPort property value about container " + uri + " cannot be found/determined");
					}
				}
			} catch (RepositoryException e) {
				throw new StartupException(e);
			} catch (MalformedQueryException e) {
				throw new StartupException(e);
			} catch (QueryEvaluationException e) {
				throw new StartupException(e);
			}
		}

		public AgentContainer getAgentContainer() {
			Runtime r = Runtime.instance();
			Properties pp = new Properties();
			AgentContainer ac;
			pp.setProperty(Profile.FILE_DIR, Files.createTempDir().toString() + File.separator);
			if (this.isMain == true) {
				System.out.println("Starting main container on port " + this.port);
				pp.setProperty(Profile.MAIN_PORT, String.valueOf(this.port));
				if (StarterRDF.this.startGUI) {
					pp.setProperty(Profile.GUI, "true");
				}
				pp.setProperty(Profile.LOCAL_HOST, "");
				ProfileImpl pi = new ProfileImpl(pp);
				//				pi.setParameter(, value)
				ac = r.createMainContainer(pi);
				return ac;
			}
			System.out.println("Starting agent container");
			ProfileImpl pi = new ProfileImpl(pp);
			ac = r.createAgentContainer(pi);
			return ac;
		}

		@Override
		public String toString() {
			return "container " + this.hashCode() + " with " + this.agentList.size() + " agents";
		}

	}

	private class AgentBean {
		protected String agentName;
		protected List<String> saplScriptList = new ArrayList<String>();
		protected List<String> roleList = new ArrayList<String>();

		public AgentBean(String uri) throws StartupException {
			try {
				List<String> scriptURIs = StarterRDF.this.giveMeAllValues(uri, conf.hasScript);
				for (String sURI : scriptURIs) {
					this.saplScriptList.add(StarterRDF.this.giveMeSingleValue(sURI, conf.hasScriptLocation));
				}

				List<String> rolesURIs = StarterRDF.this.giveMeAllValues(uri, conf.hasRole);
				for (String rURI : rolesURIs) {
					this.roleList.add(StarterRDF.this.giveMeSingleValue(rURI, conf.hasRoleName));
				}

				this.agentName = StarterRDF.this.giveMeSingleValue(uri, conf.hasAgentName);
			} catch (Exception e) {
				throw new StartupException(e);
			}
		}

		public Agent getUbiwareAgent() throws StartupException {
			if (this.saplScriptList.size() + this.roleList.size() > 0) {
				if (!this.roleList.isEmpty()) {
					this.saplScriptList.add(StarterRDF.SCRIPT_DB_STARTUP);
				}
				return new UbiwareAgent(new HashSet<String>(this.saplScriptList), new HashSet<String>(this.roleList));
			}
			throw new StartupException("Agent " + this.agentName + " has to have at least one role or at least one sapl script");
		}

		@Override
		public String toString() {
			return this.agentName + "(" + this.saplScriptList + ", " + this.roleList + ")";
		}
	}

	/*
	 * private class WebInterfaceAgentBean extends AgentBean { private final int
	 * port;
	 * 
	 * public WebInterfaceAgentBean(String uri) throws StartupException {
	 * super(uri); try { this.agentName = StarterRDF.this.giveMeSingleValue(uri,
	 * StarterRDF.P_HAS_AGENT_NAME); List<String> sss =
	 * StarterRDF.this.giveMeAllValues(uri, StarterRDF.P_RUNS_ON_PORT); if
	 * (sss.size() == 1) { this.port = Integer.parseInt(sss.get(0)); } else if
	 * (sss.size() > 1) { throw new StartupException(
	 * "You cannot specify more than one port for WebInterfaceAgent! You specified "
	 * + sss.size()); } else { this.port = WebInterfaceAgent.DEFAULT_PORT; } }
	 * catch (Exception e) { throw new StartupException(e); } }
	 * 
	 * @Override public Agent getUbiwareAgent() throws StartupException { return
	 * new WebInterfaceAgent(this.port, new
	 * HashSet<String>(this.saplScriptList)); }
	 * 
	 * @Override public String toString() { return this.agentName + "WIA(" +
	 * this.saplScriptList + ", " + this.roleList + ")"; } }
	 */
	private static final String defaultStartupFileName = "startup.rdf";

	public static void main(String[] args) throws RepositoryException, StartupException, SQLException, IOException {
		// if the original JADE debug GUI should be started
		boolean startGUI = false;
		String startupfile;
		switch (args.length) {
		case 0:
			// if no argument is specified, then we take the default startup file
			startupfile = StarterRDF.defaultStartupFileName;
			break;
		case 1:
			// startup file is the only parameter
			startupfile = args[1];
			break;
		case 2:
			// gui is the first parameter
			if ("-gui".equals(args[0])) {
				startGUI = true;
			} else {
				throw new StartupException("If you specify two arguments, then the first one must be -gui");
			}
			// startup file is the second parameter
			startupfile = args[1];
			break;
		default:
			throw new StartupException("The startup takes maximum 2 command line arguments: [-gui] startup_file");
		}

		InputStream ontologyInStream = StarterRDF.class.getResourceAsStream("/ontologies/configuration.owl");
		if (ontologyInStream == null) {
			throw new Error("configuration ontology not found.");
		}
		StarterRDF starter = new StarterRDF(startupfile, ontologyInStream, startGUI);
		starter.initialize();
		starter.startPlatform();
	}
}
