package ubiware.util;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * <p>
 * Implementation of a logger which catches the output written to the streams returned by getStandardError and getStandardOut and writes
 * this to the interceptor specified.
 * </p>
 * 
 * <p>
 * Use: (example from UbiwareAgent)
 * 
 * <pre>
 * FileOutputStream fout;
 * fout = new FileOutputStream(&quot;lastLog_&quot; + this.here().getName() + &quot;.txt&quot;);
 * Logger log = new Logger(System.out, System.err, fout);
 * System.setOut(log.getStandardOut());
 * System.setErr(log.getStandardError());
 * </pre>
 * 
 * Writes everything which gets printed to either system.out or system.err first to the file.
 * </p>
 * 
 * modified by Michael Cochez for UBIWARE project
 */
public class Logger {
	private final OutputStream oldStdOut;
	private final OutputStream oldStdErr;
	private final PrintStream interceptor;

	public Logger(OutputStream oldStdOut, OutputStream oldStdErr, OutputStream interceptor) {
		super();
		this.oldStdOut = oldStdOut;
		this.oldStdErr = oldStdErr;
		this.interceptor = new PrintStream(interceptor);
	}

	public PrintStream getStandardOut() {
		return new Pipe(this.oldStdOut);
	}

	public PrintStream getStandardError() {
		return new Pipe(this.oldStdErr);
	}

	private class Pipe extends PrintStream {

		/**
		 * Creates instance of class.
		 * 
		 * @param out
		 *            output stream where to write first
		 */
		public Pipe(OutputStream out) {
			super(out);
		}

		/**
		 * Writes data to both old stream and new stream
		 * 
		 * @param b
		 *            array of bytes that contains data to write
		 * @param off
		 *            start position of data in array
		 * @param len
		 *            number of bytes to write
		 */
		@Override
		public void write(byte b[], int off, int len) {
			//synchronization is cared about by PrintStream
			Logger.this.interceptor.write(b, off, len);
			super.write(b, off, len);
		}
	}
}