package ubiware.util;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Implementation which mimics the Python range class. It offers an iterable object which returns integers Use as follows:
 * 
 * <PRE>
 * for (Integer i : new Range(start, end, step){
 * 		//do something  with the integer
 * }
 * </PRE>
 * 
 * 24.6.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class Range implements Iterable<Integer> {
	private final int start;
	private final int end;
	private final int step;

	/**
	 * Same as {@link Range#Range(int, int, int)} with start set to 0 and step set to 1
	 * 
	 * @param end
	 */
	public Range(int end) {
		this(0, end, 1);
	}

	/**
	 * Same as {@link Range#Range(int, int, int)} with start set to start and step set to 1
	 * 
	 * @param end
	 */
	public Range(int start, int end) {
		this(start, end, 1);
	}

	/**
	 * Create an {@link Iterable} which returns an {@link Iterator} which produces a series of integers where
	 * <ul>
	 * <li>
	 * the first element = start</li>
	 * <li>
	 * then n-th element = n-1 th element + step</li>
	 * <li>
	 * The iteration stops before the integer would become greater as end.</li>
	 * 
	 * @param start
	 *            the first element
	 * @param end
	 *            limit
	 * @param step
	 *            the step between every returned element.
	 */
	public Range(int start, int end, int step) {
		this.start = start;
		this.end = end;
		this.step = step;
	}

	@Override
	public Iterator<Integer> iterator() {

		return new Iterator<Integer>() {
			private int nextElement = Range.this.start;

			@Override
			public boolean hasNext() {
				return this.nextElement < Range.this.end;
			}

			@Override
			public Integer next() {
				if (this.nextElement < Range.this.end) {
					int returnValue = this.nextElement;
					this.nextElement += Range.this.step;
					return Integer.valueOf(returnValue);
				} else {
					throw new NoSuchElementException();
				}
			}

			@Override
			public void remove() {
				throw new UnsupportedOperationException("A range is immutable");
			}
		};
	}
}